#include "utils\u_utils.h"
#include "winresrc\rc_def.h" // Ressourcen-IDs
#include "help\help_def.h"   // Help-IDs
#pragma hdrstop

#include "swintrac\m_dlg.h" // fr TCounterWindow

// Interface zur Motorsteuerung
#include "motrstrg\motrstrg.h"

// Interface zur Detektorsteuerung (sehr spartanisch)
#include "detecuse\detecuse.h"

// Dialog automatische Justage
#include "autojust\autojust.h"

#include "autojust\matrix.h"
#include "autojust\transfrm.h"
#include "autojust\m_justag.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

// Globale Definition der Motorachsen
// -> Zugriff durch alle Justage- & Tranformationsobjekte mglich
int nMotorTL= -1, nMotorDF= -1, nMotorCC= -1;

// Globales Flag fr Schreiben in Justagelog
bool bWriteLogfile;

// Statusmeldungen in Prokolldatei fr die Automatische Justage schreiben
void WriteToJustageLog(char *buf)
{
	FILE *hLogFile;

	if ( (hLogFile= fopen("Justage.log", "a+")) == NULL )
		MessageBox(GetFocus(), "WriteToJustageLog: Fehler beim ffnen von Justage.log", "fopen", MBSTOP);

	if ( fwrite(buf, 1, strlen(buf), hLogFile) != strlen(buf) )
		MessageBox(GetFocus(), "WriteToJustageLog: Fehler beim Schreiben in Justage.log", "fwrite", MBSTOP);

	if ( fclose(hLogFile) == EOF )
		MessageBox(GetFocus(), "WriteToJustageLog: Fehler beim Schlieen von Justage.log", "fclose", MBSTOP);
}

//**************** Dialog Automatische Justage*********************************
//**************** Selbststndiges Einstellen der Probe************************

TAutomaticAngleControlDlg::TAutomaticAngleControlDlg( void ): TModalDlg("AutomaticAngleControl", GetMainInstance()),
		status(0), buf(0), buf1(0), m_lnkTimer(0), Transform(0)
{
	m_lnkTimer= new TInterfaceTimer( this, 100 ); //Observer, d.h. Timer benachrichtigt uns

	nMaxString= 32767;
	status= new char[nMaxString];
	if (!status)
	{
		MessageBox("Fehler bei Speicherreservierung", "TAutomaticAngleControlDlg", MBSTOP);
		PostQuitMessage( -1);
	}
	else
	{
		buf= new char[1024];
		if (!buf)
		{
			MessageBox("Fehler bei Speicherreservierung", "TAutomaticAngleControlDlg", MBSTOP);
			PostQuitMessage( -1);
		}
		else
		{
			buf1= new char[1024];
			if (!buf1)
			{
				MessageBox("Fehler bei Speicherreservierung", "TAutomaticAngleControlDlg", MBSTOP);
				PostQuitMessage( -1);
			}
		}
	}
}

TAutomaticAngleControlDlg::~TAutomaticAngleControlDlg() {
	_FREEOBJ(m_lnkTimer);
	_FREELIST(status);
	_FREELIST(buf);
	_FREELIST(buf1);
	_FREEOBJ(Transform);
}

BOOL TAutomaticAngleControlDlg::Dlg_OnInit(HWND hwnd, HWND hwndCtl, LPARAM lParam)
{
	// Inhalt des Status-Textfeldes auslesen
	GetDlgItemText(GetHandle(), ID_Status, status, nMaxString);

	// Globale Variable fr Schreibpflicht in Justagelog
	bWriteLogfile= false;

	//Deaktivierung des Abbruchkriteriums Intensittsdifferenz
	bIntensityDiffActive= false;
	CheckDlgButton(hwnd, ID_ActivateIntenseDiff, FALSE);
	EnableWindow(GetDlgItem(hwnd, ID_TextMaxIntDiff), FALSE);
	EnableWindow(GetDlgItem(hwnd, ID_IntenseDifferenz), FALSE);

	// Test, ob Zhler korrekt im Programm initialisiert wurde
	if ( !TDetectorManager::DetectorManager().DimAvailable(0) )
	{
		strcat(status, "Counterdetektor...fehlt!\r\n");
		// im Fehlerfall wird der START-Button deaktiviert
		EnableWindow(GetDlgItem(hwnd, cm_start_justage), FALSE);
	}
	else
	{
		// verwendeter Zhler fr AutoJustage= akt. Zhler aus der Detectorliste
		Sensor= TDetectorManager::DetectorManager().GetDetector();
		strcat(status, "Counterdetektor...ok!\r\n");
		// Sichergehen, dass Zhlerfenster offen ist,
		// ansonsten werden die Zhlerwerte nicht korrekt eingelesen
		if ( !Sensor->GetCounterWnd() )
		{
			// Zhlerfenster ffnen
			(new TCounterWindow(GetMainInstance(), Sensor))->Show(); 
		}
	}

	// Test, ob der Antrieb "Beugung fein" vorhanden ist
	if ( !mlIsAxisValid(Omega) )
	{
		strcat(status, "Beugung fein...fehlt!\r\n");
		// im Fehlerfall wird der START-Button deaktiviert
		EnableWindow(GetDlgItem(hwnd, cm_start_justage), FALSE);
	}
	else
	{
		strcat(status, "Beugung fein...ok!\r\n");
		// Bestimmung der MotorID zum Antrieb Omega (Beugung fein, DF)
		nMotorDF= mlGetIdByName(Omega);
	}

	// Test, ob der Antrieb "Tilt" vorhanden ist
	if ( !mlIsAxisValid(Psi) )
	{
		strcat(status, "Tilt...fehlt!\r\n");
		// im Fehlerfall wird der START-Button deaktiviert
		EnableWindow(GetDlgItem(hwnd, cm_start_justage), FALSE);
	}
	else
	{
		strcat(status, "Tilt...ok!\r\n");
		// MotorID zuweisen
		nMotorTL= mlGetIdByName(Psi);
	}

	// Test, ob der Antrieb "Kollimator" vorhanden ist
	if ( !mlIsAxisValid(Collimator) )
	{
		strcat(status, "Kollimator...fehlt!\r\n");
		// im Fehlerfall wird der START-Button deaktiviert
		EnableWindow(GetDlgItem(hwnd, cm_start_justage), FALSE);
	}
	else
	{
		strcat(status, "Kollimator...ok!\r\n");
		// MotorID zuweisen
		nMotorCC= mlGetIdByName(Collimator);
	}

	// Abfragen des Fehlerzustands ber (de)aktivierten START-Button
	if ( !IsWindowEnabled(GetDlgItem(hwnd, cm_start_justage)) )
		strcat(status, "Fehler: Es sind nicht alle Gerte bereit!\r\n");

	// Statusmeldung in Status-Textfenster schreiben
	SetDlgItemText(GetHandle(), ID_Status, status);
	return true;
}

void TAutomaticAngleControlDlg::Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	unsigned int rc= 0;

	switch ( id )
	{
		case ID_Logfile:
			// Setzen o. Lschen der Auswahlbox fr die Protokollierung
			rc= IsDlgButtonChecked(GetHandle(), ID_Logfile);
			if(rc == 0)
				bWriteLogfile= false;
			else
				bWriteLogfile= true;
			break;

		case ID_ActivateIntenseDiff:
			// (De)Aktivierung des Parameters Max. Intensittsdifferenz
			rc= IsDlgButtonChecked(GetHandle(),ID_ActivateIntenseDiff);			
			if (rc == 0) 
				bIntensityDiffActive= false;
			else
				bIntensityDiffActive= true;
			// (De)Aktivierung der entsprechenden Dialogelemente
			EnableWindow(GetDlgItem(hwnd, ID_IntenseDifferenz), bIntensityDiffActive);
			EnableWindow(GetDlgItem(hwnd, ID_TextMaxIntDiff), bIntensityDiffActive);
			break;

		case cm_start_justage:
			// START-Button deaktivieren
			EnableWindow(GetDlgItem(hwnd, cm_start_justage), FALSE);
			// Mauszeiger in Sanduhr umwandeln
			SetDefaultCursor( IDC_WAIT );
			// Transformationsobjekt erzeugen
			Transform= new TransformationClass;
			m_NextStep.Clear(); NextStep(jpSetParameters, TRUE); //ALT FORWARD_WM_COMMAND(GetHandle(), cm_set_parameters, 0, 0, PostMessage);
			break;

		case ID_Help:
			// Aufruf der Hilfe
			WinHelp(hwnd, "SPHELP.HLP", HELP_CONTEXT, Help_AutomaticAdjustment);
			break;

		default: // auch 'Beenden'
			// alle weiteren Nachrichten werden von der Default-Methode verarbeitet
			TModalDlg::Dlg_OnCommand(hwnd, id, hwndCtl, codeNotify);
	}
}

void TAutomaticAngleControlDlg::NextStep ( EJustagePoints aNext, BOOL aStartTimer ) {
	m_NextStep.Append(aNext);
	if ( aStartTimer ) m_lnkTimer->StartTimer();
}

void TAutomaticAngleControlDlg::OnTimer ( TBasicTimer *const ) {
	m_lnkTimer->StopTimer(); // Abarbeitung abwarten; nicht parallel arbeiten

	if ( m_NextStep.GetCount()==0 ) return;

	SetDefaultCursor( IDC_WAIT );
	EJustagePoints act= (EJustagePoints)m_NextStep.GetAt(0);
	m_NextStep.Remove(0); // beim nchsten Mal nichts tun
	switch ( act ) {
		case jpSetParameters:
			// Wert fr die Toleranz des "Goldenen Schnitts" aus Dialog holen
			GetDlgItemText(GetHandle(), ID_Toleranz, buf, 4);
			// Vorzeichen eleminieren
			Transform->toleranz= fabs(atof(buf));
			// Bereichsprfung, bei Fehler: Defaultwert
			if ( (Transform->toleranz > 1.0) || (Transform->toleranz < 0.1) )
			{
				// Defaultwert
				Transform->toleranz= 1.0;
			}
			// Rckschreiben des korrigierten Wertes in den Dialog
			sprintf(buf, "%.1f", Transform->toleranz);
			SetDlgItemText(GetHandle(), ID_Toleranz, buf);

			// Wert fr die Anzahl der Algorithmus-Durchlufe aus Dialog holen
			GetDlgItemText(GetHandle(), ID_Durchlauf, buf, 4);
			// gegen Vorzeichen hilft "abs"
			durchlauf= abs(atoi(buf));
			// Bereichsprfung, bei Fehler: Defaultwert
			if ( (durchlauf < 1) || (durchlauf > 7) )
			{
				// Defaultwert
				durchlauf= 5;
			}
			// Rckschreiben des korrigierten Wertes in den Dialog
			sprintf(buf, "%d", durchlauf);
			SetDlgItemText(GetHandle(), ID_Durchlauf, buf);

			// Wert fr die Intensittsdifferenz aus Dialog holen
			// Differenz soll auch als Abbruchkriterium dienen
			GetDlgItemText(GetHandle(), ID_IntenseDifferenz, buf, 8);
			// keine negativen Werte akzeptieren
			fIntenseDifferenz= fabs(atof(buf));
			// Bereichsprfung (untere Schranke)
			if ( fIntenseDifferenz < 1.0 )
			{
				// Defaultwert
				fIntenseDifferenz= 5000.0;
			}
			// Bereichsprfung (obere Schranke)
			if ( fIntenseDifferenz > 10000.0 )
			{
				// Maximalwert
				fIntenseDifferenz= 9999.9f;
			}
			// Rckschreiben des korrigierten Wertes in den Dialog
			sprintf(buf, "%.0f", fIntenseDifferenz);
			SetDlgItemText(GetHandle(), ID_IntenseDifferenz, buf);

			// Werte fr die Intervallgrenze von DF aus dem Dialog auslesen
			GetDlgItemText(GetHandle(), ID_DF_Intervall, buf, 8);
			// Vorzeichen abfangen
			MaxDF= fabs(atof(buf));
			// Bereichsprfung, bei Fehler: Defaultwert
			if ( (MaxDF < 1.0) || (MaxDF > 300.0) )
			{
				// Defaultwert
				MaxDF= 50.0;
			}
			// Rckschreiben des korrigierten Wertes in den Dialog
			sprintf(buf, "%.0f", MaxDF);
			SetDlgItemText(GetHandle(), ID_DF_Intervall, buf);

			// Werte fr die Intervallgrenze von TL aus dem Dialog auslesen
			GetDlgItemText(GetHandle(), ID_TL_Intervall, buf, 8);
			// Vorzeichen abfangen
			MaxTL= fabs(atof(buf));
			// Bereichsprfung, bei Fehler: Defaultwert
			if ( (MaxTL < 1.0) || (MaxTL > 100.0) )
			{
				// Defaultwert
				MaxTL= 20.0;
			}
			// Rckschreiben des korrigierten Wertes in den Dialog
			sprintf(buf, "%.0f", MaxTL);
			SetDlgItemText(GetHandle(), ID_TL_Intervall, buf);

			// Werte fr die Intervallgrenze von CC aus dem Dialog auslesen
			GetDlgItemText(GetHandle(), ID_CC_Intervall, buf, 8);
			// Vorzeichen abfangen
			MaxCC= fabs(atof(buf));
			// Bereichsprfung, bei Fehler: Defaultwert
			if ( (MaxCC < 1.0) || (MaxCC > 500.0) )
			{
				// Defaultwert
				MaxCC= 150.0;
			}
			// Rckschreiben des korrigierten Wertes in den Dialog
			sprintf(buf, "%.0f", MaxCC);
			SetDlgItemText(GetHandle(), ID_CC_Intervall, buf);

			// Werte fr die Anzahl der Messungen pro Intensittsbestimmung
			// aus dem Dialog auslesen
			GetDlgItemText(GetHandle(), ID_MeasureCount, buf , 8);
			// Vorzeichen filtern
			nMeasureCount= fabs(atoi(buf));
			// Bereichsprfung (untere, obere Grenze)
			if ( nMeasureCount < 1 )
			{
				nMeasureCount= 1;
			}
			else if ( nMeasureCount > 6 )
			{
				nMeasureCount= 6;
			}
			// Rckschreiben des korrigierten Wertes in den Dialog
			sprintf(buf, "%d", nMeasureCount);
			SetDlgItemText(GetHandle(), ID_MeasureCount, buf);

			NextStep(jpInitialize); //ALT FORWARD_WM_COMMAND(GetHandle(), cm_initialize, 0, 0, PostMessage);
			break;

		case jpInitialize:
			// Initialisierung des Transform-Objektes mit den Achsen-Suchintervallen
			Transform->Initialize(MaxDF, MaxTL, MaxCC);

			// Softwareschranken fr die Maximumsuche bestimmen und abspeichern
			{
				Wertebereich= Transform->GetOrigPosBorders();
			}
			// Ausgangsintensitt auslesen
			// ohne Ausgabe in Logfile [dazu: MeasureIntensity ohne bWriteLogfile]
			if ( bWriteLogfile )
			{
				bWriteLogfile= false;
				MaxIntensity.Intensity= Transform->MeasureIntensity(nMeasureCount);
				bWriteLogfile= true;
			}
			else
				MaxIntensity.Intensity= Transform->MeasureIntensity(nMeasureCount);

			// Ausgangspositionen der Antriebe, an der die
			// Anfangsintensitt gemessen wurde, abspeichern
			MaxIntensity.TL= Wertebereich.OrigTL;
			MaxIntensity.DF= Wertebereich.OrigDF;
			MaxIntensity.CC= Wertebereich.OrigCC;

			// LastIntensity wird mit Intensitt=0 initialisiert,
			// da noch keine Justage gestartet wurde (Positionen wie MaxIntensity)
			LastIntensity= MaxIntensity;
			LastIntensity.Intensity= 0;

			// Zeit zur Bestimmung der Justagedauer
			GetLocalTime(&datum);
			GetLocalTime(&zeit);

			// Infos ins Logfile schreiben
			if (bWriteLogfile)
			{
				sprintf(buf, "########################################################\n"
						"Automatische Justage gestartet: %02i.%02i.%i %2d:%02d:%02d\n"
						"########################################################\n"
						"Motorpositionen:\n"
						"TL: %.2f Min: %.2f Max: %.2f\n"
						"DF: %.2f Min: %.2f Max: %.2f\n"
						"CC: %.2f Min: %.2f Max: %.2f\n"
						"Intensitt: %.2f\n",
						datum.wDay, datum.wMonth, datum.wYear,
						zeit.wHour, zeit.wMinute, zeit.wSecond,
						Wertebereich.OrigTL, Wertebereich.MinTL, Wertebereich.MaxTL,
						Wertebereich.OrigDF, Wertebereich.MinDF, Wertebereich.MaxDF,
						Wertebereich.OrigCC, Wertebereich.MinCC, Wertebereich.MaxCC,
						MaxIntensity.Intensity);
				WriteToJustageLog(buf);
			}

			// Bestimmung der Justageanfangszeitpunktes (Uhrzeit in Sekunden)
			justageanfang= 3600 * zeit.wHour + 60 * zeit.wMinute + zeit.wSecond;

			// Status-Textfeld auslesen - nur 72 Zeichen, da bei jeder neuen Justage
			// nur der jeweilige Justagevorgang im Statusfenster erscheinen soll
			GetDlgItemText(GetHandle(), ID_Status, status, 72);
			strcat(status, "\r\nAutomatische Justage gestartet.\r\n");
			SetDlgItemText(GetHandle(), ID_Status, status);

			// Durchgangszhler fr Optimierung
			// dient dazu, gerade und ungerade Durchgnge des Algo zu erfassen,
			// um abhngig davon abwechselnd Tilt oder Kollimator zu optimieren
			count= 0;

			// Test auf Intensittsberschreitung des Detektors
			if (!Transform->bIntensityTooHigh)
			{
				// normales Vorgehen
				NextStep(jpChooseAxis); //ALT FORWARD_WM_COMMAND(GetHandle(), cm_choose_axis, 0, 0, PostMessage);
			}
			else
			{
				// Abbruch der Justage
				NextStep(jpPostProcessing); //ALT SendMessage(GetHandle(), WM_COMMAND, cm_post_processing, 0);
			}
			break;

		case jpChooseAxis:
			// Test auf Intensittsberschreitung des Detektors
			if (Transform->bIntensityTooHigh)
			{
				// Abbruch der Justage
				NextStep(jpPostProcessing); //ALT SendMessage(GetHandle(), WM_COMMAND, cm_post_processing, 0);
				break;
			}

			// Ende der Justage ist erreicht bei: count= durchlauf * 2
			// ein Durchlauf des Algorithmus in die Justage von Tilt und Kollimator
			if (count < durchlauf * 2)
			{
				count++;

				{
					// Setzen der aktuellen Achsen-Suchbereiche
					// dazu werden die Bereiche gem den KS-Transformationen umgerechnet
					aktWertebereich= Transform->translate_PosBorders();
				}

				//alle ungeraden Durchgnge (count=1,3,5,etc.) wird CC/DF optimiert
				if (count % 2 == 1)
				{
					NextStep(jpOptimizingCC); //ALT SendMessage(GetHandle(), WM_COMMAND, cm_optimizing_CC, 0);
				}
				//alle geraden Durchgnge (count=2,4,etc.) wird TL/DF optimiert
				else
				{
					NextStep(jpOptimizingTL); //ALT SendMessage(GetHandle(), WM_COMMAND, cm_optimizing_TL, 0);
				}
			}
			else
			{
				// Ende des Algorithmus ist erreicht
				NextStep(jpPostProcessing); //ALT SendMessage(GetHandle(), WM_COMMAND, cm_post_processing, 0);
			}
			break;

		case jpOptimizingCC:
			//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// 2. Teil des Algorithmus (Kollimator/DF)
			//(nach berlegung & Test bei den Physikern ist die Variante
			// 1. CC/DF
			// 2. TL/DF
			// der rsprnglichen vorzuziehen) deshalb ist CC/DF am Anfang
			//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

			// Test auf Intensittsberschreitung des Detektors
			if (Transform->bIntensityTooHigh)
			{
				// Abbruch der Justage
				NextStep(jpPostProcessing); //ALT SendMessage(GetHandle(), WM_COMMAND, cm_post_processing, 0);
				break;
			}

			//(z-Achse)=CC: Such-Intervalle fr Goldenen Schnitt festlegen
			intervall_li= aktWertebereich.MinCC;
			intervall_re= aktWertebereich.MaxCC;

			// # Logging #
			if (bWriteLogfile)
				WriteToJustageLog("----------Optimierung CC:\r\n");

			// Goldener Schnitt: Optimierung der Z-Achse(CC)
			// step liefert die Anzahl der bentigten Intervallteilungen,
			// intervall_re/li liefern das letzte Optimierungsintervall
			step= Transform->Goldener_Schnitt(Laufachse_Z, intervall_li,
											  intervall_re, nMeasureCount);

			NextStep(jpOptimizingDF); //ALT FORWARD_WM_COMMAND(GetHandle(), cm_optimizing_DF, 0, 0, PostMessage);
			break;

		case jpOptimizingTL:
			//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			// 1. Teil des Algorithmus (Tilt/DF)
			// Erklrung, wieso der 1. Teil des Algorithmus an 2. Stelle steht,
			// nachzulesen am Anfang der Justage (2. Teil des Algorithmus)
			//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

			// Test auf Intensittsberschreitung des Detektors
			if (Transform->bIntensityTooHigh)
			{
				// Abbruch der Justage
				NextStep(jpPostProcessing); //ALT SendMessage(GetHandle(), WM_COMMAND, cm_post_processing, 0);
				break;
			}

			//(x-Achse)=TL: Such-Intervalle fr Goldenen Schnitt festlegen
			intervall_li= aktWertebereich.MinTL;
			intervall_re= aktWertebereich.MaxTL;

			//# Logging #
			if (bWriteLogfile)
				WriteToJustageLog("----------Optimierung TL:\r\n");

			// Goldener Schnitt: Optimierung der X-Achse(TL)
			// step liefert die Anzahl der bentigten Intervallteilungen,
			// intervall_re/li liefern das letzte Optimierungsintervall
			step= Transform->Goldener_Schnitt(Laufachse_X, intervall_li,
											  intervall_re, nMeasureCount);

			NextStep(jpOptimizingDF); //ALT FORWARD_WM_COMMAND(GetHandle(), cm_optimizing_DF, 0, 0, PostMessage);
			break;

		case jpOptimizingDF:
			// Nachregeln mit DF
			//(y-Achse)=DF: Such-Intervalle fr Goldenen Schnitt festlegen

			// Test auf Intensittsberschreitung des Detektors
			if (Transform->bIntensityTooHigh)
			{
				// Abbruch der Justage
				NextStep(jpPostProcessing); //ALT SendMessage(GetHandle(), WM_COMMAND, cm_post_processing, 0);
				break;
			}

			//(y-Achse)=DF: Such-Intervalle fr Goldenen Schnitt festlegen
			intervall_li= aktWertebereich.MinDF;
			intervall_re= aktWertebereich.MaxDF;

			// # Logging #
			if (bWriteLogfile)
				WriteToJustageLog("------------Nachregeln DF:\r\n");

			// Goldener Schnitt: Optimierung der Y-Achse(DF)
			// step1 liefert die Anzahl der bentigten Intervallteilungen,
			// intervall_re/li liefern das letzte Optimierungsintervall
			step1= Transform->Goldener_Schnitt(Laufachse_Y, intervall_li,
												intervall_re, nMeasureCount);

			NextStep(jpCalculate); //ALT FORWARD_WM_COMMAND(GetHandle(), cm_calculate, 0, 0, PostMessage);
			break;

		case jpCalculate:
			// aktuelle Positionen der Antriebe bestimmen, Protokollierung
			// und abspeichern in Positionen von ActIntensity

			// Test auf Intensittsberschreitung des Detektors
			if (Transform->bIntensityTooHigh)
			{
				// Abbruch der Justage
				NextStep(jpPostProcessing); //ALT SendMessage(GetHandle(), WM_COMMAND, cm_post_processing, 0);
				break;
			}

			mlSetAxis(nMotorTL);
			mGetDistance(fPosition);
			ActIntensity.TL= fPosition;
			// # Logging #
			sprintf(buf, "Reale Positionen: (Durchlauf %.1f von %d)\r\nTL: %.2f",
					(float)count / 2, durchlauf, fPosition);

			mlSetAxis(nMotorDF);
			mGetDistance(fPosition);
			ActIntensity.DF= fPosition;
			// # Logging #
			sprintf(buf1, "  DF: %.2f", fPosition);
			strcat(buf, buf1);

			mlSetAxis(nMotorCC);
			mGetDistance(fPosition);
			ActIntensity.CC= fPosition;
			// # Logging #
			sprintf(buf1, "  CC: %.2f\r\n", fPosition);
			strcat(buf, buf1);

			// Ausgabe in Statusfenster und LogFile
			GetDlgItemText(GetHandle(), ID_Status, status, nMaxString);
			strcat(status, buf);
			if (bWriteLogfile)
				WriteToJustageLog(buf);

			// aktuelle Intensitt auslesen und in ActIntensity speichern
			ActIntensity.Intensity= Transform->MeasureIntensity(nMeasureCount);
			// # Logging #
			sprintf(buf, "Intensitt: %.2f\r\nSchritte fr Suche: %d\r\n",
					ActIntensity.Intensity, step + step1);

			// Test, ob neues Intensittsmaximum erreicht wurde
			if ( ActIntensity.Intensity > MaxIntensity.Intensity )
			{
				// Falls ja: neue MaximalIntensitt merken
				MaxIntensity= ActIntensity;
				// Infos frs Statusfenster
				sprintf(buf1, " NEUES MAXIMUM GEFUNDEN\r\n");
				strcat(buf, buf1);
			// wenn Abbruch-Kriterium Intensittsdifferenz existiert
			} else if ( bIntensityDiffActive ) {
				// Test, ob akt. Intensitt kleiner als letzte Intensitt
				// dazu: Liegt Abweichung der letzten Intensitt (LastIntensity)
				// von akt. Intensitt (ActIntensity) auerhalb des Limits?
				if ((LastIntensity.Intensity - fIntenseDifferenz) > ActIntensity.Intensity)
				{
					// Falls ja: Zurcksetzen der Information der aktuellen Intensitt
					ActIntensity= LastIntensity;
					// Test, ob absoluter Maximalwert schon frher erreicht wurde
					if ( LastIntensity.Intensity < MaxIntensity.Intensity )
					{
						// Falls ja: alle Int.Variablen auf max. Intensitt setzen
						LastIntensity= MaxIntensity;
						ActIntensity= MaxIntensity;
					}

					// Zurckfahren der Antriebe an Position mit max. Intensitt
					mlSetAxis(nMotorTL);
					mMoveToDistance(LastIntensity.TL);
					while ( !mIsMoveFinish() ) ProcessMessages(1)/* wait until finish */;
							
					mlSetAxis(nMotorDF);
					mMoveToDistance(LastIntensity.DF);
					while ( !mIsMoveFinish() ) ProcessMessages(1)/* wait until finish */;

					mlSetAxis(nMotorCC);
					mMoveToDistance(LastIntensity.CC);
					while ( !mIsMoveFinish() ) ProcessMessages(1)/* wait until finish */;

					// Abbruchmeldung im Status-Textfeld & Logfile ausgeben
					sprintf(buf1, "\r\nLetzter Intensittswert zu niedrig!\r\nAbbruch und Rckkehr zur Maximalpos.\r\nReale Positionen:\r\nTL: %.2f  DF: %.2f  CC: %.2f\r\nIntensitt: %.0f\r\n",
							LastIntensity.TL, LastIntensity.DF,
							LastIntensity.CC, LastIntensity.Intensity);
					strcat(buf, buf1);
					strcat(status, buf);
					SetDlgItemText(GetHandle(), ID_Status, status);

					// ins Logfile schreiben
					if (bWriteLogfile)
						WriteToJustageLog(buf);

					// Ermittlung der Anzahl der Textzeilen im Status-Textfeld
					dwStatusZeilen= SendMessage(GetDlgItem(GetHandle(), ID_Status),
												EM_GETLINECOUNT, 0, 0L);

					// Scrollen des Textes um dwStatusZeilen minus Anzahl neuer Zeilen
					SendMessage(GetDlgItem(GetHandle(), ID_Status), EM_LINESCROLL, 0,
								dwStatusZeilen - 18L);

					// Abbruch des Justagevorgangs (Verlassen der while-Schleife)
					NextStep(jpPostProcessing); //ALT FORWARD_WM_COMMAND(GetHandle(), cm_post_processing, 0, 0, PostMessage);
					break;
				}

				// Ansonsten: die alte Intensitt mit aktueller berschreiben
				LastIntensity= ActIntensity;

			} //Ende von Flagtest bIntensityDiffActive

			// Ausgabe der erreichten Intensitt und ggf. neues Maximum
			strcat(status, buf);
			SetDlgItemText(GetHandle(), ID_Status, status);
			// # Logging #
			if (bWriteLogfile)
				WriteToJustageLog(buf);

			// Ermittlung der Anzahl der Textzeilen des Status-Textfeldes
			dwStatusZeilen= SendMessage(GetDlgItem(GetHandle(), ID_Status),
										EM_GETLINECOUNT, 0, 0L);
			// Scrollen des Textes um dwStatusZeilen Zeilen - Anzahl des neuen Textes
			SendMessage(GetDlgItem(GetHandle(), ID_Status), EM_LINESCROLL, 0,
						dwStatusZeilen - 20L);

			// Durchfhrung der Koordinatentransformation
			Transform->PosVektor.get_XYZ(x2, y2, z2);

			// # Logging #
			sprintf(buf, " PosVektor vor KS-Drehung (%.2f,%.2f,%.2f)\n", x2, y2, z2);

			if (count % 2 == 1)
				//alle ungeraden Durchgnge KS-Drehung X-Achse
				Transform->KoordinatenTransformation(Drehachse_X,
													 Transform->PosVektor);
			else
				//alle geraden Durchgnge KS-Drehung um Z-Achse
				Transform->KoordinatenTransformation(Drehachse_Z,
													 Transform->PosVektor);

			Transform->PosVektor.get_XYZ(x, y, z);
			sprintf(buf1, " neuer PositionsVektor (%.2f,%.2f,%.2f)\n", x, y, z);
			strcat (buf, buf1);
			// ins Logfile schreiben
			if (bWriteLogfile)
				WriteToJustageLog(buf);

			// Protokollierung der Koordinatensystem-Transformation
			Transform->PosVektor= Transform->translate_to_worldpoints(Transform->PosVektor);
			Transform->PosVektor.get_XYZ(x2, y2, z2);
			sprintf(buf, " PosVektor in Weltkoord. (%.2f,%.2f,%.2f)\n", x2, y2, z2);
			Transform->PosVektor= Transform->translate_from_worldpoints(Transform->PosVektor);
			Transform->PosVektor.get_XYZ(x2, y2, z2);
			sprintf(buf1, " PosVektor im letzten KS (%.2f,%.2f,%.2f)\n", x2, y2, z2);
			strcat (buf, buf1);
			// ins Logfile schreiben
			if (bWriteLogfile)
				WriteToJustageLog(buf);

			NextStep(jpChooseAxis); //ALT FORWARD_WM_COMMAND(GetHandle(), cm_choose_axis, 0, 0, PostMessage);
			break;

		case jpPostProcessing:
			// Statusfenster auslesen
			GetDlgItemText(GetHandle(), ID_Status, status, nMaxString);

			// Ende der Justage und Test ob eine Verbesserung gegenber dem
			// vor der Justage eingestellten Wert erreicht wurde
			// aktueller Intensittswert < absolutes Maximum ?
			// Toleranz 1 Prozent (kleine Messungenauigkeiten ausbgeln)
			if (ActIntensity.Intensity + (0.01*MaxIntensity.Intensity) < MaxIntensity.Intensity)
			{
				// falls ja: Zurckfahren der Antriebe an Position mit max. Intensity
				mlSetAxis(nMotorTL);
				mMoveToDistance(MaxIntensity.TL);
				while (!mIsMoveFinish())
					ProcessMessages(1)/* wait until finish */;
				mlSetAxis(nMotorDF);
				mMoveToDistance(MaxIntensity.DF);
				while (!mIsMoveFinish())
					ProcessMessages(1)/* wait until finish */;
				mlSetAxis(nMotorCC);
				mMoveToDistance(MaxIntensity.CC);
				while (!mIsMoveFinish())
					ProcessMessages(1)/* wait until finish */;

				// Infos ins Statusfenster
				sprintf(buf, "\r\nRckkehr zur Maximumposition...\r\nReale Positionen:\r\nTL: %.2f  DF: %.2f  CC: %.2f\r\nIntensitt: %.0f\r\n",
						MaxIntensity.TL, MaxIntensity.DF, MaxIntensity.CC, MaxIntensity.Intensity);
				strcat(status, buf);
				// ins Logfile schreiben
				if (bWriteLogfile)
					WriteToJustageLog(buf);
			}

			// Test auf Intensittsberschreitung des Detektors
			if (!Transform->bIntensityTooHigh)
			{
				// falls kein berlauf stattfand:
				// Nachregelung fr Achse DF

				sprintf(buf, "\r\nIntensittskorrektur fr Antrieb DF...\r\n");
				strcat(status, buf);
				// ins Statusfenster & Logfile schreiben
				SetDlgItemText(GetHandle(), ID_Status, status);
				if (bWriteLogfile)
					WriteToJustageLog(buf);

				Transform->DFCorrection(nMeasureCount, MaxIntensity.DF, MaxIntensity.Intensity);

				sprintf(buf, "Korrektur: DF:%.2f Intensitt:%.0f\r\n\r\n",
						MaxIntensity.DF, MaxIntensity.Intensity);
			}
			else
				sprintf(buf, "\r\nJustage wurde wegen berschreitung \r\nder maximal zulssigen Detektorintensitt \r\nabgebrochen. \r\nBitte vermindern Sie die Spannung \r\ndes Rntgenstrahlgenerators.\r\n\r\n");

			// Berechnung der Justagedauer
			GetLocalTime(&datum);
			GetLocalTime(&zeit);
			justageende= 3600 * zeit.wHour + 60 * zeit.wMinute + zeit.wSecond;
			justagezeit= justageende - justageanfang;
			sprintf(buf1, "Dauer der Justage: %02d:%02d.%02d\r\n",
					justagezeit / 3600, (justagezeit % 3600) / 60, (justagezeit % 3600) % 60);
			strcat(buf, buf1);
			strcat(status, buf);
			// Ins Statusfenster schreiben
			SetDlgItemText(GetHandle(), ID_Status, status);
			// geschriebene Zeilen im Statusfenster ermitteln
			dwStatusZeilen= SendMessage(GetDlgItem(GetHandle(), ID_Status),
										EM_GETLINECOUNT, 0, 0L);
			// entprechend den Scrollbar einstellen, dass das Textende zu sehen ist
			SendMessage(GetDlgItem(GetHandle(), ID_Status), EM_LINESCROLL, 0,
						dwStatusZeilen - 18L);

			// Logdaten ins Logfile schreiben
			if (bWriteLogfile)
			{
				sprintf(buf1, "########################################################\nAutomatische Justage beendet: %02i.%02i.%i %2d:%02d:%02d\n########################################################\r\n",
						datum.wDay, datum.wMonth, datum.wYear,
						zeit.wHour, zeit.wMinute, zeit.wSecond);
				// erst noch die Dauer rausschreiben
				strcat(buf, buf1);
				WriteToJustageLog(buf);
			}

			// Transformationsobjekt zerstren
			_FREEOBJ(Transform);

			SetInfo("Automatische Justage abgeschlossen ...");

			// Start-Button aktivieren
			EnableWindow(GetDlgItem(GetHandle(), cm_start_justage), TRUE);

			// Maus-Sanduhr in Zeiger umwandeln
			SetDefaultCursor( IDC_ARROW );
			// Justage beendet
			break;
	}
	if ( m_NextStep.GetCount() && act!=jpPostProcessing ) { // jetzt erst den nchsten Schritt
		m_lnkTimer->StartTimer(); 
		SetDefaultCursor( IDC_WAIT );
	} else SetDefaultCursor( IDC_ARROW ); // fertig
}

void TAutomaticAngleControlDlg::LeaveDialog(void)
{
	// Text in Programmstatuszeile schreiben
	SetInfo("Automatische Justage verlassen...");

	m_NextStep.Clear();
	NextStep(jpPostProcessing);
	
	// Detektor initialisieren
	Sensor->MeasureStop();
	Sensor->PopSettings();
	Sensor->MeasureStart();

	// Motoren anhalten
	mStopDrive(TRUE);
}
