/*
$Header: /vol/baal-vol3/projekt98/quellen/XCTL_32/DETECUSE/detecmes.cpp,v 1.38 2004/07/08 21:14:33 reinecke Exp $

Projekt   : XCTL
Subsystem : Detektoren
Autor     : Jan Picard <picard@informatik.hu-berlin.de> 2001-2002
          Institut fuer Informatik,
          Humboldt-Universitaet Berlin
Inhalt    : Code fuer die Messadapterklassen
*/

#include "utils\u_utils.h"
#include "winresrc\rc_def.h" // Ressourcen-IDs
#include <fstream.h>
#include <strstrea.h>
#include <ctype.h>
#pragma hdrstop

#include "motrstrg\motrstrg.h"

#include "detecuse\detecmes.h"
#include "detecuse\detecgui.h"
#include "detecuse\detectorcontroller.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//Verwaltungsstrukturen (Treiber)
extern DeviceList DetectorDrivers;
extern ControllerList DetectorControllers;

// time event handling
extern int nEventsToCall;
extern int nEvent;
extern int nEventsCalled;
extern float fEventIntensity;
extern HWND hEventControlWnd;
extern BOOL bEventDataValid;
//! fuer Radicon bei Continuousscan von Diffr.gruppe eingefuehrt
float fEventExposureCounts;

//##############################################################################
//######## Methoden fuer einen generischen Controller ##########################
//########     fuer nulldimensionale Detektoren       ##########################
//##############################################################################
TGenericDetector::TGenericDetector(int id) : TZeroDimDetector(id)
{
	char Ident[20];

	//Initialisieren des zugehrigen Controller-Objekts
	sprintf(Ident, "Device%i", GetId());
	LookUp(Ident);
	itoa(nBaseAddr,Ident,16);
    Hardware= dynamic_cast<TGenericController*>(GetController(GENERIC,Ident,&DetectorControllers,
		                                                       &DetectorDrivers));

	uAskTime= 70;
	exposure.TimeBounds.SetMin(0.01f);
}

TGenericDetector::~TGenericDetector()
{
}

BOOL TGenericDetector::Initialize( void )
{
	if ( Hardware->Init(fTimeCorrection) == FALSE )
	{
		MessageBox( GetFocus( ), "Die Am9513-Controllerkarte "
					"konnte nicht initialisiert werden.",
					"Fehler bei der Hardwarekommunikation", MBINFO );

		return FALSE;
	}

	DWORD param= 250;

	Hardware->IOCTL( iocConfigureAcustic, param );

	SetParameters( );

	return TRUE;
}

void TGenericDetector::LookUp(LPCSTR Section)
{
  char buf[MaxString];

  ::GetPrivateProfileString( Section, "IOAddr", "0x230", buf, MaxString, GetHWFile());
  sscanf( buf, "%x", &nBaseAddr );
  ::GetPrivateProfileString( Section, "TimeCorrection", "1.0", buf, MaxString, GetHWFile());
  sscanf( buf, "%f", &fTimeCorrection );
}

BOOL TGenericDetector::_SetParameters( void )
{
	DWORD exposureTicks= GetExposureSettings().GetExposureTime() * (DWORD)Hardware->GetTicksPerSecond();
	Hardware->IOCTL( iocSetTimeTicks, exposureTicks );

	DWORD exposureCounts= GetExposureSettings().GetExposureCounts();
	Hardware->IOCTL( iocSetCounts, exposureCounts );

	//ap - neu !!!
	Hardware->SetSound( GetSound() );

	return TRUE;
}

int TGenericDetector::MeasureStart( void )
{
	MeasureStop( );

	if ( !GetControlWnd() && !GetCounterWnd() )
	{
		// MessageBox(GetFocus(),"Kein Host.","Startfehler",MBSTOP);
		return R_Failure;
	}

	// Messung starten
	Hardware->IOCTL( iocStartCounting );
	bActive= TRUE;
	SetTimer( hWndFrame, DetectorTimerIdStart + GetId( ), uAskTime, NULL );

	return R_MeasOk;
}

int TGenericDetector::MeasureStop( void )
{
	if ( bActive )
	{
		KillTimer( hWndFrame, DetectorTimerIdStart + GetId( ) );
		Hardware->IOCTL( iocStopCounting );
	}

	bDataValid= FALSE;
	bActive= FALSE;
	return R_MeasOk;
}

int TGenericDetector::PollDetector( void )
{
	float time;
	DWORD counts, ticks;

	KillTimer( hWndFrame, DetectorTimerIdStart + GetId( ) );

	if ( !bActive )
		return R_Failure;

	TExposureSettings oExposureSettings= GetExposureSettings();

	switch ( Hardware->IOCTL( iocIsCountingReady ) )
	{
		case 0:
			SetTimer( hWndFrame, DetectorTimerIdStart + GetId( ), uAskTime, NULL );
			return R_MeasInProcess;

		case 1:      // time counter ready
			Hardware->IOCTL( iocReadCounts, counts );
			Hardware->IOCTL( iocReadTicks, ticks );
			time= ( float ) ticks / Hardware->GetTicksPerSecond( );

			if ( ticks != 0 )
				time= oExposureSettings.GetExposureTime() + time;
			break;

		case 2:      // impulse counter ready
			Hardware->IOCTL( iocReadCounts, counts );
			Hardware->IOCTL( iocReadTicks, ticks );
			time= ( float ) ticks / Hardware->GetTicksPerSecond( );

			if ( counts != 0 )
				counts= oExposureSettings.GetExposureCounts() + counts;
			break;

		case 3:      // both ready
			counts= oExposureSettings.GetExposureCounts();
			time= oExposureSettings.GetExposureTime();
			SetStatus( "BothReady" );
			break;
	}

	if ( time != 0.0 )
		fIntensity= counts / time;
	else
	{
		if ( IDYES == MessageBox( GetFocus( ), "Time= 0.0 \nNeustart ?",
								  "GenericDetector", MBASK ) )
			return R_Failure;
		else
			return 0;
	}

	bActive= FALSE;
	bDataValid= TRUE;

	UpdateViews();

	return R_MeasOk;
}

//##############################################################################
//########## Methoden fuer einen Radicon-Detektor ##############################
//##############################################################################

const TRange<WORD> TRadicon::DacThreshLimits(0, 1024);
const TRange<WORD> TRadicon::VoltageLimits(400, 900);
TRadiconController* TRadicon::TheEventHardware= 0;

TRadicon::TRadicon(int id) : DacThresh(150, 1023), TZeroDimDetector(id)
{
	//Einlesen von Rdd, Rcc, ...
	LoadDetectorSettings();
	
	//Initialisieren des zugehrigen Controller-Objekts
	char szHardwareID[256];
	itoa(Rdd,szHardwareID,16);
	Hardware= dynamic_cast<TRadiconController*>(GetController(RADICON,szHardwareID,&DetectorControllers,&DetectorDrivers));
}

TRadicon::~TRadicon( void )
{
	SaveDetectorSettings();
}

void TRadicon::LoadDetectorSettings()
{
	char buf[MaxString];

	GetPrivateProfileString( "IOAddr", "0x100", buf);
	sscanf( buf, "%x", &Rdd );
	Rcc= Rdd + 1;

	GetPrivateProfileString( "UpperThresh", "950", buf);
	WORD wUpperThresh= DacThreshLimits.ForceIntoRange(atoi(buf));

	GetPrivateProfileString( "LowerThresh", "150", buf);
	WORD wLowerThresh= DacThreshLimits.ForceIntoRange(atoi(buf));

	DacThresh= TRange<WORD>(wLowerThresh, wUpperThresh);

	GetPrivateProfileString( "HighVoltage", "640", buf);
	wHighVoltage= VoltageLimits.ForceIntoRange(atoi( buf ));
}

void TRadicon::SaveDetectorSettings() const
{
	char buf[ MaxString ];

	sprintf( buf, "%d", DacThresh.GetMax() );
	WritePrivateProfileString( "UpperThresh", buf);
	sprintf( buf, "%d", DacThresh.GetMin() );
	WritePrivateProfileString( "LowerThresh", buf);
	sprintf( buf, "%d", wHighVoltage );
	WritePrivateProfileString( "HighVoltage", buf);
}

BOOL TRadicon::Initialize( void )
{
	if (bDebug)
		SetInfo("Initialisieren des Radicon-Detektors");

	TZeroDimDetector::Initialize();

	Hardware->reset();

	if (Hardware->UploadFirmware() != 0 )
		return FALSE;

	if ( SetParameters() == FALSE )
		return FALSE;

	bActive= FALSE;
	return TRUE;
}

BOOL TRadicon::RunSpecificParametersDlg( HWND aParentWnd )
{
	TModalDlg *dlg= new TScsParametersDlg( this );
	if ( dlg ) dlg->ExecuteDialog( aParentWnd );
	_FREEOBJ(dlg);
	return TRUE;
}

void TRadicon::SetHighVoltage(WORD param)
{
	wHighVoltage= VoltageLimits.ForceIntoRange(param);
}

void TRadicon::SetDacThresh(WORD lower, WORD upper)
{
	WORD newLower= DacThreshLimits.ForceIntoRange(lower);
	WORD newUpper= DacThreshLimits.ForceIntoRange(upper);
	DacThresh= TRange<WORD>(newLower, newUpper);
}

int TRadicon::MeasureStart( void )
{
	MeasureStop();

	if ( !GetControlWnd() && !GetCounterWnd() )
		return R_Failure;

	if ( Hardware->Execute( TRadiconController::fixedImpulseCounts ) != 0 )
		return R_Failure;

	dwStartTime= timeGetTime( );

	uAskTime= 300;
	bDataValid= FALSE;
	bActive= TRUE;

	if ( bDebug )
		SetInfo( "MStart" );

	SetTimer( GetFrameHandle( ), DetectorTimerIdStart + GetId( ), uAskTime, NULL );

	return R_MeasOk;
}

int TRadicon::MeasureStop( void )
{
	if ( !bActive )
		return R_MeasOk;

	KillTimer( GetFrameHandle( ), DetectorTimerIdStart + GetId( ) );

	if ( bDebug )
		SetInfo( "MStop" );

	if ( Hardware->Execute( TRadiconController::stop ) != 0)
		return R_Failure;

	bActive= FALSE;
	bDataValid= FALSE;

	return R_MeasOk;
}

int TRadicon::PollDetector( void )
{
	DWORD counts;
	double time;

	KillTimer( GetFrameHandle( ), DetectorTimerIdStart + GetId( ) );
	bActive= FALSE;

	if ( bDebug )
	{
		SetInfo( "MPoll" );
		DelayTime( 10 );
	}

	switch ( Hardware->GetValues(TRadiconController::final, &time, &counts ) )
	{
		case -4: // Zeit-Bedingung
			dRealTime= MillisecondsToSeconds( timeGetTime( ) - dwStartTime );
			Hardware->GetValues( TRadiconController::intermediate, &time, &counts );

			if ( GetExposureSettings().GetExposureTime() > time )
			{
				bActive= TRUE;
				SetTimer( GetFrameHandle( ), DetectorTimerIdStart + GetId( ), uAskTime, NULL );
				return R_MeasInProcess;
			}

			Hardware->Execute( TRadiconController::stop );

			if ( time == 0.0 )
			{
				if ( bDebug )
					SetInfo( "time==0" );

				return R_Failure;
			}

			fIntensity= ( GetExposureSettings().GetExposureCounts() - counts ) / time;
			break;

		case 0: // Impulse-Bedingung
			if ( time == 0.0 )
			{
				if ( bDebug )
					SetInfo( "time==0" );

				return R_Failure;
			}

			fIntensity= GetExposureSettings().GetExposureCounts() / time;
			break;

		default:
			return R_Failure;
	}

	bDataValid= TRUE;
	bActive= FALSE;
	UpdateViews();

	return R_MeasOk;
}

BOOL TRadicon::_SetParameters( void )
{
	if ( Hardware->SetParameters( DacThresh.GetMax(), DacThresh.GetMin(),
								  wHighVoltage, GetExposureSettings().GetExposureTime(),
								  GetExposureSettings().GetExposureCounts(), GetSound() ) != 0 )
		return FALSE;

	DelayTime( 5 );
	return TRUE;
}

void CALLBACK TRadicon::EventHandler( UINT, UINT, DWORD, DWORD, DWORD )
{
	nEventsCalled++;

	DWORD counts;
	double time;
	TheEventHardware->GetValues( TRadiconController::intermediate, &time, &counts);

	if ( time != 0.0 )
	{
		fEventIntensity= (fEventExposureCounts - counts) / time;
		bEventDataValid= TRUE;
	}

	TheEventHardware->Execute( TRadiconController::stop );
	PostMessage( hEventControlWnd, WM_COMMAND, cm_CounterSet, nEventsCalled );

	if ( nEventsCalled == nEventsToCall )
	{
		timeKillEvent( nEvent );
		PostMessage( hEventControlWnd, WM_COMMAND, cm_SteeringReady, 0l );
	}
	else
		TheEventHardware->Execute( TRadiconController::fixedImpulseCounts );
}

BOOL TRadicon::InitializeEvent( HWND hwnd, int ec )
{
	if (hwnd == NULL)
		return FALSE;

	// check calltime
	int EventTime= 1000.0 * GetExposureSettings().GetExposureTime();
	if ( EventTime <= 100 )
	{
		MessageBox( GetFocus( ), "Messzeit wurde zu klein gewhlt !", "InitializeEvent", MBINFO );
		return FALSE;
	}

	SetControlWnd(hwnd);

	nEventsCalled= 0;
	nEventsToCall= ec;
	fEventExposureCounts= GetExposureSettings().GetExposureCounts();
	bDataValid= FALSE;

	// Die Detektor-Hardware ueber eine globale Variable fr die statische
	// EventHandler-Methode zugaengig machen, die sonst keine konkreten
	// Objekte kennt
	TheEventHardware= Hardware;

	if ( bActive )
		Hardware->Execute( TRadiconController::stop );

	if ( Hardware->Execute( TRadiconController::fixedImpulseCounts ) != 0 )
		return FALSE;

	//JP Woher sollten jetzt gueltige Daten kommen?
	bDataValid= TRUE;

	nEvent= timeSetEvent( EventTime, 1, ( LPTIMECALLBACK ) TRadicon::EventHandler, 0, TIME_PERIODIC );

	return TRUE;
}

//#############################################################################
// Testdev
//#############################################################################

const int max_cc= 9;
const int max_tl= 17;
const int max_df= 601;

//! Basis eines zweifach-indirekten Speicherblock-Zugriffs
long*** Testdev::TestdevData::data= 0;

//! Instanzenzaehler fuer die Klasse TestdevData
//! Mit ihm wird festgestellt, wann das statische Klassenobjekt data zerstoert
//! werden kann, weil es nicht mehr verwandt wird
int Testdev::TestdevData::instanceCounter= 0;

//! Der Konstruktor ist fuer das instance counting verantwortlich
Testdev::TestdevData::TestdevData()
{
	if (initData() == FALSE)
		PostQuitMessage( 0 );

	instanceCounter++;
}

//! Der Detruktor gibt bei Bedarf dynamisch Speicher frei
Testdev::TestdevData::~TestdevData()
{
	instanceCounter--;

	// rekursives freigeben des Speichers
	if ( (instanceCounter == 0) && (data != 0) )
	{
		for (int x= 0; x < max_cc; x++)
		{
			for (int y= 0; y < max_tl; y++)
			{
				_FREELIST(data[x][y]);
			}
			_FREELIST(data[x]);
		}
		_FREELIST(data);
	}
}

//! bietet Zugriff auf die Simulationsdaten
long Testdev::TestdevData::value(int x, int y, int z) const
{
	return ( (data == 0) ? 0 : data[x][y][z] );
}

//! liest die Simulationsdaten aus der testdev.dat und erstellt damit eine
//! Datenspeicherstruktur
BOOL Testdev::TestdevData::initData()
{
	if ( data == 0 )
	{
		ifstream file( GetTestDevFile() );

		if (!file.good())
		{
			MessageBox(NULL, "Kann die Datei 'testdev.dat' nicht oeffnen!",
					   "Fehler beim Oeffnen einer Datei", MB_ICONSTOP );
			return FALSE;
		}

		data= new long**[max_cc];
		if (parseCC(file) == FALSE)
		{
			_FREELIST(data);
			return FALSE;
		}
	}
	return TRUE;
}

BOOL Testdev::TestdevData::parseCC(ifstream& file)
{
	const int maxlen= 32;
	char line[maxlen];

	for (int i= 0; i < max_cc; i++)
	{
		file.getline(line, maxlen);
		if (line[0] != 'C')
			return FALSE;

		data[i]= parseTL(file);
		if ( data[i] == 0 )
			return FALSE;
	}
	return TRUE;
}

long** Testdev::TestdevData::parseTL(ifstream& file)
{
	const int maxlen= 32;
	char line[maxlen];

	long** tl_array= new long * [max_tl];

	for (int i= 0; i < max_tl; i++)
	{
		file.getline(line, maxlen);
		if (line[0] != 'T')
		{
			_FREELIST(tl_array);
			return 0;
		}

		tl_array[i]= parseDF(file);
		if ( tl_array[i] == 0 )
		{
			_FREELIST(tl_array);
			return 0;
		}
	}

	return tl_array;
}

long* Testdev::TestdevData::parseDF(ifstream& file)
{
	const int maxlen= 32;
	char line[maxlen];

	long* df_array= new long[max_df];

	for (int i= 0; i < max_df; i++)
	{
		file.getline(line, maxlen);
		if (line[0] != 'D')
		{
			_FREELIST(df_array);
			return 0;
		}

		file.getline(line, maxlen);
		if (line[0] != 'I')
		{
			_FREELIST(df_array);
			return 0;
		}

		df_array[i]= (long)( atof(line + 2) + 0.5 );
	}

	return df_array;
}

//! Der Konstruktor erstellt das Objekt mit den Simulationsdaten
Testdev::Testdev(int id) : TZeroDimDetector(id)
{
	data= new TestdevData();
}

//! Der Destruktor lscht das Objekt mit den Simulationsdaten
Testdev::~Testdev()
{
	_FREEOBJ(data);
}

//! ermittelt den aktuellen Messwert und legt ihn in fIntensity ab
int Testdev::PollDetector( void )
{
	if (!bActive)
		return R_Failure;

	KillTimer(hWndFrame, DetectorTimerIdStart + GetId());

	if (bDebug)
		SetInfo("MPoll");

	fIntensity= CalculateIntensity();
	fSigma= 0.9f * CalculateSigma( fIntensity, 0.9f );

	bDataValid= TRUE;
	bActive= FALSE;
	UpdateViews();

	return R_MeasOk;
}

//! ermittelt die aktuellen Motorpositionen, berechnet die zugehoerige Intensitaet
//! und fuegt Messfehler hinzu
double Testdev::CalculateIntensity() const
{
	// Aktuelle Motorpositionen ermitteln
	double actDF, actTL, actCC, retValue;
	mlGetDistance( mlGetIdByName( Omega ), actDF );
	mlGetDistance( mlGetIdByName( Psi ), actTL );
	mlGetDistance( mlGetIdByName( Collimator ), actCC );

	// Auf Integer-Werte runden
	int DFIndex= ( ( actDF < 0 ) ? actDF - 0.5 : actDF + 0.5 );
	int TLIndex= ( ( actTL < 0 ) ? actTL - 0.5 : actTL + 0.5 );
	int CCIndex= ( ( actCC < 0 ) ? actCC - 0.5 : actCC + 0.5 );

	// Eingrenzung auf das vermessene Gebiet (s. testdev.dat)
	DFIndex= TRange<int>( -300, 300).ForceIntoRange(DFIndex);
	TLIndex= TRange<int>( -40, 40).ForceIntoRange(TLIndex);
	CCIndex= TRange<int>( -280, 280).ForceIntoRange(CCIndex);

	// Vorbereitung der Interpolation
	int TLBias= (TLIndex + 40) % 5;

	// Umsetzung des Winkelwerts in den entsprechenden Feldindex
	DFIndex= DFIndex + 300;
	TLIndex= (TLIndex + 40) / 5;
	CCIndex= (CCIndex + 280) / 70;

	// Auslesen und u.U. Interpolation der korrespondierenden Intensitaet
	if ( TLBias )
		retValue= InterPolate( CCIndex, TLIndex, DFIndex, TLBias );
	else
		retValue= data->value(CCIndex, TLIndex, DFIndex);

	// Simulation "zufaelliger" Schwankungen (Messfehler)
	retValue += (rand() % 10 - 5);

	return retValue;
}

//! Interpolation zwischen zwei existierenden Messwerten
double Testdev::InterPolate( int ccI, int tlI, int dfI, int tlB ) const
{
	double lVal= data->value(ccI, tlI, dfI);
	double rVal= data->value(ccI, tlI + 1, dfI);
	return ((5 - tlB) * lVal + tlB * rVal) / 5.0;
}

//#############################################################################
// TZeroDimSimpleTestDetector
//#############################################################################

TZeroDimSimpleTestDetector::TZeroDimSimpleTestDetector(int id)
		: TZeroDimDetector(id)
{}

int TZeroDimSimpleTestDetector::PollDetector( void )
{
	double angx, angy, angz= 0.0;
	//! static int driftcnt= 1; //im auskomm. Teil
	//! static BOOL dir= TRUE; //im auskomm. Teil

	if ( !bActive )
		return R_Failure;

	KillTimer( hWndFrame, DetectorTimerIdStart + GetId( ) );

	if ( mlIsAxisValid( Omega ) )
	{
		mlGetDistance( mlGetIdByName( Omega ), angx );
		/*
		// Driftsimulation
		angx -= (driftcnt * 0.01104121);
		if((driftcnt == 30) || (driftcnt == 0))
		dir= !dir;
		if(dir)
		driftcnt++;
		else
		driftcnt--;
		*/
	}
	else
		angx= 0.0;

	if ( mlIsAxisValid( Psi ) )
	{
		mlGetDistance( mlGetIdByName( Psi ), angy );
		angy += angx * 0.7 - 20.0;
	}
	else
		angy= 0.0;

	angx= angx * angx / 11.3;
	angy= angy * angy / 19.1;
	// always zero !
	angz= angz * angz / 23.5;

	/*
	if(fabs(ang) > 1e5) ang= 10;
	// 0. Reflex
	fIntensity= 3.2 * 1/(1+pow(ang-2.0,2)*0.02);
	// 1. Reflex
	fIntensity += 3.2 * exp(-pow(ang-70,2)*0.03);
	// 2. Reflex
	// fIntensity += 1.0 * exp(-pow(ang+139,2)*0.007);
	*/
	fIntensity= 1 / ( 1 + 1.8 * pow( angx, 2 ) + 0.001 * pow( angy, 2 ) + \
					   0.001 * pow( angz, 2 ) );

	fIntensity *= 32000;

	fIntensity += ( rand( ) % 80 ) + 270;

	bActive= FALSE;

	fSigma= CalculateSigma( fIntensity, 0.9f );

	bDataValid= TRUE;

	UpdateViews();

	return R_MeasOk;
}

//#############################################################################
// TOneDimTestDetector
//#############################################################################

TOneDimTestDetector::TOneDimTestDetector(int id) : TOneDimDetector(id)
{}


int TOneDimTestDetector::PsdReadOut( EHowReadOutPsd eReadType)
{
	DWORD counts;
	double argum;

	LPDWORD lpdwCountBuf= (LPDWORD)hCountBuf;
	uMaximumChannel= 0;
	dwMaxCounts= 0;
	dwIntegratedCounts= 0;

	double argum2;
	mlGetDistance( mlGetIdByName( Theta ), argum2 );
	argum2 /= GetAngleStep();

	for ( int idx= 0; idx < GetChannelNumber(); idx++ )
	{
		argum= idx - ( GetChannelNumber() / 2 ) + argum2;
		argum= 1.0 / ( 1.0 + 7.2752e-3 * argum * argum );
		counts= ( DWORD ) ( (rand() % 7) + argum * 200 * MillisecondsToSeconds(dwElapsedTime) );
		//! neu akk
		if ( eReadType != AccumulationRead )
			lpdwCountBuf[idx]= 0; // akkumulierter Signalaufbau: AUS
		//! abhaengigkeit der intens. von der bereits vergangenen Messzeit
		lpdwCountBuf[idx]+= counts;
		dwIntegratedCounts += counts;

		// Kanal mit Maximum ermitteln
		CheckMaximumChannel(idx, counts);
	}
	for ( idx; idx < GetBufferSize(); idx++ ) lpdwCountBuf[idx]= 0; // auch Rest der Liste initialisieren - falls GetChannelNumber() vor dem nchsten PsdReadOut() grer wird

	return R_OK;
}

//############################################################################
// TStoePsd
//############################################################################

enum { IoWaitStoe= 42 };

// Der Stoe_PSD wird ueber eine 16Bit PIO kontrolliert.
// Zwei 8255 Chips werden parallel eingesetzt, um eine
// 16bit Ausgabe ueber PORTA und Eingabe ueber PORTB zu realisieren.
// Dabei verarbeitet Chip 1 das low-byte und Chip 2 das high-byte.
enum {
	PIO_BASEADR= 0x300,
	PIO_PORTA= 0x300,     //= PIO_BASEADR
	PIO_PORTB= 0x302,     //= PIO_BASEADR+2
	PIO_PORTC= 0x304,     //= PIO_BASEADR+4
	PIO_CTRL= 0x306,     //= PIO_BASEADR+6

	// PORTC Bit-Definitionen zum Steuern des PSD
	PSD_STOP= 0x000,     // CSR0 & CSR1 =0 : stop PSD
	PSD_CSR0= 0x002,
	PSD_CSR1= 0x004,
	PSD_NDRDY= 0x008,     // New data ready
	PSD_DTRNS= 0x010,     // Data transition
	PSD_INIT= 0x020,     // Init PSD
	PSD_DVAL= 0x100,     // Data valid
	PSD_OVF= 0x200  // Overflow
};

TStoePsd::TStoePsd(int id) : TOneDimDetector(id), hStoeBuf(0) // 30.02.2004 MEMCORRECT (verwende hStoeBuf) , hReadBuf(0)
{
	bLong= FALSE;

	LoadDetectorSettings();
}

TStoePsd::~TStoePsd( void )
{
	/* 30.02.2004 MEMCORRECT if ( hReadBuf )
		HeapFree(GetProcessHeap(),0,hReadBuf);
	hReadBuf= 0;*/
	_FREELIST(hStoeBuf);

	SaveDetectorSettings();
}

void TStoePsd::LoadDetectorSettings()
{
	char buf[ MaxString ];

	GetPrivateProfileString( "BaseAddr", "0x300", buf);
	sscanf( buf, "%x", &nBaseAddr );
}

void TStoePsd::SaveDetectorSettings() const
{
	char aBuf[ MaxString ];

	sprintf( aBuf, "0x%x", nBaseAddr ); // 29.01.2003 Kullmann+Reinecker: %d durch 0x%x ersetzt; entdeckt bei der Physik
	WritePrivateProfileString( "BaseAddr", aBuf);
}

int TStoePsd::Initialize( void )
{
	if ( ! TOneDimDetector::Initialize( ) )
		return FALSE;


	// 30.02.2004 MEMCORRECT hReadBuf= (HGLOBAL)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,(GetChannelNumber()+1) * sizeof(WORD));
	_FREELIST(hStoeBuf);
	hStoeBuf= new WORD[ GetChannelNumber()+1 ];


	return TRUE;
}

int TStoePsd::PollDetector( void )
{
	char buf[ MaxString ];
	int retval= R_MeasOk;

	KillTimer( hWndFrame, DetectorTimerIdStart + GetId( ) );

	bDataValid= FALSE;

	if ( !bActive )
		return R_Failure;

	bActive= FALSE;

	dwElapsedTime= timeGetTime( ) - dwStartTime;

	if ( MillisecondsToSeconds( dwElapsedTime ) < GetExposureSettings().GetExposureTime() )
	{
		retval= R_MeasInProcess;
		bActive= TRUE;

		if ( !bSignalGrowUp )
		{
			//*** Kein stufenweiser Signalaufbau
			SetTimer( hWndFrame, DetectorTimerIdStart + GetId( ), uAskTime, NULL );
			return R_MeasInProcess;
		}

		//*** Stufenweiser Signalaufbau
		dwElapsedTime= timeGetTime( ) - dwStartTime;

		PsdStop( );

		dRealTime= MillisecondsToSeconds( dwElapsedTime );

		if ( bFirstReadOut )
		{
			PsdReadOut( FirstRead );
			bFirstReadOut= FALSE;
		}

		else
			PsdReadOut( AccumulationRead );

		PsdStart( );

		long lWaitTime= ( timeGetTime( ) - dwStartTime ) - dwElapsedTime;

		dwStartTime += lWaitTime;

		SetTimer( hWndFrame, DetectorTimerIdStart + GetId( ), uAskTime, NULL );

		goto SendNotify;
	}

	//*** Messung ist beendet ***
	bDataValid= TRUE;

	PsdStop( );

	dRealTime= MillisecondsToSeconds( dwElapsedTime );

	PsdReadOut( AccumulationRead );

SendNotify:      //*** Benachrichtigen der entsprechenden Fenster ***
	if ( bHardOverflow )
	{
		strcpy( buf, "berlauf Psd ( Hochspannung ? )" );
		MessageBeep( 0 );
		retval= R_HardOverflow;
	}

	if ( dRealTime == 0.0 )
	{
		SetInfo( "dRealTime == 0.0 Fehler Psd" );
		DelayTime( 300 );
		dRealTime= 1.0;
		PsdStop( );
		return R_Failure;
	}

	UpdateViews();

	return retval;
}

int TStoePsd::PsdReadOut( EHowReadOutPsd eReadType )
{
	LPDWORD lpdwCountBuf= (LPDWORD)hCountBuf;

	// 30.02.2004 MEMCORRECT (verwende hStoeBuf) LPWORD lpwReadBuf= (LPWORD)hReadBuf;
	LPWORD lpwReadBuf= (LPWORD)hStoeBuf;

	int retval= PsdRead( bLong, lpwReadBuf );
	if ( retval != R_OK)
		return retval;

	uMaximumChannel= 0;
	dwMaxCounts= 0;
	dwIntegratedCounts= 0;

	for ( int idx= 0;idx < GetChannelNumber(); idx++ )
	{
		if ( eReadType != AccumulationRead )
			lpdwCountBuf[idx]= 0; // akkumulierter Signalaufbau: AUS

		lpdwCountBuf[ idx ] += lpwReadBuf[ GetChannelNumber() - idx - 1 ];
		dwIntegratedCounts += lpdwCountBuf[ idx ];

		CheckMaximumChannel(idx, lpdwCountBuf[ idx ] );
	}
	for ( idx; idx < GetBufferSize(); idx++ ) lpdwCountBuf[idx]= 0; // auch Rest der Liste initialisieren - falls GetChannelNumber() vor dem nchsten PsdReadOut() grer wird

	return retval;
}

int TStoePsd::PsdInit( void )
{
	bReadyForRead= FALSE;
	Delay( IoWaitStoe );
	Hardware.Write(6,(WORD)0x8B82);
	Delay( IoWaitStoe );
	Hardware.Write(4,(WORD)PSD_STOP);
	return R_OK;
}

int TStoePsd::PsdStart( void )
{
	bReadyForRead= FALSE;
	Delay( IoWaitStoe );
	Hardware.Write(4,(WORD)PSD_INIT);
	Delay( IoWaitStoe );
	Hardware.Write(4,(WORD)(PSD_CSR0 | PSD_CSR1));
	return R_OK;
}

int TStoePsd::PsdStop( void )
{
	bReadyForRead= TRUE;
	Delay( IoWaitStoe );

	if ( bDebug )
		return R_OK;

	Hardware.Write(4,(WORD)PSD_STOP);
	Delay( IoWaitStoe );

	WORD wValue= 0;
	Hardware.Read(4,&wValue);
	if (wValue & PSD_OVF)
	{
		bHardOverflow= TRUE;
		return R_HardOverflow;
	}

	return R_OK;
}

#ifndef _WIN32
int TStoePsd::PsdRead( int bIsLong, LPWORD buf )
{
	/*----------------------------------------------------------------------------
	Integer*2 Function PsdRead ( Long, BufPtr )
	Reads collected data from PSD interface.
	integer*2 FirstChannel [ value ]; [bp+14]
	integer*2 LastChannel [ value ]; [bp+12]
	logical*2 Long [ value ]; [bp+10]
	integer*4 BufPtr [ value ]; [bp+6]
	Data transfer is initiated by setting the bits PSD_DTRNS and
	PSD_NDRDY, then the specified channels are read out and stored.
	'BufPtr' is a far pointer to the buffer used.
	'Long'= .false. : data are stored as integer*2
	= .true. : data are stored as integer*4
	Channel numbers range from 0 to 4095 !
	The sequence to read one data word is :
	- set bits PSD_DTRNS ( data transition ),
	PSD_NDRDY ( new data ready ),
	PSD_CSR1
	- output inverted channel address to PORTA
	- clear bit PSD_NDRDY ( new data ready )
	- wait until bit PSD_DVAL ( data valid ) is set
	- read data word from PORTB and store it in Buffer
	- clear bit PSD_DTRNS ( data transition )
	When the data have been read the PSD interface is stopped.
	Returns:= 0 - no error
	> 0 - 'data valid' timeout
	returns current channel number + 1
	----------------------------------------------------------------------------*/ 
	// Symbolische Sprungmarken mit BC 3.1 nicht verfgbar
	int val1, val2;
	Delay( IoWaitStoe );

	// Altes Signal kann eventuell zur Bestimmung der Overflow-
	// Position genutzt werden
	if ( bHardOverflow )
		return R_HardOverflow;

	// Sichern, da je Zyklus nur einmal ausgelesen wird
	if ( !bReadyForRead )
		return 0;
	bReadyForRead= FALSE;

	int FirstCh= GetFirstChannel();
	int LastCh= GetLastChannel();

	_asm {
		// si,di retten
		push di
		push si
		// initiate data transfer
		mov dx, 0x304 // PC
		mov ax, PSD_DTRNS OR PSD_NDRDY
		out dx, ax
		mov bx, FirstCh
		mov cx, LastCh
		sub cx, bx
		inc cx
		mov si, bIsLong
		les di, buf
	next:
		xor ax, ax
		jcxz ready
		mov dx, 0x304 //PC
		mov ax, PSD_DTRNS OR PSD_NDRDY OR PSD_CSR1
		out dx, ax
		// jmp 0
		mov dx, 0x300 // PA
		mov ax, bx
		not ax
		and ax, 0x0FFF
		out dx, ax
		// clear PSD_NDRDY
		mov dx, 0x304 //PC
		mov ax, PSD_DTRNS OR PSD_CSR1
		out dx, ax
		mov val1, 0xFFFF
		mov val2, 15
	read1:
		mov dx, 0x304 //PC
		in ax, dx
		test ax, PSD_DVAL
		jnz read2
		dec val1
		jnz read1
		dec val2
		jnz read1
		mov ax, bx
		inc ax
		jmp short ready
	read2:
		mov dx, 0x302 // PB
		in ax, dx
		stosw
		or si, si
		jz read3
		xor ax, ax
		stosw
	read3:
		mov dx, 0x304 //PC
		mov ax, PSD_NDRDY OR PSD_CSR1 // delete PSD_DTRNS
		out dx, ax
		dec cx
		inc bx
		jmp short next
	ready:
		push ax
		mov dx, 0x304 // PC
		mov ax, PSD_STOP
		out dx, ax
		pop ax
		pop si
		pop di
	}

	return R_OK;
}
#else
int TStoePsd::PsdRead( int bIsLong, LPWORD buf )
{
	return R_OK;
}
#endif

//##############################################################################
//******************* Methoden fuer den BraunPSD ******************************
//##############################################################################

union long_to_bytes
{
	LONG LWert;
	WORD IWert[ 2 ];
	BYTE SWert[ 4 ];
};

struct MessDatenHeader
{
	long HighADCCounts;
	long LowADCCounts;
	long HighEGYCounts;
	long LowEGYCounts;
	long HighPOSCounts;
	long LowPOSCounts;
	long HSumDetCounts;
	long LSumDetCounts;
	long RateADC;
	long RateEGY;
	long RatePOS;
	long RateDET;
	long Messzeit;
	long HVParameter;
	long EndLifeTime;
	long Ueberlauf;
}
Header;

const char* TBraunError[ 40 ][ 40 ] = {
	/*  0 */ {" Detektor 1. anwaehlen" }, 
	/*  1 */ {" Messung abbrechen mit Shutter"}, 
	/*  2 */ {" Messung abbrechen ohne Shutter"}, 
	/*  3 */ {" Kein Datenfluss zum PC "}, 
	/*  4 */ {"Energieskalierung "}, 
	/*  5 */ {"Positionsskalierung "},
	/*  6 */ {"obere Grenze fuer Energiefenster "},
	/*  7 */ {"untere Grenze fuer Energiefenster "},
	/*  8 */ {"Energie-Counts fuer Countstop "},
	/*  9 */ {"Konstante f.Lifetimeberechnungen "},
	/* 10 */ {"mindest notwendigen Impulsabstand "},
	/* 11 */ {"externes Ratemeter selektieren "},
	/* 12 */ {"Multiplexerzeit f. Detektor 1 "},
	/* 13 */ {"HV-Regelung aus "},
	/* 14 */ {"HV-Regelung starten "},
	/* 15 */ {"Real/Lifetimestop "},
	/* 16 */ {"Positions-Counts f. Countstop "},
	/* 17 */ {"Messzeit einstellen "},
	/* 18 */ {"AD-Counts loeschen "},
	/* 19 */ {"Det. Counts loeschen"},
	/* 20 */ {"Positionsdaten loeschen"},
	/* 21 */ {"Messung starten"},
	/* 22 */ {"Messung gestartet ?"},
	/* 23 */ {"Messung gestoppt ?"},
	/* 24 */ {"Positionsdaten loeschen"},
	/* 25 */ {"Positions-Counts loeschen"},
	/* 26 */ {"Energie-Counts loeschen"},
	/* 27 */ {"Energiedaten senden"},
	/* 28 */ {"Positionsdaten senden"},
	/* 29 */ {"Multiplexerzeit f. Detektor 2 "},
	/* 30 */ {"Headerlaenge pruefen"},
	/* 31 */ {"Overflow d. Positionsdaten"},
	/* 32 */ {"Positionsdaten lesen"},
	/* 33 */ {"Hexfile synchronisieren "},
	/* 34 */ {"Hexfile Test auf Hexziffer"},
	/* 35 */ {"Hexfile suchen und ffnen"},
	/* 36 */ {"Hexfile Echo lesen und vergleichen"},
	/* 37 */ {"Hexfile laden mit Handshake"},
	/* 38 */ {"Speicher allokieren "},
	/* 39 */ {"Hardware nicht verfgbar"}
	};

BYTE TBraunPsd::befehl[ 30 ][ 16 ]= {
										 {3, 255, 59, 254},   // 0. Detektor1 angewaehlt
										 {4, 255, 68, 36, 254},   // 1. Messung abbrechen Shutter schliessen
										 {4, 255, 68, 37, 254},   // 2. Messung abbrechen ohne Shutter
										 {3, 255, 48, 254},   // 3. no Data to PC
										 {4, 255, 65, 0, 254},   // 4. Energieskalierung
										 {4, 255, 66, 0, 254},   // 5. Positionsskalierung
										 {5, 255, 80, 0, 0, 254},   // 6. obere Grenz. Energiefen.
										 {5, 255, 81, 0, 0, 254},   // 7. untere " "
										 {9, 255, 144, 255, 255, 255, 255, 255, 239, 254},   //8. E-Counts fuer Stop
										 {5, 255, 87, 115, 5, 254},   // 9. Konstante 0573 hex
										 {4, 255, 64, 0, 254},   // 10. notw. Impulsabstand
										 //{4, 255, 69, 0, 254},   // 11. extern. Ratemeter - computed counts (hp)
										 {4, 255, 69, 1, 254},   // 11. extern. Ratemeter - total counts (hp)
										 {5, 255, 84, 100, 0, 254},   // 12. Muxzeit Det 1. 64 hex
										 {6, 255, 103, 0, 4, 0, 254},   // 13. HV-Regelung aus
										 {6, 255, 103, 0, 4, 4, 254},   // 14. HV-Regelung ein
										 {4, 255, 70, 0, 254},   // 15. Real/Lifetimestop
										 {9, 255, 145, 255, 255, 255, 255, 255, 239, 254},   //16. Positionsstop
										 {6, 255, 96, 127, 150, 152, 254},   // 17. Messzeit =9999999 sec.
										 {3, 255, 53, 254 },   // 18. AD-Counts loeschen
										 {4, 255, 73, 0, 254},   // 19. Det. Counts loeschen
										 {5, 255, 82, 81, 82, 254},   // 20. Energiedaten loeschen
										 {3, 255, 49, 254},   // 21. Messung starten
										 {3, 255, 51, 254},   // 22. Messung gestartet ja ?
										 {3, 255, 51, 254},   // 23. Messung gestoppt ?
										 {5, 255, 83, 81, 83, 254},   // 24. Positionsdaten loeschen
										 {3, 255, 57, 254},   // 25. Positions-Counts loeschen
										 {3, 255, 55, 254},   // 26. Energie-Counts loeschen
										 {4, 255, 67, 0, 254},  	// 27. Energiedaten senden
										 {4, 255, 67, 1, 254},  	// 28. Positionsdaten senden
										 {5, 255, 85, 0, 0, 254} }; // 29. Muxzeit Det 2. 0 hex

BYTE TBraunPsd::echo[ 30 ][ 16 ]= {
									   {2, 255, 59},
									   {2, 255, 68},
									   {2, 255, 68},
									   {2, 255, 48},
									   {4, 255, 65, 254, 0},
									   {4, 255, 66, 254, 0},
									   {6, 255, 80, 254, 0, 254, 0},
									   {6, 255, 81, 254, 0, 254, 0},
									   {14, 255, 144, 254, 255, 254, 255, 254, 255, 254, 255, 254, 255, 254, 239},
									   {6, 255, 87, 254, 115, 254, 5},
									   {4, 255, 64, 254, 0},
									   {4, 255, 69, 254, 0},
									   {6, 255, 84, 254, 100, 254, 0},
									   {7, 255, 103, 0, 254, 4, 254, 0},
									   {7, 255, 103, 0, 254, 4, 254, 4},
									   {4, 255, 70, 254, 0},
									   {14, 255, 145, 254, 255, 254, 255, 254, 255, 254, 255, 254, 255, 254, 239},
									   {7, 255, 96, 127, 254, 150, 254, 152},
									   {2, 255, 53},
									   {2, 255, 73},
									   {2, 255, 82},
									   {2, 255, 49},
									   {4, 255, 51, 254, 1},
									   {4, 255, 51, 254, 0},
									   {2, 255, 83},
									   {2, 255, 57},
									   {2, 255, 55},
									   {4, 255, 67, 254, 0},
									   {4, 255, 67, 254, 1},
									   {6, 255, 85, 254, 0, 254, 0}};

typedef void (*TSetPort) ( WORD, WORD );
typedef BYTE (*TGetPort) ( WORD );
typedef void (*TSetTimeout) ( DWORD );
typedef int (*TGetData ) ( WORD, WORD, WORD, LPLONG, WORD, long& );

TSetPort lpfnSetPort;
TGetPort lpfnGetPort;
TSetTimeout lpfnSetTimeout;
TGetData lpfnGetData;

//******** Funktionen **********************************************************
TBraunPsd::TBraunPsd(int id) : TOneDimDetector(id), Hardware(0) // 31.06.2004 MEMCORRECT hReadBuf(0)
{
	long_to_bytes transform;

	LoadDetectorSettings();

	bSetError= FALSE;

	transform.IWert[ 0 ]= ( WORD ) uMuxTimeDet1;
	befehl[ 12 ][ 3 ]= transform.SWert[ 0 ];
	echo[ 12 ][ 4 ]= transform.SWert[ 0 ];
	befehl[ 12 ][ 4 ]= transform.SWert[ 1 ];
	echo[ 12 ][ 6 ]= transform.SWert[ 1 ];

	if(bRatemeter)
	{	//Ratemeter=1 -> total counts
		befehl[ 11 ][ 3 ]= 1;
		echo[ 11 ][ 4 ]= 1;
	}
	else
	{	//Ratemeter=0 -> computed counts
		befehl[ 11 ][ 3 ]= 0;
		echo[ 11 ][ 4 ]= 0;
	}

	befehl[ 15 ][ 3 ]= ( WORD ) bRealLifeTime;
	echo[ 15 ][ 4 ]= ( WORD ) bRealLifeTime;

	befehl[ 10 ][ 3 ]= ( WORD ) nDeathTime;
	echo[ 10 ][ 4 ]= ( WORD ) nDeathTime;

	/* 31.06.2004 MEMCORRECT hReadBuf= (HGLOBAL)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,GetBufferSize() * sizeof(DWORD));
	if ( NULL == hReadBuf )
	{
		bSetError= TRUE;
		nErrorCode= 38;
		// Message
		return;
	}*/
}

TBraunPsd::~TBraunPsd( void )
{
	char HeadLine[]= "Braun-Psd: Destruktor";
	int ExitList[ 3 ]= {2, 3, 13};

	bSetError= FALSE;

	for ( int i= 0;i < 3;i++ )
		if ( 0 != BuildOperation( &( befehl[ ExitList[ i ] ][ 0 ] ), &( echo[ ExitList[ i ] ][ 0 ] ), nDelayFast ) )
		{
			nErrorCode= ExitList[ i ];
			MessageBox( GetFocus( ), TBraunError[ nErrorCode ][ 0 ], HeadLine, MBFAILURE );
			bSetError= TRUE;
			break;
		}

	/* 31.06.2004 MEMCORRECT if (hReadBuf)
		HeapFree(GetProcessHeap(),0,hReadBuf);
	hReadBuf= 0;*/

	SaveDetectorSettings();
}

void TBraunPsd::LoadDetectorSettings()
{
	char buf[ MaxString ];

	GetPrivateProfileString( "DelayFast", "2", buf);
	nDelayFast= atoi( buf );

	GetPrivateProfileString( "DelaySlow", "4", buf);
	nDelaySlow= atoi( buf );

	GetPrivateProfileString( "BaseAddr", "0x300", buf);
	sscanf( buf, "%x", &nBaseAddr );

	//Initialisieren des zugehrigen HardwareIo-Objekts
	char szHardwareID[256];
	itoa(nBaseAddr,szHardwareID,16);
	Hardware= dynamic_cast<TBraunPsdController*>(GetController(BRAUN,szHardwareID,&DetectorControllers,&DetectorDrivers));

	GetPrivateProfileString( "AbbruchMitShutter", "0", buf);
	bAbbruchmitShutter= ( 0 < atoi( buf ) );

	GetPrivateProfileString( "EnergyLow", "526", buf);
	UINT lg= (UINT) atoi( buf );

	GetPrivateProfileString( "EnergyHigh", "870", buf);
	UINT hg= (UINT) atoi( buf );
	// bentigt <uEnergyScale> und <nDelaySlow>, <nDelayFast>
	SetEnergyRange(lg, hg);

	GetPrivateProfileString( "MuxTimeDet1", "40", buf);
	uMuxTimeDet1= atoi( buf );
	if ( uMuxTimeDet1 > 0xEFFF )
		uMuxTimeDet1= 0x64;

	GetPrivateProfileString( "Ratemeter", "1", buf);
	bRatemeter= ( atoi( buf ) != 0 );

	GetPrivateProfileString( "RealLifeTime", "0", buf);
	bRealLifeTime= ( atoi( buf ) != 0);

	GetPrivateProfileString( "DeathTime", "10", buf);
	nDeathTime= atoi( buf );
	if ( nDeathTime > 99 )
		nDeathTime= 10;
}

void TBraunPsd::SaveDetectorSettings() const
{
	char aBuf[ MaxString ];

	sprintf( aBuf, "0x%x", nBaseAddr ); // 29.01.2003 Kullmann+Reinecker: %d durch 0x%x ersetzt; entdeckt bei der Physik
	WritePrivateProfileString( "BaseAddr", aBuf);

	UINT lg, hg;
	GetEnergyRange(lg, hg);
	sprintf( aBuf, "%u", lg ); // umrechnen auf Scale
	WritePrivateProfileString( "EnergyLow", aBuf);

	sprintf( aBuf, "%u", hg ); // umrechnen auf Scale
	WritePrivateProfileString( "EnergyHigh", aBuf);
}

BOOL TBraunPsd::Initialize( void )
{
	if ( ! TOneDimDetector::Initialize( ) )
		return FALSE;

	return TRUE;
}

int TBraunPsd::PsdReadOut( EHowReadOutPsd eReadType )
{ 
	UINT wordstoread;
	char HeadLine[]= "Braun-Psd: PsdReadOut()";
	int retval= R_OK;
	int idx, bid;
	int nDelay= nDelaySlow;
	char *lpdwReadBuf= 0;
	long_to_bytes HVTest;

	bSetError= FALSE;

	// Bei hoher Interrupt-Rate des Controllers ist eine greres Delay zu verwenden
	if ( Header.HighADCCounts < 6000 ) nDelay += 1;
	else if ( Header.HighADCCounts < 8000 ) nDelay += 2;
	else if ( Header.HighADCCounts < 12000 ) nDelay += 3;
	else if ( Header.HighADCCounts < 15000 ) nDelay += 4;
	else nDelay= 5;

	//DelayTime( 1 );
	// sprintf(::buf,"Detektor-Rate: %ld", Header.HighADCCounts);
	// Main.SetInfoLine(::buf);

	bid= ( ( eDataType == PsdEnergyData ) ? 27 : 28);

	if ( BuildOperation( &( befehl ) [ bid ][ 0 ], &( echo ) [ bid ][ 0 ], nDelay ) )
	{
		bSetError= TRUE;
		nErrorCode= bid;
		MessageBox(GetFocus(),TBraunError[nErrorCode][0],HeadLine, MBFAILURE);
		return R_Failure;
	}

	// Daten vom Psd lesen
	IrpParamsGetData DataBuf;

	//24.06.2004 if (eDataType == PsdEnergyData) wordstoread= ( 8192 >> uEnergyScale );
	//24.06.2004 else wordstoread= ( 8192 >> uPositionScale );
	// immer fr Scale= 0 auslesen (Werte werden spter zusammengefasst)
	wordstoread= GetBufferSize()/2; // Gre des reservierten Speichers (in Byte) umrechnen in Word (besteht aus zwei Bytes)

	// 31.06.2003 MEMCORRECT lpdwReadBuf= (LPDWORD)HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,wordstoread * 2);
	lpdwReadBuf= new char[GetBufferSize()+1]; // Speicher in Bytes (zwei Byte = ein Word) - GetBufferSize muss restfrei durch 4 teilbar sein!

	if(!lpdwReadBuf)
	{
		bSetError= TRUE;
		nErrorCode= 1;
		return R_Failure;
	}
	for (idx= 0; idx<GetBufferSize()+1; idx++) lpdwReadBuf[idx]= 0; // sicherheitshalber initialisieren, falls nicht alle Messwerte gelesen werden knnen
	DataBuf.pBuffer= (PUCHAR)lpdwReadBuf;
	DataBuf.MaxCount= wordstoread;

	//Daten vom PSD einlesen
	if ( Hardware ) Hardware->GetData(&DataBuf);

	if ( !Hardware || DataBuf.MaxCount != wordstoread)
	{
		/* 31.06.2004 MEMCORRECT if(lpdwReadBuf)
			HeapFree(GetProcessHeap(),0,lpdwReadBuf);
		lpdwReadBuf= 0;*/
		_FREELIST(lpdwReadBuf);

		bSetError= TRUE;

		if ( !Hardware ) nErrorCode= 39;
		else nErrorCode= 31;
		MessageBox(GetFocus(),TBraunError[nErrorCode][0],HeadLine, MBFAILURE);
		return R_Failure;
	}

	// berlauf aufgetreten ?
	memcpy((LPLONG)&Header, (LPLONG)lpdwReadBuf, sizeof(MessDatenHeader)); 

	if (Header.Ueberlauf & 0x80)
	{
		/* 31.06.2004 MEMCORRECT if(lpdwReadBuf)
			HeapFree(GetProcessHeap(),0,lpdwReadBuf);
		lpdwReadBuf= 0;*/
		_FREELIST(lpdwReadBuf);

		nErrorCode= 30;
		bSetError= TRUE;
		MessageBox( GetFocus( ), TBraunError[ nErrorCode ][ 0 ], HeadLine, MBFAILURE );
		bHardOverflow= TRUE;
		return R_HardOverflow;
	};

	if (GetHVControl())
	{
		float val;

		HVTest.LWert= Header.HVParameter;
		val= ( float ) ( int ) ( HVTest.IWert[ 0 ] - HVTest.IWert[ 1 ] ) / HVTest.IWert[ 0 ] * 100.0;
		char buf[ MaxString ];
		sprintf( buf, "dHV : %.1f %%", val );
		SetStatus( buf );
		nHVControl_OK= ( int ) val;
	}
	else
		nHVControl_OK= 0;

	// Speicherreservierung bernehmen
	LPDWORD lpdwCountBuf= (LPDWORD)hCountBuf;

	// 31.06.2004 MEMCORRECT LPSTR lpReadBufOffset= (LPSTR)( lpdwReadBuf + 16 );

	unsigned long *caster= (unsigned long*)( lpdwReadBuf + sizeof(MessDatenHeader) );
	// Anzahl der Kanle fr GetData() setzen
	if ( eDataType != PsdEnergyData ) // Positionsspektrum (Energiespektrum hat immer volle Kanalanzahl)
		caster+= max(0, GetFirstChannel() << uPositionScale);

	//die ersten 16 Kanle (Sonderkanle) sind schon ausgewertet und lpReadBufOffset entsprechend angepat
	//memcpy( (LPSTR) lpdwCountBuf, lpReadBufOffset,GetChannelNumber() * sizeof(long));
	// 31.06.2004 MEMCORRECT memcpy( (LPSTR) lpdwCountBuf, lpReadBufOffset, (GetChannelNumber()-16) * sizeof(long)); 	

	dwIntegratedCounts= 0l;
	uMaximumChannel= 0; 
	dwMaxCounts= 0; 
	if ( eDataType == PsdEnergyData ) // Energiespektrum
	{
		// ausgelesen wurden Messwerte fr <MaxChannels>-Kanle
		// hier: berspringen von Kanlen, entsprechend <EnergyScale>
		// d.h. bei Scale= 0 wird jeder, bei Scale= 1 nur jeder zweite, bei Scale= 2 nur jeder vierte Messwert bernommen, usw.
		// NDERUNG: Die Messwerte werden nicht bersprungen, es wird das Maximum aus den bersprungenen Messwerten gebildet.
		for ( idx= 0; idx < GetChannelNumber(); idx++ ) 
		{
			// Umrechnung auf Messwert (ggf. Auslassung von Kanlen entsprechend Scale)
			int channel= idx << uEnergyScale; 

			// Intensitt in diesem Kanalfenster
			unsigned long intens= caster[channel];						
			for ( int window= 1; window < (1<<uEnergyScale); window++ ) 
			{
				intens= max( caster[channel+window], intens ); // nehme maximale Intensitt in diesem Fenster
			}

			// Werte permanent bernehmen, integrale Intensitt berechnen und auf Maximalintensitt (und -kanal) prfen
			if ( eReadType != AccumulationRead )
				lpdwCountBuf[idx]= 0; // akkumulierter Signalaufbau: AUS

			lpdwCountBuf[idx]+= intens;
			dwIntegratedCounts+= intens;
			CheckMaximumChannel(idx, lpdwCountBuf[idx]);
		}
	} else { // Positionsspektrum
		// ausgelesen wurden Messwerte fr <MaxChannels>-Kanle
		// hier: Zusammenfassen von Kanlen, entsprechend <PositionScale>
		// d.h. bei Scale= 0 wird ein, bei Scale= 1 werden zwei, bei Scale= 2 werden vier Messwerte zusammengefasst (addiert), usw.
		for ( idx= 0; idx < GetChannelNumber(); idx++ ) 
		{
			// Umrechnung auf Messwert (ggf. Auslassung von Kanlen entsprechend Scale)
			int channel= idx << uPositionScale; 

			// Intensitt in diesem Kanalfenster
			unsigned long intens= caster[channel];						
			for ( int window= 1; window < (1<<uPositionScale); window++ ) 
			{
				intens+= caster[channel+window]; // addiere Messwerte zu einem Gesamtwert
			}

			// Werte permanent bernehmen, integrale Intensitt berechnen und auf Maximalintensitt (und -kanal) prfen
			if ( eReadType != AccumulationRead )
				lpdwCountBuf[idx]= 0; // akkumulierter Signalaufbau: AUSakkumulierter Signalaufbau: AUS
			lpdwCountBuf[idx]+= intens;
			dwIntegratedCounts+= intens;
			CheckMaximumChannel(idx, lpdwCountBuf[idx]);
		}
	}

	for ( idx; idx < GetBufferSize(); idx++ ) lpdwCountBuf[idx]= 0; // auch Rest der Liste initialisieren - falls GetChannelNumber() vor dem nchsten PsdReadOut() grer wird

	/* 31.06.2003 MEMCORRECT if ( lpdwReadBuf )
		HeapFree(GetProcessHeap(), 0, lpdwReadBuf);
	lpdwReadBuf= 0;*/
	_FREELIST(lpdwReadBuf);

	return retval;
}

int TBraunPsd::PsdInit( void )
{
	char HeadLine[ ]= "Braun-Psd: Init()";
	int i;
	int nDelay= nDelayFast;
	const int CntLst= 21;
	//! BOOL FirstTime= FALSE;
	int InitList[ CntLst ]= {1, 2, 3, 0, 4, 5, 6, 7, 8, 16, 9, 20, 10, 11, 12, 13, 14, 15, 17, 26, 29};

	bSetError= FALSE;

	if ( bDebug )
	{
		SetInfo( "Debug BPsd" );
		return R_OK;
	}

	/*// FIX: PSD-INIT nur mit Scale= 0 mglich
	UINT realEnergyScale= SetEnergyScale(0);
	UINT realPositionScale= SetPositionScale(0);*/

	Hardware->SetTimeout(10000);
	ResetDelayTime( );

	if ( LoadHexFile( ) != R_OK )
	{
		MessageBox( GetFocus( ), TBraunError[ nErrorCode ][ 0 ], HeadLine, MBFAILURE );

		/*// FIX: PSD-INIT nur mit Scale= 0 mglich
		SetEnergyScale(realEnergyScale);
		SetPositionScale(realPositionScale);*/

		return R_Failure;
	}

	//warten, bis Hexfile im Controller verarbeitet
	Sleep(1000);

	// entsprechend der Einstellungen im INI-File initialisieren
	for ( i= 0;i < CntLst;i++ )
	{
		if ( ( ! GetHVControl() ) && ( InitList[ i ] == 14 ) )
			continue;

		if ( ( GetHVControl() ) && ( InitList[ i ] == 13 ) )
			continue;

		if ( ( !bAbbruchmitShutter ) && ( InitList[ i ] == 1 ) )
			continue;

		if ( ( bAbbruchmitShutter ) && ( InitList[ i ] == 2 ) )
			continue;

		if ( (InitList[i] == 3) || (InitList[i] == 4) )
			nDelay += 1;

		if ( 0 != BuildOperation( &( befehl ) [ InitList[ i ] ][ 0 ], &( echo ) [ InitList[ i ] ][ 0 ], nDelay ) )
		{
			nErrorCode= InitList[ i ];
			bSetError= TRUE;
			MessageBox( GetFocus( ), TBraunError[ nErrorCode ][ 0 ], HeadLine, MBFAILURE );

			/*// FIX: PSD-INIT nur mit Scale= 0 mglich
			SetEnergyScale(realEnergyScale);
			SetPositionScale(realPositionScale);*/

			return R_Failure;
		}
	}

	// Initialisierung wie im ASA-Programm (nur an dieser Stelle!!!)
	{ // ohne Fehlerabfrage aus PsdStart() bernommen
		int StartList[ 7 ]= { 18, 19, 21, 22, 24, 25, 26};
		int CntLst= 7; // Anzahl der Kommandos
		int nDelay= nDelayFast;
		for (int i= 0; i < CntLst; i++ ) {
			if ( StartList[i]==22 || StartList[i]==24 ) nDelay += 1;
			BuildOperation( &( befehl ) [ StartList[i] ][ 0 ], &( echo ) [ StartList[i] ][ 0 ], nDelay );
		}
	}

	/*// FIX: PSD-INIT nur mit Scale= 0 mglich
	SetEnergyScale(realEnergyScale);
	SetPositionScale(realPositionScale);*/

	return R_OK;
}

int TBraunPsd::PsdStart( void )
{
	char HeadLine[ ]= "Braun-Psd: PsdStart()";
	int StartList[ 8 ]= { /* Konstanter Teil */ 18, 19, 21, 22,
						 /* Variabler Teil */ 24,    25,    26,    20};
	// old {0,18,19,20,24,25,26,21,22};
	int CntLst= 7; // Anzahl der Kommandos
	int nDelay= nDelayFast;

	// Regelung luft wahrscheinlich stndig autark im Controller
	// Mode Energiedaten anzeigen 20
	// Mode Positionsdaten anzeigen 24
	// Regelung sollen Energiedaten nicht gelscht werden

	if ( eDataType == PsdEnergyData )
		CntLst= 8;

	bSetError= FALSE;

	for ( int i= 0; i < CntLst; i++ )
	{
		switch ( StartList[ i ] )
		{
			case 24:
			case 20:
			case 22:
				nDelay += 1;
				break;
		}

		if ( 0 != BuildOperation( &( befehl ) [ StartList[ i ] ][ 0 ], &( echo ) [ StartList[ i ] ][ 0 ], nDelay ) )
		{
			nErrorCode= StartList[ i ];
			bSetError= TRUE;
			MessageBox( GetFocus( ), TBraunError[ nErrorCode ][ 0 ], HeadLine, MBFAILURE );
			return R_Failure;
		}
	}

	return R_OK;
}

int TBraunPsd::PsdStop( void )
{
	char HeadLine[ ]= "Braun-Psd: PsdStop";
	int nDelay= nDelayFast;
	const int CntLst= 3;
	int StopList[ CntLst ]= {2, 1, 23}; // 2,1,23,28

	bSetError= FALSE;

	for ( int i= 0;i < CntLst;i++ )
	{
		if ( StopList[i] == 23 )
			nDelay += 1;

		if ( ( !bAbbruchmitShutter ) && ( StopList[ i ] == 1 ) )
			continue;

		if ( ( bAbbruchmitShutter ) && ( StopList[ i ] == 2 ) )
			continue;

		if ( 0 != BuildOperation( &( befehl ) [ StopList[ i ] ][ 0 ], &( echo ) [ StopList[ i ] ][ 0 ], nDelay ) )
		{
			nErrorCode= StopList[ i ];
			bSetError= TRUE;
			MessageBox( GetFocus( ), TBraunError[ nErrorCode ][ 0 ], HeadLine, MBFAILURE );
			return R_Failure;
		}
	}

	return R_OK;
}

///////////////////////////////////////////////////////////////////////////////
// GET-/ SET-Methoden
///////////////////////////////////////////////////////////////////////////////

UINT TBraunPsd::SetEnergyScale(UINT aEnergyScale) {
	UINT result= TOneDimDetector::SetEnergyScale(aEnergyScale);

	// Werte immer fr Scale= 0 auslesen (Werte werden am Ende von PsdReadOut zusammengefasst)
	befehl[ 4 ][ 3 ]= ( BYTE ) 0;
	echo[ 4 ][ 4 ]= ( BYTE ) 0;
	return result;
};

UINT TBraunPsd::SetPositionScale(UINT aPositionScale) {
	UINT result= TOneDimDetector::SetPositionScale(aPositionScale);

	// Werte immer fr Scale= 0 auslesen (Werte werden am Ende von PsdReadOut zusammengefasst)
	befehl[ 5 ][ 3 ]= ( BYTE ) 0;
	echo[ 5 ][ 4 ]= ( BYTE ) 0;
	return result;
};


void TBraunPsd::GetEnergyRange( UINT& ler, UINT& her ) const 
{
	// auf Scale umrechnen
	ler= uEnergyLow >> uEnergyScale;
	her= uEnergyHigh >> uEnergyScale;
}

BOOL TBraunPsd::SetEnergyRange( UINT uLowerRange, UINT uHigherRange )
{
	char HeadLine[ ]= "Braun-Psd: SetEnergieRange";
	int nDelay= nDelayFast;
	long_to_bytes transform;
	const int CntLst= 2;
	int CmdList[ CntLst ]= {6, 7};

	// nochrechnen auf <MaxChannels>-Kanle - siehe PsdReadOut
	TRange<int> limit(0, nMaxChannels-1);
	uLowerRange= limit.ForceIntoRange(uLowerRange << uEnergyScale);
	uHigherRange= limit.ForceIntoRange(uHigherRange << uEnergyScale);

	uEnergyHigh= max( uLowerRange, uHigherRange );
	transform.IWert[ 0 ]= ( WORD ) uEnergyHigh;
	befehl[ 6 ][ 3 ]= transform.SWert[ 0 ];
	echo[ 6 ][ 4 ]= transform.SWert[ 0 ];
	befehl[ 6 ][ 4 ]= transform.SWert[ 1 ];
	echo[ 6 ][ 6 ]= transform.SWert[ 1 ];

	uEnergyLow= min( uLowerRange, uHigherRange );
	transform.IWert[ 0 ]= ( WORD ) uEnergyLow;
	befehl[ 7 ][ 3 ]= transform.SWert[ 0 ];
	echo[ 7 ][ 4 ]= transform.SWert[ 0 ];
	befehl[ 7 ][ 4 ]= transform.SWert[ 1 ];
	echo[ 7 ][ 6 ]= transform.SWert[ 1 ];

	bSetError= FALSE;

	for ( int i= 0;i < CntLst;i++ )
	{
		if ( 0 != BuildOperation( &( befehl ) [ CmdList[ i ] ][ 0 ], &( echo ) [ CmdList[ i ] ][ 0 ], nDelay ) )
		{
			nErrorCode= CmdList[ i ];
			bSetError= TRUE;
			MessageBox( GetFocus( ), TBraunError[ nErrorCode ][ 0 ], HeadLine, MBFAILURE );
			return FALSE;
		}
	}

	return TRUE;
}

int TBraunPsd::GetBufferSize( void ) const {
	//return max( ( 4096 >> uPositionScale ) + 18, ( 4096 >> uEnergyScale ) + 18 ) + 1; // unabhngig von uPositionScale und uEnergyScale, weil diese nun vernderlich sind
	int result= sizeof(MessDatenHeader) + (nMaxChannels*sizeof(unsigned long)); // GetChannelNumber() nicht verwenden, weil sich dieser Wert vergrern knnte ... dann ist nicht gengend Speicher reserviert
	if ( result%4==1 ) result+= 3;
	if ( result%4==2 ) result+= 2;
	if ( result%4==3 ) result+= 1;
	return result; // nur in 4er Schritten (ggf. aufrunden) - Grund siehe PsdReadOut
}

///////////////////////////////////////////////////////////////////////////////
// Detektorsteuerung
///////////////////////////////////////////////////////////////////////////////

int TBraunPsd::BuildOperation( BYTE* zgrbefehl, BYTE* zgrecho, int delay )
{
	if ( bDebug )
	{
		SetInfo( "Debug BPsd" );
		return 0;
	}
	delay= max(0, delay); // keine negativen Wartezeiten

	int j= ( int ) zgrbefehl[ 0 ];

	int i;
	for ( i= 1;i < j + 1;i++ )
	{
		Hardware->Write(0,zgrbefehl[i]);
		Sleep(2);
	}

	j= ( int ) zgrecho[ 0 ];
	BYTE EchoRead[ 18 ];

	for ( i= 0;i < j;i++ )
	{
		if ( Look_till_BaseAddr1( delay ) )
			break;

		Hardware->Read(2,&(EchoRead[i+2]));
		Hardware->Read(3,&(EchoRead[i+1]));

		i++;

		if ( ( i == 3 ) && ( j == 7 ) )
		{
			EchoRead[ 3 ]= EchoRead[ 4 ];
			i--;
		}
	}

	EchoRead[ 0 ]= ( BYTE ) j;
	return strncmp( ( char* ) EchoRead, ( char* ) zgrecho, j + 1 );
}

void TBraunPsd::ResetDelayTime( void )
{
	if ( bDebug )
	{
		SetInfo( "Debug BPsd" );
		return;
	}

	SYSTEMTIME t1, t2;
	GetLocalTime( &t1 );

	do
	{
		GetLocalTime( &t2 );
		Hardware->Write(1,(BYTE)0x00);
	}
	while ( ( abs( t2.wSecond - t1.wSecond ) ) < 2 );
}

int TBraunPsd::Look_till_BaseAddr1( int Delay )
{
	if ( bDebug )
	{
		SetInfo( "Debug BPsd" );
		return 0;
	}

	int k= 0;

	BYTE Value= 0;
	Hardware->Read(1,&Value);
	while(Value > 0x7F)
	{
		Sleep(1);
		if ( k > Delay )
			return 1;
		Hardware->Read(1,&Value);
		k++;
	}
	return 0;
}

int TBraunPsd::SynchronHexFile( BYTE& W_String, BYTE& R_String )
{
	if ( bDebug )
	{
		SetInfo( "Debug BPsd" );
		return 0;
	}

	W_String= 0x3A;

	int zaehler= 0;
	do
	{
		Hardware->Write(0,(BYTE)0x3A);
		Look_till_BaseAddr1( 1 );
		Hardware->Read(3,&R_String);	
		Hardware->Read(2,&R_String);

		if ( zaehler > 99 )
			return 1;

		zaehler++;
	}
	while ( R_String != 0x3A );

	return 0;
}

BYTE TBraunPsd::konvert( char z )
{
	BYTE zb= static_cast<BYTE>(z);

	if ( isdigit( zb ) )
		return ( BYTE ) ( zb - 48 );

	if ( z <= 70 )
		return ( BYTE ) ( zb - 55 );

	return ( BYTE ) ( zb - 87 );
}

int TBraunPsd::LoadHexFile( void )
{
	//! char HeadLine[]= "Braun-Psd: Load Hex-File";
	BYTE zeichen, HighWert, LowWert;
	int laenge= 0, i;
	//! int k=0;
	int anzahl= 0;
	//! int l=0;
	BOOL ReadError= FALSE;
	BYTE Buffer[ 80 ];
	BYTE ReadString[ 70 ];
	BYTE WriteString[ 70 ];
	float anzln= 350;
	int acln= 0;
	char Filename[ ]= "asa23.hex";

	bSetError= FALSE;

	ifstream myfile( Filename, ios::in | ios::nocreate );

	if ( !myfile )
	{
		bSetError= TRUE;
		nErrorCode= 35;
		return R_Failure;
	}

	char buf[ MaxString ];

	do
	{
		if ( ReadError )
			myfile.seekg( ( myfile.tellg( ) ) - ( laenge + 2 ) );

		myfile.getline ( Buffer, 80, '\n' );

		if ( !( ++acln % 40 ) )
		{
			sprintf( buf, "HEX-File laden: %.1f %% sind geladen.", (float)(acln / anzln)*100);
			SetInfo( buf );
		}

		if ( myfile.eof( ) )
			break;

		laenge= strlen( ( char* ) Buffer );
		istrstream mystream((char*)Buffer, laenge );

		if ( ( mystream.get( ) ) == 0x3A )
		{
			if ( SynchronHexFile( WriteString[ 0 ], ReadString[ 0 ] ) )
			{
				bSetError= TRUE;
				nErrorCode= 33;
				return R_Failure;
			};

			for ( i= 0;i < ( laenge - 1 ) / 2;i++ )
			{
				mystream.get( HighWert );

				if ( !isxdigit( HighWert ) )
				{
					bSetError= TRUE;
					nErrorCode= 34;
					return R_Failure;
				};

				mystream.get( LowWert );

				if ( !isxdigit( LowWert ) )
				{
					bSetError= TRUE;
					nErrorCode= 34;
					return R_Failure;
				}

				zeichen= ( BYTE ) ( ( konvert( HighWert ) << 4 ) + konvert( LowWert ) );
				WriteString[ i + 1 ]= zeichen;

				Hardware->Write(0,zeichen);

				if ( Look_till_BaseAddr1( 1 ) )
				{
				  bSetError= TRUE;
				  nErrorCode= 37;
				  return R_Failure;
				}

				BYTE Value= 0;
				Hardware->Read(3,&Value);	//Einlesen, um Blockade zu verhindern, vermutlich
				Hardware->Read(2,&(ReadString[i+1]));
			}

			if ( strncmp( ( char* ) WriteString, ( char* ) ReadString, i + 1 ) )
			{
				ReadError= TRUE;
				anzahl++;

				if ( anzahl >= 3 )
				{
					bSetError= TRUE;
					nErrorCode= 36;
					return R_Failure;
				}
			}

			else
			{
				ReadError= FALSE;
				anzahl= 0;
			}
		}

		else
		{
			ReadError= TRUE;

			if ( anzahl >= 3 )
			{
				bSetError= TRUE;
				nErrorCode= 36;
				return R_Failure;
			}

			anzahl++;
		}
	}
	while ( !myfile.eof( ) || ReadError );

	SetInfo( "HEX-File erfolgreich geladen." );

	return R_OK;
}

