/*
$Header: /vol/baal-vol3/projekt98/quellen/XCTL_32/INCLUDE/DETECUSE/Range.h,v 1.2 2004/04/15 21:41:53 reinecke Exp $

Projekt   : XCTL
Subsystem : alle
Autor     : Jan Picard <picard@informatik.hu-berlin.de> 2001-2002
          Institut fuer Informatik,
          Humboldt-Universitaet Berlin
Inhalt    : Eine Klasse, die ein Intervall mit Ober- und Untergrenzen
          realisiert
*/

#ifndef __RANGE_H
#define __RANGE_H

#include <windows.h>

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//! Implementation of Martin Fowlers "Range Pattern"
template <class TRangeType>
class TRange
{
	//! The range limits belong the range! So its possible
	//! to force a value into a given range by setting it to
	//! to the lower or upper limit.
	//! The TRangeType class must provide operator<

public:
	TRange(TRangeType min, TRangeType max)
	{
		if ( min < max)
		{
			_min= min;
			_max= max;
		}

		else
		{
			_min= max;
			_max= min;
		}
	}

	TRangeType ForceIntoRange(TRangeType value) const
	{
		if ( value < _min)
			return _min;
		else
			if ( _max < value)
				return _max;
			else
				return value;
	}

	void SetMin(TRangeType value)
	{
		if (_max < value)
			_min= _max;
		else
			_min= value;
	}

	void SetMax(TRangeType value)
	{
		if (value < _min)
			_max= _min;
		else
			_max= value;
	}

	TRangeType GetMin() const
	{
		return _min;
	}

	TRangeType GetMax() const
	{
		return _max;
	}

	BOOL includes(TRangeType value) const
	{
		if ( (value < _min) || (_max < value) )
			return FALSE;
		else
			return TRUE;
	}

private:
	TRangeType _min;
	TRangeType _max;
};

#endif // __RANGE_H


