/*
$Header: /vol/baal-vol3/projekt98/quellen/XCTL_32/INCLUDE/DETECUSE/detecmes.h,v 1.26 2004/07/23 22:23:53 reinecke Exp $

Projekt   : XCTL
Subsystem : Detektoren
Autor     : Jan Picard <picard@informatik.hu-berlin.de> 2001-2002
          Institut fuer Informatik,
          Humboldt-Universitaet Berlin
Inhalt    : Interface-Deklaration fuer die Messadapterklassen
*/

#ifndef __M_DEVHW_H
#define __M_DEVHW_H

#include <fstream.h>
#include "detecuse\detecuse.h"
#include "detecuse\detectorcontroller.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//vormals in evrythng.h
#define  R_MeasInProcess   205
#define  R_HardOverflow    217

//liefert den der spezifizierten id entsprechenden Controller aus der spezifizierten
//Controllerliste -> instantiiert ggf. einen neuen Controller
Controller* GetController(EDeviceType DeviceID, LPTSTR HardwareID,
									 ControllerList* Controllers, DeviceList* Drivers, DWORD* index= 0);

//#############################################################################
// 0-dimensionale Detektoren
//#############################################################################

class TGenericDetector : public TZeroDimDetector
{
public:
	TGenericDetector(int id);
	virtual  ~TGenericDetector();

	BOOL Initialize( void );
	EDetectorType GetDetectorType() const
	{
		return GenericDetector;
	};

	int MeasureStart( void );
	int MeasureStop( void );
	int PollDetector( void );

	void LookUp(LPCSTR Section);

protected:
	BOOL _SetParameters( void );

private:
	//! Das Hardware-Objekt, das fuer die Hardwarekommunikation zustaendig ist
	TGenericController* Hardware;
	int nBaseAddr;
	float fTimeCorrection;
};

//#############################################################################

class TRadicon : public TZeroDimDetector
{

public:
	TRadicon(int id);
	virtual  ~TRadicon( );

	BOOL Initialize( void );
	BOOL InitializeEvent( HWND, int );
	static void CALLBACK EventHandler( UINT, UINT, DWORD, DWORD, DWORD );
	EDetectorType GetDetectorType() const
	{
		return CounterDetector;
	};

	//! Gibt den Wert fuer die Beschleunigungsspannung zurueck
	WORD GetHighVoltage() const
	{
		return wHighVoltage;
	};

	//! Setzt den Wert fuer die Beschleunigungsspannung
	void SetHighVoltage(WORD);

	//JP?
	void SetDacThresh(WORD, WORD);

	//JP?
	const TRange<WORD>& GetDacThresh() const
	{
		return DacThresh;
	};

	int MeasureStart( void );
	int MeasureStop( void );
	int PollDetector( void );
	BOOL RunSpecificParametersDlg( HWND );
	BOOL HasSpecificParametersDlg( void ) const
	{
		return TRUE;
	};

private:
	BOOL _SetParameters( void );
	BOOL SetSound( void );

// nicht berschrieben, wenn auch gleiche Signatur (weil in Basisklasse nicht virtuell)
private:	
	void LoadDetectorSettings();
	void SaveDetectorSettings() const;

// ATTRIBUT(E)
private:
	double dRealTime;

	//! IO-Port, der fuer die Datenuebergabe verwandt wird
	int Rdd;

	//! IO-Port, die fuer die Hardwaresteuerung verwandt wird
	int Rcc;

	//JP? Intervall fuer gultige Werte fuer
	TRange<WORD> DacThresh;
	static const TRange<WORD> DacThreshLimits;

	//! Intervall, in dem Werte fuer die Beschleuningungsspannung liegen duerfen
	static const TRange<WORD> VoltageLimits;

	//! Wert fuer die Beschleunigungsspannung
	WORD wHighVoltage;

	//! Das Hardware-Objekt, das fuer die Hardwarekommunikation zustaendig ist
	TRadiconController* Hardware;

	//! Zeiger auf das Hardware-Objekt, dass fuer den Event-Mechanismus verwandt
	//! wird
	static TRadiconController* TheEventHardware;
};

//#############################################################################
// Test-Detektoren
//#############################################################################

class Testdev : public TZeroDimDetector
{
	/*!
	  Testdev simuliert einen nulldimensionalen Roentgendetektor, dessen
	  Messdaten von den aktuellen Positionen der Motoren CC, TL und DF ab-
	  haengen.
	  Um dieses Verhalten zu realisieren, wurden die Messdaten einer realen
	  Messung in einer Datei abgelegt. Diese Daten werden in den Speicher ge-
	  lesen und zusammen mit einem simulierten Messfehler zurueckgegeben.
	*/

	class TestdevData
	{
		/*!
		  TestdevData ist eine eingebettete Klasse, die die Verwaltung der
		  Simulationsdaten kapseln soll. Sie stellt in der Schichtenarchtiktur des
		  Detektoren-Subsystems die Hardware-Klasse, die Herkunft der Messdaten,
		  dar.
		*/

	public:
		TestdevData();
		virtual  ~TestdevData();

		//! Liefert den Messwert fuer die angegebene Koordinaten
		long value(int x, int y, int z) const;

	private:
		//! Parst eine Abschnitt von Daten fuer CC in der testdev.dat
		BOOL parseCC(ifstream& file);

		//! Parst eine Abschnitt von Daten fuer TL in der testdev.dat
		long** parseTL(ifstream& file);

		//! Parst eine Abschnitt von Daten fuer DF in der testdev.dat
		long* parseDF(ifstream& file);

		//! Zaehlt die Anzahl der Instanzen, die das TestdevData-Objekt
		//! verwenden. Durch das instance counting kann der Speicher
		//! mit den Simulationsdaten freigegeben werdem, wenn er nicht
		//! mehr gebraucht wird
		static int instanceCounter;

		//! Wurzel des Baums mit den Simulationsdaten
		static long ***data;

		//! Methode parst die testdev.dat und erstellt den Baum mit den
		//! Simulationsdaten
		BOOL initData();
	};

public:
	Testdev(int id);
	virtual  ~Testdev();

	int PollDetector( void );
	EDetectorType GetDetectorType() const
	{
		return CounterDetector;
	};

private:
	//! Das Objekt, das die Simulationsdaten verwaltet
	TestdevData* data;

	//! Ermittelt die Intensitaet
	double CalculateIntensity() const;

	//! Interpoliert den Messwert fuer nicht vorhandene Simulationsdaten
	double InterPolate( int ccI, int tlI, int dfI, int tlB ) const;
};

//#############################################################################

class TZeroDimSimpleTestDetector : public TZeroDimDetector
{
public:
	TZeroDimSimpleTestDetector(int id);
	virtual  ~TZeroDimSimpleTestDetector() {}

	int PollDetector( void );
};

//#############################################################################

class TOneDimTestDetector : public TOneDimDetector
{
public:
	TOneDimTestDetector(int id);
	virtual  ~TOneDimTestDetector() {}

	int PsdReadOut( EHowReadOutPsd eReadType);
};

//#############################################################################
// 1-dimensionale Detektoren
//#############################################################################

class TStoePsd : public TOneDimDetector
{
public:
	TStoePsd(int id);
	virtual  ~TStoePsd( void );

	int PollDetector( void );

	EDetectorType GetDetectorType() const
	{
		return PsdDetector;
	};

// nicht berschrieben, wenn auch gleiche Signatur (weil in Basisklasse nicht virtuell)
private:	
	void LoadDetectorSettings();
	void SaveDetectorSettings() const;

// StoePSD untersttzt nur PositionScale= 0 (kein EnergyScale)
public:
	virtual void SetDataType( EPsdDataType ) {
		eDataType= PsdPositionData;
	};
	virtual UINT SetEnergyScale(UINT) {
		return TOneDimDetector::SetEnergyScale(0);
	};
	virtual UINT SetPositionScale(UINT) {
		return TOneDimDetector::SetPositionScale(0);
	}

// ATTRIBUT(E)
private:
	int nBaseAddr;
	BOOL bLong;
	BOOL Initialize( void );
	int PsdReadOut( EHowReadOutPsd );
	int PsdRead( int, LPWORD );
	virtual int PsdInit( void );
	virtual int PsdStart( void );
	int PsdStop( void );


	// 30.02.2004 MEMCORRECT (verwende hStoeBuf) HGLOBAL hReadBuf;
	LPWORD hStoeBuf;


	int nReadBufItems;

	//! Aehnliches Konzept wie bActive, ist zusaetzlich auch vor der ersten
	//! Messung auf FALSE gesetzt
	BOOL bReadyForRead;

	HardwareIo Hardware;
};

//#############################################################################

class TBraunPsd : public TOneDimDetector
{
public:
	TBraunPsd(int id);
	virtual  ~TBraunPsd( void );

	BOOL Initialize( void );
	//int PollDetector( void );
	virtual int PsdReadOut( EHowReadOutPsd );
	virtual int PsdInit( void );
	virtual int PsdStart( void );
	virtual int PsdStop( void );

	void GetEnergyRange( UINT& ler, UINT& her ) const;
	BOOL SetEnergyRange( UINT, UINT );
	virtual int GetBufferSize( void ) const;

// EnergyScale und PositionScale
public:
	// setzt <EnergyScale>
	virtual UINT SetEnergyScale(UINT aEnergyScale);

	// setzt <PositionScale>
	virtual UINT SetPositionScale(UINT aPositionScale);

private:
	int nErrorCode;
	BOOL bSetError;
	int nHVRegelung_OK;
	EDetectorType GetDetectorType() const
	{
		return PsdDetector;
	};

protected:
	int BuildOperation( BYTE*, BYTE*, int );
	int LoadHexFile( void );
	void ResetDelayTime( void );
	BYTE konvert( char );
	int SynchronHexFile( BYTE&, BYTE& );
	int Look_till_BaseAddr1( int );

// nicht berschrieben, wenn auch gleiche Signatur (weil in Basisklasse nicht virtuell)
private:	
	void LoadDetectorSettings();
	void SaveDetectorSettings() const;

// ATTRIBUT(E)
private:
	int nBaseAddr;

	static BYTE echo[ 30 ][ 16 ];
	static BYTE befehl[ 30 ][ 16 ];
	UINT uMuxTimeDet1;
	LONG PositionsDatenHeader[ 16 ];
	UINT uEnergyLow; // hochgerechnet auf <MaxChannels>-Kanle - siehe PsdReadOut
	UINT uEnergyHigh; // hochgerechnet auf <MaxChannels>-Kanle - siehe PsdReadOut
	//21.06.2004 LONG lPositionStop; // Countstop fuer Positionskanaele
	// 0..EFFFFFFFFFFF hex
	BOOL bAbbruchmitShutter; // boolscher Wert
	int nDeathTime; // min. Impulsabstand 0..99 dec [0..99 *E-6 sec]
	BOOL bRatemeter; // computed=00, total=01
	BOOL bRealLifeTime; // Realtime=00, Lifetime=01
	//21.06.2004 LONG lMeasTime; // Messzeit 1..9999999 dec [sec]

	// 31.06.2004 MEMCORRECT HGLOBAL hReadBuf;

	int nDelayFast;
	int nDelaySlow;
	int nReadBufItems;
	int nHVControl_OK;

	TBraunPsdController* Hardware;
};
#endif // __M_DEVHW_H
