/*//////////////////////////////////////////////////////////////////////////////
//                                                                            //
// DETECTORCONTROLLER.H                                                       //
//                                                                            //
// Subsystem : Detektornutzung / Hardwarezugriff							  //
//----------------------------------------------------------------------------//
// Autoren:  Ren Harder, Alexander Paschold (2003)		                      //
//                                                                            // 
// Stand  : 23.05.2003                                                        //
// letzter vollstndiger Test:                                                //
//                                                                            //
//////////////////////////////////////////////////////////////////////////////*/

#ifndef _DETECTORCONTROLLER_H
#define _DETECTORCONTROLLER_H

#include "hardware\hwio.h"

#ifndef _COUNTERCLASS 
#if defined (Build_Counters)
#define _COUNTERCLASS __declspec(dllexport)
#elif defined(Use_Counters)
#define _COUNTERCLASS __declspec(dllimport)
#else
#define _COUNTERCLASS
#endif
#endif


/////////////////////////////////////////////////////////////////
// Controller fr BraunPsd
class _COUNTERCLASS TBraunPsdController : public Controller
{
private:
	BraunPsdIo* SpecialHardware;

public:
	TBraunPsdController(EDeviceType DeviceID, LPTSTR HardwareID, DeviceList* Devices);
	virtual ~TBraunPsdController();

	virtual BOOL Check();

	BOOL SetTimeout(unsigned long Cycles);
	BOOL GetData(IrpParamsGetData *Params); 
};

/////////////////////////////////////////////////////////////////
//Controller fr Radicon
class _COUNTERCLASS TRadiconController : public Controller
{
public:
	//! how to readout the measured values
	enum EReadoutMode
	{
		final,					// receive result time and pulses
		intermediate			// receive current time and pulses
	};

	//! tells the detector's controller what to do
	enum EOperationMode
	{
		fixedExposureCounts,	// count pulses during exposure time
		fixedImpulseCounts,		// count time when number of pulses is set
		intensimeter,			// intensimeter
		stop					// stop controller
	};


	TRadiconController(EDeviceType DeviceID, LPTSTR HardwareID, DeviceList* Devices);
	virtual ~TRadiconController();

	virtual BOOL Check();

	int UploadFirmware(void);
	int SetParameters(unsigned short upperThreshold, unsigned short lowerThreshold,
		              int highvoltage, double exposureTime, unsigned long impulseCount,
					  BOOL bSound);
	int Execute(EOperationMode mode);
	int GetValues(EReadoutMode, double *exposureTime, unsigned long *impulseCount);
	void reset();

private:
	int TransmitMessage(int nFunctionCode, unsigned char* lpszMessage= 0, int nMessageLength= 0);
	int ReceiveMessage(int nFunctionCode, unsigned char* lpszMessage, int nMaxMessageLength,
		               int& nRealMessageLength);
	int TransmitFunctionCode(int nFunctionCode);
	BOOL out_byte(unsigned char d);
	BOOL in_byte(unsigned char &d);
	BOOL IsReadyToRead();
	BOOL IsReadyToWrite();
	DWORD CurrentTime(void);

	DWORD controlPort;
	DWORD dataPort;
};


/////////////////////////////////////////////////////////////////
// Controller fr Detektor an TAm9513 Steuerkarte

enum EIOCCmd
{
	iocSetCounts= 1,
	iocSetTimeTicks,
	iocStartCounting,
	iocStopCounting,
	iocIsCountingReady,
	iocReadTicks,
	iocReadCounts,
	iocConfigureAcustic,
	iocStopCCDTiming,
	iocStartCCDTiming,
	iocStartCCDShot
};

class _COUNTERCLASS TGenericController : public Controller
{
public:
	TGenericController(EDeviceType DeviceID, LPTSTR HardwareID, DeviceList* Devices);
	virtual ~TGenericController();

	virtual BOOL Check();

	int Init(float fTimeCorrection);
	int IOCTL( EIOCCmd, DWORD& );
	int IOCTL( EIOCCmd );
	void DigitalOut( BYTE );
	void DigitalIn( BYTE* );
	void SetSound(BOOL param);	//ap - neu !!!
	DWORD GetTicksPerSecond( void );
	void ClearToggleOut( BYTE );
	void SetToggleOut( BYTE );

protected:
	BOOL SelectChip();
	void WriteCmd( BYTE cmd );
	void ChooseDataPtr( BYTE group, BYTE reg );
	void WriteData( WORD );
	WORD ReadData( void );
	WORD ReadStatus( void );
	void LoadAndArmC( BYTE );
	void LoadC( BYTE );
	void DisarmAndSaveC( BYTE );
	void DisarmC( BYTE );
	void LatchToHoldC( BYTE );
	void ArmC( BYTE );
	void Reset( void );
	
	BYTE nbChipId;

private:
	float fTimeCorrection;
	BOOL SplitNumber( DWORD, WORD&, WORD& );

	BOOL bIsLatched;
	BOOL bIsStopped;
	WORD wLowTicks;
	WORD wHighTicks;
	WORD wLowCounts;
	WORD wHighCounts;
};

#endif