//##############################################################################
//                                                                            //
// A_EXTERN.H                                                                 //
//                                                                            //
// Subsystem: Diffraktometrie/ Reflektometrie                                 //
//            DAS MDI-Fenster fr AreaScan                                    //
// Benutzung durch andere Subsysteme erforderlich: JA                         //
//                                                                            //
//##############################################################################

#ifndef _A_EXTERN_H
#define _A_EXTERN_H

#include "difrkmty\d_extern.h" // TPlotWindow
#include "detecuse\detecuse.h" // TDetector, TOneDimDetector

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
// TAreaScanParameters
//##############################################################################

class TAreaScanParameters
{
private:
	//! struktur haelt Info zu reportkurven
	//! (welche zusatzinfo existieren und welche nicht)
	struct TReportUse
	{
		BOOL bMonitor; //! Monitorintensitaet
		BOOL bAbsorber; //! Absorberposition
		BOOL bTime; //! reale Messzeit
		BOOL bIntegral; //! integrale Intensitaet
	};
	/* 21.04.2004 enum ENotifyValue
	{
		nvMonitorIntensity= 1, nvAbsorberPosition, nvPeakPosition, nvIntegralIntensity,
		nvMeasurementTime
	};*/

public:
	TAreaScanParameters( void );

	TDetector* GetDetector()
	{
		return Detector;
	};
	TDetector* GetMonitor()
	{
		return Monitor;
	};

	void SetDetector(TDetector* detectorParam)
	{
		Detector= detectorParam;
	};
	void SetMonitor(TDetector* monitorParam)
	{
		Monitor= monitorParam;
	};

	//! Gibt den Messdetektor als Psd zurueck, wenn es ein Psd ist,
	//! andernfalls 0
	TOneDimDetector* GetPsd();

protected:
	virtual void ResetPosition();

//ATTRIBUT(E)
public:
	// ehemalig extern-Verweise aus D_INTERN.CPP
	char DataFile[ 40 ];
	BOOL bAquisitionActive;
	int nLinePoints;
	BOOL bSetDXZero,
		 bSetDYZero,
		 bSetRLSave;
	int nEnvironment;

	// ehemalig static in A_EXTERN.CPP
	int nFirstReadColumn;
	int nLastReadColumn;
	float fPsdRange;
	char szODF[ 7 ], szOWF[ 7 ], szTDF[ 7 ], szTWF[ 7 ];
	char DismantleFile[ _MAX_PATH+1 ]; //! Dateiname zum Zerlegen bzw. Zusammenfassen einer Datenbasis

	//! enthaelt Nummer des Omega- sowie Thetamotors in der Motorenliste
	int nOmega, nTheta;

	//! enthaelt Anfangs- und Endwinkel sowie die Schrittweite des Omegamotors
	//! fuer einen Areascan
	double dOmegaMin, dOmegaMax, dOmegaWidth;

	//! enthaelt Anfangs- und Endwinkel , die Schrittweite sowie den Offset des
	//! Thetamotors fuer einen Areascan
	//! NEU dThetaStep
	double dThetaMin, dThetaMax, dThetaWidth, dThetaOffset;

	//! gibt den Thetabereich eines Spektrums an
	double dThetaWindow;

	//! gibt das Schrittverhaeltnis zwischen Omega und Theta an
	double dMoveRelation;

	//! gibt den Scantyp an (omega2Theta/StandardScan)
	ExScanType eScanType;

	//! gibt an in welcher Form die Messwerte in Datei abgelegt werden sollen
	//! (nur StandardFile verwendet, erweitert um bkFile, dtnFile)
	EFileFormat eSaveFormat;

	//! Speichern nach Beendigung der Messung
	BOOL bSaveOnReady;

	//! Speichern waehrend der Messung
	BOOL bSaveContinuous;

	//! Aenderung DB_Zerlegung
	//25.04.2004 BOOL bSaveEachScan;

	//! bisher ueberfluessig (bei anderen Speicherformaten verwendet)
	BOOL bThetaAddsPsd;

	//! zeigt an ob AreaScan mit Absorber durchzufuehren
	BOOL bAbsorberUsed;

	//! gibt an wieviele Kanaele des Psd zu einem zusammenzufassen sind
	int nAddedChannels;

	float fMaxTime; // Maximale Messzeit
	DWORD dwMaxCounts; // Maximale Anzahl Impulse

	//! gibt an welche zusatzdaten zu einem Spektrum im Scanreport zu speichern
	//! sind, zur Zeit nur IntegralIntensity moeglich
	//21.04.2004 TNotifyValue eNotifyValue;

	//! gibt die gewuenschte Thetastartposition bei O2T-Scans bzw. die feste
	//! position beim Standardscan an
	double d2ThetaStart;

	//! Aenderung DB_Zerlegung
	char FirstFile[ _MAX_PATH+1 ], LastFile[ _MAX_PATH+1 ];
	int MaxFiles;

	//! gibt den Omega- bzw. Thetaoffset an (differenz zwischen gewuenschtem und
	//! realem winkel)
	double dOffsetOmega, dOffsetTheta;

	//! gibt Winkelwert an, der sich aus der Nummer des Messkanals und der
	//! breite eines Psd-kanals ergibt; um diesen winkelwert muss man verschieben,
	//! um den Messkanal auf der 2Thetaposition zu positionieren
	double dPsdOffset;

	//! gibt Messkanal an; Kanal des Psd der sich immer auf der 2Thetaposition
	//! befinden soll
	int nMeasurementChannel;

	//! gibt an welche Zusatzinfos zu der Datenbasis anzulegen sind bzw. bei
	//! nachgeladenen welche Infos existieren
	TReportUse ReportUse;

	//! neu akk
	//! gibt art der Darstellung waehrend eines AreaScans mit Psd an
	//! akkumulierte Anzeige (TRUE) oder
	//! Anzeige am Ende der Messzeit (FALSE)
	BOOL bAccumulatedDisplay;

protected:
	//! gibt ausgewaehlten Detektor an
	TDetector* Detector; // nicht im Destruktor freigeben, wird durch Detektorsubsystem verwaltet

	//! gibt ausgewaehlten Monitordetektor an (ist bisher nicht auswaehlbar)
	TDetector* Monitor; // nicht im Destruktor freigeben, wird durch Detektorsubsystem verwaltet
};

//##############################################################################
// TAreaScanWindow
//##############################################################################

class TAreaScanWindow : public TAreaScanParameters, public TPlotWindow
{
	//Kullmann+Reinecker: zahlreiche unntige friend-Deklarationen entfernt
	friend class TScanCmd;
	friend class TAreaScanCmd;
	friend class TSaveDataCmd;
	friend class TChooseScanDlg;

public:
	TAreaScanWindow( HINSTANCE aInstance );
	virtual  ~TAreaScanWindow( void );

// berschriebenes von TAreaScanParameters
private:
	virtual void ResetPosition();

// berschriebenes aus der (Fenster-)Basisklasse
private:
	virtual void OnCreate();
	virtual LRESULT OnCommand( WPARAM, LPARAM );
	LRESULT OnEvent ( HWND, UINT, WPARAM, LPARAM );

	//Mausbedienung
	virtual void OnPopupMenuInit ( WPARAM, LPARAM );
	virtual void OnLButtonUp( WPARAM, LPARAM );
	virtual void OnLButtonDown( WPARAM, LPARAM );

	virtual BOOL SetTitle( void );
	virtual void UpdateWnd( EPaintType= ptEntire );
	virtual void SetRanges( void );

	//! Ablaufsteuerung
	virtual void SteeringReady( LPARAM ); // Ende
	virtual void SteeringReset( LPARAM ); // Abbruch	
	virtual void Interrupt(); //! unterbricht Scanvorgang

	//! erhaelt Messpunkt, stellt ihn dar, speichert ihn ggf.
	virtual void CounterSetRequest( LPARAM );

	virtual BOOL CanNew()
	{ return (!bAreaScanStarted || bInterrupted) && !bShowPsdContinuous; /*Messung beendet und Spekrenanzeige aus*/ }
	virtual BOOL CanOpenFile()
	{ return (!bAreaScanStarted || bInterrupted) && !bShowPsdContinuous; /*Messung beendet und Spekrenanzeige aus*/ }
	virtual BOOL CanSave()
	{ return (!bAreaScanStarted || bInterrupted) && !bShowPsdContinuous; /*Messung beendet und Spekrenanzeige aus*/ }
	virtual BOOL IsScanning(BOOL &PsdContinious) 
	{ PsdContinious= bShowPsdContinuous; return bAreaScanStarted; /*Messung luft*/ }

	virtual BOOL New( void );
	virtual BOOL LoadOldData();
	virtual BOOL SaveFile( EAskType );
	virtual BOOL UpdateFile( void );

	//! liefert den Klassennamen zurueck
	virtual LPCSTR ClassName()
	{
		return "AreaScan";
	};

	//! liefert Klassencharakteristik
	virtual LPCSTR GetCharacteristic()
	{
		return "AreaScan";
	};

	//24.04.2004 void DrawMenu( void );
	//24.04.2004 void StopMeasurement( void );

// berschriebenes aus TPlotWindow
private:
	virtual BOOL SetMeasurementArea( TKSystem & );

	// gibt die Beschriftung der X- und Y-Achse zurck
	virtual void GetTitleX(int, int, char*, bool);
	virtual void GetTitleY(int, int, char*, bool);

	virtual bool CanSwitchChannelAngle() { 
		if ( eOutputType==otReciprokeLatticeBitmap ) return false; // nicht mglich
		else if ( bShowPsdContinuous && m_PsdContinousType==PsdEnergyData ) return false; // nicht mglich, wenn Energiespektrum aktiv
		else return TPlotWindow::CanSwitchChannelAngle(); 
	}

	//! Funktion liefert den absoluten Thetawert als Rueckkehrcode
	//! 1.Param. scid gibt Kurvennr. an, deren absoluter Thetawert gesucht ist
	//! nur bei RL-Bitmap benutzt
	virtual double GetThetaOffset( int );

//SONSTIGES
private:
	BOOL LoadReport( void );
	BOOL SaveReport( void );

	void StartSensorContinuous( EPsdDataType, BOOL= FALSE ); // wenn Spektrenanzeige gestartet werden kann, wird die im Parameter angegebene Art gestartet
	void StopSensorContinuous(); // wenn Spektrenanzeige aktiv, wird aktuelle Art gestoppt
	void ShowSensorContinuous( EPsdDataType ); // Start/ Stop der Spektrenanzeige, Aufruf durch Kontextmenpunkte 'Energiespektrum anzeigen' und 'Positionsspektrum anzeigen'

	BOOL LoadMeasurementInfo( TTxtRead& );
	// BOOL LoadMeasurementInfo( HFILE );
	BOOL SaveMeasurementInfo( BOOL );

	int GetAdditionalColumns( void );
	int GetShift( int ScanNumber );

	void CalibratePsd( void );	
	void InitializeAreaScanTask();
	void DoDataAquisition();

	// Read/Write (Aenderung DB_Zerlegung)
	BOOL ComposeDB( void );
	BOOL SaveDismantleCurve( int );	
	BOOL DismantleDB( void );
	
	// Protokoll
	//! klier 01.03.2003 Protokollbuch
	//! Speichert Informationen im Protokollbuch
	void SetMeasurementProtocolParameter( void );

	// berechnet den Dateinamen der Reportdatei
	LPCSTR GetRepFileName( void );

	double GetActAngle( const LPCSTR, BOOL& ) const;

//ATTRIBUT(E)
private:
	//! nur bei benutzung von 0 dim. Detekt. von Bedeutung
	//! zeigt an ob der Detektor den Thetabereich durchlaufen hat
	BOOL bThetaScanCompleted;

	//! Kurve enthaelt zu jedem Spektrum der Datenbasis die Omega-,Thetapos.
	//! und einen Reportwert (integrale Intensitaet)
	LPCurve ScanReport; // nicht im Destruktor freigeben, wird durch lpDatabase verwaltet

	//! Kurve enthaelt weitere Zusatzinformationen zu den Kurven der Datenbasis
	//! Absorberposition, Monitorintensitaet, reale Messzeit
	LPCurve ScanReport2; // im Destruktor freigeben

	//! gibt Nummer der Kurve in Datenbasis an, die gerade aktiv ist
	int nActiveScan;

	// wie bMeasurementActive, aber bereits beim anfahren der Startposition == TRUE
	BOOL bAreaScanStarted;
	
	BOOL bShowPsdContinuous; //! zeigt an, ob kont. Spektrumsanzeige aktiv ist
	EPsdDataType m_PsdContinousType; // Art der zuletzt benutzen Spektrenart (zum Stoppen der aktuellen Anzeige bentigt)

	//! zeigt an, ob die Psdkalibrierung gerade durchgefuehrt wird
	BOOL bCalibrationActive;

	//! zeigt an, ob gerade das erste Spektrum eines AreaScans aktiv ist
	//! wird benutzt, da die Datenbasis bereits mit einer Kurve init. wird
	//! aber in den folgenden Schritten immer eine neue Kurve anlegen muss
	BOOL bFirstScan;

	//! soll AreaScanstart vor Aufruf einer Einstellungsbox verhindern
	//! FIX Fehler 2
	BOOL bScanSetupOk;

	// bei Energiespektrum vorherige Beschriftung der Abszisse merken
	BOOL bSavedSecondaryCoor;

	//! Aenderung DB_Zerlegung
	BOOL bComposeDB;

	// Platz fr den Dateinamen der Reportdatei; wird bei GetRepFileName() in Abhngigkeit von TMDIWindow::Filename neu berechnet
	char m_RepFileName[ _MAX_PATH+1 ];

	TDetector *oldDetector; // nicht im Destruktor freigegeben, wird durch das Detektorsubsystem verwaltet
};

#endif // _A_EXTERN_H

//#############################################################################
// LastLine
//#############################################################################