//##############################################################################
//                                                                            //
// MJ_OFUNK.H                                                                 //
//                                                                            //
// Subsystem : alte Manuelle Justage / Funktionalitt                         //
// Benutzung durch andere Subsysteme erforderlich: NEIN                       //
//----------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                                //
// Stand  : 20.08.2002                                                        //
//                                                                            //
//##############################################################################

#ifndef __MJ_OFUNK_H
#define __MJ_OFUNK_H

#include "utils\u_utils.h"

#include "motrstrg\motrstrg.h" // TAxisType
#include "detecuse\detecuse.h" // TDetector

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//#############################################################################
// TAngleCtl
//#############################################################################

class TAngleCtl
{
	//******************************************************************************
	// Konstruktor/Destruktor
public:
	TAngleCtl ( void );
	virtual  ~TAngleCtl ( void );

	//******************************************************************************
	// Sonstiges
public:
	BOOL DoInit ( const int );       // initialisiert m_MotorList und whlt den angegebenen Antrieb aus
	BOOL DoInitMotor ( const int, const BOOL ); // setzt die Bewegungsgeschwindigkeit des angegebenen Antriebs, auf MaxSpeed oder m_MotorList[<Index>]

	//******************************************************************************
	// Antriebssteuerung
public:
	BOOL DoDirect ( const int, const double ); // Bewegung im Direktbetrieb
	BOOL DoStep ( const int, const BOOL );   // Bewegung im Fahrbetrieb
	BOOL DoDrive ( const int, const BOOL );   // Bewegung im Schrittbetrieb

	BOOL DoStop ( const int ); // stoppen der Bewegung, unabhngig von der Betriebsart
	void DoStopEverything ( const int ); // stoppen aller Antriebe, der Halbwertsbreitenmessung und der Detektormessung

	//******************************************************************************
	// Statusinformationen
public:
	BOOL IsIndexValid ( const int ) const;        // gltiger Index der Antriebsliste ?
	BOOL HasOffset ( const int, BOOL& ) const; // Offset gesetzt, d.h. (Offset != 0) ?
	BOOL IsMoving ( const int, BOOL& ) const; // Antrieb in Bewegung ?
	BOOL IsCalibrated ( const int, BOOL& ) const; // Antriebsposition kalibriert?

	//******************************************************************************
	// GET-Methoden
public:   // Delegate an C-Interface des Motors
	// Antriebspositionen unabhngig von der relativen Null
	double GetAngle ( const int, BOOL& ) const; // aktuelle Antriebsposition
	double GetAngleMin ( const int, BOOL& ) const; // minimale Antriebsposition
	double GetAngleMax ( const int, BOOL& ) const; // maximale Antriebsposition
	double GetSpeed ( const int, BOOL& ) const; // Geschwindigkeit
	double GetAngleWidth ( const int, BOOL& ) const; // Schrittweite

	UINT GetDigits ( const int, BOOL& ) const; // lesen der Nachkommastellen
	LPCSTR GetUnit ( const int, BOOL& ) const; // lesen der Einheit

	int GetMotorIdx ( const EAxisType ) const;  // Index des angegebenen Antriebs; ggf. -1
	LPCSTR GetMotorName ( const int, BOOL& ) const; // Antriebsbezeichnung per standardisierter Antriebsbezeichnung
	UINT GetMotorCount ( void ) const;       // Anzahl der Antriebe in der Antriebsliste
	int GetActualMotor ( void ) const;       // Index des ausgewhlten Antriebs

	//******************************************************************************
	// SET-Methoden
public:
	BOOL SetSpeed ( const int, double& ); // Geschwindigkeit
	BOOL SetAngleWidth ( const int, double& ); // Schrittweite
	BOOL SetCorrectionState ( const int ); // CorrectionState

	//*****************************************************************************
	// Relative Null
public:
	BOOL SetRelativeZero ( const int ); // setzen
	BOOL ResetRelativeZero ( const int ); // aufheben

	//*****************************************************************************
	// Halbwertsbreitenmessung
public:
	BOOL DoStartMeasure ( const HWND ); // starten
	BOOL DoStopMeasure ( const int );       //stoppen

	BOOL IsMeasuring ( void ) const;  // ==> TRUE <--> Messung aktiv
	BOOL IsMeasureReset ( void ) const;  // Messung abgebrochen?
	double GetMeasureHwb ( void ) const;  // ==> HWB

	BOOL MeasureStopped ( const int );   // kennzechnet, dass die Messung beendet ist (cm_SteeringReady wurde empfangen)
	void UpdateDetector ( void ) const;  //informiert die Makroverarbeitung, dass neue Messwerte vorliegen

	//******************************************************************************
	// Zustand fr Bewegung ermitteln
public:
	// knnen die folgenden Funktionalitten durchgefhrt werden?
	// das Freigeben bzw. Sperren und Ausgrauen von Steuerelementen
	BOOL CanDoDirect ( const int ) const;       // Direktbetrieb mglich ?
	BOOL CanDoStep ( const int, const BOOL ) const; // Fahrbetrieb mglich ?
	BOOL CanDoDrive ( const int, const BOOL ) const; // Schrittbetrieb mglich ?

	BOOL CanDoStop ( const int ) const; // kann Bewegung gestoppt werden ?

	//*****************************************************************************
	// Zustand zum Setzen von Parametern
public:
	BOOL CanSetSpeed ( const int ) const; // setzen der Geschwindigkeit mglich ?
	BOOL CanSetAngleWidth ( const int ) const; // setzen der Sollposition mglich ?

	BOOL CanSetOffset ( const int ) const; // setzen der Relativen Null bzw. setzen des Offset fr <Antrieb> mglich?
	BOOL CanResetRelativeZero ( const int ) const; // aufheben der Relativen Null mglich ?

	BOOL CanDoStartMeasure ( void ) const; // starten der Halbwertsbreitenmessung mglich ?
	BOOL CanDoStopMeasure ( void ) const; // stoppen der Halbwertsbreitenmessung mglich ?

	//*****************************************************************************
	// private
private:
	BOOL DoSelectMotor ( const int ) const;       // whlt den angegebenen Antrieb aus; Methoden des C-Interfaces zur Antriebssteuerung arbeiten dann mit diesem Motor
  BOOL Moves( const int, BOOL& ) const; // hnlich IsMoving, jedoch ohne den Parnterantrieb zu bercksichtigen

	//*****************************************************************************
	// ATTRIBUTE
private:
	TDetector *m_Sensor;  // der ausgewhlte Detektor
	UINT m_MotorCount;    // Anzahl der Geschwindigkeiten in m_MoveSpeed
	double *m_MoveSpeed;  // Bewegungsgeschwindigkeit des jeweiligen Antriebs
	BOOL m_IsMeasuring;   // Status der HWB-Messung
};

#endif //__MJ_OFUNK.H

//#############################################################################
// LastLine
//#############################################################################