#ifndef __MOTRSTRG_H
#define __MOTRSTRG_H

#include "motrstrg\m_motcom.h" // TMotor
#include "motrstrg\msimstat.h" // Motorsimulator

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--


#if defined (Build_Motors)
#define _MOTORCLASS __declspec(dllexport)
#elif defined(Use_Motors)
#define _MOTORCLASS __declspec(dllimport)
#else
#define _MOTORCLASS
#endif


/* Benutzt in m_motcom.h, m_layer.h/cpp */
enum EValueType
{
	Distance= 1, MinDistance, MaxDistance, MinDistanceRel, MaxDistanceRel, Speed, MinSpeed, MaxSpeed, Width, MinWidth, Acceleration
	
};

/*
 * High Level layer der Motoransteuerung.
 * Funktionen von ausserhalb sollten diese benutzen und nicht die
 * aus motors.h
 *
 * mFunctionName = funktion benutzt den aktivierten Motor
 * mlFunctionName= funktion benutzt den Motor mit der uebergebenen
 *                  MotorID (mid)
 * Achse= ein Motor, der eine bestimmte Achse bewegt
 *
 * Viele hier benutzte Typen werden in comhead.h definiert, auf
 * jeden Fall einmal anschauen.
 * In m_motcom.h stehen einige Kurzroutinen!!
 *
 * Kommentare: Jens-Uwe Rumstich, 18. Februar 1999
 *             Stefan Ltzkendorf, Februar 2001
 */

#ifdef __cplusplus
extern "C"
{
#endif
	// prototypes for export functions

	/*******************************************************************************
	 ******************** MotorListen-Funktionen ***********************************
	 ******************************************************************************/

	/* initialisiere  DLL.
	 * TRUE = initialisierung erfolgreich
	 * FALSE= sonst
	 */
	BOOL _MOTORCLASS WINAPI mlInitializeMotorsDLL( void );

	/* Auswahl der aktiven Axis (Motor).
	 * Auswahl des Motors, der danach direkt ueber die mFuntionsname
	 * Funktionen angesprochen wird.
	 *
	 * der Parameter id endspricht der Position des Motors in der
	 * Motoren-Liste bzw. dem X in [MotorX] im ini-file
	 *
	 * fuer id muss gelten: 0 <= id && id < mlGetAxisNumber()
	 *
	 * standardmaessig ist Achse 0 aktiv
	 *
	 * TRUE = Auswahl erfolgreich
	 * FALSE= Auswahl schlug fehl (id ausserhalb des gueltigen Bereiches)
	 */
	BOOL _MOTORCLASS WINAPI mlSetAxis( int id );

	/* Ermittle derzeit aktivierte Achse
	 * siehe mlSetAxis fuer mehr
	 */
	int _MOTORCLASS WINAPI mlGetAxis( void );

	/* liefert die id der Achse die einem Achsentype zugeordnet ist
	 *
	 * liefert -1 falls diese Achse nicht existiert bzw. nicht initialisiert wurde
	 * s.a. mlIsAxisValid
	 *
	 * moegliche achsen: X, Y, Z, Omega, Theta, Phi, Spi, Encoder, Monochromator,
	 *                   Absorber, Collimator, DF, DC, Tilt, Rotation
	 *                   (siehe auch comhead.h)
	 */
	int _MOTORCLASS WINAPI mlGetIdByName( EAxisType axis);

	//! neu Kullmann+Reinecker: Ermittelt die Nachkommastellengenauigkeit fr Angle- und Speed-Ausgaben (siehe DFmt)
	//ACHTUNG die Schrittweite muss eine Nachkommastelle mehr haben!!!
	//ACHTUNG der Index dex Antriebs (erster Parameter) wird nicht auf seine Gltigkeit hin berprft!
	UINT _MOTORCLASS WINAPI mlGetDigits ( int );

	/* returns the value of TValueType of the motor.
	 * TValueType might be one of :
	 *  Distance - aktuelle Position in Nutzereinheiten (incl. AngleBias)
	 *  MinDistance - kleinstmgliche Position in Nutzereinheiten
	 *  MinDistanceRel - WIEVOR, jedoch incl. AngleBias
	 *  MaxDistance - grtmgliche Position in Nutzereinheiten
	 *  MaxDistanceRel - WIEVOR, jedoch incl. AngleBias
	 *  Speed - aktuelle Geschwindigkeit in Nutzereinheiten
	 *  MinSpeed - minimale Geschwindigkeit in Nutzereinheiten/Sekunde
	 *  MaxSpeed - maximale Geschwindigkeit in Nutzereinheiten/Sekunde
	 *  Width - Schrittweite in Nutzereinheiten
	 *  MinWidth - minimale Schrittweite in Nutzereinheiten
	 *  Acceleration - Beschleunigung in Nutzereinheiten/(Sekunde*Sekunde)
	 * fuer TValueType siehe auch comhead.h
	 *
	 * s.a. mGetValue
	 */
	double _MOTORCLASS WINAPI mlGetValue( int, EValueType );

	//Neu : Kullmann, Reinecker (22.10.2002); minimale, relative Antriebsposition in Nutzereinheiten
	double _MOTORCLASS WINAPI mlGetAngleRelativeMin( int );

	//Neu : Kullmann, Reinecker (22.10.2002); maximale, relative Antriebsposition in Nutzereinheiten
	double _MOTORCLASS WINAPI mlGetAngleRelativeMax( int );

	/* Bewegt den gegebenen Motor zur gegebenen Position (in Nutzereinheiten)
	 */
	BOOL _MOTORCLASS WINAPI mlMoveToDistance( int mid, double position);

	/* Bewegt aktuellen Motor relativ zur aktuellen Position um eine
	 * bestimmte Distanz.
	 * Distanz wird im Winkelmass (Nutzereinheiten) uebergeben
	 */
	BOOL _MOTORCLASS WINAPI mMoveByDistance( double distance);


	/* Feststellen, ob sich der Motor noch bewegt.
	 * TRUE  : motor finished to move (stands still)
	           oder befindet sich in einem toten Bereich
	 */
	BOOL _MOTORCLASS WINAPI mlIsMoveFinish( int );

	/* returns Offset (dAngleBias)
	 * Offset, dass die Verschiebung der Relativen zur Absoluten Null angibt
	 * Wenn Offset != 0, dann beziehen sich alle Positionsangaben auf relative Null.
	 */
	double _MOTORCLASS WINAPI mlGetOffset( int );

	/* Setzen der Defaults fuer Winkel (dAngleBias auf 0 ) und der
	 * (erlaubten ?)Winkelbereiche (SetAngleRange). Die Werte brauchen nicht
	 * uebergeben zu werden.
	 */
	void WINAPI mSetAngleDefault( void );

	/* Umwandeln eines Strings in einen AxisType.
	 * Allows more than one name for the same axis type. for example will
	 * all of AzimutalRotation, AZ, Rotation and Azimute get to axis type
	 * Rotation.
	 * wird zum Beispiel ueber mehrere Ecken von ParsingCmd in m_steering.cpp
	 * benutzt. useless ??
	 */
	EAxisType _MOTORCLASS WINAPI mlParsingAxis( LPSTR );

	/* herausfinden, ob eine bestimmte Achse initialisiert wurde
	 * TRUE = Achse wurde bereits initialisiert
	 * FALSE= sonst
	 */
	BOOL _MOTORCLASS WINAPI mlIsAxisValid( EAxisType );

	/* liefert die Anzahl der Achsen/Motoren in der Motoren Liste
	 *
	 * gueltige Motor-IDs liegen im Bereich von 0 bis mlGetAxisNumber()-1
	 */
	int _MOTORCLASS WINAPI mlGetAxisNumber( void );

	/* Abspeichern der Einstellungen aller Motoren im ini-File
	 *  - v.a. der aktuellen Position fuer den Programm-Neustart
	 *  - setzt RestartPossible=TRUE um eine ordnugsgemaesses schliessen
	 *    des Programmes zu signalisieren
	 *  - stoppt die Motoren
	 * (motors.cpp)
	 */
	void _MOTORCLASS WINAPI mlSaveModuleSettings( void );

	/* Setzen der Defaults fuer Winkel (dAngleBias auf 0 ) und der
	 * (erlaubten ?)Winkelbereiche (SetAngleRange). Die Werte brauchen nicht
	 * uebergeben zu werden.
	 */
	void _MOTORCLASS WINAPI mlSetAngleDefault( void );

	/* Rueckgabe der Versionsnummer von der m_layer-Schicht
	 */
	LPCSTR _MOTORCLASS WINAPI mlGetVersion( void );

	/* returns instance of module
	 */
	HINSTANCE _MOTORCLASS WINAPI mlGetInstance( void );

	/*******************************************************************************
	 ************************ Motor-Funktionen *************************************
	 ******************************************************************************/

	/* Bewegt den aktuellen Motor zur gegebenen Position (in Nutzereinheiten)
	 * Die Position wird im Bogenmass (Nutzereinheiten) uebergeben
	 */
	BOOL _MOTORCLASS WINAPI mMoveToDistance( double position);

	/* Feststellen, ob sich der Motor noch bewegt.
	 * TRUE  : motor finished to move (stands still)
				  oder befindet sich in einem toten Bereich
	 */
	BOOL _MOTORCLASS WINAPI mIsMoveFinish( void );

	/* Feststellen, ob ein Motor (vielleicht auch nur frueher berechneter
	 * Winkel, wird in Translate gesetzt) ausserhalb des erlaubten Bereich
	 * ist.
	 * TRUE = Ausserhalb des erlaubten Bereichs, Fehler!
	 * FALSE= Innerhalb des erlaubten Bereichs, kein Problem :-)
	 */
	BOOL _MOTORCLASS WINAPI mIsRangeHit( void );

	/* Test ob Motor kalibriert ist und Werte ber einen gltigen Referenzpunktlauf
	 * vorhanden sind.
	 *
	 * FALSE= die Positionsangaben kommen aus einem ungltigen Koordinatensystem
	 * TRUE = die Positionsangaben sind relativ zur Absoluten bzw. zur Relativen Null
	 *
	 * == ini_InitialMove OR (ini_RestartPossible OR Referenzkunktlauf_ist_erfolgt)
	 */
	BOOL _MOTORCLASS WINAPI mlIsCalibrated( int axis );
	BOOL _MOTORCLASS WINAPI mIsCalibrated( void );

	/* TRUE = WinkelOffset != 0, sprich der Winkel bezieht sich auf
	 *         relative Null
	 * FALSE= WinkelOffset= 0.
	 */
	BOOL _MOTORCLASS WINAPI mlIsDistanceRelative( int axis );
	BOOL _MOTORCLASS WINAPI mIsDistanceRelative( void );

	/* Wenn FALSE, dann wird Befehl auf der 812 ignoriert !!!  BUG ??
	 * Stoppen des Motors.
	 * Der Bool-Wert gibt eventuell an, ob der Motor wieder neu gestartet
	 * werden soll. Leider Source ohne Doc nicht lesbar :-(
	 */
	void _MOTORCLASS WINAPI mStopDrive( BOOL );

	/* Ermittle Distance des Motors mit der gegebenen Motor ID.
	 * TRUE = Ok, Distanz konnte ermittelt werden
	         -> Distanz steht im Double-Parameter!!
	 * FALSE= sorry, funktion ging schief
	 */
	BOOL _MOTORCLASS WINAPI mlGetDistance( int, double& );
	BOOL _MOTORCLASS WINAPI mGetDistance( double& );

	// WIEVOR, jedoch auch in Bewegung mglich
	double _MOTORCLASS WINAPI mlGetDistanceProcess( int axis );
	double _MOTORCLASS WINAPI mGetDistanceProcess( void );

	//! neu Kullmann+Reinecker: Ermittelt die Nachkommastellengenauigkeit fr Angle- und Speed-Ausgaben (siehe DFmt)
	//ACHTUNG die Schrittweite muss eine Nachkommastelle mehr haben!!!
	UINT _MOTORCLASS WINAPI mGetDigits ( void );

	/* s. mlGetValue
	 */
	double _MOTORCLASS WINAPI mGetValue( EValueType );

	//Neu : Kullmann, Reinecker (22.10.2002); minimale, relative Antriebsposition in Nutzereinheiten
	double _MOTORCLASS WINAPI mGetAngleRelativeMin( void );

	//Neu : Kullmann, Reinecker (22.10.2002); maximale, relative Antriebsposition in Nutzereinheiten
	double _MOTORCLASS WINAPI mGetAngleRelativeMax( void );

	/* returns Unittype.
	 * this might be one of Grad,Minuten,Sekunden,Millimeter,Mikrometer
	 * Channel. See comhead.h for deklaration of TUniType
	 */
	EUnitType _MOTORCLASS WINAPI mlGetUnit( int axis );
	EUnitType _MOTORCLASS WINAPI mGetUnit( void );

	/* set value of a special ValueType. TValueType might be Speed or Width.
	 * Distance, MaxDistance, MinDistance allways return false an can not be set!
	 *
	 * wenn valType == Speed
	 *    Einheit ist Unit / Sekunde (s. ini-Params. Unit, SpeedScale)
	 *    der Wert wird automatisch durch MaxVelocity beschrnkt
	 *
	 * TRUE : Function successfull
	 * FALSE: Function failed
	 * This function depends on a LowLevel-Driver to return false/true correctly 
	 */
	BOOL _MOTORCLASS WINAPI mlSetValue( int axis, EValueType vtype, double value );
	BOOL _MOTORCLASS WINAPI mSetValue( EValueType valType, double value);

	/* Aktiviere einen Motor (Ansteuerungskarte?)
	 * Dieses geschieht direkt der Hardware Ebene
	 */
	void _MOTORCLASS WINAPI mActivateDrive( void );

	/* der Correctionstate wird entweder ON oder OFF (oder 0) geschaltet
	 */
	void _MOTORCLASS WINAPI mlSetCorrectionState( int axis, BOOL onoff );
	void _MOTORCLASS WINAPI mSetCorrectionState( BOOL );

	/* liefert die Bezeichnung des aktuellen motors (name-parameter in ini-file)
	 */
	LPCSTR _MOTORCLASS WINAPI mlGetAxisName( int axis );
	LPCSTR _MOTORCLASS WINAPI mGetAxisName( void );

	/* ausgabe von szUnit. Wird aus ini-file ausgelesen*/
	LPCSTR _MOTORCLASS WINAPI mlGetUnitName( int mid );
	LPCSTR _MOTORCLASS WINAPI mGetUnitName( void );

	/* rueckgabe eines String. Siehe DFmt in motors.cpp
	 */
	LPCSTR _MOTORCLASS WINAPI mGetDF( void );

	/* rueckgabe eines String. Siehe SFmt in motors.cpp
	 */
	LPCSTR _MOTORCLASS WINAPI mGetSF( void );

	/* setzen der relativen Null.
	 * Wenn Bool= TRUE -> Die relative Null wird um offset verschoben; ist bereits
	 *                     eine relative Null gesetzt, wird die relative Null
	 *                     relativ zur bisherigen relative Null gesetzt.
	 *          = FALSE-> Die relative Null wird zurckgesetzt, d.h. sie wird
	 *                     gleich der absoluten Null gesetzt.
	 */
	void _MOTORCLASS WINAPI mlSetRelativeZero( int axis, BOOL yes, double value );
	void _MOTORCLASS WINAPI mSetRelativeZero( BOOL, double offset);

	/* Ausfuehren des Kommandos in LPSTR direkt auf der Karte/dem Motor.
	 *
	 * sollte solcherlei direktes Ansprechen der Hardware in diesem
	 * High-Level-Layer nicht unmoeglich sein?
	 */
	int _MOTORCLASS WINAPI mExecuteCmd( LPSTR );

	/* Zwischenspeichern von dAngle, dAngleMin, dAngleMax, dAngleWidth und
	 * dSpeed. Achtung, es werden eventuell frueher gespeicherte Werte
	 * ueberschrieben. Zwischen 2 PushSettings muss also ein PopSettings
	 * erfolgen, sonst gehen diese Werte verloren!
	 * Frage: ist es wirklich sinnvoll, eine solche Routine nach aussen
	 * anzubieten? Sollte so etwas nicht irgendwie abgeschirmt werden?
	 */
	void _MOTORCLASS WINAPI mPushSettings( void );

	/* Zurueckschreiben der (letzten) mit PushSettings gespeicherten
	 * Werte in die entsprechenden Variablen!
	 */
	void _MOTORCLASS WINAPI mPopSettings( EMParameter );

	/*******************************************************************************
	 ****** "MoveScan"-Funktionen **************************************************
	 *******************************************************************************
	 *
	 * Ein MoveScan setzt den Motor mit den aktuellen Parametern
	 * in Bewegung und tastet die Position in regelmigen Abstnden
	 * ab. Der Entstehende Vektor von Positionsangaben steht danach zur
	 * Verfgung.
	 *
	 */

	/* liefert die maximale Anzahl von Abtastungen die durchgefhrt wird,
	 * d.h. die Gre des Vektors
	 * (liefert konstant 150)
	 */
	int _MOTORCLASS WINAPI mGetScanSize( void );

	/* liefert die tatschliche Anzahl von Abtastungen die durchgefhrt wurden
	 * d.h. die Anzahl der Werte im Vektor die gltig sind
	 */
	int _MOTORCLASS WINAPI mGetMoveFinishIdx( void );

	/* liefert einen Zeiger auf den Vektor der abgetasteten Positionswerte
	 * der Verktor hat die Gre mGetScanSize() und der mGetMoveFinishIdx()
	 * gltige Werte enthlt
	 */
	LPLONG _MOTORCLASS WINAPI mGetMoveScan( void );

	/*******************************************************************************
	 ************************ Motor-Dialoge ****************************************
	 ******************************************************************************/
	/* ich habe alles mit DLG im Namen hierhin an das Ende verbannt. Alles mit
	 * Dialogen hat hier nichts zu suchen und sollte an eine entsprechende
	 * Files bewegt werden!!
	 * should be moved out of m_layer !!!
	 */

	/* start InquireReferencePoint-Dialog
	 */
	void _MOTORCLASS WINAPI mlInquireReferencePointDlg( BOOL ); // Parameter gendert int -> BOOL (08.08.02)

	/* start PositionControl-Dialog
	 */
	void _MOTORCLASS WINAPI mlPositionControlDlg( void );

	/* start SetParameter-Dialog
	 */
	void _MOTORCLASS WINAPI mlSetParametersDlg ( void );

	/* send cm_CallExecuteScan and start Optimizing-Dialog
	 */
	void _MOTORCLASS WINAPI mlOptimizingDlg( void );
#ifdef __cplusplus

}
#endif


#endif 
// __LastLine__


