//******************************************************************************
//Datei     : protocol.cpp
//Projekt   : XCTL
//Subsystem : Protokollbuch
//Autor     : Jens Klier <klier@informatik.hu-berlin.de>
//            Institut fuer Informatik,
//            Humboldt-Universitaet Berlin
//Inhalt    : Deklaration der Exportfunktionen der protocol.dll
//******************************************************************************
//!neu klier Protokollbuch

#include "protocol\TProtocolLanguage.h"
#include "protocol\TProtocol.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

#if defined (Build_Protocol)
#define _PROTOCOLCLASS __declspec(dllexport)
#elif defined(Use_Protocol)
#define _PROTOCOLCLASS __declspec(dllimport)
#else
#define _PROTOCOLCLASS
#endif

BOOL SetProtocolValue( TProtocol*, int, LPCSTR );
BOOL SetProtocolValue( TProtocol*, int, LPCSTR, float );

#ifdef __cplusplus
extern "C"
{
#endif

	// Initialisiert die vom Protokollbuch benoetigten Variablen
	void _PROTOCOLCLASS WINAPI InitializeProtocol( HWND, TMeasurementParameter* );

	// Funktion erzeugt einen ProtokollTopographieDialog und zeigt ihn an,
	// beim ersten Aufruf wird das ProtocolTopography Objekt initialisiert
	BOOL _PROTOCOLCLASS WINAPI ViewOnProtocolTopographyDlg( void );
	BOOL _PROTOCOLCLASS WINAPI ViewProtocolTopographyDlg( void );

	// Funktion erzeugt einen ProtokollDiffractometryDialog und zeigt ihn an,
	// beim ersten Aufruf wird das ProtocolDiffractometry Objekt initialisiert
	BOOL _PROTOCOLCLASS WINAPI ViewOnProtocolDiffractometryDlg( void );
	BOOL _PROTOCOLCLASS WINAPI ViewProtocolDiffractometryDlg( void );

	// Diese Methoden werden von m_topo.cpp aufgerufen
	// und setzen einen ProtokollTopographyParameter
	//-------------------------------------------------------------
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyOn( void );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyOff( void );
	BOOL _PROTOCOLCLASS WINAPI IsProtocolTopographyOn( void );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyKKruemmung( float ); //Kollimator Krmmung
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyImpMax( float );    //Maximale Anzahl Impulse
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyHWB( float );        //Halbwertsbreite
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyImpsvor( float );    //Impulse vor Messung
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyImpsnach( float );   //Impulse nach Messung
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographySTE( void );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyCONLIMIT( void );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyCRT( float );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyINCDF( float );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyNBEXPOS( WORD );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyRSA( double );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyABE( float );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyEXPT( long );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyCONPOINT( float );  //Arbeitspunkt
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyDFvor( float Value );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyDFnach( float Value );
	BOOL _PROTOCOLCLASS WINAPI SaveProtocolTopographyParameter( void );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolTopographyReadOnly( void );
	//-------------------------------------------------------------
	// Diese Methoden werden von m_scan.cpp und m_arscan aufgerufen
	// und setzen einen ProtokollDiffractometryParameter
	//-------------------------------------------------------------
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryOn( void );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryOff( void );
	BOOL _PROTOCOLCLASS WINAPI IsProtocolDiffractometryOn( void );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMessdatei( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryNutzer( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScanart( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryOrientierung( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryUReflex( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySpannung( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryWellenlaenge( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScantyp( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScanmethode( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryStrom( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScanachse1( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySOffset1( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMinimum1( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMaximum1( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySchrittweite1( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryGeschwindigkeit1( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryScanachse2( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySOffset2( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMinimum2( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMaximum2( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometrySchrittweite2( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryGeschwindigkeit2( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryDetektor( LPCSTR );
	// BOOL WINAPI SetProtocolDiffractometryDetektor( int );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMonitor( LPCSTR );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMesszeit( float );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryImpulse( DWORD );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryWinkelbereich( float );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryKanalabstand( float );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryMesskanal( float );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryAddiere( int );
	BOOL _PROTOCOLCLASS WINAPI SaveProtocolDiffractometryParameter( void );
	BOOL _PROTOCOLCLASS WINAPI SetProtocolDiffractometryReadOnly( void );
	//-------------------------------------------------------------

#ifdef __cplusplus
}
#endif

// LastLine


