//##############################################################################
//                                                                            //
// TP_FUNK.H                                                                  //
//                                                                            //
// Subsystem : Topographie / Funktionalitt                                   //
// Benutzung durch andere Subsysteme erforderlich: NEIN                       //
//----------------------------------------------------------------------------//
// Autoren:  Thomas Kullmann, Gnther Reinecker (2002)                        //
//                                                                            //
// Stand  : 11.01.2003                                                        //
// letzter vollstndiger Test: 11.01.2003                                     //
//                                                                            //
//##############################################################################

#ifndef _TP_FUNK_H
#define _TP_FUNK_H

#include "utils\u_utils.h"
#pragma hdrstop

#include "detecuse\detecuse.h" // TDetector
//#include "datavisa\datavisa.h"
#include "workflow\workflow.h" // extern Steering
#include "motrstrg\motrstrg.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
// globale Typdeklarationen
//##############################################################################

enum EFormat { eDF= 1, eSF= 2 };

//##############################################################################
// TTopography
//##############################################################################

class TTopography : private IMainEvents // neu! Kullmann+Reinecker (31.01.2004) Singleton-Objekt freigeben, wenn Hauptprogramm beendet wird
{
	//------------------------------------------------------------------------------
	// Konstruktoren
private:
	TTopography( void );
	TTopography( const TTopography& ) {} // kein Copy-Konstruktor
	TTopography& operator= ( const TTopography& ); // kein Zuweisungsoperator
	//------------------------------------------------------------------------------
	// Methode zur Erzeugung einer Singleton-Instanz
public:
	static TTopography *GetInstance();
	//------------------------------------------------------------------------------
	// Accessor-Methoden
public:
	TDetector *GetDetector ( void )
	{
		return m_lnkDetector;
	};
	TDetector *GetActDetector ( void ); // gibt Zeiger auf den aktuellen Detektor zurck
	TDetector *GetMonitor ( void )
	{
		return m_lnkMonitor;
	};
	TMacroTag *GetMWorkPoint ( void )
	{
		return m_lnkMWorkPoint;
	};

	int GetMotor ( void )
	{
		return m_Motor;
	};
	LPCSTR GetMotorName ( int );  // Rckgabe der Antriebsbezeichnung
	LPCSTR GetMotorUnit ( void ); // Rckgabe der Einheit des aktuellen Motors
	LPCSTR GetDigitFormat ( const EFormat ); // Rckgabe eines Zahlenformats mit Digits oder Digits+1 Nachkommastellen des aktuellen Motors
	int GetActMotor ( void ); // gibt den Index des aktuellen Motors zurck
	int GetMotorCount ( void ); // gibt die Anzahl der verfuegbaren Motoren zurck
	double GetStartAngle ( void )
	{
		return m_dStartAngle;
	};
	double GetAngle ( void ); // gibt die Winkelposition des aktuellen Motors zurck
	UINT GetMotorDigits ( const EFormat ); // Rckgabe der Nachkommastellengenauigkeit des ausgewhlten Antriebs; entweder Digits (fr Positionsangaben) oder Digits+1 (fr Schrittweiten)

	LONG GetMeasurementTime ( void )
	{
		return m_lMeasurementTime;
	};
	DWORD GetCurTime ( void )
	{
		return m_dwCurrentTime;
	};
	DWORD GetStartTime ( void )
	{
		return m_dwStartTime;
	};
	float GetMaxTime ( void )
	{
		return m_fMaxTime;
	};

	DWORD GetMaxCounts ( void )
	{
		return m_dwMaxCounts;
	};
	WORD GetNumberCycle ( void )
	{
		return m_nNumberCycle;
	};
	WORD GetRestShots ( void )
	{
		return m_nRestShots;
	};

	float GetControlStep ( void )
	{
		return m_fControlStep;
	};
	float GetControlRange ( void )
	{
		return m_fControlRange;
	};
	float GetMoveStep ( void )
	{
		return m_fMoveStep;
	};
	float GetMaxAngleEscape ( void )
	{
		return m_fMaxAngleEscape;
	};
	float GetWorkPoint ( void )
	{
		return m_fWorkPoint;
	};
	float GetAngleBetweenShots ( void )
	{
		return m_fAngleBetweenShots;
	};
	//------------------------------------------------------------------------------
	// Mutator-Methoden
public:
	BOOL SetMotor( EAxisType );
	BOOL SetMotor( int );
	void SetDetector( TDetector *aDetector )
	{
		m_lnkDetector= aDetector;
	};
	void SetMonitor ( TDetector *aMonitor )
	{
		m_lnkMonitor= aMonitor;
	};

	BOOL SetMeasurementTime( LONG );
	void SetStartTime ( DWORD aTime )
	{
		m_dwStartTime= aTime;
	};
	void SetCurrentTime ( DWORD aTime )
	{
		m_dwCurrentTime= aTime;
	};
	BOOL SetMaxTime ( float );

	BOOL SetNumberCycle ( WORD );
	BOOL SetMaxCounts ( LONG );
	void SetRestShots ( WORD aShots )
	{
		m_nRestShots= aShots;
	};

	BOOL SetControlRange ( float );
	BOOL SetAngleBetweenShots( float );
	BOOL SetStartAngle ( double );
	BOOL SetControlStep ( float );
	BOOL SetMoveStep ( float );
	BOOL SetAngleWidth ( float );
	BOOL SetMaxAngleEscape ( float );
	BOOL SetWorkPoint ( float );
	//------------------------------------------------------------------------------
	// Setzen logischer Ausdrcke
public:
	void SetTimeFinish ( BOOL aValid )
	{
		m_bTimeFinish= aValid;
	};
	void SetTimeRunning ( BOOL aValid )
	{
		m_bTimeRunning= aValid;
	};
	void SetAdditionalTime ( BOOL aValid )
	{
		m_bAdditionalTime= aValid;
	};
	void SetStartPointOk ( BOOL aValid )
	{
		m_bStartPointOk= aValid;
	};
	void SetExceptionOccured( BOOL aValid )
	{
		m_bExceptionOccured= aValid;
	};
	void SetControlActive ( BOOL aValid )
	{
		m_bControlActive= aValid;
	};
	void SetMonitorUsed ( BOOL aValid )
	{
		m_bMonitorUsed= aValid;
	};
	void SetMultipleShot ( BOOL aValid )
	{
		m_bMultipleShot= aValid;
	};
	void SetSmallAngleSide ( BOOL aValid )
	{
		m_bSmallAngleSide= aValid;
	};
	//------------------------------------------------------------------------------
	// Lesen logischer Ausdrcke
public:
	BOOL HasMotor ( void )
	{
		return (m_Motor != -1);
	};
	BOOL HasAdditionalTime( void )
	{
		return m_bAdditionalTime;
	};
	BOOL IsMultipleShot ( void )
	{
		return m_bMultipleShot;
	};
	BOOL IsControlActive ( void )
	{
		return m_bControlActive;
	};
	BOOL IsSmallAngleSide ( void )
	{
		return m_bSmallAngleSide;
	};
	BOOL IsSetupOk ( void )
	{
		return m_bSetupOk;
	};
	BOOL IsStartPointOk ( void )
	{
		return m_bStartPointOk;
	};
	BOOL IsMonitorUsed ( void )
	{
		return m_bMonitorUsed;
	};
	BOOL IsTimeRunning ( void )
	{
		return m_bTimeRunning;
	};
	BOOL IsExeptionOccured( void )
	{
		return m_bExceptionOccured;
	};
	//------------------------------------------------------------------------------
	// Lesen und Setzen von Attributen der Ablaufsteuerung
public:
	void SetStrngStartPoint ( double );

	double GetStrngStartPoint ( void );
	double GetStrngDistance ( void );
	BOOL DetectorRequest ( void );
	float GetStrngPeakIntensity( void );
	float GetStrngStartIntensity( void );
	float GetStrngIntensity ( void );
	double GetStrngHwb ( void );
	//------------------------------------------------------------------------------
	// Auslsen von Aktionen
public:
	BOOL DoInitDetectors( void );  // Initialisierung des aktuellen Detektors und Monitors
	BOOL DoInitMotor( void );            // Initialisierung des Motors (bevorzugt Omega)
	void DoInitWorkPoint( void );        // Ablaufsteuerung initialisieren
	BOOL DoStartMeasure ( ECmdId, int, int, LPSTR, HWND ); // Durchfhrung einer Messung mittels Kommandoverarbeitung
	BOOL DoStartMeasure ( TMacroTag*, HWND ); // Durchfhrung einer Messung mittels Macroverarbeitung
	BOOL DoStopMeasure( void );     // Unterbrechung einer Messung
	void DoSetDetectorParams ( float, DWORD, BOOL ); // setzt Parameter fr die Belichtung
	void DoResetDetectorParams( void );  // lschen der im Detektor gespeicherten Daten zur Topography
	BOOL DoSaveMonitorSignal( void );
	BOOL DoSaveMotorSettings( void ); // Settings des aktuellen Motors sichern
	BOOL DoLoadMotorSettings( void ); // Settings des aktuellen Motors zurckschreiben

	//------------------------------------------------------------------------------
	// privates
private:
	BOOL DoInitMotor( EAxisType, BOOL ); // angegebenen Antrieb initialisieren
	virtual void  OnLeave ( ); // neu! Kullmann+Reinecker (31.01.2004) Singleton-Objekt freigeben, wenn Hauptprogramm beendet wird

	//------------------------------------------------------------------------------
	// Attribute
private:
	TDetector *m_lnkDetector;      // Messgeraet
	TDetector *m_lnkMonitor;       // Zweiter Zaehler fuer Monitor-Signal
	TMacroTag *m_lnkMWorkPoint;    // Makro zum Arbeitspunkt anfahren

	WORD m_nNumberCycle;       // Anzahl der Mess-Zyklen
	float m_fWorkPoint;         // Arbeitspunkt
	float m_fControlRange;      // Regel-Bereich nach Intensitaet
	float m_fControlStep;       // Regel-Schrittweite
	float m_fMoveStep;          // Schrittweite zum Anfahren des Arbeitspunktes
	float m_fMaxAngleEscape;    // Bei Weglaufen der Regelung
	int m_Motor;              // Regelmotor
	DWORD m_dwMaxCounts;        // Maximale Anzahl Impulse
	double m_dStartAngle;        // Startwinkel relativ zum Peak
	float m_fAngleBetweenShots; // Abstand bei Mehrfachbelichtung
	LONG m_lMeasurementTime;   // Mess-Zeit in Sekunden je Zyklus
	float m_fMaxTime;           // Maximale Messzeit
	DWORD m_dwCurrentTime;      // aktuelle Zeit einer Messung
	DWORD m_dwStartTime;        // Startzeit einer Messung

	BOOL m_bSmallAngleSide;    // Klein-Winkel= TRUE
	BOOL m_bMonitorUsed;       // ob Monitor-Detektor genutzt werden soll
	BOOL m_bMultipleShot;      // Mehrfach-Belichtungen
	BOOL m_bControlActive;     // ob die Messung laeuft
	BOOL m_bSetupOk;           // ob Motor und Detektor korrekt gesetzt
	BOOL m_bStartPointOk;      // ob der Arbeitspunkt angefahren wurde
	WORD m_nRestShots;         // Anzahl der restlichen Zyklen
	BOOL m_bTimeFinish;
	BOOL m_bAdditionalTime;    // ob zusaetzliche Zeit entstanden ist
	BOOL m_bExceptionOccured;  // ob Fehler aufgetreten
	BOOL m_bTimeRunning;

	static TTopography *s_Instance; // enthlt die EINZIGE Instanz
};

#endif //_TP_FUNK_H 
//------------------------------------------------------------------------------
// __LastLine__
