//#############################################################################
//                                                                           //
// U_FILES.H                                                                 //
// Subsystem: Utilities - Dateizugriff                                       //
// Benutzung durch andere Subsysteme erforderlich: JA                         //
// Implementation: U_FILES.CPP                                               //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand:   03.08.2002                                                       //
//                                                                           //
//#############################################################################

#ifndef __U_FILES_H
#define __U_FILES_H

#include "utils\u_values.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//#############################################################################
// globale Methoden (Lesen/ Schreiben von String-, Double- und Long-Werten aus/ in ini-Dateien)
//#############################################################################

// Signatur: IniReadString( out aReturn, in aFile, in aSection, in aKey,
//                          in aDef, in aLength )
// liest einen String-Wert aus der ini-Datei <aFile>, in Sektion <aSection>, Schlssel <aKey>
// und gibt das Ergebnis in <aReturn> zurck.
// Hat der gelesene Wert mehr als <aLength> Zeichen, wird der Rest abgeschnitten.
// Ist der Eintrag nicht vorhanden, wird <aDef> zurckgegeben.
void _CURVECLASS WINAPI IniReadString ( char*,
		LPCSTR, LPCSTR, LPCSTR,
		LPCSTR, const UINT );

// Signatur: IniWriteString( in aFile, in aSection, in aKey,
//                           in aString )
// schreibt <aString> in die ini-Datei <aFile>, in Sektion <aSection>, Schlssel <aKey>
// ==> TRUE <--> Wert erfolgreich geschrieben
BOOL _CURVECLASS WINAPI IniWriteString ( LPCSTR, LPCSTR, LPCSTR,
		LPCSTR );

// Signatur: IniReadLong( in aFile, in aSection, in aKey,
//                        in aDef )
// ==> der Bool-Wert aus der ini-Datei <aFile>, in Sektion <aSection>, Schlssel <aKey>
// Ist der Eintrag kein Bool-Wert (weder "0" noch "1"), wird <aDef> zurckgegeben.
// Hat der gelesene Wert Nachkommastellen, werden diese ignoriert.
BOOL _CURVECLASS WINAPI IniReadBool ( LPCSTR, LPCSTR, LPCSTR,
		const BOOL );

// Signatur: IniWriteBool( in aFile, in aSection, in aKey,
//                         in aBool )
// schreibt <aBool> in die ini-Datei <aFile>, in Sektion <aSection>, Schlssel <aKey>
// ==> TRUE <--> Wert erfolgreich geschrieben
BOOL _CURVECLASS WINAPI IniWriteBool ( LPCSTR, LPCSTR, LPCSTR,
		const BOOL );

// Signatur: IniReadLong( in aFile, in aSection, in aKey,
//                        in aDef )
// ==> der Long-Wert aus der ini-Datei <aFile>, in Sektion <aSection>, Schlssel <aKey>
// Hat der gelesene Wert Nachkommastellen, werden diese verworfen.(ABRUNDEN).
// Ist der Eintrag nicht vorhanden oder kein Long-Wert, wird <aDef> zurckgegeben.
// ACHTUNG: Im Gegensatz zur bisherigen Konvertierung mittels atof(...) wird "1a234" nicht als
//          Long-Wert akzeptiert! Es wird also <aDef> zurckgegeben.
LONG _CURVECLASS WINAPI IniReadLong ( LPCSTR, LPCSTR, LPCSTR,
		const LONG );

// Signatur: IniReadLong( in aFile, in aSection, in aKey,
//                        in aMin, in aDef, in aMax )
// ==> WIEVOR, der zurckzugebene Wert wird jedoch zustzlich ins
//     Intervall [<aMin>..<aMax>]
LONG _CURVECLASS WINAPI IniReadLong ( LPCSTR, LPCSTR, LPCSTR,
		const LONG, const LONG, const LONG );

// Signatur: IniWriteLong( in aFile, in aSection, in aKey,
//                         in aLong )
// schreibt <aLong> in die ini-Datei <aFile>, in Sektion <aSection>, Schlssel <aKey>
// ==> TRUE <--> Wert erfolgreich geschrieben
BOOL _CURVECLASS WINAPI IniWriteLong ( LPCSTR, LPCSTR, LPCSTR,
		const LONG );

// Signatur: IniReadFloat( in aFile, in aSection, in aKey,
//                         in aDef, in aDigits )
// WIE IniReadDouble, jedoch fr Float
float _CURVECLASS WINAPI IniReadFloat ( LPCSTR, LPCSTR, LPCSTR,
		const float, const UINT );

// Signatur: IniReadFloat( in aFile, in aSection, in aKey,
//                         in aMin, in aDef, in aMax, in aDigits )
// ==> WIEVOR, der zurckzugebene Wert wird jedoch zustzlich ins
//     Intervall [<aMin>..<aMax>] eingepasst
float _CURVECLASS WINAPI IniReadFloat ( LPCSTR, LPCSTR, LPCSTR,
		const float, const float, const float, const UINT );

// Signatur: IniWriteFloat( in aFile, in aSection, in aKey,
//                          in aFloat, in aDigits )
// WIE IniWriteDouble, jedoch fr Float
BOOL _CURVECLASS WINAPI IniWriteFloat ( LPCSTR, LPCSTR, LPCSTR,
		const float, const UINT );

// Signatur: IniReadDouble( in aFile, in aSection, in aKey,
//                          in aDef, in aDigits )
// ==> der Double-Wert aus der ini-Datei <aFile>, in Sektion <aSection>, Schlssel <aKey>
// Hat der gelesene Wert mehr als <aDigits> Nachkommastellen, werden nachfolgendene verworfen.(ABRUNDEN)
// Ist der Eintrag nicht vorhanden oder kein Double-Wert, wird <aDef> zurckgegeben.
// ACHTUNG: Im Gegensatz zur bisherigen Konvertierung mittels atof(...) wird "1a234" nicht als
//          Double-Wert akzeptiert! Es wird also <aDef> zurckgegeben.
double _CURVECLASS WINAPI IniReadDouble ( LPCSTR, LPCSTR, LPCSTR,
		const double, const UINT );

// Signatur: IniReadDouble( in aFile, in aSection, in aKey,
//                          in aDef, in aDigits )
// ==> WIEVOR, der zurckzugebene Wert wird jedoch zustzlich ins
//     Intervall [<aMin>..<aMax>]
double _CURVECLASS WINAPI IniReadDouble ( LPCSTR, LPCSTR, LPCSTR,
							  const double, const UINT );


// Signatur: IniReadDouble( in aFile, in aSection, in aKey,
//                          in aMin, in aDef, in aMax, in aDigits )
// ==> WIEVOR, der zurckzugebene Wert wird jedoch zustzlich ins
//     Intervall [<aMin>..<aMax>] eingepasst
double _CURVECLASS WINAPI IniReadDouble ( LPCSTR, LPCSTR, LPCSTR,
		const double, const double, const double, const UINT );

//### Statt der Nachkommastellengenauigkeit kann auch NKAUTO+<x> angegeben werden. Dann wird die Genauigkeit anhand des Zahlenwertes berechnet, per <x> kann man die Genauigkeit weiter vergrern.
// Signatur: IniWriteDouble( in aFile, in aSection, in aKey,
//                           in aDouble, in aDigits### )
// schreibt <aDouble> in die ini-Datei <aFile>, in Sektion <aSection>, Schlssel <aKey>
// Nach der zu schreibende Wert mehr als <aDigits> Nachkommastellen, werden nachfolgende verworfen.(ABRUNDEN)
// ==> TRUE <--> Wert erfolgreich geschrieben
BOOL _CURVECLASS WINAPI IniWriteDouble ( LPCSTR, LPCSTR, LPCSTR,
		const double, const UINT );

//#############################################################################
// TTxtRead
//#############################################################################

class _CURVECLASS TTxtRead
{
private:
	TTxtRead ( TTxtRead& ) {} // kein Copy-Konstruktor

public:
	// versucht die Datei zu ffnen und zhlt die Zeilen in der Datei
	TTxtRead ( const char* );

	virtual  ~TTxtRead ( void );

public:
	// beginnt wieder am Anfang der Datei
	void Restart();

	// liest die nchste Zeile und gibt sie in <result> zurck
	// ==> FALSE <--> Fehler oder Dateiende
	BOOL NextLine (string &result);

	// WIEVOR, die Zeile wird jedoch per Rckgabewert zurckgegeben ( 0 <--> Fehler oder Dateiende )
	// der zurckgegebene Wert darf nicht freigegeben werden, das erledigt TTxtRead
	char *NextLine ();

	// ==> TRUE <--> Dateiende erreicht oder Datei nicht vorhanden
	BOOL IsEOF ();

	// ==> TRUE <--> Datei ist vorhanden
	BOOL Exists ();

	// ==> Anzahl der Zeilen in dieser Datei; 0 bei leeren oder nicht vorhandenen Dateien
	UINT GetLines ();

private:
	FILE *m_File;
	char *m_Line;
	UINT m_Lines;
	BOOL m_Eof;
	BOOL m_Exists;
};

#endif //__U_FILES_H

//#############################################################################
// LASTLINE
//#############################################################################
