//#############################################################################
//                                                                           //
// U_VALUES.H                                                                //
// Subsystem: Utilities - Tools fr Zahlen und Strings                       //
// Benutzung durch andere Subsysteme erforderlich: JA                         //
// Implementation: U_VALUES.CPP                                              //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand:   07.11.2002                                                       //
//                                                                           //
//#############################################################################

#ifndef __U_VALUES_H
#define __U_VALUES_H

#if defined (Build_Library)
#define _CURVECLASS __declspec(dllexport)
#elif defined(Use_Library)
#define _CURVECLASS __declspec(dllimport)
#else
#define _CURVECLASS
#endif

#define STRICT
#ifdef __cplusplus
	#include <windows.h>
	#include <string.h>
	#include <windowsx.h>
	#include <mmsystem.h>
	#include <stdio.h>
	#include <math.h>
	#include <dos.h>
	#include <string.h>
	#include <string>
	typedef std::string string;
	#include <stdlib.h>
	#include <stdarg.h>
#endif

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//#############################################################################
// globale Typ- und Konstantendeklarationen
//#############################################################################

//gibt <toFree> frei und setzt <toFree> 0
#define _FREEOBJ(toFree) { if (toFree) { delete toFree; toFree= 0; }; }

//gibt das Feld <toFree> frei und setzt <toFree> 0
#define _FREELIST(toFree) { if (toFree) { delete []toFree; toFree= 0; }; }

const UINT _MAXLENDOUBLE= 50; // 50 Zeichen fr Double-Werte sollte bei weitem reichen

	// TMemoryPolice-Objekt nach einer Felddeklaration (array) erzeugen
	// wird der letzte Index des Feldes berschritten, liefert CheckStillContant des TMemoryPolice-Objekts FALSE, weil der Inhalt seines Members berschrieben wurde
	// Realisierung einmal im Steak und Heap mglich - TStackPolice bzw. THeapPolice
	class _CURVECLASS TMemoryPolice { 
	private:
		TMemoryPolice(const TMemoryPolice& aList) {}; // kein Copy-Konstruktor
		TMemoryPolice &operator= (const TMemoryPolice&) { return *this; }; // kein Zuweisungsoperator
	public:
		TMemoryPolice() {};
	protected:
		virtual bool CheckStillConstant()=0;
	};

#ifdef _DEBUG

	const char DEFENDSTR[]= "This line should stay as it is. Supervise this by a CheckStillConstant to proof that nobody exceedes his memory and overwrites your personal data.";

	class _CURVECLASS THeapPolice : protected TMemoryPolice {
	public:
		THeapPolice() : m_ToDefend(0) {
			m_ToDefend= new char[strlen(DEFENDSTR)+1];
			strcpy(m_ToDefend, DEFENDSTR);
		}
		~THeapPolice() {
			_FREELIST(m_ToDefend);
		}

		virtual bool CheckStillConstant() {
			return ( strcmp(m_ToDefend, DEFENDSTR)==0 );
		}

	private:
		char *m_ToDefend;	
	};
	class _CURVECLASS TStackPolice : protected TMemoryPolice {
	public:
		TStackPolice() {
			strcpy(m_ToDefend, DEFENDSTR);
		}

		virtual bool CheckStillConstant() {
			return ( strcmp(m_ToDefend, DEFENDSTR)==0 );
		}

	private:
		char m_ToDefend[147];	
	};

	// Klassen, von denen Objekte als Memoryleak zurckbleiben knnten, von TLeakPolice ableiten
	// in der DEBUG-Version wird bei Programmbeendigung 'LEAKPOLICE #<x>' ausgegeben, wenn wirklich noch Objekte dieses Typs im Speicher liegen
	// <x> gibt dann die Nummer an, wann das Objekt erzeugt wurde - Breakpoints im Konstruktor (Bedingung m_Counter==<x>) kann dann die Aufrufbedingung des Leak-Objekts ber die Steakaufrufliste analysiert werden
	class _CURVECLASS TLeakPolice {
		char m_LeakName[30];
		UINT m_Counter;
		static UINT s_Counter;

	public:
		TLeakPolice(LPCSTR aName="") : m_Counter(s_Counter++) {
			if ( !aName || strlen(aName)==0 ) sprintf(m_LeakName, "LEAKPOLICE #%d", aName, m_Counter);
			else sprintf(m_LeakName, "%s #%d", aName, m_Counter);
		}		
		~TLeakPolice() {
		}
	};

#else

	class _CURVECLASS THeapPolice : protected TMemoryPolice {
	public:
		virtual bool CheckStillConstant() {
			return true;
		}
	};
	class _CURVECLASS TStackPolice : protected TMemoryPolice {
	public:
		virtual bool CheckStillConstant() {
			return true;
		}
	};

	class _CURVECLASS TLeakPolice {
	public:
		TLeakPolice(LPCSTR aName= "") {}		
	};

#endif

////////////////////////////////////////////////////////////////////////////////
// Trace
////////////////////////////////////////////////////////////////////////////////

// Signatur: TraceMsg( in aStr )
// gibt <aStr> in einer MessageBox mit dem Titel "Trace" aus
void _CURVECLASS WINAPI TraceMsg ( LPCSTR );

// Signatur: TraceMsg( in aDouble )
// gibt <aDouble> mit der Nachkommastellenganauigkeit von 1 in einer MessageBox mit dem Titel "Trace" aus
void _CURVECLASS WINAPI TraceMsg ( const double );

// Signatur: Trace( in aStr ) bzw. TraceErr( in aStr )
// NUR DEBUG-VERSION: sendet <aStr> an den aktuellen Debugger, um dort angezeigt zu werden (funktioniert erst ab BC 5.02)
void _CURVECLASS WINAPI Trace ( LPCSTR aStr ); // fr kurzfristiges Degugging, im CVS sollte also nie eine Trace-Benutzung auftauchen
void _CURVECLASS WINAPI TraceErr ( LPCSTR aStr ); // fr dauerhafte Warnungen fr DEBUG-Anwender

// Signatur: Trace( in aDouble )
// NUR DEBUG-VERSION: sendet <aDouble> mit der Nachkommastellenganauigkeit von 1 an den aktuellen Debuffer, um dort angezeigt zu werden (funktioniert erst ab BC 5.02)
void _CURVECLASS WINAPI Trace ( const double aDouble );

////////////////////////////////////////////////////////////////////////////////
// Runden, Abrunden von Double-Werten
////////////////////////////////////////////////////////////////////////////////

// Signatur: DoubleRound( in aDouble, in aDigits )
// ==> <aDouble> gerundet auf <aDigits> Nachkommastellen
double _CURVECLASS WINAPI DoubleRound ( const double, const UINT );

// Signatur: DoubleRoundUp( in aDouble, in aDigits )
// ==> <aDouble> mit <aDigits> Nachkommastellen (AUFGERUNDET)
double _CURVECLASS WINAPI DoubleRoundUp ( const double, const UINT );

// Signatur: DoubleRoundOff( in aDouble, in aDigits )
// ==> <aDouble> mit <aDigits> Nachkommastellen, nachfolgende werden verworfen (ABRUNDEN)
double _CURVECLASS WINAPI DoubleRoundOff ( const double, const UINT );

// Signatur: Double2Float( in aDouble, in aDigits )
// ==> <aDouble> als Float-Wert mit <aDigits> Nachkommastellen (ohne das blde Ungenauigkeitsphnomen), nachfolgende werden verworfen (ABRUNDEN)
// ACHTUNG: Die Genauigkeit von Float ist geringer als von Double!!!
float _CURVECLASS WINAPI Double2Float ( const double, const UINT );

// Signatur: Float2Double( in aFloat, in aDigits )
// ==> <aFloat> als Double-Wert mit <aDigits> Nachkommastellen (ohne das blde Ungenauigkeitsphnomen), nachfolgende werden verworfen (ABRUNDEN)
double _CURVECLASS WINAPI Float2Double ( const float, const UINT );


////////////////////////////////////////////////////////////////////////////////
// Stringmanipulation
////////////////////////////////////////////////////////////////////////////////
// *** der Aufrufer ist dafr zustndig, dass der zurckgegebene String freigegeben wird !!!

// siehe GetFormatedStringMaxLen mit <aMaxLen>= 1024
// ==> formatierter Text ***
LPSTR _CURVECLASS WINAPI GetFormatedString( LPCSTR, ... );
LPSTR _CURVECLASS WINAPI GetFormatedString( LPCSTR, va_list& ) ;

// Signatur: GetFormatedStringMaxLen( in aFormat, in aMaxLen, ... )
// reserviert soviel Speicher, dass sprintf den in <aFormat> angegebenen String 
// mit der variablen Parameterliste ausgeben kann (max. jedoch <aMaxLen> Zeichen)
// ==> formatierter Text ***
LPSTR _CURVECLASS WINAPI GetFormatedStringMaxLen( LPCSTR aFormat, const UINT aMaxLen, ... );
LPSTR _CURVECLASS WINAPI GetFormatedStringMaxLen( LPCSTR aFormat, const UINT aMaxLen, va_list& );

////////////////////////////////////////////////////////////////////////////////
// Stringmanipulation
////////////////////////////////////////////////////////////////////////////////

// Signatur: StringReplace( in/out aString, in aOld, in aNew )
// ersetzt <aOld> durch <aNew> in <aString> und gibt das Ergebnis wieder in <aString> zurck
void _CURVECLASS WINAPI StringReplace ( char*, const char, const char );

// Signatur: StringIgnore( in/out aString, in aIgnore )
// entfernt alle <aIgnore>-Zeichen aus <aString> und gibt das Ergebnis wieder in <aString> zurck
// ist <aIgnore> die Nullterminierung, wird nichts gendert
void _CURVECLASS WINAPI StringIgnore ( char *, const char );

// Signatur: DoubleErr( in aString, in aDecimal, in aThousand )
// prft ob <aString> in eine Double-Wert konvertiert werden kann
// <aDecimal> ist das Kommazeichen (falls abweichend von '.')
// Als Tausendertrennzeichen wird <aThousand> verwendet. Ist dieses Zeichen die Nullterminierung, wird kein Tausendertrennzeichen verwendet.
// ==> -1 wenn Konvertierung mglich, SONST erste Position in <aString>, die die Konvertierung verhindert
int _CURVECLASS WINAPI DoubleErr ( LPCSTR, const char= '.', const char= '\0' );

// Signatur: IsDouble( in aString, in aDecimal, in aThousand )
// <aDecimal> ist das Kommazeichen (falls abweichend von '.')
// Als Tausendertrennzeichen wird <aThousand> verwendet. Ist dieses Zeichen die Nullterminierung, wird kein Tausendertrennzeichen verwendet.
// ==> TRUE <--> Konvertierung von <aString> in einen Double-Wert mglich
BOOL _CURVECLASS WINAPI IsDouble ( LPCSTR, const char= '.', const char= '\0' );

////////////////////////////////////////////////////////////////////////////////
// Konvertierung zwischen String und Double
////////////////////////////////////////////////////////////////////////////////

// Signatur: String2Double( in aString, in aDigits, in aRound, in aDecimal, in aThousand )
// ==> <aString> konvertiert in einen Double-Wert - gerundet bzw. abgerundet auf <aDigits> Nachkommastellen (<aRound>==TRUE bzw. <aRound>==FALSE)
// Das zur Markierung der Kommastelle verwendete Zeichen, steht in <aDecimal>.
// Als Tausendertrennzeichen wird <aThousand> verwendet. Ist dieses Zeichen die Nullterminierung, wird kein Tausendertrennzeichen verwendet.
double _CURVECLASS WINAPI String2Double ( LPCSTR, const UINT, const BOOL= TRUE, const char= '.', const char= '\0' );

const UINT NKAUTO= 1000;

//### Statt der Nachkommastellengenauigkeit kann auch NKAUTO+<x> angegeben werden. Dann wird die Genauigkeit anhand des Zahlenwertes berechnet, per <x> kann man die Genauigkeit weiter vergrern.
// Signatur: Double2String( out aReturn, in aDouble, in aDigits###, in aRound, in aDecimal, in aThousand )
// konvertiert <aDouble> in einen String-Wert und gibt das Ergebnis
// auf <aDigits> Nachkommastellen gerundet bzw. abgerundet (<aRound>==TRUE bzw. <aRound>==FALSE)
// in <aReturn> zurck
// Das zur Markierung der Kommastelle verwendete Zeichen, steht in <aDecimal>.
// Als Tausendertrennzeichen wird <aThousand> verwendet. Ist dieses Zeichen die Nullterminierung, wird kein Tausendertrennzeichen verwendet.
void _CURVECLASS WINAPI Double2String ( char*, const double, UINT, const BOOL= TRUE, const char= '.', const char= '\0' );

////////////////////////////////////////////////////////////////////////////////
// Formatierte Zeitausgabe HH:MM:SS bei Eingabe in Millisekunden
////////////////////////////////////////////////////////////////////////////////

void _CURVECLASS WINAPI GetTimeString( char*, long );

////////////////////////////////////////////////////////////////////////////////
// Min, Max
////////////////////////////////////////////////////////////////////////////////

// Signatur: SetInRange( in/out aValue, in aBorder1, in aBorder2 )
// passt <aValue> in das Intervall [aBorder1, aBorder2]
// es ist dabei egal ob <aBorder1> kleiner als <aBorder2> oder umgekehrt
// <aValue> wird bei berschreitung auf die jeweils berschrittete Grenze korrigiert
// das Ergebnis ist TRUE, wenn keine Anpassung ntig war, ansonsten FALSE
BOOL _CURVECLASS WINAPI SetInRange ( double&, double, double );
BOOL _CURVECLASS WINAPI SetInRange ( long&, long, long );

//Datum: 16.08.2002
/*
int        WINAPI maxi( int, int );
long       WINAPI maxl( long, long );
float      WINAPI maxf( float, float );
double     WINAPI maxd( double, double );
int        WINAPI mini( int, int );
long       WINAPI minl( long, long );
float      WINAPI minf( float, float );
double     WINAPI mind( double, double );
*/

#endif // __U_VALUES_H
