//#############################################################################
//                                                                           //
// MJ_GUI.CPP                                                                //
//                                                                           //
// Subsystem : neue Manuelle Justage / Oberflche                            //
// Benutung durch andere Subsysteme erforderlich: JA                         //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand  : 21.04.2002                                                       //
//                                                                           //
//#############################################################################

#include "internls\evrythng.h" // GermanVersion
#include "winresrc\rc_def.h"   // Ressourcen-IDs
#include "help\help_def.h"     // Help-IDs
#include <assert.h> // assert()

#include "manjust\mj_gui.h" // SCHNITTSTELLE fr diese Datei

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

extern HINSTANCE hModuleInstance;

//#############################################################################
// sprachspezifische Konstanten
//#############################################################################

static const char _NODRIVE[]= "-"; // sprachunabhngige Kennzeichung fr "kein Antrieb" in der ini-Datei

static const UINT USR_OTHERCONTROLS= WM_USER + 100;
static const UINT USR_TB_CONTROLS= WM_USER + 102;
static const UINT USR_TB_KINDOFMOVE= WM_USER + 103;
static const UINT USR_TB_STARTSTOP= WM_USER + 104;
static const UINT USR_TB_MOTIONPARAM= WM_USER + 105;
static const UINT USR_TB_POSITIONS= WM_USER + 106;
static const UINT USR_TB_OFFSET= WM_USER + 107;
static const UINT USR_TB_UNITS= WM_USER + 108;
static const UINT USR_TB_CONTROLSENABLE= WM_USER + 109;
static const UINT USR_TB_MOTORLISTENABLE= WM_USER + 110;
static const UINT USR_TB_KINDOFMOVEENABLE= WM_USER + 111;
static const UINT USR_TB_STARTSTOPENABLE= WM_USER + 112;
static const UINT USR_TB_MOTIONPARAMENABLE= WM_USER + 113;
static const UINT USR_TB_POSITIONSENABLE= WM_USER + 114;
static const UINT USR_TB_OFFSETENABLE= WM_USER + 115;
static const UINT USR_TB_MOTORCHANGED= WM_USER + 116;

//#############################################################################
// TManJustageDlg
//#############################################################################

//*****************************************************************************
// KONSTRUKTOR-DESTRUKTOR
//*****************************************************************************

TManJustageDlg::TManJustageDlg ( void ) : TModalDlg( "MANUALADJUSTMENTDLG", GetMainInstance() ), m_lnkManJustage( 0 ), m_lnkTimer( 0 ), m_Selection( 0 )
{
	m_lnkManJustage= new TManJustage(); //POLLING, d.h. KEINE Referenz auf uns bergeben
	m_lnkTimer= new TInterfaceTimer( this, 100 ); //Observer, d.h. Timer benachrichtigt uns

	// enthlt fr jeden Teilbereich, welcher Index zuletzt ausgewhlt war
	m_Selection= new UINT[ TB_Count() ];
	for (UINT i= 0; i < TB_Count(); i++)
		m_Selection[i]= m_lnkManJustage->GetMotorCount() + 1; // bislang ist nichts ausgewhlt (sonst keine nderung bei SetSelection und Aktualisierung bleibt aus)

	m_Parallel= IniReadBool(GetCFile(), "ManualAdjustment", "Parallel", TRUE);
	m_StateRefresh= IniReadBool(GetCFile(), "ManualAdjustment", "StateRefresh", TRUE);
	m_DisplayNoActionMsg= IniReadBool(GetCFile(), "ManualAdjustment", "DisplayNoActionMsg", FALSE);
};
//-----------------------------------------------------------------------------

TManJustageDlg::~TManJustageDlg ( void )
{
	IniWriteBool(GetCFile(), "ManualAdjustment", "Parallel", m_Parallel);
	IniWriteBool(GetCFile(), "ManualAdjustment", "StateRefresh", m_StateRefresh);
	IniWriteBool(GetCFile(), "ManualAdjustment", "DisplayNoActionMsg", m_DisplayNoActionMsg);
	_FREEOBJ( m_Selection );
	_FREEOBJ( m_lnkManJustage );
	_FREEOBJ( m_lnkTimer );
};

//*****************************************************************************
// EREIGNISSE DER BASISKLASSE
//*****************************************************************************

BOOL TManJustageDlg::Dlg_OnInit ( HWND, HWND, LPARAM )
{
	m_lnkManJustage->BeginUpdate();
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
	{ // alle drei Teilbereiche
		UINT i; // zum Durchsuchen der Liste der verfgbaren Antriebe

		// Antriebs-Auswahllisten mit verfgbaren Antrieb und "kein Antrieb" fllen
		BOOL bValid;
		for (i= 0; i < m_lnkManJustage->GetMotorCount(); i++)
			CtrlAddString( GetResId(tb, riMotorList), m_lnkManJustage->GetMotorName(i, bValid) );
		CtrlAddString( GetResId(tb, riMotorList), sth_mj_NoMotor );

		// Bezeichnung des zuletzt verwendeten Antriebs aus ini-Datei laden, _NODRIVE ("-"= kein Antrieb) oder "" (erster Aufruf dieses Dialogfensters)
		char tbName[ 13 ]; // Teilbereich<Ziffer> + Nullterminierung
		TB_GetName( tb, tbName );
		char mName[ _MAXLENCHARACTERISTIC + 1 ]; // + Nullterminierung
		IniReadString( mName, GetCFile(), "ManualAdjustment", tbName, "", _MAXLENCHARACTERISTIC );

		// Index des auszuwhlenden Antriebs ermitteln
		int mSel;
		if ( strcmp(mName, _NODRIVE) != 0 )
		{
			// Index des zuletzt ausgewhlten Antriebs ermitteln; Antrieb darf nicht mehrfach angezeigt werden
			if ( strcmp(mName, "") == 0 )
				mSel= -1; // -1 <--> erster Aufruf dieses Dialogfensters
			else
				mSel= m_lnkManJustage->GetMotorIdx( mName ); // -1 <--> Antrieb ist nicht mehr vorhanden
			if ( Selection2TB(mSel) != ETB(0) )
				mSel= -1; // Antrieb wird bereits angezeigt
			// falls der Index noch nicht feststeht UND der Antrieb noch nicht angezeigt wird: whle den ersten verfgbaren Antrieb, der noch nicht angezeigt wird
			for (i= 0; i < m_lnkManJustage->GetMotorCount(); i++)
				if ( (mSel == -1) && (Selection2TB(i) == (ETB)0) && (CanSetSelection(tb,i)) )
					mSel= i;
			if ( mSel == -1 )
				mSel= m_lnkManJustage->GetMotorCount(); // kein "erster verfgbarer Antrieb": whle "kein Antrieb"
		}
		else
			mSel= m_lnkManJustage->GetMotorCount(); // "kein Antrieb", darf auch mehrfach angezeigt werden

		// Antrieb auswhlen, Inhalt der Steuerelemente im Teilbereich aktualisieren, freigeben bzw. sperren und ausgrauen
		SetSelection( tb, mSel );
	};

	// Halbwertsbreite beschriften und Steuerelemenete (auerhalb der Teilbereiche) freigeben bzw. sperren und ausgrauen
	Notice( USR_OTHERCONTROLS, (ETB)0 );
	m_lnkManJustage->EndUpdate();

	// Beschriftung von '<Psd>-Offset...'
	BOOL bValid;
	LPCSTR dName= m_lnkManJustage->GetDetectorName( bValid );
	CtrlSetFormat( cm_PsdOffset, sth_mj_PsdOffset, dName );

	// HotKey's fr Betriebsarten in den Teilbereichen UND zum Wechseln zwischen den Teilbereichen (nur Win32)
	LoadHotKeys( LoadAccelerators(GetMainInstance(), MAKEINTRESOURCE(ACC_ManualAdjustment)) ); // Accelerator aus Ressourcen laden
	return TRUE;
};
//-----------------------------------------------------------------------------

LRESULT TManJustageDlg::OnEvent ( HWND aWnd, UINT aId, WPARAM wParam, LPARAM lParam )
{
	if ( TBasicDialog::OnEvent(aWnd, aId, wParam, lParam) )
		return TRUE; // Botschaft verarbeitet
	switch ( aId )
	{ // die folgenden Nachrichten erlauben das Fllen und Freigeben bzw. Sperren und Ausgrauen der Steuerelemente im Hintergrund
		case USR_OTHERCONTROLS:
			OtherControls();
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_CONTROLS:
			OnTB_Controls( (ETB)wParam, (BOOL)lParam );
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_KINDOFMOVE:
			OnTB_KindOfMove( (ETB)wParam, (BOOL)lParam );
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_STARTSTOP:
			OnTB_StartStop( (ETB)wParam, (BOOL)lParam );
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_MOTIONPARAM:
			OnTB_MotionParam( (ETB)wParam, (BOOL)lParam );
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_POSITIONS:
			OnTB_Positions( (ETB)wParam, (BOOL)lParam );
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_OFFSET:
			OnTB_Offset( (ETB)wParam, (BOOL)lParam );
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_UNITS:
			OnTB_Units( (ETB)wParam );
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_CONTROLSENABLE:
			OnTB_ControlsEnable( (ETB)wParam );
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_MOTORLISTENABLE:
			OnTB_MotorListEnable( (ETB)wParam );
			return TRUE; // Botschaft wurde verarbeitet
		
		case USR_TB_KINDOFMOVEENABLE:
			OnTB_KindOfMoveEnable( (ETB)wParam );
			return TRUE; // Botschaft wurde verarbeitet
		
		case USR_TB_STARTSTOPENABLE:
			OnTB_StartStopEnable( (ETB)wParam );
			return TRUE; // Botschaft wurde verarbeitet
		
		case USR_TB_MOTIONPARAMENABLE:   // Spezialfall: siehe Notice()
			OnTB_MotionParamEnable( (ETB)wParam );
			return TRUE; // Botschaft wurde verarbeitet
		
		case USR_TB_POSITIONSENABLE:
			OnTB_PositionsEnable( (ETB)wParam );
			return TRUE; // Botschaft wurde verarbeitet
		
		case USR_TB_OFFSETENABLE:
			OnTB_OffsetEnable( (ETB)wParam );
			return TRUE; // Botschaft wurde verarbeitet

		case USR_TB_MOTORCHANGED:
			TB_MotorChanged( (ETB)wParam );
			return TRUE; // Botschaft wurde verarbeitet
	}
	return FALSE; // Botschaft nicht verarbeitet
};
//-----------------------------------------------------------------------------

void TManJustageDlg::Dlg_OnCommand ( HWND aHWnd, int aId, HWND aCtrlWnd, UINT aCode )
{
	if ( aCode==ONENTER ) return;
	// prfe zuerst die Steuerelemente in den Teilbereichen, diese wurden in drei Gruppen eingeteilt:
	// LINKS: Auswahlliste Antrieb, Betriebsarten
	// MITTE: Start/ Stop und Betriebsarten, wenn [ENTER] gedrckt wurde
	// RECHTS: Relative Null setzen, aufheben; Offset setzen
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
	{
		if (  ( (TB_CheckLeft(tb, aId)) || (TB_CheckMiddle(tb, aId)) || (TB_CheckRight(tb, aId)) ) ||
			   ( (aCode == ONEXIT) && (TB_CheckExit(tb, aId)) )  )
			return; // Botschaft verarbeitet
	}

	// prfe dann die Steuerelemente auerhalb der Teilbereiche
	switch ( aId )
	{
		case cm_PsdOffset:
			OnPsdOffset();
			return; // Botschaft verarbeitet

		case cm_HwbMeasure:
			if ( !m_lnkManJustage->IsMeasuring() )
			{ // dann starten
				if ( m_lnkManJustage->DoStartMeasure( GetHandle() ) )
					m_lnkTimer->StartTimerIm(); // wenn Makroverarbeitung gestartet sorgt der Timer fr Bildschirmaktualisierung
				else
					OnNoAction(); // Messung kann nicht gestartet werden
			}
			else if ( m_lnkManJustage->DoStopMeasure() )
				m_lnkTimer->Immediately(); // Makroverarbeitung stoppen: der Timer stoppt automatisch, wenn nichts mehr zu berwachen ist
			else
				OnNoAction(); // Messung kann nicht gestoppt werden
			return; // Botschaft verarbeitet

		case cm_ChooseTB:   // Tastenkombination "Ctrl+Tab"
			OnChooseTB();
			return; // Botschaft verarbeitet

		case cm_Help:
			WinHelp(GetHandle(), GetHelpFile(), HELP_CONTEXT, Help_ManualAdjustmentDlg);
			return; // Botschaft verarbeitet
			
		case cm_CounterSet: // Aktualisieren des Detektorfensters
			m_lnkManJustage->UpdateDetector();
			return; // Botschaft verarbeitet

		case IDOK:
			return; // [ENTER]-Botschaft ignorieren; SONST schliet sich das Dialogfenster

		default:
			TModalDlg::Dlg_OnCommand( aHWnd, aId, aCtrlWnd, aCode );
	};
};
//-----------------------------------------------------------------------------

void TManJustageDlg::Dlg_OnHScrollBar ( HWND, HWND aCtrl, UINT aCode, int aPos )
{
	BOOL bValid;
	// in welchem Teilbereich wurde die Bildlaufleiste benutzt?
	ETB tb= ETB(0); // "unbekannte Bildlaufleiste"
	ETB tblist= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tblist) )
		if ( aCtrl == GetCtrl(tblist, riRelativePos) )
			tb= tblist; // alle drei Teilbereiche

	// welcher Index ist in dem Teilbereich ausgewhlt?
	UINT mSel= TB2Selection( tb ); // ggf. GetMotorCount(), d.h.
	EMotionType mt= m_lnkManJustage->GetMotionType( mSel, bValid );
	if ( !bValid )
		return; // "unbekannter Teilbereich" oder "kein Antrieb"

	// Spezialfall: Direktbetrieb
	if ( mt == mtDirect )
	{
		if ( (aCode != SB_THUMBTRACK) && (aCode != SB_ENDSCROLL) )
			MessageBox( sth_mj_NoScroll, sth_mj_ErrTitle, MBSTOP ); // THUMBTRACK und SB_ENDSCROLL werden ausgeschlossen, nach einer Interaktion des Anwenders erscheinen sonst mehrere Meldungsfenster
		return; //Steuerung nur im Fahr- und Schrittbetrieb
	}

	// aufwrts, merkwrdig ist die widersprchliche Benennung der Scrollcodes
	if ( ( (aCode == SB_LINEDOWN) || (aCode == SB_PAGEDOWN) ) || (aCode == SB_BOTTOM) || ( (aCode == SB_THUMBPOSITION) && (aPos > ScrollBar_GetPos(aCtrl)) ) )
	{
		if ( mt == mtDrive )
		{
			if ( m_lnkManJustage->DoDrive(mSel, dUp) )
				m_lnkTimer->StartTimerIm();
			else
				OnNoAction(); // Fahrbetrieb nicht mglich
		}
		if ( mt == mtStep )
		{
			if ( m_lnkManJustage->DoStep(mSel, dUp) )
				m_lnkTimer->StartTimerIm();
			else
				MessageBeep( -1); // Schrittbetrieb nicht mglich
		}
	}

	// abwrts, merkwrdig ist die widersprchliche Benennung der Scrollcodes
	if ( ( (aCode == SB_LINEUP) || (aCode == SB_PAGEUP) ) || (aCode == SB_TOP) || ( (aCode == SB_THUMBPOSITION) && (aPos < ScrollBar_GetPos(aCtrl)) ) )
	{
		if ( mt == mtDrive )
		{
			if ( m_lnkManJustage->DoDrive(mSel, dDown) )
				m_lnkTimer->StartTimerIm();
			else
				OnNoAction(); // Fahrbetrieb nicht mglich
		}
		if ( mt == mtStep )
		{
			if ( m_lnkManJustage->DoStep(mSel, dDown) )
				m_lnkTimer->StartTimerIm();
			else
				MessageBeep( -1); // Schrittbetrieb nicht mglich
		}
	}

	//Fahrbetrieb: Bewegung endet, wenn [Maus-]Taste losgelassen wird (Bildlaufpfeil nicht mehr gedrckt wird)
	if ( (aCode == SB_ENDSCROLL) && (mt == mtDrive) )
	{
		if ( m_lnkManJustage->DoStop(mSel) )
			m_lnkTimer->Immediately(); // Bewegung stoppen: der Timer stoppt automatisch, wenn nichts mehr zu aktualisieren ist
		else
			OnNoAction(); // Stoppen der Bewegung im Fahrbetrieb nicht mglich
	}
};
//-----------------------------------------------------------------------------

void TManJustageDlg::LeaveDialog ( void )
{
	m_lnkManJustage->DoStopEverything();
};

//*****************************************************************************
// EREIGNISSE
//*****************************************************************************

void TManJustageDlg::OnTimer ( TBasicTimer *const )
{
	BOOL bValid;
	BOOL bStopTimer= TRUE; //Standard: alles ruht, Timer wird nicht mehr bentigt

	m_lnkManJustage->BeginUpdate();
	BOOL isMeasuring= m_lnkManJustage->IsMeasuring();
	BOOL kzMeasuring= m_lnkManJustage->GetKzMeasuring();
	// zuerst prfen wir was die Antriebe machen; aber nur wenn KEIN Makro verarbeitet wird (Makroverarbeitung == alle Antriebe reserviert fr Bewegung)
	// nur die Antriebe in den Teilbereichen berprfen
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	if ( !isMeasuring )
		while ( TB_Next(tb) )
		{ //05.02.2003 wir prfen nicht mehr alle Antriebe, sondern nur die angezeigten in den Teilbereichen
			int i= TB2Selection(tb); // dieser Antrieb wird angezeigt
			if ( i == m_lnkManJustage->GetMotorCount() )
				continue; // kein Antrieb ist ausgewhlt
			BOOL isMoving= m_lnkManJustage->IsMoving(i, bValid);
			BOOL kzMoving= m_lnkManJustage->GetKzMoving(i, bValid);
			// seit dem letzten OnTimer wurde dieser Antrieb gestartet
			if ( (isMoving) && (!kzMoving) && (bValid) )
			{
				bStopTimer= FALSE; // die Bewegung dieses Antriebs weiter verfolgen
				m_lnkManJustage->SetKzMoving( i, TRUE ); // jetzt hat auch die Oberflche erfahren, dass der Antrieb gestartet wurde
				OnMotionStarts( i );

				// der Antrieb ist immernoch in Bewegung
			}
			else if ( (isMoving) && (kzMoving) && (bValid) )
			{
				bStopTimer= FALSE; // die Bewegung dieses Antriebs weiter verfolgen
				OnMotionProgress( i );

				// seit dem letzten OnTimer hat dieser Antrieb gestoppt
			}
			else if ( (!isMoving) && (kzMoving) && (bValid) )
			{
				// diesen Antrieb brauchen wir nicht weiter berwachen
				m_lnkManJustage->SetKzMoving( i, FALSE ); // jetzt hat auch die Oberflche erfahren, dass der Antrieb gestoppt wurde
				OnMotionStops( i );
			}
		}

	// dann noch die Makroverarbeitung prfen
	// seit dem letzten OnTimer wurde die Makroverarbeitung gestartet
	if ( (isMeasuring) && (!kzMeasuring) )
	{
		bStopTimer= FALSE; // die Makroverarbeitung wollen wir weiter verfolgen
		OnMeasureStarts();
	}
	else if ( (isMeasuring) && (kzMeasuring) )
	{ // das Makro luft immernoch
		bStopTimer= FALSE; // die Makroverarbeitung wollen wir weiter verfolgen
		OnMeasureProgress();
	}
	else if ( (!isMeasuring) && (kzMeasuring) )
	{ // seit dem letzten OnTimer hat die Makroverarbeitung gestoppt
		OnMeasureStops(); 
		// Makroverarbeitung nicht weiter berwachen
	} 
	m_lnkManJustage->EndUpdate();
	if (bStopTimer)
		m_lnkTimer->StopTimer(); // wenn alle Antriebe stehen und kein Makro verarbeitet wird
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnMotionStarts ( const UINT aSel )
{
	ETB tb= Selection2TB(aSel);
	if ( tb == (ETB)0 )
		return;

	OnTB_ControlsEnable( tb ); // zuerst die Steuerelemente freigeben bzw. sperren und ausgrauen
	Notice( USR_TB_STARTSTOP, tb ); // "Start" mit "Stop" beschriften
	Notice( USR_OTHERCONTROLS, (ETB)0 ); // Steuerelemente auerhalb der Teilbereiche
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnMotionProgress ( const UINT aSel )
{
	ETB tb= Selection2TB(aSel);
	if ( tb != (ETB)0 )
		OnTB_Positions( tb, FALSE ); // Notice( USR_TB_POSITIONS, Selection2TB(aSel) ); ist zu langsam
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnMotionStops ( const UINT aSel )
{
	ETB tb= Selection2TB(aSel);
	if ( tb == (ETB)0 )
		return;

	OnTB_ControlsEnable( tb ); // zuerst die Steuerelemente freigeben bzw. sperren und ausgrauen
	Notice( USR_TB_POSITIONS, tb ); // Positionen aktualisieren
	Notice( USR_TB_STARTSTOP, tb ); // "Stop" mit "Start" beschriften
	Notice( USR_OTHERCONTROLS, (ETB)0 ); // Steuerelemente auerhalb der Teilbereiche
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnMeasureStarts ( void )
{
	m_lnkManJustage->SetKzMeasuring( TRUE ); // jetzt hat auch die Oberflche erfahren, dass die Makroverarbeitung gestartet wurde

	// "Halbwertsbreite wird gemessen..." anzeigen
	CtrlSetText( id_HwbState, sth_mj_IsMeasuring );

	// alle Steuerelemente aktualisieren, sperren und ausgrauen
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
	{ // alle drei Teilbereiche
		Notice( USR_TB_STARTSTOP, tb ); // "Start" mit "Stop" beschriften
		Notice( USR_TB_CONTROLSENABLE, tb ); // alle Steuerelemente speeren
	}
	Notice( USR_OTHERCONTROLS, (ETB)0 );
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnMeasureProgress ( void )
{
	// Fortschritt zeigen
	int idx, count;
	char *progress= new char [ MaxString + 1 ]; //siehe TCmd::GetShowData (und abgeleitete Klassen)
	if ( !m_lnkManJustage->GetMeasureProgress(idx, progress, count) ) {
		_FREELIST(progress);
		return; // Messung ist bereits beendet
	}

	CtrlSetFormat( id_HwbState, sth_mj_Progress, idx + 1, count, progress );
	_FREELIST(progress);
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnMeasureStops ( void )
{
	m_lnkManJustage->SetKzMeasuring( FALSE ); // jetzt hat auch die Oberflche erfahren, dass die Makroverarbeitung gestoppt hat oder wurde

	BOOL bValid;
	// Wert der Halbwertsbreitenmessung anzeigen
	if ( !m_lnkManJustage->IsMeasureReset() )
	{ // Makroverarbeitung und Messung erfolgreich beendet
		double hwb= m_lnkManJustage->GetMeasureHwb();
		char *value= new char[ _MAXLENDOUBLE + 1 ]; // Double-Werte haben max. 50 Zeichen + Nullterminierung
		Double2String( value, hwb, m_lnkManJustage->GetDigits( m_lnkManJustage->GetHwbAxis(), bValid ), FALSE, _DECIMAL); // wenn zu viele Nachkommastellen: Rest abschneiden
		LPCSTR unit= m_lnkManJustage->GetUnit( m_lnkManJustage->GetMotorIdx(m_lnkManJustage->GetHwbAxis()), bValid );

		MessageBeep( -1 );
		CtrlSetFormat( id_HwbState, sth_mj_MeasureValue, value, unit ); 
		_FREELIST(value);
		char buf[MaxString+1];
		CtrlGetText( id_HwbState, buf, MaxString);
		SetInfo( buf );
		// Messung wurde abgebrochen
	}
	else
		CtrlSetText( id_HwbState, sth_mj_MeasureInterrupted );

	// alle Steuerelemente aktualisieren, freigeben bzw. sperren und ausgrauen
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
	{ // alle drei Teilbereiche
		Notice( USR_TB_POSITIONS, tb );  // Positionen aktualisieren
		Notice( USR_TB_STARTSTOP, tb ); // "Stop" mit "Start" beschriften
		Notice( USR_TB_CONTROLSENABLE, tb ); // alle Steuerelemente speeren
	}
	Notice( USR_OTHERCONTROLS, (ETB)0 );
};

//*****************************************************************************
// Verhalten aller Steuerelemente auerhalb der Teilbereiche
//*****************************************************************************

void TManJustageDlg::OnPsdOffset ( void )
{
	int idx= m_lnkManJustage->GetDetectorAxisIdx();
	if ( !m_lnkManJustage->CanSetOffset(idx) )
	{ // PSD-Offsetdialog nur anzeigen wenn wir drfen; includes m_lnkManJustage->HasDetectorAxis()
		OnNoAction();
		return;
	}

	//Dialog anzeigen
	TModalDlg *dlg= new TPsdOffsetDlg( idx, m_lnkManJustage );
	if ( dlg ) dlg->ExecuteDialog( GetHandle() );
	_FREEOBJ(dlg);

	m_lnkManJustage->BeginUpdate();
	// ALLE Teilbereiche aktualisieren, wir wissen nicht wo <_PSDAXIS> angezeigt wird
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
	{
		Notice( USR_TB_POSITIONS, tb );    // Positionen aktualisieren,
		Notice( USR_TB_OFFSET, tb, TRUE ); // Steuerelemnete fr Relative Null, Offset freigeben bzw. sperren und ausgrauen; ggf. "OFFSET" anzeigen
	}
	m_lnkManJustage->EndUpdate();
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnChooseTB ( void )
{
	// wir ermitteln welcher Teilbereich derzeit fokussiert ist und fokusieren die Antriebsauswahlliste des nchsten Teilbereichs (nach dem dritten folgt der erste)
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
		if ( (CtrlHasFocus(GetResId(tb, riMotorList))) || (CtrlHasFocus(GetResId(tb, riDirectMode))) || (CtrlHasFocus(GetResId(tb, riDriveMode))) || (CtrlHasFocus(GetResId(tb, riStepMode))) || (CtrlHasFocus(GetResId(tb, riStart))) ||
				(CtrlHasFocus(GetResId(tb, riRelativePos))) || (CtrlHasFocus(GetResId(tb, riAngleDest))) || (CtrlHasFocus(GetResId(tb, riSpeed))) || (CtrlHasFocus(GetResId(tb, riStepWidth))) ||
				(CtrlHasFocus(GetResId(tb, riSetRelativeZero))) || (CtrlHasFocus(GetResId(tb, riResetRelativeZero))) || (CtrlHasFocus(GetResId(tb, riAngle))) || (CtrlHasFocus(GetResId(tb, riOffset))) )
		{
			if ( !TB_Next(tb) )
				break; // der letzte Teilbereich ist fokussiert: siehe nach der while-Schleife
			CtrlSetFocus( GetResId(tb, riMotorList) );
			return; // gefunden
		}
	// Steuerelement nicht innerhalb der ersten beiden Teilbereiche, also im dritten oder auerhalb: fokussiere den ersten Teilbereich
	tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	TB_Next(tb);
	CtrlSetFocus( GetResId(tb, riMotorList) );
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnNoAction ( void )
{
	if ( !m_DisplayNoActionMsg )
		return;

	SetFocus(NULL); // <kein Steuerelement> fokussieren, sonst erscheint die Meldung u.U. mehrfach
	MessageBox( sth_mj_NoAction, sth_mj_ErrTitle, MBINFO );
};

//*****************************************************************************
// Verhalten aller Steuerelemente innerhalb der Teilbereiche
//*****************************************************************************

BOOL TManJustageDlg::TB_CheckLeft ( const ETB aTB, const int aId )
{
	// Auswahlliste Antrieb
	if ( aId == GetResId(aTB, riMotorList) )
	{
		// weil der Regressionstest SENDMESSAGE zum Auswhlen des Antriebs verwendet, wartet ATOS auf return TRUE; d.h. die Meldung "Antrieb ist bereits ausgewhlt" wre nicht testbar
		// Verarbeitung der riMotorList-Botschaft deshalb quasiparallel
		Notice( USR_TB_MOTORCHANGED, aTB );
		return TRUE; // Botschaft verarbeitet
	}

	// Betriebsart
	EMotionType mt= (EMotionType)0;
	if ( aId == GetResId(aTB, riDirectMode) )
		mt= mtDirect;
	if ( aId == GetResId(aTB, riDriveMode) )
		mt= mtDrive;
	if ( aId == GetResId(aTB, riStepMode) )
		mt= mtStep;
	if ( mt != (EMotionType)0 )
	{ // NUR wenn eines der Betriebsarten-Optionsfelder
		BOOL bValid= m_lnkManJustage->SetMotionType( TB2Selection(aTB), mt );
		Notice( USR_TB_STARTSTOPENABLE, aTB );   // "Start" freigeben (Direktbetrieb) bzw. sperren und ausgrauen
		Notice( USR_TB_KINDOFMOVE, aTB, TRUE );  // Betriebsart auswhlen und freigeben bzw. sperren und ausgrauen
		Notice( USR_TB_MOTIONPARAMENABLE, aTB ); // Bewegungsparameter freigeben bzw. sperren und ausgrauen
		if ( bValid )
		{ // wenn die Betriebsart wirklich gesetzt wurde, entsprechenden Bewegungsparameter fokusieren (Eingabe hier sofort mglich)
			if ( mt == mtDirect )
				CtrlSetFocus( GetResId(aTB, riAngleDest) );
			if ( mt == mtDrive )
				CtrlSetFocus( GetResId(aTB, riSpeed) );
			if ( mt == mtStep )
				CtrlSetFocus( GetResId(aTB, riStepWidth) );
		}
		else
			OnNoAction();
		return TRUE; // Botschaft verarbeitet
	};
	return FALSE; // Botschaft NICHT verarbeitet
};
//-----------------------------------------------------------------------------

BOOL TManJustageDlg::TB_CheckMiddle ( const ETB aTB, const int aId )
{
	BOOL bValid;
	EMotionType mt= m_lnkManJustage->GetMotionType( aTB, bValid );
	if ( aId == GetResId(aTB, riStart) )
	{ // Start bzw. Stop
		TB_StartStopClicked( aTB );
		return TRUE; // Botschaft verarbeitet
	};

	// [ENTER] wurde gedrckt
	if ( aId == IDOK )
	{
		if ( CtrlHasFocus(GetResId(aTB, riAngleDest)) )
		{ // [ENTER] bei Sollposition
			CtrlSetFocus( GetResId(aTB, riStart) ); // Versuch Start fokussieren (beim Verlassen wird TB_CheckExit gerufen)
			return TRUE; // Botschaft verarbeitet
		}
		if ( CtrlHasFocus(GetResId(aTB, riSpeed)) )
		{ // [ENTER] bei Geschwindigkeit
			if ( mt==mtDrive && bValid ) CtrlSetFocus( GetResId(aTB, riRelativePos) ); // Versuch Bildlaufleiste fokussieren wenn Fahrbetrieb (beim Verlassen wird TB_CheckExit gerufen)
			else if ( mt==mtDirect && bValid ) CtrlSetFocus( GetResId(aTB, riStart) ); // Versuch Start fokussieren wenn Direktbetrieb (beim Verlassen wird TB_CheckExit gerufen)
			return TRUE; // Botschaft verarbeitet
		}
		if ( CtrlHasFocus(GetResId(aTB, riStepWidth)) )
		{ // [ENTER] bei Schrittweite
			CtrlSetFocus( GetResId(aTB, riRelativePos) ); // Versuch Bildlaufleiste fokussieren (beim Verlassen wird TB_CheckExit gerufen)
			return TRUE; // Botschaft verarbeitet
		}
		if ( CtrlHasFocus(GetResId(aTB, riStart)) )
		{ // Start bzw. Stop
			TB_StartStopClicked( aTB );
			return TRUE; // Botschaft verarbeitet
		}
	}
	return FALSE; // Botschaft nicht verarbeitet
};
//-----------------------------------------------------------------------------

BOOL TManJustageDlg::TB_CheckRight ( const ETB aTB, const int aId )
{
	int mSel= TB2Selection(aTB); // den ausgewhlten Antrieb in diesem Teilbereich ermitteln
	// Setzen/ Aufheben der Relativen Null und Offset
	if ( aId == GetResId(aTB, riSetRelativeZero) )
	{
		if ( !m_lnkManJustage->SetRelativeZero(mSel) )
			OnNoAction(); // Relative Null konnte nicht gesetzt werden

		Notice( USR_TB_POSITIONS, aTB );    // Positionen aktualisieren
		Notice( USR_TB_OFFSET, aTB, TRUE ); // Steuerelemente fr Offset, Relative Null setzen und aufheben freigeben bzw. sperren und ausgrauen; ggf. <_HASOFFSET> anzeigen
		return TRUE; // Botschaft verarbeitet
	}
	if ( aId == GetResId(aTB, riResetRelativeZero) )
	{
		if ( !m_lnkManJustage->ResetRelativeZero(mSel) )
			OnNoAction(); // Relative Null konnte nicht aufgehoben werden

		Notice( USR_TB_POSITIONS, aTB );    // Positionen aktualisieren
		Notice( USR_TB_OFFSET, aTB, TRUE ); // Steuerelemente fr Offset, Relative Null setzen und aufheben freigeben bzw. sperren und ausgrauen; ggf. <_HASOFFSET> anzeigen
		return TRUE; // Botschaft verarbeitet
	}
	if ( aId == GetResId(aTB, riOffset) )
	{
		if ( !m_lnkManJustage->CanSetOffset(mSel) )
		{ // Offset darf nicht definiert werden
			OnNoAction();
			return TRUE;
		}

		// Motoroffset-Dialog anzeigen
		TModalDlg *dlg= new TMotorOffsetDlg( mSel, m_lnkManJustage );
		if ( dlg ) dlg->ExecuteDialog( GetHandle() );
		_FREEOBJ(dlg);

		Notice( USR_TB_POSITIONS, aTB );    // Positionen aktualisieren
		Notice( USR_TB_OFFSET, aTB, TRUE ); // Steuerelemente fr Offset, Relative Null setzen und aufheben freigeben bzw. sperren und ausgrauen; ggf. "Offset" anzeigen
		return TRUE; // Botschaft verarbeitet
	}
	return FALSE; // Botschaft NICHT verarbeitet
};
//-----------------------------------------------------------------------------

BOOL TManJustageDlg::TB_CheckExit ( const ETB aTB, const int aId )
{
	if ( aId == GetResId(aTB, riAngleDest) )
	{
		TB_AngleDestEntered( aTB );
		return TRUE;
	}
	else if ( aId == GetResId(aTB, riSpeed) )
	{
		TB_SpeedEntered( aTB );
		return TRUE;
	}
	else if ( aId == GetResId(aTB, riStepWidth) )
	{
		TB_StepWidthEntered( aTB );
		return TRUE;
	}
	else
		return FALSE;
};

//*****************************************************************************
// Verhalten fr ausgewhlter Steuerelemente innerhalb der Teilbereiche
//*****************************************************************************

void TManJustageDlg::TB_StartStopClicked ( const ETB aTB )
{
	BOOL bValid;
	if ( !m_lnkManJustage->IsMeasuring() )
	{ // SONST darf sich nicht's ndern
		int mSel= TB2Selection(aTB); // den ausgewhlten Antrieb in diesem Teilbereich ermitteln
		if ( (!m_lnkManJustage->IsMoving(mSel, bValid)) && (bValid) )
		{ // dann starten
			if ( m_lnkManJustage->DoDirect(mSel) )
				m_lnkTimer->StartTimerIm(); // wenn Bewegung gestartet wird sorgt der Timer fr die Bildschirmaktualisierung
			else
				OnNoAction(); // Direktbetrieb kann nicht gestartet werden
		}
		else if ( m_lnkManJustage->DoStop(mSel) )
			m_lnkTimer->Immediately(); // Bewegung stoppen: der Timer stoppt automatisch, wenn nichts mehr zu aktualisieren ist
		else
			OnNoAction(); // beliebige Bewegung konnte nicht gestoppt werden
	}
};
//-----------------------------------------------------------------------------

void TManJustageDlg::TB_AngleDestEntered ( const ETB aTB )
{
	BOOL bValid, bSuccess= TRUE;
	int mSel= TB2Selection(aTB); // den ausgewhlten Antrieb in diesem Teilbereich ermitteln
	UINT iDigits= m_lnkManJustage->GetDigits( mSel, bValid ); //Nachkommastellen schonmal im vorhinein ermitteln
	if ( (m_lnkManJustage->IsIndexValid(mSel)) && (!m_lnkManJustage->IsMeasuring()) )
	{ // SONST darf sich nichts ndern
		TAngle value= CtrlGetDouble( GetResId(aTB, riAngleDest), iDigits, _DECIMAL, bValid );
		bSuccess= ( (bValid) && (m_lnkManJustage->SetAngleDest(mSel, value)) );
	}
	OnTB_MotionParam( aTB, FALSE ); // Bewegungsparameter immer aktualisieren (wenn der gewnschte Wert nicht in die Funktionalitt bernommen wurde); Notice ist zu langsam
	if ( !bSuccess )
		OnNoAction();
};
//-----------------------------------------------------------------------------

void TManJustageDlg::TB_SpeedEntered ( const ETB aTB )
{
	BOOL bValid, bSuccess= TRUE;
	int mSel= TB2Selection(aTB); // den ausgewhlten Antrieb in diesem Teilbereich ermitteln
	UINT iDigits= m_lnkManJustage->GetDigits( mSel, bValid ); //Nachkommastellen schonmal im vorhinein ermitteln
	if ( (m_lnkManJustage->IsIndexValid(mSel)) && (!m_lnkManJustage->IsMeasuring()) )
	{ // SONST darf sich nichts ndern
		EMotionType mt= m_lnkManJustage->GetMotionType( mSel, bValid );
		TAngle value= CtrlGetDouble( GetResId(aTB, riSpeed), iDigits, _DECIMAL, bValid );
		bSuccess= ( (bValid) && (m_lnkManJustage->SetSpeed(mSel, value)) );
	}
	OnTB_MotionParam( aTB, FALSE ); // Bewegungsparameter immer aktualisieren (wenn der gewnschte Wert nicht in die Funktionalitt bernommen wurde); Notice ist zu langsam
	if ( !bSuccess )
		OnNoAction();
};
//-----------------------------------------------------------------------------

void TManJustageDlg::TB_StepWidthEntered ( const ETB aTB )
{
	BOOL bValid, bSuccess= TRUE;
	int mSel= TB2Selection(aTB); // den ausgewhlten Antrieb in diesem Teilbereich ermitteln
	UINT iDigits= m_lnkManJustage->GetDigits( mSel, bValid ); //Nachkommastellen schonmal im vorhinein ermitteln
	if ( (m_lnkManJustage->IsIndexValid(mSel)) && (!m_lnkManJustage->IsMeasuring()) )
	{ // SONST darf sich nichts ndern
		TAngle value= CtrlGetDouble( GetResId(aTB, riStepWidth), iDigits + 1, _DECIMAL, bValid ); // plus eine Nachkommastelle
		bSuccess= ( (bValid) && (m_lnkManJustage->SetAngleWidth(mSel, value)) );
	}
	OnTB_MotionParam( aTB, FALSE ); // Bewegungsparameter immer aktualisieren (wenn der gewnschte Wert nicht in die Funktionalitt bernommen wurde); Notice ist zu langsam
	if ( !bSuccess )
		OnNoAction();
};
//-----------------------------------------------------------------------------

void TManJustageDlg::TB_MotorChanged ( const ETB aTB )
{
	// whrend der Halbwertsbreitenmessung darf kein anderer Antrieb ausgewhlt werden
	int sel= TB2Selection(aTB); // Index des zuvor ausgewhlten Antriebs ermitteln; ggf. GetMotorCount() - d.h. 'kein Antrieb'
	if ( m_lnkManJustage->IsMeasuring() )
	{ // whrend der Halbwertsbreitenmessung wird der zuletzt ausgewhlte Antrieb wieder ausgewhlt
		SetSelection( aTB, sel );
		OnNoAction();
		return; // fertig
	}

	// welcher Index wurde ausgewhlt, hat sich was gendert?
	int mSel= CtrlGetSel( GetResId(aTB, riMotorList) );

	// Geschwindigkeitsoptimierung: Wenn Block bei DetermineMoving wieder einkommentiert wurde, kann dieser Block auch wieder auskommentiert werden.
	// whrend sich der Antrieb bewegt, darf kein anderer ausgewhlt werden
	BOOL bValid= FALSE;
	if ( (mSel!=sel) && (m_lnkManJustage->IsMoving(sel, bValid)) ) {
		SetSelection( aTB, sel );
		MessageBox( sth_mj_DontChange, sth_mj_ErrTitle, MBSTOP );
		return; // Abbruch
	} 

	// versuchen neuen Antrieb auszuwhlen und Inhalt der Steuerelemente aktualisieren und Steuerelemenete freigeben bzw. sperren und ausgrauen
	if ( SetSelection(aTB, mSel) )
		return; // Antrieb erfolgreich ausgewhlt

	// zuvor ausgewhlten Antrieb wieder herstellen (damit ist die Dopplung beseitigt) und anschlieend Meldungsfenster ausgeben
	// !!! Reihenfolge ist unbedingt zu beachten, SONST erscheint die Meldung mehrfach
	SetSelection( aTB, sel );

	char *stDoubleMotor= LoadString(sth_mj_DoubleMotor);
	char *msg= new char[ strlen(stDoubleMotor) + _MAXLENCHARACTERISTIC + 1 ]; //max. Lnge einer Antriebsbezeichnung nicht vergessen
	sprintf( msg, stDoubleMotor, m_lnkManJustage->GetMotorName(mSel, bValid) );
	_FREELIST(stDoubleMotor);

	MessageBox( msg, sth_mj_ErrTitle, MBSTOP );
	_FREELIST(msg);
};

//*****************************************************************************
// quasiparallele Aktualisierung (Inhalt und Zustand) aller Steuerelemente
//*****************************************************************************

void TManJustageDlg::Notice( const int aCommand, const ETB aTB, const BOOL aEnableDisable )
{
	if ( (aCommand == USR_TB_MOTORCHANGED) ||   // immer quasiparallel, wegen ATOS
			((m_Parallel) && (aCommand != USR_TB_MOTIONPARAMENABLE)) )
		PostMessage( GetHandle(), aCommand, aTB, aEnableDisable ); // quasiparallele Verarbeitung (USR_TB_MOTIONPARAMENABLE NIE quasiparallel)
	else
		OnEvent( GetHandle(), aCommand, aTB, aEnableDisable ); // quasiparallele Verarbeitung ist zu langsam (Eingabefelder knnen z.B. fokussiert werden bevor sie freigegeben werden)
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_Controls ( const ETB aTB, const BOOL aEnableDisable )
{
	m_lnkManJustage->BeginUpdate();
	if ( aEnableDisable )
		Notice( USR_TB_CONTROLSENABLE, aTB ); // ZUERST freigeben bzw. sperren und ausgrauen

	Notice( USR_TB_POSITIONS, aTB );
	Notice( USR_TB_KINDOFMOVE, aTB );
	Notice( USR_TB_STARTSTOP, aTB );
	Notice( USR_TB_MOTIONPARAM, aTB );
	Notice( USR_TB_OFFSET, aTB );

	// gltiger Referenzpunktlauf?
	UINT sel= TB2Selection(aTB);
	BOOL bValid;
	if ( (!m_lnkManJustage->IsCalibrated(sel, bValid)) && (bValid) )
		CtrlSetText( GetResId(aTB, riRefPoint), sth_mj_NoRefPoint ); // bei "kein Antrieb" ==> !bValid
	else
		CtrlSetText( GetResId(aTB, riRefPoint), sth_mj_HasRefPoint );
	m_lnkManJustage->EndUpdate();
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_KindOfMove ( const ETB aTB, const BOOL aEnableDisable )
{
	BOOL bValid;
	EMotionType mt= m_lnkManJustage->GetMotionType( TB2Selection(aTB), bValid ); // wenn die Betriebsart nicht ermittelt werden konnte (bValid==FALSE), wird mtDirect zurckgegeben
	if ( bValid )
	{ // entweder Direkt-/ Fahr- oder Schrittbetrieb auswhlen
		CtrlSetChecked( GetResId(aTB, riDirectMode), mt == mtDirect );
		CtrlSetChecked( GetResId(aTB, riDriveMode), mt == mtDrive );
		CtrlSetChecked( GetResId(aTB, riStepMode), mt == mtStep );
	}
	if ( aEnableDisable )
		Notice( USR_TB_KINDOFMOVEENABLE, aTB );
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_StartStop ( const ETB aTB, const BOOL aEnableDisable )
{
	BOOL bValid;
	BOOL moves= m_lnkManJustage->IsMoving(TB2Selection(aTB), bValid);
	if ( bValid )
	{ // Start/ Stop?
		if ( (moves) && (!m_lnkManJustage->IsMeasuring()) )
			CtrlSetText( GetResId(aTB, riStart), sth_mj_Stop );
		else
			CtrlSetText( GetResId(aTB, riStart), sth_mj_Start );
	}
	if ( aEnableDisable )
		Notice( USR_TB_STARTSTOPENABLE, aTB );
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_MotionParam ( const ETB aTB, const BOOL aEnableDisable )
{
	BOOL bValid;
	UINT mSel= TB2Selection( aTB );
	UINT iDigits= m_lnkManJustage->GetDigits(mSel, bValid); // wenn die Nachkommastellengenauigkeit nicht ermittelt werden (bValid==FALSE), wird 2 zurckgegeben
	if ( bValid )
	{ // Bewegungsparameter (Sollposition auch bei TB_Positions)
		CtrlSetDouble( GetResId(aTB, riAngleDest), m_lnkManJustage->GetAngleDest(mSel, bValid), iDigits, _DECIMAL ); // siehe auch OnTB_Positions
		CtrlSetDouble( GetResId(aTB, riSpeed), m_lnkManJustage->GetSpeed(mSel, bValid), iDigits, _DECIMAL );
		CtrlSetDouble( GetResId(aTB, riStepWidth), m_lnkManJustage->GetAngleWidth(mSel, bValid), iDigits + 1, _DECIMAL );
	}
	if ( aEnableDisable )
		Notice( USR_TB_MOTIONPARAMENABLE, aTB );
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_Positions ( const ETB aTB, const BOOL aEnableDisable )
{
	SCROLLINFO scrInfo;
	scrInfo.cbSize= sizeof(SCROLLINFO);
	scrInfo.fMask= SIF_POS | SIF_RANGE;

	BOOL bValid;
	UINT mSel= TB2Selection( aTB );
	UINT iDigits= m_lnkManJustage->GetDigits( mSel, bValid );
	if ( bValid )
	{ // Sollposition und Min-, Act-, Maxposition
		CtrlSetDouble( GetResId(aTB, riAngleDest), m_lnkManJustage->GetAngleDest(mSel, bValid), iDigits, _DECIMAL ); // siehe bei OnTB_MotionParam

		// Min-, Act-, Maxposition auslesen
		TAngle angleMin= m_lnkManJustage->GetAngleMin( mSel, bValid );
		TAngle angle= m_lnkManJustage->GetAngle( mSel, bValid );
		TAngle angleMax= m_lnkManJustage->GetAngleMax( mSel, bValid );

		// Min-, Act-, Maxposition aktualisieren
		CtrlSetDouble( GetResId(aTB, riRelativePosMin), angleMin, iDigits, _DECIMAL );
		CtrlSetDouble( GetResId(aTB, riAngle), angle, iDigits, _DECIMAL );
		CtrlSetDouble( GetResId(aTB, riRelativePosMax), angleMax, iDigits, _DECIMAL );

		// Bildlaufleiste aktualisieren
		int posMin, pos, posMax;
		ScrollbarCalculate( angleMin, angle, angleMax, posMin, pos, posMax );
		scrInfo.nPos= pos;
		scrInfo.nMin= posMin;
		scrInfo.nMax= posMax;
		SetScrollInfo(GetCtrl(aTB, riRelativePos),SB_CTL,&scrInfo,TRUE);
	}
	if ( aEnableDisable )
		Notice( USR_TB_POSITIONSENABLE, aTB );
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_Offset ( const ETB aTB, const BOOL aEnableDisable )
{
	BOOL bValid;
	BOOL offset= m_lnkManJustage->HasOffset(TB2Selection(aTB), bValid);
	if ( bValid )
	{ // "OFFSET" definiert ?
		if ( offset )
			CtrlSetText( GetResId(aTB, riOffsetState), sth_mj_HasOffset );
		else
			CtrlSetText( GetResId(aTB, riOffsetState), sth_mj_NoOffset );
	}
	if ( aEnableDisable )
		Notice( USR_TB_OFFSETENABLE, aTB );
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_Units ( const ETB aTB )
{
	BOOL bValid;
	LPCSTR unit= m_lnkManJustage->GetUnit( TB2Selection(aTB), bValid );
	if ( bValid )
	{ // Auslesen der Einheit erfolgreich ?
		char *stSec= LoadString(sth_mj_Sec);
		char *unitSec= new char[ strlen(unit) + strlen(stSec) + 1 ]; //+ Nullterminierung
		strcpy( unitSec, unit );
		strcat( unitSec, stSec );
		_FREELIST(stSec);

		CtrlSetText( GetResId(aTB, riRelativePosUnit), unit );
		CtrlSetText( GetResId(aTB, riAngleDestUnit), unit );
		CtrlSetText( GetResId(aTB, riSpeedUnit), unitSec );
		CtrlSetText( GetResId(aTB, riStepWidthUnit), unit );
		CtrlSetText( GetResId(aTB, riAngleUnit), unit );
		_FREELIST(unitSec);
	}
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_ControlsEnable ( const ETB aTB )
{
  m_lnkManJustage->BeginUpdate();
	Notice( USR_TB_MOTORLISTENABLE, aTB );
	Notice( USR_TB_STARTSTOPENABLE, aTB );
	Notice( USR_TB_KINDOFMOVEENABLE, aTB );
	Notice( USR_TB_MOTIONPARAMENABLE, aTB );
	Notice( USR_TB_POSITIONSENABLE, aTB );
	Notice( USR_TB_OFFSETENABLE, aTB );
  m_lnkManJustage->EndUpdate();
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_MotorListEnable ( const ETB aTB ) const
{
	// Antriebs-Auswahlliste
	CtrlSetEnabled( GetResId(aTB, riMotorList), (!m_StateRefresh) || (!m_lnkManJustage->IsMeasuring()) ); // nur whrend der Halbwertsbreitenmessung inaktiv (whrend der Bewegung und bei "kein Antrieb" freigeben)
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_KindOfMoveEnable ( const ETB aTB ) const
{
	BOOL bValid;
	UINT mSel= TB2Selection( aTB );
	BOOL enable= (!m_StateRefresh) || (!m_lnkManJustage->IsMoving(mSel, bValid)) && (bValid); // bei "kein Antrieb" kann nichts ausgewhlt werden
	CtrlSetEnabled( GetResId(aTB, riDirectMode), enable );
	CtrlSetEnabled( GetResId(aTB, riDriveMode), enable );
	CtrlSetEnabled( GetResId(aTB, riStepMode), enable );
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_StartStopEnable ( const ETB aTB ) const
{
	UINT mSel= TB2Selection( aTB );
	CtrlSetEnabled( GetResId(aTB, riStart), (!m_StateRefresh) || (m_lnkManJustage->CanDoDirect(mSel)) || (m_lnkManJustage->CanDoStop(mSel)) ); // bei "kein Antrieb" kann weder gestartet noch gestoppt werden
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_MotionParamEnable ( const ETB aTB ) const
{
	UINT mSel= TB2Selection( aTB );
	// Sollposition auch bei TB_PositionsEnable
	CtrlSetEnabled( GetResId(aTB, riAngleDest), (!m_StateRefresh) || (m_lnkManJustage->CanSetAngleDest(mSel)) ); // bei "kein Antrieb" kann nichts eingegeben werden
	CtrlSetEnabled( GetResId(aTB, riSpeed), (!m_StateRefresh) || (m_lnkManJustage->CanSetSpeed(mSel)) ); // bei "kein Antrieb" kann nichts eingegeben werden
	CtrlSetEnabled( GetResId(aTB, riStepWidth), (!m_StateRefresh) || (m_lnkManJustage->CanSetAngleWidth(mSel)) ); // bei "kein Antrieb" kann nichts eingegeben werden
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_PositionsEnable ( const ETB aTB ) const
{
	UINT mSel= TB2Selection( aTB );
	// Sollposition auch bei TB_MotionParamEnable
	CtrlSetEnabled( GetResId(aTB, riAngleDest), (!m_StateRefresh) || (m_lnkManJustage->CanSetAngleDest(mSel)) ); // bei "kein Antrieb" kann nichts eingegeben werden
	// Min-, Act- und Maxposition sind und bleiben gesperrt und ausgegraut
	// Bildlaufleiste
	CtrlSetEnabled( GetResId(aTB, riRelativePos), (!m_StateRefresh) || (!m_lnkManJustage->IsMeasuring()) && (m_lnkManJustage->IsIndexValid(mSel)) ); // bei "kein Antrieb" oder Makroverarbeitung sperren und ausgrauen; im Direktbetrieb erscheint eine Fehlermeldung
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OnTB_OffsetEnable ( const ETB aTB ) const
{
	UINT mSel= TB2Selection( aTB );
	BOOL canSet= (!m_StateRefresh) || (m_lnkManJustage->CanSetOffset(mSel));
	CtrlSetEnabled( GetResId(aTB, riSetRelativeZero), canSet ); // bei "kein Antrieb" kann die Null nicht gesetzt werden
	CtrlSetEnabled( GetResId(aTB, riResetRelativeZero), (!m_StateRefresh) || (m_lnkManJustage->CanResetRelativeZero(mSel)) ); // bei "kein Antrieb" kann die Null nicht aufgehoben werden
	CtrlSetEnabled( GetResId(aTB, riOffset), canSet ); // bei "kein Antrieb" kann die Null nicht aufgehoben werden
};
//-----------------------------------------------------------------------------

void TManJustageDlg::OtherControls ( void ) const
{
	if ( m_lnkManJustage->IsMeasuring() )
		CtrlSetText( cm_HwbMeasure, sth_mj_StopMeasure );
	else
		CtrlSetText( cm_HwbMeasure, sth_mj_StartMeasure );

	int idx= m_lnkManJustage->GetDetectorAxisIdx();
	CtrlSetEnabled( cm_PsdOffset, (!m_StateRefresh) || (m_lnkManJustage->CanSetOffset(idx)) ); // includes m_lnkManJustage->HasDetectorAxis()
	CtrlSetEnabled( cm_HwbMeasure, (!m_StateRefresh) || (m_lnkManJustage->CanDoStartMeasure()) || (m_lnkManJustage->CanDoStopMeasure()) );
};

//*****************************************************************************
// Verwaltung der Teilbereiche
//*****************************************************************************

UINT TManJustageDlg::TB2Selection ( const ETB aTB ) const
{
	int idx= TB_GetId(aTB);
	if ( idx != -1 )
		return m_Selection[ idx ];
	else
		return m_lnkManJustage->GetMotorCount(); // unbekannter Teilbereich: "kein Antrieb"
};
//-----------------------------------------------------------------------------

ETB TManJustageDlg::Selection2TB( const UINT aSel ) const
{
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
		if ( TB2Selection(tb) == aSel )
			return tb; // gefunden (!!! "kein Antrieb" kann auch in mehreren Teilbereichen angezeigt werden)
	return (ETB) 0; // irgends angezeigt
};
//-----------------------------------------------------------------------------

BOOL TManJustageDlg::CanSetSelection ( const ETB aTB, const UINT aSel ) const
{
	// Status der Halbwertsbreitenmessung darf hier NICHT bercksichtigt werden, sonst kann SetSelection(...) whrend der Messung den Antrieb nicht wiederherstellen!!!
	if ( !m_lnkManJustage->IsIndexValid(aSel) )
		return TRUE; // "kein Antrieb" darf mehrfach angezeigt werden

	// der ausgewhlte Index darf nicht bereits in einem anderen Teilbereich angezeigt werden
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) ) {
		if ( (TB2Selection(tb)==aSel) && (tb!=aTB) ) return FALSE; 

		// Geschwindigkeitsoptimierung: Wenn Bock bei DetermineMoving wieder einkommentiert wurde, kann dieser Block auch wieder auskommentiert werden.
		// Partnerantriebe drfen nicht gleichzeitig sichtbar sein
		int df= m_lnkManJustage->GetMotorIdx(DF);
		int dc= m_lnkManJustage->GetMotorIdx(DC);
		if ( (aSel==df) && (TB2Selection(tb)==dc) && (tb!=aTB) ) return FALSE;
		if ( (aSel==dc) && (TB2Selection(tb)==df) && (tb!=aTB) ) return FALSE;
	}
	return TRUE; // Index darf ausgewhlt werden
};
//-----------------------------------------------------------------------------

BOOL TManJustageDlg::SetSelection ( const ETB aTB, const UINT aSel )
{
	UINT sel= min( aSel, m_lnkManJustage->GetMotorCount() ); // im Zweifelsfall: "kein Antrieb"
	if ( !CanSetSelection( aTB, sel ) )
		return FALSE; // Antrieb darf nicht doppelt ausgewhlt werden

	// Antrieb in Auswahlliste auswhlen und Auswahl merken
	CtrlSetSel( GetResId(aTB, riMotorList), sel ); // Antrieb auswhlen (auch wenn KEINE nderung!!!)
	if ( m_Selection[ TB_GetId(aTB) ] == sel )
		return TRUE; // keine nderung: weder speichern noch aktualisieren

	// ausgewhlten Antriebs in der ini-Datei speichern
	m_Selection[ TB_GetId(aTB) ]= sel;
	BOOL bValid;
	char *mName= new char[ _MAXLENCHARACTERISTIC + 1 ];
	if ( !m_lnkManJustage->IsIndexValid(sel) )
		strcpy(mName, _NODRIVE); // "kein Antrieb"
	else
		strcpy(mName, m_lnkManJustage->GetMotorName(sel, bValid));
	char tbName[ 12 + 1 ]; // "Teilbereich<Nummer des Teilbereichs, beginnend bei 1>" + Nullterminierung
	TB_GetName( aTB, tbName );
	IniWriteString( GetCFile(), "ManualAdjustment", tbName, mName );
	_FREELIST(mName);

  m_lnkManJustage->BeginUpdate();
	// Teilbereich aktualisieren
	Notice( USR_TB_CONTROLS, aTB, TRUE ); // Steuerelemente mit Inhalt fllen und freigeben bzw. sperren und ausgrauen
	Notice( USR_TB_UNITS, aTB ); // Einheiten anzeigen; wenn diese nicht ermittelt werden konnte (weil "kein Antrieb" angezeigt wird), wird diese auch nicht angezeigt
  m_lnkManJustage->EndUpdate();
	m_lnkTimer->StartTimerIm(); //05.02.2003 Antrieb knnte sich bewegen (KzMoving setzen, Oberfl. ist schon aktualisiert)
	return TRUE; // erfolgreich
};
//-----------------------------------------------------------------------------

HWND TManJustageDlg::GetCtrl ( const ETB aTB, const ERessourceId aCtrl ) const
{
	return TBasicDialog::Ctrl( GetResId(aTB, aCtrl) );
};
//-----------------------------------------------------------------------------

int TManJustageDlg::GetResId ( const ETB aTB, const ERessourceId aCtrl )
{
	// zugunsten einer dreifachen Ausfhrungsgeschwindigkeit musste hier nach Teilbereichen unterscheiden werden
	if ( aTB == tb1 )
	{
		switch ( aCtrl )
		{
			case riMotorList:
				return id_p1_MotorList;

			case riStart:
				return cm_p1_Start;

			case riSetRelativeZero:
				return cm_p1_SetRelativeZero;

			case riResetRelativeZero:
				return cm_p1_ResetRelativeZero;
			
			case riOffset:
				return cm_p1_Offset;
			
			case riDirectMode:
				return id_p1_DirectMode;
			
			case riDriveMode:
				return id_p1_DriveMode;
			
			case riStepMode:
				return id_p1_StepMode;
			
			case riAngleDest:
				return id_p1_AngleDest;
			
			case riSpeed:
				return id_p1_Speed;
			
			case riStepWidth:
				return id_p1_StepWidth;
			
			case riAngle:
				return id_p1_Angle;
			
			case riOffsetState:
				return id_p1_OffsetState;
			
			case riRelativePos:
				return id_p1_RelativePos;
			
			case riRelativePosMin:
				return txt_p1_RelativePosMin;
			
			case riRelativePosMax:
				return txt_p1_RelativePosMax;
			
			case riAngleDestUnit:
				return txt_p1_AngleDestUnit;
			
			case riSpeedUnit:
				return txt_p1_SpeedUnit;
			
			case riStepWidthUnit:
				return txt_p1_StepWidthUnit;
			
			case riAngleUnit:
				return txt_p1_AngleUnit;
			
			case riRelativePosUnit:
				return txt_p1_RelativePosUnit;
			
			case riRefPoint:
				return txt_p1_RefPoint;
		}
	}
	else if ( aTB == tb2 )
	{
		switch ( aCtrl )
		{
			case riMotorList:
				return id_p2_MotorList;
			
			case riStart:
				return cm_p2_Start;
			
			case riSetRelativeZero:
				return cm_p2_SetRelativeZero;
			
			case riResetRelativeZero:
				return cm_p2_ResetRelativeZero;
			
			case riOffset:
				return cm_p2_Offset;
			
			case riDirectMode:
				return id_p2_DirectMode;
			
			case riDriveMode:
				return id_p2_DriveMode;
			
			case riStepMode:
				return id_p2_StepMode;
			
			case riAngleDest:
				return id_p2_AngleDest;
			
			case riSpeed:
				return id_p2_Speed;
			
			case riStepWidth:
				return id_p2_StepWidth;
			
			case riAngle:
				return id_p2_Angle;
			
			case riOffsetState:
				return id_p2_OffsetState;
			
			case riRelativePos:
				return id_p2_RelativePos;
			
			case riRelativePosMin:
				return txt_p2_RelativePosMin;
			
			case riRelativePosMax:
				return txt_p2_RelativePosMax;
			
			case riAngleDestUnit:
				return txt_p2_AngleDestUnit;
			
			case riSpeedUnit:
				return txt_p2_SpeedUnit;
			
			case riStepWidthUnit:
				return txt_p2_StepWidthUnit;
			
			case riAngleUnit:
				return txt_p2_AngleUnit;
			
			case riRelativePosUnit:
				return txt_p2_RelativePosUnit;
			
			case riRefPoint:
				return txt_p2_RefPoint;
		}
	}
	else if ( aTB == tb3 )
	{
		switch ( aCtrl )
		{
			case riMotorList:
				return id_p3_MotorList;
			
			case riStart:
				return cm_p3_Start;
			
			case riSetRelativeZero:
				return cm_p3_SetRelativeZero;
			
			case riResetRelativeZero:
				return cm_p3_ResetRelativeZero;
			
			case riOffset:
				return cm_p3_Offset;
			
			case riDirectMode:
				return id_p3_DirectMode;
			
			case riDriveMode:
				return id_p3_DriveMode;
			
			case riStepMode:
				return id_p3_StepMode;
			
			case riAngleDest:
				return id_p3_AngleDest;
			
			case riSpeed:
				return id_p3_Speed;
			
			case riStepWidth:
				return id_p3_StepWidth;
			
			case riAngle:
				return id_p3_Angle;
			
			case riOffsetState:
				return id_p3_OffsetState;
			
			case riRelativePos:
				return id_p3_RelativePos;
			
			case riRelativePosMin:
				return txt_p3_RelativePosMin;
			
			case riRelativePosMax:
				return txt_p3_RelativePosMax;
			
			case riAngleDestUnit:
				return txt_p3_AngleDestUnit;
			
			case riSpeedUnit:
				return txt_p3_SpeedUnit;
			
			case riStepWidthUnit:
				return txt_p3_StepWidthUnit;
			
			case riAngleUnit:
				return txt_p3_AngleUnit;
			
			case riRelativePosUnit:
				return txt_p3_RelativePosUnit;
			
			case riRefPoint:
				return txt_p3_RefPoint;
		}
	};
	return -1; //unbekannter Teilbereich oder unbekannte Ressourcen-ID
};

//*****************************************************************************
// zur Verwaltung der Teilbereiche
//*****************************************************************************

BOOL TManJustageDlg::TB_Next ( ETB &aTB )
{
	switch ( aTB )
	{
		case (ETB)0:
			aTB= tb1;
			return TRUE;

		case tb1:
			aTB= tb2;
			return TRUE;

		case tb2:
			aTB= tb3;
			return TRUE;

		default:
			return FALSE; // nach tb3 gehts nicht mehr weiter
	}
};
//-----------------------------------------------------------------------------

UINT TManJustageDlg::TB_Count ( void )
{
	UINT result= 0;
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
		result++;
	return result;
};
//-----------------------------------------------------------------------------

UINT TManJustageDlg::TB_GetId ( const ETB aTB )
{
	UINT result= 0; // beginnend bei 0
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
	{
		if ( tb == aTB )
			return result;
		result++;
	}
	return -1; // unbekannter Teilbereich
};
//-----------------------------------------------------------------------------

void TManJustageDlg::TB_GetName ( const ETB aTB, LPSTR aReturn )
{
	strcpy( aReturn, "" ); // unbekannter Teilbereich, falls nicht gefunden
	ETB tb= (ETB)0; // nur so gibt TB_Next beim ersten Aufruf tb1 zurck
	while ( TB_Next(tb) )
		if ( aTB == tb )
			sprintf( aReturn, "Section%d", TB_GetId(aTB) + 1 ); // nicht abhngig von GermanVersion, weil Eintrag aus der ini-Datei
};

//*****************************************************************************
// Winkelwerte fr Bildlaufleiste umrechnen
//*****************************************************************************

void TManJustageDlg::ScrollbarCalculate ( const TAngle angleMin, const TAngle angle, const TAngle angleMax,
		int &posMin, int &pos, int &posMax )
{
	posMin= 0;
	float skale;
	try 
	{
		skale= (float) ( 32000 / (angleMax - angleMin)); 
	} 
	catch (...) 
	{
		skale= 0;
	}	
	pos= (int)( (angle - angleMin) * skale );
	posMax= (int)( (angleMax - angleMin) * skale );
};

//#############################################################################
// TMotorOffsetDlg
//#############################################################################

//-----------------------------------------------------------------------------
// Konstruktor

TMotorOffsetDlg::TMotorOffsetDlg ( const UINT aIdx, TManJustage *aManJustage ) : TModalDlg ( "MOTOROFFSETDLG", GetMainInstance() )
{
	BOOL bValid= FALSE;
	//Aufrufer muss sicherstellen, dass gltige Referenzen bergeben werden
	assert( aManJustage );
	m_lnkManJustage= aManJustage;
	assert( m_lnkManJustage->IsIndexValid(aIdx) );
	m_Idx= aIdx;
	m_Digits= m_lnkManJustage->GetDigits(m_Idx, bValid); // Nachkommastellen lesen
};
//-----------------------------------------------------------------------------
// Initialisierung

BOOL TMotorOffsetDlg::Dlg_OnInit ( HWND, HWND, LPARAM )
{
	BOOL bValid= FALSE;
	// Titelleiste des Dialogfensters
	LPCSTR mName= m_lnkManJustage->GetMotorName( m_Idx, bValid );

	char *stMotorOffsetTitle= LoadString( sth_mj_MotorOffsetTitle );
	char *title= new char[ strlen(stMotorOffsetTitle) + _MAXLENCHARACTERISTIC + 1 ]; // + Nullterminierung
	sprintf( title, stMotorOffsetTitle, mName );
	_FREELIST(stMotorOffsetTitle);

	SetWindowText( GetHandle(), title );
	_FREELIST(title);
	// Einheiten anzeigen
	LPCSTR unit= m_lnkManJustage->GetUnit( m_Idx, bValid );
	CtrlSetText( txt_AngleUnit, unit );
	CtrlSetText( txt_OffsetUnit, unit );
	CtrlSetText( txt_AngleDestUnit, unit );
	CtrlSetText( txt_OffsetDestUnit, unit );
	// aktuelle Position und aktueller Offset anzeigen
	CtrlSetDouble( id_Angle, m_lnkManJustage->GetAngle(m_Idx, bValid), m_Digits, _DECIMAL);
	CtrlSetDouble( id_Offset, m_lnkManJustage->GetOffset(m_Idx, bValid), m_Digits, _DECIMAL);
	// zu Beginn immer Winkeleingabe aktiviert
	ChooseAngleMode( TRUE );
	// Offset von 0 anzeigen
	CtrlSetDouble( id_AngleDest, m_lnkManJustage->GetAngle(m_Idx, bValid), m_Digits, _DECIMAL);
	CalcValue(FALSE);
	return TRUE;
};

//-----------------------------------------------------------------------------
// Eingabeverarbeitung

void TMotorOffsetDlg::Dlg_OnCommand ( HWND aHWnd, int aId, HWND aCtrlWnd, UINT aCode )
{
	switch ( aId )
	{
		case id_AngleMode:
		case id_OffsetMode:   // entweder Winkel ODER Offset-Eingabe freigegeben,
			ChooseAngleMode( aId == id_AngleMode );
			return;

		case id_AngleDest:
		case id_OffsetDest:   // Eingabefelder 'neuer Winkel' und 'Offset'
			if ( aCode == ONEXIT )
				CalcValue(FALSE);
			return;

		case IDOK:   // Eingabebesttigung [ENTER]
			if ( (CtrlHasFocus(id_AngleDest)) || (CtrlHasFocus(id_OffsetDest)) )
			{
				if ( CalcValue(FALSE) )
					CtrlSetFocus( IDOK );
				return; // verarbeitet
				// bei 'OK' Werte bernehmen
			}
			else if ( (CtrlHasFocus(IDOK)) && (!CalcValue(TRUE)) )
				return; // nur bei gltiger Eingabe weiter bei default (Dialg schlieen)
			// hier kein break, sondern TModalDlg::Dlg_OnCommand

		default:
			TModalDlg::Dlg_OnCommand( aHWnd, aId, aCtrlWnd, aCode );
	};
};
//-----------------------------------------------------------------------------
// Wechsel der Eingabemglichkeit

void TMotorOffsetDlg::ChooseAngleMode( BOOL aAngleMode )
{
	CtrlSetChecked( id_AngleMode, aAngleMode );
	CtrlSetChecked( id_OffsetMode, !aAngleMode );
	CtrlSetEnabled( id_AngleDest, IsAngleMode() );
	CtrlSetEnabled( id_OffsetDest, !IsAngleMode() );
	if ( IsAngleMode() )
		CtrlSetFocus( id_AngleDest );
	else
		CtrlSetFocus( id_OffsetDest );
};
//-----------------------------------------------------------------------------
// lokale Berechnung von Offset oder Winkel, bernahme wenn Parameter==TRUE

BOOL TMotorOffsetDlg::CalcValue( BOOL aTakeOver )
{
	BOOL bValid= FALSE;
	int inputResId= 0, outputResId= 0;
	if ( IsAngleMode() )
	{ // neuer Winkel eingegeben
		inputResId= id_AngleDest;
		outputResId= id_OffsetDest;
	}
	else
	{ // neuer Offset eingegeben
		inputResId= id_OffsetDest;
		outputResId= id_AngleDest;
	};
	TAngle inputValue= CtrlGetDouble( inputResId, m_Digits, _DECIMAL, bValid );
	if (!bValid)
		return FALSE; // wenn Fehler beim lesen dann raus

	TAngle outputValue= 0.0;
	if ( IsAngleMode() )
		outputValue= m_lnkManJustage->CalcOffsetFromAngle(m_Idx, inputValue);
	else
		outputValue= m_lnkManJustage->CalcAngleFromOffset(m_Idx, inputValue);
	if ( aTakeOver )
	{ // TRUE <--> bernahme
		if ( IsAngleMode() )
			m_lnkManJustage->SetOffset(m_Idx, outputValue);
		else
			m_lnkManJustage->SetOffset(m_Idx, inputValue);
	};
	CtrlSetDouble( inputResId, inputValue, m_Digits, _DECIMAL);
	CtrlSetDouble( outputResId, outputValue, m_Digits, _DECIMAL);
	return TRUE;
};
//-----------------------------------------------------------------------------

BOOL TMotorOffsetDlg::IsAngleMode ( void )
{
	return CtrlIsChecked( id_AngleMode );
};

//#############################################################################
// TPsdOffsetDlg
//#############################################################################

//-----------------------------------------------------------------------------
// KONSTRUKTOR-DESTRUKTOR

TPsdOffsetDlg::TPsdOffsetDlg ( const UINT aIdx, TManJustage *aManJustage ) : TModalDlg ( "PSDOFFSETDLG", GetMainInstance() )
{
	BOOL bValid= FALSE;
	// Aufrufer muss sicherstellen, dass gltige Referenzen bergeben werden
	assert( aManJustage );
	assert( aManJustage->GetDetector() ); // eindim. Detektor (z.B. PSD) vorhanden?
	m_lnkManJustage= aManJustage;
	assert( m_lnkManJustage->IsIndexValid(aIdx) );
	m_Idx= aIdx;
	m_Digits= m_lnkManJustage->GetDigits(m_Idx, bValid); // Nachkommastellen lesen
	m_PSDDigits= m_lnkManJustage->GetDetectorDigits(bValid); // Kanalbreite Nachkommastellen lesen
};

//-----------------------------------------------------------------------------
// EREIGNISSE DER BASISKLASSE

BOOL TPsdOffsetDlg::Dlg_OnInit ( HWND, HWND, LPARAM )
{
	BOOL bValid= FALSE;
	//Titelleiste des Dialogfensters
	LPCSTR dName= m_lnkManJustage->GetDetectorName( bValid );

	char *stPsdOffsetTitle= LoadString( sth_mj_PsdOffsetTitle );
	char *title= new char[ strlen(stPsdOffsetTitle) + _MAXLENCHARACTERISTIC + 1 ]; // + Nullterminierung
	sprintf( title, stPsdOffsetTitle, dName );
	_FREELIST(stPsdOffsetTitle);

	SetWindowText( GetHandle(), title );
	_FREELIST(title);
	// Beschriftung von "Offset fr <Psd>"
	LPCSTR mName= m_lnkManJustage->GetMotorName( m_Idx, bValid );

	char *stPsdActPos= LoadString( sth_mj_PsdActPos );
	title= new char[ strlen(stPsdActPos) + _MAXLENCHARACTERISTIC + 1 ]; // + Nullterminierung
	sprintf( title, stPsdActPos, mName );
	_FREELIST(stPsdActPos);
	
	CtrlSetText(txt_PsdAngle, title);
	_FREELIST(title);

	//Beschriftung von "<Psd>-Kaneloffset"
	char *stPsdOffsetDest= LoadString( sth_mj_PsdOffsetDest );
	title= new char[ strlen(stPsdOffsetDest) + _MAXLENCHARACTERISTIC + 1 ]; // + Nullterminierung
	sprintf( title, stPsdOffsetDest, dName );
	_FREELIST(stPsdOffsetDest);
	CtrlSetText(txt_PsdOffsetDest, title);
	_FREELIST(title);
	
	// Einheiten anzeigen
	LPCSTR unit= m_lnkManJustage->GetUnit( m_Idx, bValid );
	CtrlSetText( txt_PsdAngleUnit, unit );
	CtrlSetText( txt_PsdAnglePerChannelUnit, unit );
	CtrlSetText( txt_PsdOffsetDestUnit, unit );
	CtrlSetText( txt_PsdAngleDestUnit, unit );
	// aktuelle Position und Winkel/Kanal anzeigen
	CtrlSetDouble( id_PsdAngle, m_lnkManJustage->GetAngle(m_Idx, bValid), m_Digits, _DECIMAL);
	CtrlSetDouble( id_PsdAnglePerChannel, m_lnkManJustage->GetAnglePerChannel(), m_PSDDigits, _DECIMAL);
	// aktuellen Kanal anzeigen
	CtrlSetLong( id_ChannelDest, m_lnkManJustage->GetChannel());
	CalcValue(FALSE);
	return TRUE;
};
//-----------------------------------------------------------------------------

void TPsdOffsetDlg::Dlg_OnCommand ( HWND aHWnd, int aId, HWND aCtrlWnd, UINT aCode )
{
	switch ( aId )
	{
		case id_ChannelDest:   // Eingabefelder 'neu zugeordneter Kanal'
			if ( aCode == ONEXIT )
				CalcValue(FALSE);
			return;

		case IDOK:   // Eingabebesttigung [ENTER]
			if ( CtrlHasFocus(id_ChannelDest) )
			{
				if ( CalcValue(FALSE) )
					CtrlSetFocus( IDOK );
				return; // verarbeitet
				// bei 'OK' Werte bernehmen
			}
			else if ( (CtrlHasFocus(IDOK)) && (!CalcValue(TRUE)) )
				return; // nur bei gltiger Eingabe weiter bei default (Dialg schlieen)
			// hier kein break, sondern TModalDlg::Dlg_OnCommand

		default:
			TModalDlg::Dlg_OnCommand( aHWnd, aId, aCtrlWnd, aCode );
	};
};
//-----------------------------------------------------------------------------
// lokale Berechnung von PSD-Offset und Thetaposition, bernahme wenn Parameter==TRUE

BOOL TPsdOffsetDlg::CalcValue ( BOOL aTakeOver )
{
	BOOL bValid= FALSE;
	int inputValue= CtrlGetLong( id_ChannelDest, bValid );
	if (!bValid)
		return FALSE; // wenn Fehler beim lesen dann raus

	TAngle OffsetValue= m_lnkManJustage->CalcChannelOffset( inputValue );
	TAngle AngleValue= m_lnkManJustage->CalcChannelAngle( inputValue );
	if ( aTakeOver )
		m_lnkManJustage->SetChannel( inputValue ); // TRUE <--> bernahme
	CtrlSetLong( id_ChannelDest, inputValue );
	CtrlSetDouble( id_PsdOffsetDest, OffsetValue, m_PSDDigits, _DECIMAL);
	CtrlSetDouble( id_PsdAngleDest, AngleValue, m_Digits, _DECIMAL);
	return TRUE;
};
//-----------------------------------------------------------------------------
// LastLine
