//#############################################################################
//                                                                           //
// MJ_OFUNK.CPP                                                              //
//                                                                           //
// Subsystem : alte Manuelle Justage / Funktionalitt                        //
// Benutzung durch andere Subsysteme erforderlich: NEIN                      //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand  : 12.10.2002                                                       //
//                                                                           //
//#############################################################################

#include "manjust\mj_ofunk.h" // SCHNITTSTELLE fr diese Datei

#include "workflow\workflow.h" // extern Steering

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
// externe Abhngigkeiten
//##############################################################################

extern TSteering Steering;
extern BOOL bManualMovesCorrected;

//##############################################################################
// TAngleCtl
//##############################################################################

//*****************************************************************************
// Konstruktor/Destruktor
//*****************************************************************************

TAngleCtl::TAngleCtl ( void )
{
	m_IsMeasuring= FALSE; // Halbwertsbreite wird nicht gemessen
	m_MotorCount= mlGetAxisNumber();
	m_MoveSpeed= new double [GetMotorCount()]; // Liste der Geschwindigkeiten: Lnge=Anzahl der Motoren
	for (UINT i= 0; i < GetMotorCount(); i++)
		m_MoveSpeed[i]= 0;
	m_Sensor= TDetectorManager::DetectorManager().GetDetector(); // Pointer auf den akt. Sensor
}
//-----------------------------------------------------------------------------

TAngleCtl::~TAngleCtl ( void )
{
	_FREELIST( m_MoveSpeed );
}

//******************************************************************************
// Sonstiges
//******************************************************************************

BOOL TAngleCtl::DoInit ( const int aIndex )
{
	if ( !IsIndexValid(aIndex) )
		return FALSE;

	for (UINT cnt= 0; cnt < GetMotorCount(); cnt++)
	{ // Sichern der Motor-Einstellungen
		mlSetAxis(cnt);
		mPushSettings();
		m_MoveSpeed[cnt]= mGetValue(Speed); // zuletzt benutze Geschwindigkeit := aktuelle Bewegungsgeschwindigkeit
	}
	return DoSelectMotor( aIndex );
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::DoInitMotor ( const int aIndex, const BOOL aMaxSpeed )
{
	if ( !IsIndexValid(aIndex) )
		return FALSE;

	if ( aMaxSpeed )
		mlSetValue(aIndex, Speed, 10000.0);  // Im Schrittbetrieb Maximalgeschwingkeit verwenden; BESSER MaxVelocity verwenden!!!
	else
		mlSetValue(aIndex, Speed, m_MoveSpeed[aIndex]); // sonst zuletzt eingegebe Geschwindigkeit
	return TRUE;
}

//******************************************************************************
// Antriebssteuerung
//******************************************************************************

BOOL TAngleCtl::DoDirect ( const int aIndex, const double aPos )
{
	if ( !CanDoDirect(aIndex) )
		return FALSE;

	DoSelectMotor(aIndex);
	mActivateDrive(); // Motor aktivieren (sonst bewegt sich der Antrieb nicht, wenn er gestartet wird)
	mMoveToDistance(aPos);
	BOOL bValid;
	return IsMoving(aIndex, bValid);
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::DoStep ( const int aIndex, const BOOL aUp )
{
	if ( !CanDoStep(aIndex, aUp) )
		return FALSE;

	DoSelectMotor(aIndex);
	mActivateDrive(); // Motor aktivieren (sonst bewegt sich der Antrieb nicht, wenn er gestartet wird)
	BOOL bValid;
	if ( aUp )
		mMoveToDistance( GetAngle(aIndex, bValid) + GetAngleWidth(aIndex, bValid) );
	else
		mMoveToDistance( GetAngle(aIndex, bValid) - GetAngleWidth(aIndex, bValid) );
	return IsMoving(aIndex, bValid);
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::DoDrive ( const int aIndex, const BOOL aUp )
{
	if ( !CanDoDrive(aIndex, aUp) )
		return FALSE;

	DoSelectMotor(aIndex);
	mActivateDrive(); // Motor aktivieren (sonst bewegt sich der Antrieb nicht, wenn er gestartet wird)
	BOOL bValid;
	if ( aUp )
		mMoveToDistance( GetAngleMax(aIndex, bValid) );
	else
		mMoveToDistance( GetAngleMin(aIndex, bValid) );
	return IsMoving(aIndex, bValid);
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::DoStop ( const int aIndex )
{
	if ( !CanDoStop(aIndex) )
		return FALSE;

	DoSelectMotor(aIndex);
	mStopDrive(TRUE);
	DelayTime(5);
	BOOL bValid;
	return !IsMoving(aIndex, bValid);
}
//-----------------------------------------------------------------------------

void TAngleCtl::DoStopEverything ( const int aIndex )
{
	//zuerst die Halbwertsbreitenmessung stoppen
	DoStopMeasure(0);
	//Detektor stoppen
	if (m_Sensor != TDetectorManager::DetectorManager().GetDetector(0, 1))
	{
		m_Sensor->MeasureStop();
		m_Sensor->PopSettings();
		m_Sensor->MeasureStart();
	}
	//letztendlich auch die Antriebe stoppen
	for (UINT cnt= 0; cnt < GetMotorCount(); cnt++)
		if ( DoSelectMotor(cnt) )
		{
			mPopSettings(ThisPosition);
			mSetCorrectionState(FALSE);
			mStopDrive(TRUE);
		}
	DoSelectMotor(aIndex);
}

//******************************************************************************
// Statusinformationen
//******************************************************************************

BOOL TAngleCtl::IsIndexValid ( const int aIndex ) const
{ 
	return ( (0 <= aIndex) && (aIndex < (int)GetMotorCount()) );
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::HasOffset ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return FALSE;

	aValid= TRUE;
	return mlIsDistanceRelative(aIndex);
}
//------------------------------------------------------------------------------

BOOL TAngleCtl::IsMoving ( const int aIndex, BOOL &aValid ) const
{
	// ist der Partnerantrieb in Bewegung?
	int idDF= GetMotorIdx(DF);
	int idDC= GetMotorIdx(DC);
	if ( (aIndex == idDF) && (Moves(idDC, aValid)) && (aValid) )
		return TRUE;
	else if ( (aIndex == idDC) && (Moves(idDF, aValid)) && (aValid) )
		return TRUE;

	return Moves(aIndex, aValid);
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::IsCalibrated ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return FALSE;

	aValid= TRUE;
	return mlIsCalibrated(aIndex);
};

//******************************************************************************
// Get-Methoden
//******************************************************************************

double TAngleCtl::GetAngle ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return 0.0;

	double result;
	if ( Moves(aIndex, aValid) )
	{
		aValid= TRUE;
		result= mlGetDistanceProcess(aIndex);
	}
	else
		aValid= mlGetDistance(aIndex, result);
	return result;
}
//-----------------------------------------------------------------------------

double TAngleCtl::GetAngleMin ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return 0.0;

	aValid= TRUE;
	return min( mlGetAngleRelativeMin(aIndex), mlGetAngleRelativeMax(aIndex) );
}
//-----------------------------------------------------------------------------

double TAngleCtl::GetAngleMax ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return 0.0;

	aValid= TRUE;
	return max( mlGetAngleRelativeMin(aIndex), mlGetAngleRelativeMax(aIndex) );
}
//-----------------------------------------------------------------------------

double TAngleCtl::GetSpeed ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return 0.0;

	aValid= TRUE;
	return mlGetValue(aIndex, Speed);
}
//-----------------------------------------------------------------------------

double TAngleCtl::GetAngleWidth ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return 0.0;

	aValid= TRUE;
	return mlGetValue(aIndex, Width);
}
//-----------------------------------------------------------------------------

UINT TAngleCtl::GetDigits ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return 2;

	aValid= TRUE;
	return mlGetDigits(aIndex);
}
//-----------------------------------------------------------------------------

LPCSTR TAngleCtl::GetUnit ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return "";

	aValid= TRUE;
	return mlGetUnitName(aIndex);
}
//-----------------------------------------------------------------------------

int TAngleCtl::GetMotorIdx ( const EAxisType aAxis ) const
{
	if ( aAxis == (EAxisType)0 )
		return -1;
	else
		return mlGetIdByName(aAxis);
}
//-----------------------------------------------------------------------------

LPCSTR TAngleCtl::GetMotorName ( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return "";

	aValid= TRUE;
	return mlGetAxisName(aIndex);
}
//-----------------------------------------------------------------------------

UINT TAngleCtl::GetMotorCount ( void ) const
{
	return m_MotorCount;
}
//-----------------------------------------------------------------------------

int TAngleCtl::GetActualMotor ( void ) const
{
	return mlGetAxis();
}

//******************************************************************************
// SET-Methoden
//******************************************************************************

BOOL TAngleCtl::SetSpeed ( const int aIndex, double &aSpeed ) 
{
	if ( !CanSetSpeed(aIndex) )
	{
		BOOL bValid;
		if ( IsIndexValid(aIndex) )
			aSpeed= GetSpeed(aIndex, bValid);
		return FALSE;
	}

	mlSetValue(aIndex, Speed, aSpeed);
	aSpeed= mlGetValue(aIndex, Speed);
	m_MoveSpeed[aIndex]= aSpeed;
	return TRUE;
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::SetAngleWidth ( const int aIndex, double &aAngleWidth )
{
	if ( !CanSetAngleWidth(aIndex) )
	{
		BOOL bValid;
		if ( IsIndexValid(aIndex) )
			aAngleWidth= GetAngleWidth(aIndex, bValid);
		return FALSE;
	}

	mlSetValue(aIndex, Width, aAngleWidth);
	aAngleWidth= mlGetValue(aIndex, Width);
	return TRUE;
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::SetCorrectionState ( const int aIndex )
{
	if ( !IsIndexValid(aIndex) )
		return FALSE;

	mlSetCorrectionState(aIndex, TRUE);
	return TRUE;
}

//*****************************************************************************
// Relative Null
//*****************************************************************************

BOOL TAngleCtl::SetRelativeZero ( const int aIndex )
{
	if ( !CanSetOffset(aIndex) )
		return FALSE;

	BOOL bValid;
	mlSetRelativeZero(aIndex, TRUE, GetAngle(aIndex, bValid)); // relative Null setzen
	//FEHLER: zu jedem Push muss es ein POP geben mPushSettings(); // Motoreinstellungen sichern
	return TRUE;
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::ResetRelativeZero ( const int aIndex )
{
	if ( !CanResetRelativeZero(aIndex) )
		return FALSE;

	mlSetRelativeZero(aIndex, FALSE, 0.0); // relative Null aufheben
	return TRUE;
}

//*****************************************************************************
// Halbwertsbreitenmessung
//*****************************************************************************

BOOL TAngleCtl::DoStartMeasure ( const HWND aWnd )
{
	if ( !CanDoStartMeasure() )
		return FALSE;

	m_IsMeasuring= TRUE;
	Steering.Reset();
	Steering.StartUp(aWnd, 0, m_Sensor); // Fensterhandle zum Fenster, das die Bildschirmaktualisierung des Zhlerfensters bernimmt; ausgewhlten Detektor setzen; zweiter Paramater ("Beugung fein" wird vom Makro selbst ausgewhlt) wird nicht bentigt
	Steering.Visualising( TRUE, FALSE );
	Steering.StartMacroExecution(Steering.GetMacroById(InquireHwb), aWnd); // Makro "Inquire Hwb" ausfhren
	return TRUE;
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::DoStopMeasure ( const int aIndex )
{
	if ( !CanDoStopMeasure() )
	{
		DoSelectMotor(aIndex);
		return FALSE;
	}

	m_IsMeasuring= FALSE;
	Steering.Reset(); // Reset ist besser als ToggleInterrupt
	return DoSelectMotor(aIndex);
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::IsMeasuring ( void ) const
{
	return m_IsMeasuring;
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::IsMeasureReset ( void ) const
{
	return Steering.IsReset();
}
//-----------------------------------------------------------------------------

double TAngleCtl::GetMeasureHwb ( void ) const
{
	return Steering.GetHwb();
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::MeasureStopped ( const int aIndex )
{
	if ( (!IsIndexValid(aIndex)) || (!m_IsMeasuring) )
		return FALSE;

	m_IsMeasuring= FALSE;
	return DoSelectMotor(aIndex);
}
//-----------------------------------------------------------------------------

void TAngleCtl::UpdateDetector ( void ) const
{
	m_Sensor->UpdateViews(TRUE);
	Steering.DetectorRequest(); // wird diese Methode nicht gerufen, verharrt die Halbwertsbreitenmessung in einer Endlosschleife
}

//******************************************************************************
// Zustand fr Bewegung ermitteln
//******************************************************************************

BOOL TAngleCtl::CanDoDirect ( const int aIndex ) const
{
	BOOL bValid;
	return ( (!IsMoving(aIndex, bValid)) && (bValid) && (!m_IsMeasuring) );
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::CanDoStep ( const int aIndex, const BOOL aUp ) const
{
	BOOL bValid;
	if ( (IsMoving(aIndex, bValid)) || (m_IsMeasuring) )
		return FALSE;

	//Ist die Zielposition des nchsten Schrittes im Intervall
	if ( aUp )
		return GetAngle(aIndex, bValid) + GetAngleWidth(aIndex, bValid) < GetAngleMax(aIndex, bValid);
	else
		return GetAngleMin(aIndex, bValid) < GetAngle(aIndex, bValid) - GetAngleWidth(aIndex, bValid);
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::CanDoDrive ( const int aIndex, const BOOL aUp ) const
{
	BOOL bValid;
	if ( (IsMoving(aIndex, bValid)) || (m_IsMeasuring) )
		return FALSE;

	//Zielposition bereits erreicht?
	if ( aUp )
		return ( GetAngle(aIndex, bValid) < GetAngleMax(aIndex, bValid) );
	else
		return ( GetAngleMin(aIndex, bValid) < GetAngle(aIndex, bValid) );
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::CanDoStop ( const int aIndex ) const
{
	BOOL bValid;
	return ( (IsMoving(aIndex, bValid)) && (!m_IsMeasuring) );
}

//*****************************************************************************
// Zustand zum Setzen von Parametern
//*****************************************************************************

BOOL TAngleCtl::CanSetSpeed ( const int aIndex ) const
{
	BOOL bValid;
	return ( (!IsMoving(aIndex, bValid)) && (bValid) && (!m_IsMeasuring) );
}
//-----------------------------------------------------------------------------
BOOL TAngleCtl::CanSetAngleWidth ( const int aIndex ) const
{
	BOOL bValid;
	return ( (!IsMoving(aIndex, bValid)) && (bValid) && (!m_IsMeasuring) );
}

//-----------------------------------------------------------------------------

BOOL TAngleCtl::CanSetOffset ( const int aIndex ) const
{
	BOOL bValid;
	return ( (!IsMoving(aIndex, bValid)) && (bValid) && (!m_IsMeasuring) );
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::CanResetRelativeZero ( const int aIndex ) const
{
	BOOL bValid;
	return ( (!IsMoving(aIndex, bValid)) && (bValid) && (!m_IsMeasuring) && (HasOffset(aIndex, bValid)) );
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::CanDoStartMeasure ( void ) const
{
	BOOL bValid;
	for (UINT cnt= 0; cnt < GetMotorCount(); cnt++)
		if ( IsMoving(cnt, bValid) )
			return FALSE; // alle Antriebe mssen still stehen
	return ( (mlIsAxisValid(Omega)) && (Steering.GetMacroById(InquireHwb)) && (!m_IsMeasuring) );
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::CanDoStopMeasure ( void ) const
{
	// es knnte sein, dass die Halbwertsbreitenmessung gerade einen Antrieb bewegt: dann trotzdem stoppen
	return (m_IsMeasuring);
}

//*****************************************************************************
// private
//*****************************************************************************

BOOL TAngleCtl::DoSelectMotor ( const int aIndex ) const
{
	if ( !IsIndexValid(aIndex) )
		return FALSE;

	mlSetAxis(aIndex);
	return TRUE;
}
//-----------------------------------------------------------------------------

BOOL TAngleCtl::Moves( const int aIndex, BOOL &aValid ) const
{
	aValid= FALSE;
	if ( !IsIndexValid(aIndex) )
		return FALSE;

	aValid= TRUE;
	if ( (!mlIsMoveFinish(aIndex)) || (IsMeasuring()) )
		return TRUE;
	return FALSE;
}

