//******************************************************************************
//Datei     : mespara.cpp
//Projekt   : XCTL
//Subsystem : Protokollbuch
//Autor     : Jens Klier <klier@informatik.hu-berlin.de>
//            Institut fuer Informatik,
//            Humboldt-Universitaet Berlin
//Inhalt    : Implementation der Klasse TMeasurementParameter
//******************************************************************************
//!neu klier Protokollbuch

#include "internls\evrythng.h" // extern Main

#include "mespara\mespara.h"
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

TMeasurementParameter::TMeasurementParameter()
{};

void TMeasurementParameter::Load( LPCSTR IniFile )
{
	char buf[MaxString];

	strcpy(szApplication, "Steuerprogramm");
	GetPrivateProfileString(szApplication, "User", "Nutzer", sUser, MaxString, IniFile );
	GetPrivateProfileString(szApplication, "Target", "Probe", sTarget, MaxString, \
							IniFile );
	GetPrivateProfileString(szApplication, "Substrat", "Si", sTargetBulk, MaxString, \
							IniFile );
	GetPrivateProfileString(szApplication, "WaveLength", "1.534", buf, MaxString, \
							IniFile );
	SetWaveLength(buf);
	GetPrivateProfileString(szApplication, "Reflection", "[001]", buf, 10, IniFile);
	SetReflection(buf);
	GetPrivateProfileString(szApplication, "Orientation", "[001]", buf, 10, IniFile);
	SetOrientation(buf);
	GetPrivateProfileString(szApplication, "Comment", "", sComment, 2*MaxString, IniFile);
	SetCurrent( GetPrivateProfileInt(szApplication, "Current", 10, IniFile) );
	SetVoltage( GetPrivateProfileInt(szApplication, "Voltage", 20, IniFile) );
};

void TMeasurementParameter::Save( LPCSTR IniFile )
{
	char buf[MaxString];

	WritePrivateProfileString(szApplication, "WaveLength", GetWaveLength(buf), IniFile);
	WritePrivateProfileString(szApplication, "User", sUser, IniFile);
	WritePrivateProfileString(szApplication, "Target", sTarget, IniFile);
	WritePrivateProfileString(szApplication, "Substrat", sTargetBulk, IniFile);
	WritePrivateProfileString(szApplication, "Reflection", GetReflection(buf), IniFile);
	WritePrivateProfileString(szApplication, "Orientation", GetOrientation(buf), IniFile);
	WritePrivateProfileString(szApplication, "Comment", sComment, IniFile);
	WritePrivateProfileString(szApplication, "Current", GetCurrent(buf), IniFile);
	WritePrivateProfileString(szApplication, "Voltage", GetVoltage(buf), IniFile);
};

LPCSTR TMeasurementParameter::GetComment()
{
	return sComment;
};

void TMeasurementParameter::SetComment( LPCSTR sCom )
{
	strcpy(sComment, sCom);
};

LPSTR TMeasurementParameter::GetReflection( LPSTR sRef )
{
	GetHKL(thklReflection, sRef);
	return sRef;
};

LPSTR TMeasurementParameter::GetReflection( THKL& thklRef, LPSTR sRef )
{
	thklRef= thklReflection;
	GetHKL(thklReflection, sRef);
	return sRef;
};

BOOL TMeasurementParameter::SetReflection( LPCSTR sRef )
{
	return SetHKL(thklReflection, sRef);
};

LPSTR TMeasurementParameter::GetOrientation( LPSTR sOri )
{
	GetHKL(thklOrientation, sOri);
	return sOri;
};

LPSTR TMeasurementParameter::GetOrientation( THKL& thklOri, LPSTR sOri )
{
	thklOri= thklOrientation;
	GetHKL(thklOrientation, sOri);
	return sOri;
};

BOOL TMeasurementParameter::SetOrientation( LPCSTR sOri )
{
	return SetHKL(thklOrientation, sOri);
};

LPCSTR TMeasurementParameter::GetUser()
{
	return sUser;
};

void TMeasurementParameter::SetUser( LPCSTR sUse )
{
	strcpy(sUser, sUse);
};

LPCSTR TMeasurementParameter::GetTarget()
{
	return sTarget;
};

void TMeasurementParameter::SetTarget( LPCSTR sTar )
{
	strcpy(sTarget, sTar);
};

LPCSTR TMeasurementParameter::GetTargetBulk()
{
	return sTargetBulk;
};

void TMeasurementParameter::SetTargetBulk( LPCSTR sTaB )
{
	strcpy(sTargetBulk, sTaB);
};

LPSTR TMeasurementParameter::GetWaveLength( LPSTR sWaL)
{
	sprintf(sWaL, "%.3f", fWaveLength);
	return sWaL;
};

float TMeasurementParameter::GetWaveLength()
{
	return fWaveLength;
};

BOOL TMeasurementParameter::SetWaveLength( LPCSTR sWaL )
{
	char *stopstring;

	fWaveLength= strtod(sWaL, &stopstring);
	if (strlen(stopstring) == 0)
		return TRUE;
	else
		return FALSE;
};

LPSTR TMeasurementParameter::GetVoltage( LPSTR sVol)
{
	sprintf(sVol, "%d", iVoltage);
	return sVol;
};

int TMeasurementParameter::GetVoltage()
{
	return iVoltage;
};

BOOL TMeasurementParameter::SetVoltage( LPCSTR sVol )
{
	iVoltage= atoi(sVol);
	if ( iVoltage != 0)
		return TRUE;
	else
		return FALSE;
};

BOOL TMeasurementParameter::SetVoltage( const int& iVol )
{
	iVoltage= iVol;
	if ( iVoltage != 0)
		return TRUE;
	else
		return FALSE;
};

LPSTR TMeasurementParameter::GetCurrent( LPSTR sCur )
{
	sprintf(sCur, "%d", iCurrent);
	return sCur;
};

int TMeasurementParameter::GetCurrent()
{
	return iCurrent;
};

BOOL TMeasurementParameter::SetCurrent( LPCSTR sCur )
{
	iCurrent= atoi(sCur);
	if ( iCurrent != 0)
		return TRUE;
	else
		return FALSE;
};

BOOL TMeasurementParameter::SetCurrent( const int& iCur )
{
	iCurrent= iCur;
	if ( iCurrent != 0)
		return TRUE;
	else
		return FALSE;
};

//! bestimmt 3 Integerwerte zwischen -9 und 9 aus einem String
//! wird nur in Verbindung mit Orientation,Reflection benutzt
//! 1.Parameter(ausgabe) Struktur von 3 integerwerten
//! 2.Parameter(eingabe) String
BOOL TMeasurementParameter::SetHKL( THKL& thklHKL, LPCSTR sHKL )
{
	int i, k, digit, alpha;
	THKL tmp;
	BOOL bNoError;

	i= digit= alpha= 0;
	tmp.h= tmp.k= tmp.l= 0;
	bNoError= TRUE;
	for (k= 0; k < strlen(sHKL); k++)
	{
		if (isdigit(sHKL[k]))
			digit++;
		if (isalpha(sHKL[k]))
			alpha++;
	}
	if ((digit != 3) || (alpha != 0))
	{
		thklHKL= tmp;
		return FALSE;
	}
	while ( '[' == sHKL[ i ] )
		i++;
	while ( ' ' == sHKL[ i ] )
		i++;
	if ( '-' == sHKL[ i ] )
		tmp.h= max( '0' - sHKL[ ++i ], -9 );
	else
		tmp.h= min( sHKL[ i ] - '0', 9 );
	if (!isdigit(sHKL[i]))
	{
		bNoError= FALSE;
		tmp.h= 0;
	}
	i++;
	while ( ' ' == sHKL[ i ] )
		i++;
	if ( '-' == sHKL[ i ] )
		tmp.k= max( '0' - sHKL[ ++i ], -9 );
	else
		tmp.k= min( sHKL[ i ] - '0', 9 );
	if (!isdigit(sHKL[i]))
	{
		bNoError= FALSE;
		tmp.k= 0;
	}
	i++;
	while ( ' ' == sHKL[ i ] )
		i++;
	if ( '-' == sHKL[ i ] )
		tmp.l= max( '0' - sHKL[ ++i ], -9 );
	else
		tmp.l= min( sHKL[ i ] - '0', 9 );
	if (!isdigit(sHKL[i]))
	{
		bNoError= FALSE;
		tmp.l= 0;
	}
	thklHKL= tmp;
	return bNoError;
};


//! bildet aus 3 Integerwerten einen String
//! wird nur in Verbindung mit Orientation,Reflection benutzt
//! 1.Parameter(eingabe) Struktur von 3 integerwerten
//! 2.Parameter(ausgabe) String
void TMeasurementParameter::GetHKL( THKL& thklHKL, LPSTR sHKL )
{
	sprintf( sHKL, "[%d%d%d]", thklHKL.h, thklHKL.k, thklHKL.l );
};






