#include "utils\u_utils.h"
#include "winresrc\rc_def.h" // Ressourcen-IDs
#pragma hdrstop

#include "motrstrg\motrstrg.h"
#include "motrstrg\m_layer.h"
#include "motrstrg\m_motcom.h"
#include "motrstrg\m_mothw.h"

#include "hardware\hwio.h"
#include "hardware\hwguids.h"

#include "utils\CheckMemory.h"	// CHECKMEM

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

HINSTANCE hModuleInstance;

static char mlVersion[ ]= "V 1.51 ("__DATE__")";

static int bModulLoaded= FALSE;
static BOOL bReadyHit;
static int MoveFinishIdx;
static int nTimeTicks= 20;
static int nCallBackAction= 0;
static const nScanSize= 150;
static long Scan[ nScanSize + 1 ];
static UINT nEvent;
static UINT nCycle= nScanSize;
static BOOL bScanReady= TRUE;
static BOOL bShowEncoder= FALSE;

DeviceList MotorDrivers;
ControllerList MotorControllers;

extern int nC812;

char buf[ MaxString ];

LPMList _MOTORCLASS lpMList= NULL; // Kullmann, Reinecker (02.08.02) exportiert

LPCSTR _MOTORCLASS WINAPI mlGetVersion( void )
{
	return ( LPCSTR ) mlVersion;
};

HINSTANCE _MOTORCLASS WINAPI mlGetInstance( void )
{
	return hModuleInstance;
};

//******************************************************************************
//***** Routines for optimizing the motors *************************************
//******************************************************************************

// forward declaration, s.u.
void _MOTORCLASS CALLBACK mSavePosition( UINT, UINT, DWORD, DWORD, DWORD );

int _MOTORCLASS WINAPI mGetScanSize( void )
{
	return nScanSize;
};

// VORSICHT: 2. int-Parameter ist nicht deklariert
void _MOTORCLASS WINAPI mStartMoveScan( int tic, int )
{
	if ( !bScanReady )
		return;
	nCycle= 0;
	memset( Scan, 0, nScanSize );
	bReadyHit= FALSE;
	bScanReady= FALSE;
	nTimeTicks= tic;
	lpMList -> MP() -> StartCheckScan();
};

void TC_812ISA::StartCheckScan( void )
{
	// timeBeginPeriod(1);
	nCallBackAction= 1;
	nEvent= timeSetEvent( nTimeTicks, 1, (LPTIMECALLBACK)mSavePosition, 0,
						   TIME_PERIODIC );
	MoveByPosition( wPositionWidth );
};

void TC_832::StartCheckScan( void )
{
	// timeBeginPeriod(1);
	nCallBackAction= 1;
	nEvent= timeSetEvent( nTimeTicks, 1, (LPTIMECALLBACK)mSavePosition, 0,
						   TIME_PERIODIC );
	MoveByPosition( wPositionWidth );
};

LPLONG _MOTORCLASS WINAPI mGetMoveScan( void )
{
	bScanReady= TRUE;
	return ( LPLONG ) Scan;
};

int _MOTORCLASS WINAPI mGetMoveFinishIdx( void )
{
	return MoveFinishIdx;
};

void _MOTORCLASS CALLBACK mSavePosition( UINT nEvent, UINT, DWORD, DWORD, DWORD )
{
	switch ( nCallBackAction )
	{
		case 1:
			lpMList -> MP() -> GetPosition( FALSE );
			Scan[ nCycle++ ]= lpMList -> MP() -> GetPosition(); //Neu : Kullmann, Reinecker (27.07.02) lPosition ersetzt
			if ( !bReadyHit )
				if ( lpMList -> MP() -> IsMoveFinish() )
				{
					bReadyHit= TRUE;
					MoveFinishIdx= nCycle - 1;
				}
			if ( nCycle >= nScanSize )
			{
				timeKillEvent( nEvent );
				nCycle= 0;
				PostMessage( GetScanHandle(), WM_COMMAND, cm_MoveScanReady, 0 );
			}
			break;

		case 2:
			nCycle= 0;
			MoveFinishIdx= 1;
			PostMessage( GetScanHandle(), WM_COMMAND, cm_MoveScanReady, 0 );
			break;
	}
};
//******************************************************************************
//******************************************************************************

//******************************************************************************
//***** C-Call interface for TMList ********************************************
//******************************************************************************
BOOL _MOTORCLASS WINAPI mlInitializeMotorsDLL( void )
{
	bModulLoaded= FALSE;
	bShowEncoder= GetPrivateProfileInt( "Steuerprogramm", "ArrangeMode", 0,
										 GetCFile( ) );
	if ( !lpMList->InitializeModule( ) )
		return FALSE;
	// Motors.dll initialisiert, ini-Datei erfolgreich eingelesen
	bModulLoaded= TRUE;
	return TRUE;
};

BOOL _MOTORCLASS WINAPI mlSetAxis( int axis )
{
	// bestimmte Achse wird aktiviert
	if ( lpMList->SetAxis( axis ) )
		return TRUE;
	lpMList->SetAxis( 0 );
	return FALSE;
};

int _MOTORCLASS WINAPI mlGetAxis( void )
{
	// Rueckgabe der aktiven Achse
	return lpMList->GetAxis( );
	//TMList::GetAxis() ist inline in m_motcom.h definiert
};

int _MOTORCLASS WINAPI mlGetIdByName( EAxisType at )
{
	int id= -1;
	// Rckgabe der Achsen-ID
	switch ( at )
	{
		case X:
			id= lpMList->AD_X;
			break;

		case Y:
			id= lpMList->AD_Y;
			break;
		
		case Z:
			id= lpMList->AD_Z;
			break;
		
		case Omega:
			id= lpMList->AD_Omega;
			break;
		
		case Theta:
			id= lpMList->AD_Theta;
			break;
		
		case Phi:
			id= lpMList->AD_Phi;
			break;
		
		case Psi:
			id= lpMList->AD_Psi;
			break;
		
		case Encoder:
			id= lpMList->AD_Encoder;
			break;
		
		case Monochromator:
			id= lpMList->AD_Monochromator;
			break;
		
		case Absorber:
			id= lpMList->AD_Absorber;
			break;
		
		case Collimator:
			id= lpMList->AD_Collimator;
			break;
		
		case DF:
			id= lpMList->AD_DF;
			break;
		
		case DC:
			id= lpMList->AD_DC;
			break;
		
		case Tilt:
			id= lpMList->AD_Tilt;
			break;
		
		case Rotation:
			id= lpMList->AD_Rotation;
			break;
	}
	return id;
};

EAxisType _MOTORCLASS WINAPI mlParsingAxis( LPSTR axisname )
{
	return lpMList->ParsingAxis( axisname );
};

BOOL _MOTORCLASS WINAPI mlIsAxisValid( EAxisType type )
{
	switch ( type )
	{
		case X:
			return ( lpMList->AD_X > -1 );
		
		case Y:
			return ( lpMList->AD_Y > -1 );
		
		case Z:
			return ( lpMList->AD_Z > -1 );
		
		case Omega:
			return ( lpMList->AD_Omega > -1 );
		
		case Theta:
			return ( lpMList->AD_Theta > -1 );
		
		case Phi:
			return ( lpMList->AD_Phi > -1 );
		
		case Psi:
			return ( lpMList->AD_Psi > -1 );
		
		case Encoder:
			return ( lpMList->AD_Encoder > -1 );
		
		case Monochromator:
			return ( lpMList->AD_Monochromator > -1 );
		
		case Absorber:
			return ( lpMList->AD_Absorber > -1 );
		
		case Collimator:
			return ( lpMList->AD_Collimator > -1 );
		
		case DF:
			return ( lpMList->AD_DF > -1 );
		
		case DC:
			return ( lpMList->AD_DC > -1 );
		
		case Tilt:
			return ( lpMList->AD_Tilt > -1 );
		
		case Rotation:
			return ( lpMList->AD_Rotation > -1 );
		
		default:
			return FALSE;
	}
};

BOOL _MOTORCLASS WINAPI mlIsServerOK( void )
{
	return bModulLoaded;
};

int _MOTORCLASS WINAPI mlGetAxisNumber( void )
{
	return lpMList->GetAxisNumber( );
};

double _MOTORCLASS WINAPI mlGetOffset( int mid )
{
	return lpMList->MP( mid ) ->GetAngleBias(); //Neu : Kullmann, Reinecker (27.07.02) dAngleBias ersetzt
};

void _MOTORCLASS WINAPI mlSetAngleDefault( void )
{
	lpMList->SetAngleDefault( );
};

void _MOTORCLASS WINAPI mlOptimizingDlg( void )
{
	PostMessage( GetFrameHandle( ), WM_COMMAND, cm_CallExecuteScan, 0l );
	lpMList->MP( ) ->OptimizingDlg( );
};

UINT _MOTORCLASS WINAPI mlGetDigits ( int mid )
{
	return lpMList->MP( mid ) ->GetDigits();
}

double _MOTORCLASS WINAPI mlGetValue( int mid, EValueType vtype )
{
	switch ( vtype )
	{
		case Distance:
  			return lpMList->MP( mid ) -> GetAngle(); //Neu : Kullmann, Reinecker (27.07.02) dAngle ersetzt

		case MinDistance:
			//Neu : Kullmann, Reinecker (27.07.02) dAngleMin ersetzt
			return min( lpMList->MP( mid ) -> GetAngleMin(),
	  					lpMList->MP( mid ) -> GetAngleMax() ); //29.01.2003 Kullmann+Reinecker: GetAngleMin ersetzt durch min(AngleMin, AngleMax)

		case MaxDistance:
			//Neu : Kullmann, Reinecker (27.07.02) dAngleMax ersetzt
			return max( lpMList->MP( mid ) -> GetAngleMin(),
					    lpMList->MP( mid ) -> GetAngleMax() ); //29.01.2003 Kullmann+Reinecker: GetAngleMax ersetzt durch max(AngleMin, AngleMax)

		case MinDistanceRel:
	  		//Neu : 17.07.2003
  			return min( lpMList->MP( mid ) -> GetAngleRelativeMin(),
	  					lpMList->MP( mid ) -> GetAngleRelativeMax() ); 

		case MaxDistanceRel:
			//Neu : 17.07.2003
			return max( lpMList->MP( mid ) -> GetAngleRelativeMin(),
					    lpMList->MP( mid ) -> GetAngleRelativeMax() ); 

		case Speed:
			return lpMList->MP( mid ) -> GetSpeed();

		case MinSpeed:
			return lpMList->MP( mid ) -> GetMinSpeed();

		case MaxSpeed:
			return lpMList->MP( mid ) -> GetMaxSpeed();

		case Width:
  			return lpMList->MP( mid ) -> GetAngleWidth();

		case MinWidth:
			return lpMList->MP( mid ) -> GetMinAngleWidth();

		case Acceleration:
			return lpMList->MP( mid ) -> GetAcceleration();

		default:
			return 1.0;
	}
};

double _MOTORCLASS WINAPI mlGetAngleRelativeMin( int mid )
{
	return lpMList->MP( mid ) -> GetAngleRelativeMin();
};

double _MOTORCLASS WINAPI mlGetAngleRelativeMax( int mid )
{
	return lpMList->MP( mid ) -> GetAngleRelativeMax();
};

BOOL _MOTORCLASS WINAPI mlMoveToDistance( int mid, double distance )
{
	// Wurde motors.dll korrekt initialisiert ?
	// wenn INI-Datei nicht erfolgreich geladen -> FALSE
	if ( !bModulLoaded )
		return FALSE;
	// VORSICHT: keine Fehlerkontrolle bei Fkt.-Ausfuehrung
	// distance ist ein Winkelmass und muss in MoveToAngel() in Position
	// uebertragen ( Translate()-Fkt. ) werden
	// Keine Behandlung von MoveToAngle()-Rueckgabewert FALSE
	// (Winkel distance ist keine gueltige Position bzw.
	//  MoveAbsolute() in MoveToPosition() des akt. Motors schlaegt fehl)
	lpMList->MP( mid ) ->MoveToAngle( distance );
	// VORSICHT: Fkt. gibt immer TRUE zurueck - Fehler ??
	return TRUE;
};

BOOL _MOTORCLASS WINAPI mlIsMoveFinish( int mid )
{
	return lpMList->MP( mid ) ->IsMoveFinish( );
};

void _MOTORCLASS WINAPI mlSetParametersDlg( void )
{
	lpMList->SetParametersDlg( );
};

void _MOTORCLASS WINAPI mlPositionControlDlg( void )
{
	lpMList->PositionControlDlg( );
};

void _MOTORCLASS WINAPI mlSaveModuleSettings( void )
{
	lpMList->SaveModuleSettings( );
};

void _MOTORCLASS WINAPI mlInquireReferencePointDlg( BOOL task ) // Parameter gendert int -> BOOL (08.08.02)
{
	lpMList->InquireReferencePointDlg( task );
};

//******************************************************************************
//***** C-Call interface for TMotor ********************************************
//******************************************************************************
BOOL _MOTORCLASS WINAPI mSetLine( int number, BOOL state )
{
	// wurde motors.dll korrekt initialisiert ? NEIN -> Abbruch
	if ( !bModulLoaded )
		return FALSE;
	return lpMList->MP( nC812 ) ->SetLine( number, state );
};

void _MOTORCLASS WINAPI mlSetRelativeZero( int axis, BOOL yes, double value )
{
	if ( yes )
		// lpMList->MP(axis) -> dAngleBias -= value;
		lpMList->MP(axis) -> SetAngleBias( lpMList->MP(axis)->GetAngleBias() - value );  //Neu : Kullmann, Reinecker (27.07.02) dAngleBias ersetzt
	else
		// lpMList->MP(axis) -> dAngleBias= 0.0;
		lpMList->MP(axis) -> SetAngleBias( 0.0 );  //Neu : Kullmann, Reinecker (27.07.02) dAngleBias ersetzt
}
void _MOTORCLASS WINAPI mSetRelativeZero( BOOL yes, double value )
{
	if ( yes )
		// lpMList->MP( ) -> dAngleBias -= value;
		lpMList->MP( ) -> SetAngleBias( lpMList->MP( )->GetAngleBias() - value );  //Neu : Kullmann, Reinecker (27.07.02) dAngleBias ersetzt
	else
		// lpMList->MP( ) -> dAngleBias= 0.0;
		lpMList->MP( ) -> SetAngleBias( 0.0 );  //Neu : Kullmann, Reinecker (27.07.02) dAngleBias ersetzt
}

BOOL _MOTORCLASS WINAPI mlIsDistanceRelative( int axis )
{
	return ( lpMList->MP(axis) ->GetAngleBias() != 0.0 ); //Neu : Kullmann, Reinecker (27.07.02) dAngleBias ersetzt
}
BOOL _MOTORCLASS WINAPI mIsDistanceRelative( void )
{
	return ( lpMList->MP( ) ->GetAngleBias() != 0.0 ); //Neu : Kullmann, Reinecker (27.07.02) dAngleBias ersetzt
}

BOOL _MOTORCLASS WINAPI mIsRangeHit( void )
{
	return lpMList->MP( ) -> IsRangeHit( );
};

BOOL _MOTORCLASS WINAPI mMoveByDistance( double angle )
{
	// wurde motors.dll korrekt initialisiert ? NEIN -> Abbruch
	if ( !bModulLoaded )
		return FALSE;
	// VORSICHT: keine Fehlerkontrolle bei Fkt.-Ausfuehrung
	lpMList->MP( ) ->MoveByAngle( angle );
	// VORSICHT: Fkt. gibt immer TRUE zurueck - Fehler ??
	return TRUE;
};

BOOL _MOTORCLASS WINAPI mMoveToDistance( double angle )
{
	// Wurde motors.dll korrekt initialisiert ?
	// wenn INI-Datei nicht erfolgreich geladen -> FALSE
	if ( !bModulLoaded )
		return FALSE;
	// VORSICHT: keine Fehlerkontrolle bei Fkt.-Ausfuehrung
	// distance ist ein Winkelmass und muss in MoveToAngel() in Position
	// uebertragen ( Translate()-Fkt. ) werden
	// Keine Behandlung von MoveToAngle()-Rueckgabewert FALSE
	// (Winkel distance ist keine gueltige Position bzw.
	//  MoveAbsolute() in MoveToPosition() des akt. Motors schlaegt fehl)
	lpMList->MP( ) -> MoveToAngle( angle );
	// VORSICHT: Fkt. gibt immer TRUE zurueck - Fehler ??
	return TRUE;
};

BOOL _MOTORCLASS WINAPI mIsMoveFinish( void )
{
	// wurde motors.dll korrekt initialisiert ?
	// VORSICHT: im Fehlerfall (keine korrekte Init.) Rueckgabe TRUE ??
	if ( !bModulLoaded )
		return TRUE;
	return lpMList->MP( ) ->IsMoveFinish( );
};

BOOL _MOTORCLASS WINAPI mlGetDistance( int mid, double& distance )
{
	// Falls Motor nicht in Fahrt & kein Auslesefehler
	// dann ist in MemberVar dAngel akt. Winkel gespeichert
	if ( lpMList->MP( mid ) ->GetAngle( TRUE ) )
	{
		// Distanz wird als Winkel dAngel zurueckgeliefert
		distance= lpMList->MP( mid ) ->GetAngle(); //Neu : Kullmann, Reinecker (27.07.02) dAngle ersetzt
		return TRUE;
	}
	distance= 0.0;
	return FALSE;
};
BOOL _MOTORCLASS WINAPI mGetDistance( double &ang )
{
	// Falls Motor nicht in Fahrt & kein Auslesefehler
	// dann ist in MemberVar dAngel akt. Winkel gespeichert
	if ( lpMList->MP( ) ->GetAngle( TRUE ) )
	{
		// Distanz wird als Winkel dAngel zurueckgeliefert
		ang= lpMList->MP( ) ->GetAngle(); //Neu : Kullmann, Reinecker (27.07.02) dAngle ersetzt
		if ( !bShowEncoder )
			return TRUE;
		sprintf( buf, "%ld", lpMList->MP( ) -> GetPosition() ); //Neu : Kullmann, Reinecker (27.07.02) lPosition ersetzt
		SetStatus( buf );
		return TRUE;
	}
	ang= 0.0;
	return FALSE;
};

double _MOTORCLASS WINAPI mlGetDistanceProcess( int axis )
{
	lpMList->MP(axis) ->GetAngle( 0 );
	return lpMList->MP(axis) -> GetAngle(); //Neu : Kullmann, Reinecker (27.07.02) dAngle ersetzt
};
double _MOTORCLASS WINAPI mGetDistanceProcess( void )
{
	lpMList->MP( ) ->GetAngle( 0 );
	return lpMList->MP( ) -> GetAngle(); //Neu : Kullmann, Reinecker (27.07.02) dAngle ersetzt
};

//! neu Kullmann+Reinecker: Statt GetDF() verwenden wir lieber eine andere Konvertierungsfkt.
UINT _MOTORCLASS WINAPI mGetDigits ( void )
{
	return lpMList->MP( ) ->GetDigits();
};

void _MOTORCLASS WINAPI mStopDrive( BOOL restart )
{
	lpMList->MP( ) ->StopDrive( restart );
};

double _MOTORCLASS WINAPI mGetValue( EValueType vtype )
{
	switch ( vtype )
	{
		case Distance:
			return lpMList->MP() -> GetAngle(); //Neu : Kullmann, Reinecker (27.07.02) dAngle ersetzt

		case MinDistance:
			//Neu : Kullmann, Reinecker (27.07.02) dAngleMin ersetzt
			return min( lpMList->MP() -> GetAngleMin(),
				        lpMList->MP() -> GetAngleMax() ); //29.01.2003 Kullmann+Reinecker: GetAngleMin ersetzt durch min(AngleMin, AngleMax)

		case MaxDistance:
			//Neu : Kullmann, Reinecker (27.07.02) dAngleMax ersetzt
			return max( lpMList->MP() -> GetAngleMin(),
				        lpMList->MP() -> GetAngleMax() ); //29.01.2003 Kullmann+Reinecker: GetAngleMax ersetzt durch max(AngleMin, AngleMax)

		case MinDistanceRel:
	  		//Neu : 17.07.2003
  			return min( lpMList->MP() -> GetAngleRelativeMin(),
	  					lpMList->MP() -> GetAngleRelativeMax() ); 

		case MaxDistanceRel:
			  //Neu : 17.07.2003
			  return max( lpMList->MP() -> GetAngleRelativeMin(),
						  lpMList->MP() -> GetAngleRelativeMax() );

		case Speed:
			return lpMList->MP() -> GetSpeed();

		case MinSpeed:
			return lpMList->MP() -> GetMinSpeed();

		case MaxSpeed:
			return lpMList->MP() -> GetMaxSpeed();

		case Width:
			return lpMList->MP() -> GetAngleWidth();

		case MinWidth:
			return lpMList->MP() -> GetMinAngleWidth();

		case Acceleration:
			return lpMList->MP() -> GetAcceleration();

		default:
			return 1.0;
	}
};

double _MOTORCLASS WINAPI mGetAngleRelativeMin( void )
{
	return lpMList->MP() -> GetAngleRelativeMin();
}

double _MOTORCLASS WINAPI mGetAngleRelativeMax( void )
{
	return lpMList->MP() -> GetAngleRelativeMax();
}

BOOL _MOTORCLASS WINAPI mlSetValue( int axis, EValueType vtype, double value )
{
	switch ( vtype )
	{
		case Speed:
			return lpMList->MP(axis) -> SetSpeed( value );

		case Width:
			return lpMList->MP(axis) -> SetAngleWidth( value );

		// VORSICHT: nicht zu setzende Cases: immer FALSE
		default:
			return FALSE;
	}
};
BOOL _MOTORCLASS WINAPI mSetValue( EValueType vtype, double value )
{
	switch ( vtype )
	{
		case Speed:
			return lpMList->MP( ) -> SetSpeed( value );

		case Width:
			return lpMList->MP( ) -> SetAngleWidth( value );

		// VORSICHT: nicht zu setzende Cases: immer FALSE
		default:
			return FALSE;
	}
};

EUnitType _MOTORCLASS WINAPI mlGetUnit( int axis ) {
	return lpMList->MP(axis)->GetUnit(); //Neu : Kullmann, Reinecker (27.07.02) eUnit ersetzt
}
EUnitType _MOTORCLASS WINAPI mGetUnit( void )
{
	return lpMList->MP( )->GetUnit(); //Neu : Kullmann, Reinecker (27.07.02) eUnit ersetzt
};

BOOL _MOTORCLASS WINAPI mlIsCalibrated( int axis )
{
	return lpMList->MP(axis) ->IsCalibrated( );
};
BOOL _MOTORCLASS WINAPI mIsCalibrated( void )
{
	return lpMList->MP( ) ->IsCalibrated( );
};

void _MOTORCLASS WINAPI mActivateDrive( void )
{
	lpMList->MP( ) ->ActivateDrive( );
};

void _MOTORCLASS WINAPI mlSetCorrectionState( int axis, BOOL onoff )
{
	lpMList->MP(axis) ->SetCorrectionState( onoff );
};
void _MOTORCLASS WINAPI mSetCorrectionState( BOOL onoff )
{
	lpMList->MP( ) ->SetCorrectionState( onoff );
};

LPCSTR _MOTORCLASS WINAPI mlGetAxisName( int axis )
{
	return lpMList->MP(axis) ->pCharacteristic( );
};
LPCSTR _MOTORCLASS WINAPI mGetAxisName( void )
{
	return lpMList->MP( ) ->pCharacteristic( );
};

LPCSTR _MOTORCLASS WINAPI mlGetUnitName( int mid ) {
	return lpMList->MP( mid )->GetUnitName();
}
LPCSTR _MOTORCLASS WINAPI mGetUnitName( void )
{
	return ( LPCSTR ) lpMList->MP( ) ->GetUnitName(); //Neu : Kullmann, Reinecker (27.07.02) szUnit ersetzt
};

LPCSTR _MOTORCLASS WINAPI mGetSF( void )
{
	return lpMList->MP( ) ->pSF( );
};

LPCSTR _MOTORCLASS WINAPI mGetDF( void )
{
	return lpMList->MP( ) ->pDF( );
};

int _MOTORCLASS WINAPI mExecuteCmd( LPSTR cmd )
{
	return lpMList->MP( ) ->ExecuteCmd( cmd );
};

void _MOTORCLASS WINAPI mPushSettings( void )
{
	lpMList->MP( ) ->PushSettings( );
};

void _MOTORCLASS WINAPI mPopSettings( EMParameter mp )
{
	lpMList->MP( ) ->PopSettings( mp );
};

//******************************************************************************
//******************************************************************************
TMFunctionTable MFT;

BOOL WINAPI DllMain(HINSTANCE hModule, DWORD dwFunction, LPVOID)
{
	if ( !lpMList )
		lpMList= ( LPMList ) new TMList( 10 ); // TMList zweiten Parameter, hier: NULL, entfernt (08.08.02)

	switch ( dwFunction )
	{
		case DLL_PROCESS_ATTACH:
			hModuleInstance= hModule;
			if ( !lpMList )
				lpMList= ( LPMList ) new TMList( 10 ); // TMList zweiten Parameter, hier: NULL, entfernt (08.08.02)
			// Initialize function array.
			MFT.InitializeMotorsDLL= ( TInitializeMotorsDLL ) mlInitializeMotorsDLL;
			MFT.SetAxis= ( TSetAxis ) mlSetAxis;
			MFT.GetAxis= ( TGetAxis ) mlGetAxis;
			MFT.MoveToDistance= ( TMoveToDistance ) mMoveToDistance;
			MFT.MoveByDistance= ( TMoveByDistance ) mMoveByDistance;
			MFT.SetLine= ( TSetLine ) mSetLine;
			MFT.IsMoveFinish= ( TIsMoveFinish ) mIsMoveFinish;
			MFT.GetDistance= ( TGetDistance ) mGetDistance;
			MFT.IsServerOK= ( TIsServerOK ) mlIsServerOK;

			RegisterDrivers(&MotorDrivers,GUID_C812,C812);
			RegisterDrivers(&MotorDrivers,GUID_C832,C832);
			break;

		case DLL_PROCESS_DETACH:
			bModulLoaded= FALSE;
			//CHECKMEM("MOTORS.DLL");	//hp
			break;
	}

	return TRUE;
}

Controller* GetController(EDeviceType DeviceID, LPTSTR HardwareID, ControllerList* Controllers, DeviceList* Drivers,
									 DWORD* index)
{
	char szDeviceName[257];
	memset(szDeviceName, '\0', 257);

	Controller* pCon= Controllers->GetController(DeviceID, HardwareID);

	if (!pCon)
	{ //neuen Controller anlegen
		switch (DeviceID)
		{
			case C812:
				strcpy(szDeviceName, "C-812");
				pCon= new TC_812Controller(DeviceID, HardwareID, Drivers);
				break;

			case C832:
				strcpy(szDeviceName, "C-832");
				pCon= new TC_832Controller(DeviceID, HardwareID, Drivers);
				break;

			default:
				strcpy(szDeviceName, "Unbekanntes Gert");
				pCon= new StandardController(DeviceID, HardwareID, Drivers);
		}

		if (!pCon)
		{
			MessageBox(0, "Fehler bei Initialisierung des Controllers", szDeviceName, MBINFO);
			return (0);
		}

		if (!pCon->AddClient())
		{
			MessageBox(0, "Maximale Anzahl von Clients berschritten.\nEs liegt eine Fehlkonfiguration vor.", szDeviceName, MBINFO);
			return (0);
		}

		Controllers->Add(pCon);
	}

	if (index)
		*index= Controllers->GetControllerIndex(DeviceID, HardwareID);

	return (pCon);
}



// __LastLine__
