//******************************************************************************
//Datei     : TProtocolManagePrintDlg.cpp
//Projekt   : XCTL
//Subsystem : Protokollbuch
//Autor     : Jens Klier <klier@informatik.hu-berlin.de>
//            Institut fuer Informatik,
//            Humboldt-Universitaet Berlin
//Inhalt    : Implementation der Klasse TProtocolManagePrintDlg
//******************************************************************************
//!neu klier Protokollbuch
#include "internls\evrythng.h"
#include "winresrc\rc_prtow.h" // Ressourcen-IDs
#include <commdlg.h>
#include <commctrl.h>

#include "protocol\TProtocolLanguage.h"
#include "protocol\TProtocol.h"
#include "protocol\TProtocolDlg.h"
#include "protocol\TProtocolParameterDlg.h"
#include "protocol\TProtocolManagePrintDlg.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

extern TProtocolLanguage *ProtocolLanguage;

TProtocolManagePrintDlg::TProtocolManagePrintDlg(int Feldelemente, int x, int y) : Felder(Feldelemente),
		PrintX(x), PrintY(y), bDelete(FALSE), bSearchActiv(FALSE)
{}
;

void TProtocolManagePrintDlg::EnableButton( BOOL Status )
{
	EnableWindow( GetDlgItem(hwndDlg, IDSEARCH), Status);
	EnableWindow( GetDlgItem(hwndDlg, IDVIEW), Status);
	if (!Protocol->IsProtocolOn())
	{
		EnableWindow( GetDlgItem(hwndDlg, IDDELETE), Status);
		EnableWindow( GetDlgItem(hwndDlg, IDIMPORT), Status);
		EnableWindow( GetDlgItem(hwndDlg, IDPRINT), Status);
		EnableWindow( GetDlgItem(hwndDlg, id_Parameter), Status);
	}
};

void TProtocolManagePrintDlg::SearchNew( )
{
	if (bSearchActiv)
		SendMessage(hwndDlg, WM_COMMAND, IDSEARCH, 0);
};


BOOL TProtocolManagePrintDlg::PListView_InsertColumn(HWND hLVCtrl, int iCol, LPCSTR ColText)
{
    LV_COLUMN lvc;
	char buf[MaxString];
 
    // Initialize the LV_COLUMN structure. 
    lvc.mask= LVCF_TEXT | LVCF_FMT; 
    lvc.fmt= LVCFMT_LEFT; 
    lvc.pszText= buf; 

    // Add the columns. 
    lvc.iSubItem= iCol;
 	strcpy(buf, ColText);
	lvc.cx=ListView_GetStringWidth(hLVCtrl,buf);
    if (ListView_InsertColumn(hLVCtrl, iCol, &lvc) == -1) 
            return FALSE;

    ListView_SetColumnWidth(hLVCtrl,iCol,LVSCW_AUTOSIZE_USEHEADER);

	return TRUE;
};

void TProtocolManagePrintDlg::PListView_InsertItem(HWND hLVCtrl, int iIndex, LPCSTR ItemText)
{
	char buf[MaxString];
	int iCol=0,cx;
	LVITEM pitem;
 
    pitem.iItem= iIndex;
    pitem.iSubItem= 0;
    pitem.mask= LVIF_TEXT;
    pitem.state= LVIS_FOCUSED;
	pitem.stateMask= NULL;
	pitem.cchTextMax= MaxString;
	pitem.iImage= NULL;
	pitem.lParam= NULL;
	pitem.iIndent= 1;
	pitem.pszText= buf;

	strcpy(buf, ItemText);
	ListView_InsertItem(hLVCtrl, &pitem);

    if( ListView_GetColumnWidth(hLVCtrl,0) < 
	    (cx=(ListView_GetStringWidth(hLVCtrl,buf)+15)) )
    ListView_SetColumnWidth(hLVCtrl,0,cx);
};

void TProtocolManagePrintDlg::PListView_SetItemText(HWND hLVCtrl, int iIndex, int iCol, LPSTR ItemText)
{
    int cx;

	ListView_SetItemText(hLVCtrl, iIndex, iCol, ItemText);

    if( ListView_GetColumnWidth(hLVCtrl,iCol) < 
	    (cx=(ListView_GetStringWidth(hLVCtrl,ItemText)+15)) )
    ListView_SetColumnWidth(hLVCtrl,iCol,cx);
};

BOOL TProtocolManagePrintDlg::Dlg_OnInit(HWND hwnd, HWND hwndCtl, LPARAM lParam)
{
	ProtocolDlg->InitializeManagePrintDlg(this);
	hwndDlg= hwnd;
	hSuchList= GetDlgItem( hwnd, id_SuchList );
	if (Protocol->IsProtocolOn())
	{
		EnableWindow( GetDlgItem(hwndDlg, IDDELETE), DISABLE);
		EnableWindow( GetDlgItem(hwndDlg, IDIMPORT), DISABLE);
		EnableWindow( GetDlgItem(hwndDlg, IDPRINT), DISABLE);
		EnableWindow( GetDlgItem(hwndDlg, id_Parameter), DISABLE);
	}
	return TRUE;
};

void TProtocolManagePrintDlg::Dlg_OnNotify(HWND hwnd, int message, UINT id, UINT wParam)
{
    char buf[MaxLine], buf2[MaxString];
	int SelCount;
	
    if ( message!=WM_NOTIFY || id!=id_SuchList ) return;

	if ( (SelCount= ListView_GetSelectedCount(hSuchList)) > 0 )
	{
		strcpy(buf, itoa(SelCount, buf2, 10));
		if (SelCount != 1)
			strcat(buf, ProtocolLanguage->GetMessage(42));
		else
			strcat(buf, ProtocolLanguage->GetMessage(43));
		strcat(buf, itoa(GetPages(SelCount), buf2, 10));
		strcat(buf, ProtocolLanguage->GetMessage(44));
		SetDlgItemText(hwnd, id_PrnStatusLine, buf);
	}		
};

BOOL TProtocolManagePrintDlg::Dlg_OnCommand(HWND hwnd, int id, UINT codeNotify)
{
	char buf[MaxLine], buf2[MaxString], lLfdNr[MaxString], tmpLfdNr[MaxString];
	TSearchDate lDatum;
	TSearchParameter lParameter;
	int Count, SelCount, i;
	BOOL bTest, bIsDelete;
	TModalDlg *dlg= 0;

	bTest= TRUE;
	switch (id)
	{
		case IDSEARCH:
			bSearchActiv= TRUE;
			GetDlgItemText(hwnd, id_PrnDatum1, lDatum[0], MaxString);
			if ((strlen(lDatum[0]) > 0) && !Protocol->TestDate(lDatum[0]))
			{
				SetDlgItemText(hwnd, id_PrnStatusLine,
							   ProtocolLanguage->GetMessage(58));
				MessageBox(GetFocus(), ProtocolLanguage->GetMessage(58),
						   ProtocolLanguage->GetFailure(), MBFAILURE);
				SetFocus(GetDlgItem(hwnd, id_PrnDatum1));
				break;
			}
			GetDlgItemText(hwnd, id_PrnDatum2, lDatum[1], MaxString);
			if ((strlen(lDatum[1]) > 0) && !Protocol->TestDate(lDatum[1]))
			{
				SetDlgItemText(hwnd, id_PrnStatusLine,
							   ProtocolLanguage->GetMessage(58));
				MessageBox(GetFocus(), ProtocolLanguage->GetMessage(58),
						   ProtocolLanguage->GetFailure(), MBFAILURE);
				SetFocus(GetDlgItem(hwnd, id_PrnDatum2));
				break;
			}
			SetDlgItemText(hwnd, id_PrnStatusLine, "");
            ListView_DeleteAllItems(hSuchList);
			GetDlgItemText_LfdNr(hwnd, lLfdNr);
			strcpy(tmpLfdNr, Protocol->GetLfdNr(buf));
			if (Protocol->InitializeSearchProtocolString(lLfdNr))
			{
				GetDlgItemText_SearchParameter( hwnd, lParameter );
				Count= 0;
				while (SearchProtocolString( lLfdNr, lDatum, lParameter ))
				{
					if ( bDelete == Protocol->IsProtocolDelete() )
					{
						strcpy(buf, itoa(++Count, buf2, 10));
						if (Count != 1)
							strcat(buf, ProtocolLanguage->GetMessage(59));
						else
							strcat(buf, ProtocolLanguage->GetMessage(60));
						SetDlgItemText(hwnd, id_PrnStatusLine, buf);

                        MakeListString(Count-1);
					}
				}
			}
			if ( (Count= ListView_GetItemCount(hSuchList)) == 0 )
			{
				SetDlgItemText(hwnd, id_PrnStatusLine,
							   ProtocolLanguage->GetMessage(61));
			}
			else
			{
				strcpy(buf, itoa(Count, buf2, 10));
				if (Count != 1)
					strcat(buf, ProtocolLanguage->GetMessage(59));
				else
					strcat(buf, ProtocolLanguage->GetMessage(60));
				SetDlgItemText(hwnd, id_PrnStatusLine, buf);
			}
			Protocol->ReloadProtocolString(tmpLfdNr);
			break;

		case IDVIEW:
			if ( (Count= ListView_GetItemCount(hSuchList)) > 0 )
			{
				if ( (SelCount= ListView_GetSelectedCount(hSuchList)) > 0 )
				{
					if (ProtocolDlg->TestChangeDlg())
						break;
					SetDlgItemText(hwnd, id_PrnStatusLine, "");
                    SelCount=ListView_GetNextItem(hSuchList,-1,LVIS_SELECTED);
					ProtocolDlg->ShowLfdNr(GetSelLfdNr(lLfdNr, SelCount));
				}
				else
					SetDlgItemText(hwnd, id_PrnStatusLine,
								   ProtocolLanguage->GetMessage(45));
			}
			break;
		
		case IDPRINT:
			if ( (Count= ListView_GetItemCount(hSuchList)) > 0 )
			{
				if ( (SelCount= ListView_GetSelectedCount(hSuchList)) > 0 )
				{
					PrintSelection(hwnd, SelCount, GetColumn());
				}
				else
				{
					ListBox_GetText(hSuchList, 0, buf);
					strcpy(buf, ProtocolLanguage->GetMessage(30));
					strcat(buf, itoa(Count, buf2, 10));
					if (Count != 1)
						strcat(buf, ProtocolLanguage->GetMessage(31));
					else
						strcat(buf, ProtocolLanguage->GetMessage(32));
					strcat(buf, itoa(GetPages(Count), buf2, 10));
					strcat(buf, ProtocolLanguage->GetMessage(33));
					if (MessageBox(GetFocus(), buf,
								   ProtocolLanguage->GetQuestion(), MBASK) == IDYES)
					{
						for(i=0; i<Count; i++)
                            ListView_SetItemState(hSuchList,i,LVIS_SELECTED,LVIS_SELECTED);
						PrintSelection(hwnd, Count, GetColumn());
					}
				}
				SetDlgItemText(hwnd, id_PrnStatusLine, "");
			}
			break;
		
		case id_Parameter:
			if (!ProtocolDlg->GetPasswort(hwnd))
				break;
			dlg= (TProtocolParameterDlg *)new TProtocolParameterDlg("ProtocolParameterDlg", Protocol, ProtocolDlg, this);
			if ( dlg ) dlg->ExecuteDialog(hwnd);
			_FREEOBJ(dlg);
			SetFocus(hwnd);
			break;
		
		case IDIMPORT:
			if (!ProtocolDlg->GetPasswort(hwnd))
				break;
			SetDlgItemText(hwnd, id_PrnStatusLine, ProtocolLanguage->GetMessage(46));
			if (Protocol->ImportProtocolFile())
				SetDlgItemText(hwnd, id_PrnStatusLine,
							   ProtocolLanguage->GetMessage(47));
			else
				SetDlgItemText(hwnd, id_PrnStatusLine,
							   ProtocolLanguage->GetMessage(48));
			break;
		
		case id_Delete:
			if (IsDlgButtonChecked(hwnd, id_Delete))
			{
				bDelete= TRUE;
				SetDlgItemText(hwnd, IDDELETE, ProtocolLanguage->GetMessage(49));
			}
			else
			{
				bDelete= FALSE;
				SetDlgItemText(hwnd, IDDELETE, ProtocolLanguage->GetMessage(50));
			}
			SearchNew();
			break;
		
		case IDDELETE:
			bIsDelete= FALSE;
			if ( (Count= ListView_GetItemCount(hSuchList)) > 0 )
			{
				if ( (SelCount= ListView_GetSelectedCount(hSuchList)) > 0 )
				{
					if (!ProtocolDlg->GetPasswort(hwnd))
						break;
					SetDlgItemText(hwnd, id_PrnStatusLine,
								   ProtocolLanguage->GetMessage(46));
					strcpy(tmpLfdNr, Protocol->GetLfdNr(buf));
					Count=-1;
					for (i= 0; i < SelCount; i++)
					{
                        Count=ListView_GetNextItem(hSuchList,Count,LVIS_SELECTED);
						GetSelLfdNr(lLfdNr, Count);
						if (strcmp(tmpLfdNr, lLfdNr) == 0)
							bIsDelete= TRUE;
					}
					if (bIsDelete)
						if (ProtocolDlg->TestChangeDlg())
							break;
					Count=-1;
					for (i= 0; i < SelCount; i++)
					{
                        Count=ListView_GetNextItem(hSuchList,Count,LVIS_SELECTED);
						GetSelLfdNr(lLfdNr, Count);
						if ( Protocol->LoadProtocolString(lLfdNr) == 0 )
							if (!bDelete)
								Protocol->SetProtocolDelete();
							else
								Protocol->SetProtocolReadOnly();
					}
					ProtocolDlg->ShowLfdNr(tmpLfdNr);
					strcpy(buf, itoa(SelCount, buf2, 10));
					if (!bDelete)
					{
						if (SelCount != 1)
							strcat(buf, ProtocolLanguage->GetMessage(51));
						else
							strcat(buf, ProtocolLanguage->GetMessage(52));
					}
					else
					{
						if (SelCount != 1)
							strcat(buf, ProtocolLanguage->GetMessage(53));
						else
							strcat(buf, ProtocolLanguage->GetMessage(54));
					}
					SetDlgItemText(hwnd, id_PrnStatusLine, buf);
					SearchNew();
				}
				else
				{
					ListBox_GetText(hSuchList, 0, buf);
					SetDlgItemText(hwnd, id_PrnStatusLine,
								   ProtocolLanguage->GetMessage(34));
					MessageBox(GetFocus(), ProtocolLanguage->GetMessage(34),
							   ProtocolLanguage->GetWarning(), MBSTOP);
				}
			}
			break;
		
		default:
			bTest= FALSE;
	}
	return bTest;
};

void TProtocolManagePrintDlg::AbortPrintSelection(HWND hwnd)
{
	EnableWindow(hwnd, TRUE);
	Protocol->ReloadProtocolString(tmpLfdNr);
};

BOOL TProtocolManagePrintDlg::PrintSelection(HWND hwnd, int SelCount, int IniSpalten)
{
	char buf[MaxLine], PrintLfdNr[3][MaxString];
	int i, k, Seite, Seiten, Count;

	strcpy(tmpLfdNr, Protocol->GetLfdNr(buf));
	Seite= 0;
	if ((Spalten= IniSpalten) > 4)
		Spalten= 4;
	Seiten= SelCount / (Spalten - 1);
	if (SelCount % (Spalten - 1) > 0)
		Seiten++;
	EnableWindow(hwnd, FALSE);
	if (!InitializePrint(hwnd))
	{
		AbortPrintSelection(hwnd);
		return FALSE;
	}
	for (k= 0; k < 3; k++)
		strcpy(PrintLfdNr[k], "");
	Count=-1;
	for (i= 0; i < SelCount; i++)
	{
        Count=ListView_GetNextItem(hSuchList,Count,LVIS_SELECTED);
		GetSelLfdNr(PrintLfdNr[i % (Spalten - 1)], Count);
		if ( i % (Spalten - 1) == (Spalten - 2) )
		{
			sprintf(buf, "%s%d%s%d%s", ProtocolLanguage->GetMessage(55), ++Seite,
					ProtocolLanguage->GetMessage(56), Seiten,
					ProtocolLanguage->GetMessage(57));
			SetDlgItemText(hwnd, id_PrnStatusLine, buf);
			if (!Print(PrintLfdNr[0], PrintLfdNr[1], PrintLfdNr[2]))
			{
				AbortPrintSelection(hwnd);
				return FALSE;
			}
			for (k= 0; k < 3; k++)
				strcpy(PrintLfdNr[k], "");
		}
	}
	if (SelCount % (Spalten - 1) > 0)
	{
		sprintf(buf, "%s%d%s%d%s", ProtocolLanguage->GetMessage(55), ++Seite,
				ProtocolLanguage->GetMessage(56), Seiten,
				ProtocolLanguage->GetMessage(57));
		SetDlgItemText(hwnd, id_PrnStatusLine, buf);
	}
	if (!EndPrint(PrintLfdNr[0], PrintLfdNr[1], PrintLfdNr[2]))
	{
		AbortPrintSelection(hwnd);
		return FALSE;
	}
	AbortPrintSelection(hwnd);
	return TRUE;
};

void TProtocolManagePrintDlg::PrintError(LPCSTR Message)
{
	AbortDoc(pd.hDC);
	DeleteDC(pd.hDC);
	MessageBox(GetFocus(), Message, ProtocolLanguage->GetFailure(), MBFAILURE);
};

BOOL TProtocolManagePrintDlg::InitializePrint(HWND hwnd)
{
	SIZE size;

	// Initialize the PRINTDLG members.
	memset( &pd, '\0', sizeof(PRINTDLG) );
	pd.lStructSize= sizeof(PRINTDLG);
	pd.hDevMode= (HANDLE) NULL;
	pd.hDevNames= (HANDLE) NULL;
	pd.Flags= PD_RETURNDC;
	pd.hwndOwner= hwnd;
	pd.hDC= (HDC) NULL;
	pd.nFromPage= 1;
	pd.nToPage= 1;
	pd.nMinPage= 0;
	pd.nMaxPage= 0;
	pd.nCopies= 1;
	pd.hInstance= (HINSTANCE) NULL;
	pd.lCustData= 0L;
	pd.lpfnPrintHook= NULL;
	pd.lpfnSetupHook= NULL;
	pd.lpPrintTemplateName= (LPSTR) NULL;
	pd.lpSetupTemplateName= (LPSTR) NULL;
	pd.hPrintTemplate= (HANDLE) NULL;
	pd.hSetupTemplate= (HANDLE) NULL;

	// Display the PRINT dialog box.
	if ( !PrintDlg(&pd) )
		return FALSE;

	if (!(GetDeviceCaps(pd.hDC, RASTERCAPS) & RC_BITBLT))
	{
		PrintError(ProtocolLanguage->GetMessage(35));
		return FALSE;
	}

	// Initialize the members of a DOCINFO structure.
	memset( &di, '\0', sizeof(DOCINFO) );
	di.cbSize= sizeof(DOCINFO);
	di.lpszDocName= "Protokollbuch";
	di.lpszOutput= NULL;

	Rect.top= Rect.left= 0;
	Rect.right= PrintX;
	Rect.bottom= PrintY;
	xmax= GetDeviceCaps(pd.hDC, HORZRES);
	ymax= GetDeviceCaps(pd.hDC, VERTRES);
	if ( xmax <= 0 || ymax <= 0 )
	{
		PrintError(ProtocolLanguage->GetMessage(36));
		return FALSE;
	}
	xmax--;
	ymax--;
	zoom= (xmax / 4) / Rect.right;
	if (zoom == 0)
		zoom= 1;
	Rect.right= Rect.right * zoom;
	Rect.bottom= Rect.bottom * zoom;

	// Begin a print job by calling the StartDoc  function.
	if ( StartDoc(pd.hDC, &di) == SP_ERROR )
	{
		PrintError(ProtocolLanguage->GetMessage(37));
		return FALSE;
	}

	GetTextExtentPoint(pd.hDC, "---- ", 5, &size);
	xRand= size.cx;
	xmax -= xRand;

	return TRUE;
};

BOOL TProtocolManagePrintDlg::PrintNewPage( int &dy, int n)
{
	if ( EndPage(pd.hDC) <= 0)
	{
		PrintError(ProtocolLanguage->GetMessage(38));
		return FALSE;
	}
	if ( StartPage(pd.hDC) <= 0)
	{
		PrintError(ProtocolLanguage->GetMessage(39));
		return FALSE;
	}

	TextOut(pd.hDC, 0, ymax / 2, "---- ", 5);

	MoveToEx(pd.hDC, xRand, 0, NULL);
	LineTo(pd.hDC, xRand, ymax);
	LineTo(pd.hDC, xmax, ymax);
	LineTo(pd.hDC, xmax, 0);
	LineTo(pd.hDC, xRand, 0);

	MoveToEx(pd.hDC, xRand - 1, 1, NULL);
	LineTo(pd.hDC, xRand - 1, ymax);
	LineTo(pd.hDC, xmax - 1, ymax - 1);
	LineTo(pd.hDC, xmax - 1, 1);
	LineTo(pd.hDC, xRand - 1, 1);

	for (int k= 1; k <= n; k++)
	{
		MoveToEx(pd.hDC, ((xmax - xRand) / Spalten)*k + xRand - 1, 0, NULL);
		LineTo(pd.hDC, ((xmax - xRand) / Spalten)*k + xRand - 1, ymax);
		MoveToEx(pd.hDC, ((xmax - xRand) / Spalten)*k + xRand - 2, 0, NULL);
		LineTo(pd.hDC, ((xmax - xRand) / Spalten)*k + xRand - 2, ymax);
	}

	dy= 0;

	return TRUE;
};

BOOL TProtocolManagePrintDlg::EndPrint(LPCSTR lLfdNr1, LPCSTR lLfdNr2, LPCSTR lLfdNr3)
{
	if (!Print(lLfdNr1, lLfdNr2, lLfdNr3))
		return FALSE;

	if ( EndDoc(pd.hDC) <= 0)
	{
		PrintError(ProtocolLanguage->GetMessage(40));
		return FALSE;
	}

	// Delete the printer DC
	DeleteDC(pd.hDC);

	return TRUE;
};

BOOL TProtocolManagePrintDlg::Print(LPCSTR lLfdNr1, LPCSTR lLfdNr2, LPCSTR lLfdNr3)
{
	char Parameter[MaxParameterString], ParameterValue[MaxParameterString];
	char c, t[MaxString], kLfdNr[3][MaxLfdNr];
	int i, k, l, m, x, y, dx, dy, tmpdy, dymax, n;
	SIZE size;

	n= 0;
	if (strlen(lLfdNr1) != 0)
	{
		strcpy(kLfdNr[n++], lLfdNr1);
		if (strlen(lLfdNr2) != 0)
		{
			strcpy(kLfdNr[n++], lLfdNr2);
			if (strlen(lLfdNr3) != 0)
				strcpy(kLfdNr[n++], lLfdNr3);
		}
	}
	else
		return TRUE;

	if (Spalten < (n + 1))
		Spalten= n + 1;

	if ( StartPage(pd.hDC) <= 0)
	{
		PrintError(ProtocolLanguage->GetMessage(39));
		return FALSE;
	}

	TextOut(pd.hDC, 0, ymax / 2, "---- ", 5);

	MoveToEx(pd.hDC, xRand, 0, NULL);
	LineTo(pd.hDC, xRand, ymax);
	LineTo(pd.hDC, xmax, ymax);
	LineTo(pd.hDC, xmax, 0);
	LineTo(pd.hDC, xRand, 0);

	MoveToEx(pd.hDC, xRand - 1, 1, NULL);
	LineTo(pd.hDC, xRand - 1, ymax);
	LineTo(pd.hDC, xmax - 1, ymax - 1);
	LineTo(pd.hDC, xmax - 1, 1);
	LineTo(pd.hDC, xRand - 1, 1);

	for (k= 1; k <= n; k++)
	{
		MoveToEx(pd.hDC, ((xmax - xRand) / Spalten)*k + xRand - 1, 0, NULL);
		LineTo(pd.hDC, ((xmax - xRand) / Spalten)*k + xRand - 1, ymax);
		MoveToEx(pd.hDC, ((xmax - xRand) / Spalten)*k + xRand - 2, 0, NULL);
		LineTo(pd.hDC, ((xmax - xRand) / Spalten)*k + xRand - 2, ymax);
	}

	dy= 0;
	for (i= 0; i < Felder - 1; i++)
	{

		if (PrintSkizze(i))
		{
			strcpy(Parameter, "Lageskizze:");
			TextOut(pd.hDC, xRand, dy, Parameter, strlen(Parameter));

			for (k= 0; k < n; k++)
			{
				Protocol->LoadPicFile(kLfdNr[k]);
				MoveToEx(pd.hDC, Rect.left + ((xmax - xRand) / Spalten)*(k + 1) + xRand, Rect.top + dy, NULL);
				LineTo(pd.hDC, Rect.right + ((xmax - xRand) / Spalten)*(k + 1) + xRand, Rect.top + dy);
				LineTo(pd.hDC, Rect.right + ((xmax - xRand) / Spalten)*(k + 1) + xRand, Rect.bottom + dy);
				LineTo(pd.hDC, Rect.left + ((xmax - xRand) / Spalten)*(k + 1) + xRand, Rect.bottom + dy);
				LineTo(pd.hDC, Rect.left + ((xmax - xRand) / Spalten)*(k + 1) + xRand, Rect.top + dy);
				Protocol->ResetXY();
				while (Protocol->GetXY( c, x, y, t ))
				{
					switch (c)
					{
						case 'M':
							MoveToEx(pd.hDC, x*zoom + ((xmax - xRand) / Spalten)*(k + 1) + xRand, y*zoom + dy, NULL);
							break;
						
						case 'L':
							LineTo(pd.hDC, x*zoom + ((xmax - xRand) / Spalten)*(k + 1) + xRand, y*zoom + dy);
							break;

						case 'T':						
							TextOut(pd.hDC, x*zoom + ((xmax - xRand) / Spalten)*(k + 1) + xRand, y*zoom + dy, t, strlen(t));
							break;

						default:
							MessageBox(GetFocus(), ProtocolLanguage->GetMessage(23),
									   ProtocolLanguage->GetFailure(), MBFAILURE);
					}
				}
			}
			dy += Rect.bottom;
			if (dy >= ymax)
				if (!PrintNewPage(dy, n))
					return FALSE;

			dy++;
			MoveToEx(pd.hDC, xRand, dy, NULL);
			LineTo(pd.hDC, xmax, dy);
			dy += 4;
		}

		if (dy >= ymax)
			if (!PrintNewPage(dy, n))
				return FALSE;
		tmpdy= dy;
		dymax= 0;
		for (k= -1; k < n; k++)
		{

			if (k == -1)
				GetParameterName(i, Parameter);
			else if (!GetParameter(i, Parameter, kLfdNr[k]))
			{
				PrintError(ProtocolLanguage->GetMessage(41));
				return FALSE;
			}

			dx= ((xmax - xRand) / Spalten) * (k + 1) + xRand;
			GetTextExtentPoint(pd.hDC, Parameter, strlen(Parameter), &size);
			while ( ((xmax - xRand) / Spalten) < size.cx )
			{
				strcpy(ParameterValue, Parameter);
				strcpy(Parameter, "");
				l= strlen(ParameterValue) - 1;
				m= 0;
				while ( (((xmax - xRand) / Spalten) < size.cx) && (l > 0) )
				{
					Parameter[m++]= ParameterValue[l];
					Parameter[m]= ParameterValue[l--]= '\0';
					GetTextExtentPoint(pd.hDC, ParameterValue, strlen(ParameterValue), &size);
					if ( ( (strrchr(ParameterValue, ' ') != NULL) ||
							(strrchr(ParameterValue, '-') != NULL) ) &&
							(l > 0) && (Parameter[m - 1] != ' ') && (Parameter[m - 1] != '-') )
					{
						while ( (l > 0) && (ParameterValue[l] != ' ') &&
								(ParameterValue[l] != '-') )
						{
							Parameter[m++]= ParameterValue[l];
							Parameter[m]= ParameterValue[l--]= '\0';
						}
						GetTextExtentPoint(pd.hDC, ParameterValue, strlen(ParameterValue), &size);
					}
				}
				strrev(Parameter);
				TextOut(pd.hDC, dx, tmpdy, ParameterValue, strlen(ParameterValue));
				tmpdy += size.cy;
				GetTextExtentPoint(pd.hDC, Parameter, strlen(Parameter), &size);
			}
			TextOut(pd.hDC, dx, tmpdy, Parameter, strlen(Parameter));
			tmpdy += size.cy;
			if (dymax < tmpdy)
				dymax= tmpdy;
			tmpdy= dy;
		}
		dy= dymax;
		if (dy >= ymax)
			if (!PrintNewPage(dy, n))
				return FALSE;

		if ( i < (Felder - 2) )
		{
			MoveToEx(pd.hDC, xRand, dy, NULL);
			LineTo(pd.hDC, xmax, dy);
			dy += 2;
		}
		if ( PrintLine(i) )
		{
			MoveToEx(pd.hDC, xRand, dy + 1, NULL);
			LineTo(pd.hDC, xmax, dy + 1);
			dy++;
		}
		dy += 2;

		if (dy >= ymax)
			if (!PrintNewPage(dy, n))
				return FALSE;
	}
	if ( EndPage(pd.hDC) <= 0)
	{
		PrintError(ProtocolLanguage->GetMessage(38));
		return FALSE;
	}
	return TRUE;
};

// LastLine


