//******************************************************************************
//Datei     : TProtocolParameterDlg.cpp
//Projekt   : XCTL
//Subsystem : Protokollbuch
//Autor     : Jens Klier <klier@informatik.hu-berlin.de>
//            Institut fuer Informatik,
//            Humboldt-Universitaet Berlin
//Inhalt    : Implementation der Klasse TProtocolParameterDlg
//******************************************************************************
//!neu klier Protokollbuch
#include "internls\evrythng.h"
#include "winresrc\rc_prtow.h" // Ressourcen-IDs
#include <commdlg.h>

#include "protocol\TProtocolLanguage.h"
#include "protocol\TProtocol.h"
#include "protocol\TProtocolDlg.h"
#include "protocol\TProtocolManagePrintDlg.h"
#include "protocol\TProtocolParameterDlg.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

extern TProtocolLanguage *ProtocolLanguage;
extern HINSTANCE hModuleInstance;

TProtocolParameterDlg::TProtocolParameterDlg(LPSTR Dlg, TProtocol *Proto, TProtocolDlg *ProtoDlg, TProtocolManagePrintDlg *ProtoPrintDlg) : \
		TModalDlg(Dlg,hModuleInstance), Protocol(Proto), ProtocolDlg(ProtoDlg), ProtocolManagePrintDlg(ProtoPrintDlg)
{}
;


BOOL TProtocolParameterDlg::Dlg_OnInit(HWND hwnd, HWND, LPARAM)
{
	if ( GetPrivateProfileInt("Protocol", "Debug", 0, GetCFile()) == 1)
		bDebug= TRUE;
	else
		bDebug= FALSE;

	bSortDate= Protocol->IsProtocolSortDate();
	bNoSortNew= Protocol->IsProtocolNoSortNew();
	strcpy(szProtocolFile, Protocol->GetPFile());
	strcpy(szProtocolBackupDirectory, Protocol->GetPBackupDirectory());

	SetDlgItemText(hwnd, id_ProtoFile, szProtocolFile);
	SetDlgItemText(hwnd, id_ProtoBackupPath, szProtocolBackupDirectory);

	if (bSortDate)
		CheckDlgButton(hwnd, id_SortDate, 1);
	else
		CheckDlgButton(hwnd, id_SortLfdNr, 1);
	if (bNoSortNew)
		CheckDlgButton(hwnd, id_NoSortNew, 1);

	ProtocolLanguage->SetParameterDlgLanguage(hwnd);
	return TRUE;
};

void TProtocolParameterDlg::Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	OPENFILENAME	ofn;
	char buf[_MAX_PATH], lLfdNr[MaxLfdNr];
	BOOL bViewNew= FALSE;

	switch (id)
	{
		case id_SortLfdNr:
			bSortDate= FALSE;
			break;

		case id_SortDate:
			bSortDate= TRUE;
			break;
		
		case id_NoSortNew:
			if (IsDlgButtonChecked(hwnd, id_NoSortNew))
				bNoSortNew= TRUE;
			else
				bNoSortNew= FALSE;
			break;
		
		case id_Durchsuchen:
			sprintf(buf, "%s%s", Protocol->GetPName(), ".CSV");
			// Initialize the structure
			memset( &ofn, '\0', sizeof( OPENFILENAME ) );
			ofn.lpstrTitle= ProtocolLanguage->GetMessage(17);
			ofn.lStructSize= sizeof( OPENFILENAME );
			ofn.hwndOwner= GetFocus();
			ofn.lpstrFilter= "csv-Dateien (*.CSV)\0*.CSV\0\0";
			ofn.nFilterIndex= 1;
			ofn.lpstrFile= buf;
			ofn.nMaxFile= _MAX_PATH;
			ofn.lpstrInitialDir= Protocol->GetPDirectory();
			ofn.Flags= OFN_HIDEREADONLY;
			if ( !GetOpenFileName( &ofn ) )
				break;
			strcpy(szProtocolFile, buf);
			SetDlgItemText(hwnd, id_ProtoFile, szProtocolFile);
			break;
		
		case IDOK:
			if (ProtocolDlg->TestChangeDlg())
				break;
			Protocol->GetLfdNr(lLfdNr);
			Protocol->SetProtocolNoSortNew(bNoSortNew);
			GetDlgItemText(hwnd, id_ProtoBackupPath, szProtocolBackupDirectory, _MAX_PATH);
			if (strcmp(szProtocolBackupDirectory, Protocol->GetPBackupDirectory()) != 0)
			{
				strcpy( buf, Protocol->GetPDirectory());
				if (!Protocol->SetPBackupDirectory(szProtocolBackupDirectory))
					Protocol->SetPBackupDirectory(buf);
				if ( bDebug )
					MessageBox("Backupdirectory gendert!", "Debug Info", MBINFO);
			}
			GetDlgItemText(hwnd, id_ProtoFile, szProtocolFile, _MAX_PATH);
			if (strcmp(szProtocolFile, Protocol->GetPFile()) != 0)
			{
				strcpy( buf, Protocol->GetPFile());
				if (!Protocol->SetPFile(szProtocolFile))
				{
					Protocol->SetPFile(buf);
				}
				else
				{
					if (!Protocol->ReloadProtocol())
					{
						Protocol->SetPFile(buf);
						while (!Protocol->ReloadProtocol())
							/* wait for loading the protocol*/;
					}
					bViewNew= TRUE;
				}
				if ( bDebug )
					MessageBox("Protokolldatei neu geladen!", "Debug Info", MBINFO);
			}
			if (Protocol->IsProtocolSortDate() != bSortDate)
			{
				Protocol->SetProtocolSortDate(bSortDate);
				bViewNew= TRUE;
			}
			if (bViewNew)
			{
				ProtocolDlg->ShowLfdNr(lLfdNr);
				ProtocolManagePrintDlg->SearchNew();
			}
			// Kullmann+Reinecker: Hier muss kein break hin, weiter bei TModalDlg::Dlg_OnCommand!
		
		default:
			TModalDlg::Dlg_OnCommand(hwnd, id, hwndCtl, codeNotify);
	}
};

// LastLine


