//##############################################################################
//                                                                            //
// TP_GUI.CPP                                                                 //
//                                                                            //
// Subsystem : Topographie / Oberflche                                       //
// Benutzung durch andere Subsysteme erforderlich: NEIN                       //
//----------------------------------------------------------------------------//
// Autoren:  Thomas Kullmann, Gnther Reinecker (2002)                        //
//                                                                            //
// Stand  : 20.01.2003                                                        //
//                                                                            //
//##############################################################################

#include "internls\evrythng.h" // GermanVersion
#include "winresrc\rc_def.h"   // Ressourcen-IDs

#include "topogrfy\tp_gui.h" // SCHNITTSTELLE fr diese Datei

#include "mespara\mespara.h"   //! neu: klier Allgemeine Einstellungen
#include "protocol\protocol.h" //! neu: klier Protokollbuch

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//#############################################################################
// TTopographyExecDlg
//#############################################################################

//-----------------------------------------------------------------------------
// Konstruktor

TTopographyExecDlg::TTopographyExecDlg() : TModalDlg( "TOPOGRAPHYEXECDLG", GetMainInstance() ), m_lnkTopography( 0 ), m_lnkTimerMeasuring( 0 ), m_lnkTimerMultiExpose( 0 )
{
	m_lnkTopography= TTopography::GetInstance(); // einzige globale Instanz der Klasse //POLLING, d.h. KEINE Referenz auf uns bergeben
	m_lnkTimerMeasuring= new TInterfaceTimer( this, 2000 ); //Oberserver, d.h. Timer benachrichtigt uns
	m_lnkTimerMultiExpose= new TInterfaceTimer( this, 500 ); //Oberserver, d.h. Timer benachrichtigt uns
	// falls Detektor noch nicht gesetzt, auf aktuellen Detektor setzen
	m_lnkTopography->DoInitDetectors();
	// Motor (wenn mglich) auf Omega, sonst auf aktuellen Motor setzen
	// aktuelle Settings im Motor speichern, um diese beim Verlassen wiederherzustellen
	// FALSE -> Omega ist nicht verfgbar
	if ( !m_lnkTopography->DoInitMotor() )
		MessageBox(sth_tp_NoOmega, sth_tp_MsgBoxError, MBSTOP);

	m_lnkTopography->SetCurrentTime( 0 );
	m_lnkTopography->SetStartTime( 0 );
	m_lnkTopography->SetRestShots( m_lnkTopography->GetNumberCycle() );
	m_lnkTopography->DoInitWorkPoint(); // Parameter des Makro initialisieren
	m_lnkTopography->SetStartPointOk( FALSE );
};

//-----------------------------------------------------------------------------
// Initialisierung zur Darstellung des 'Topographie Durchfuehrung' Fensters

BOOL TTopographyExecDlg::Dlg_OnInit(HWND, HWND, LPARAM)
{
	char buf[MaxString];
	if ( !m_lnkTopography->GetDetector()->GetCounterWnd() )
		(new TCounterWindow(GetMainInstance(), m_lnkTopography->GetDetector()))->Show(); 

	// Initialisieren der Parameter, Messzeit, usw...
	// Beschriftung und Werte im Fenster
	GetTimeString(buf, m_lnkTopography->GetCurTime());
	CtrlSetText( id_MeasurementTime, buf);
	CtrlSetFloat( id_Angle, 0, m_lnkTopography->GetMotorDigits(eDF), _DECIMAL );
	CtrlSetLong( id_RemainderCyclus, m_lnkTopography->GetNumberCycle());
	if ( m_lnkTopography->IsMultipleShot() )
		CtrlSetText( cm_SwitchControl, sth_tp_StartMultiExposure );
	else
		CtrlSetText( cm_SwitchControl, sth_tp_StartSingleExposure );
	Dlg_OnInitialize();
	Dlg_OnParamSet();
	return TRUE;
};
//-----------------------------------------------------------------------------
// Zeitueberpruefung fuer die Mehrfachbelichtung und Beepton nach Messzeitende

void TTopographyExecDlg::OnTimer( TBasicTimer *const aTimer )
{
	// Gesamtmesszeit war abgelaufen; Ausgabe eines regelmaessigen Beeptons
	if (aTimer == m_lnkTimerMeasuring)
	{
		MessageBeep(0);
		DelayTime(100);
		MessageBeep(0);
		return;
	};
	// Zeitueberpruefung waehrend der Mehrfachbelichtung
	if (aTimer == m_lnkTimerMultiExpose)
	{
		Dlg_OnParamSet();
		Dlg_OnMultipleExpose();
		return;
	};
};
//-----------------------------------------------------------------------------
// Timeout fr die Mehrfachbelichtung

void TTopographyExecDlg::Dlg_OnMultipleExpose( void )
{
	char buf[MaxString];

	if ( (m_lnkTopography->GetCurTime() / 1000) < (DWORD)m_lnkTopography->GetMeasurementTime() )
		return; // ob noch innerhalb der Messzeit

	m_lnkTimerMultiExpose->StopTimer();
	m_lnkTopography->SetRestShots( m_lnkTopography->GetRestShots() - 1 ); // Messzyklen verringern
	// falls noch Messzyklen vorhanden
	if ( m_lnkTopography->GetRestShots() )
	{
		CtrlSetLong( id_RemainderCyclus, m_lnkTopography->GetRestShots());
		// Weiterfuehrung der Probe, der Timer wird durch den case Zweig cm_SteeringReady
		// in Dlg_OnCommand (nach Ende der Steering Aktion) neu gesetzt
		sprintf(buf, m_lnkTopography->GetDigitFormat(eSF), m_lnkTopography->GetAngleBetweenShots() );
		m_lnkTopography->DoStartMeasure(MoveToPoint, Relative, 0, buf, GetHandle() );
		return;
	};
	// alle Messzyklen beendet
	CtrlSetLong( id_RemainderCyclus, m_lnkTopography->GetRestShots());
	m_lnkTopography->SetExceptionOccured( TRUE );
	m_lnkTopography->SetAdditionalTime( TRUE ); // Zusatzzeit; Hinweisbox anzeigen

	m_lnkTimerMeasuring->StartTimer(); // Beep Timer
	MessageBox(sth_tp_MeasureTimeEnd, sth_tp_MsgBoxInfo, MBINFO);
	PostMessage( GetHandle(), WM_COMMAND, cm_SwitchControl, 0 );
	m_lnkTimerMeasuring->StopTimer();
};
//-----------------------------------------------------------------------------
// Behandlung der Nachrichten cm_SwitchControl

void TTopographyExecDlg::Dlg_OnSwitchControl( void )
{
	char buf[2*MaxString];

	// Ende der Messzeit erreicht, Benutzer hat Hinweisfenster quittiert -> Nachregelmechanismus anhalten
	if ( m_lnkTopography->IsControlActive() )
	{
		if ( m_lnkTopography->IsMultipleShot() )
			m_lnkTimerMultiExpose->StopTimer();
		else
			m_lnkTopography->DoStopMeasure();
		// Herstellen des Ausgangszustandes
		m_lnkTopography->SetTimeFinish( FALSE );
		m_lnkTopography->SetTimeRunning( FALSE );
		m_lnkTopography->SetAdditionalTime( FALSE );
		m_lnkTopography->SetStartTime( 0 );
		m_lnkTopography->SetCurrentTime( 0 );
		// Buttonbeschriftung von stoppen auf starten aendern
		if ( m_lnkTopography->IsMultipleShot() )
			CtrlSetText( cm_SwitchControl, sth_tp_StartMultiExposure);
		else
			CtrlSetText( cm_SwitchControl, sth_tp_StartSingleExposure);

		m_lnkTopography->SetControlActive( !m_lnkTopography->IsControlActive() );
		Dlg_OnParamSet();
		m_lnkTopography->SetControlActive( FALSE );

		// Klier Protokollbuch  gendert 01.03.2003
		if ( IsProtocolTopographyOn() )
		{
			if ( !m_lnkTopography->IsMultipleShot() )
				SetProtocolTopographyImpsnach( m_lnkTopography->GetStrngIntensity() );
			SetProtocolTopographyDFvor( m_lnkTopography->GetStrngStartPoint() );
			SetProtocolTopographyDFnach( m_lnkTopography->GetAngle() );
			SaveProtocolTopographyParameter();
			ViewOnProtocolTopographyDlg();
			if ( IsProtocolTopographyOn() )
				SetProtocolTopographyReadOnly();
			SetProtocolTopographyOff();
		}
		CtrlSetEnabled(cm_GotoWorkPoint, TRUE); // aktiviert Button 'Startpositon einstellen';
		return;
	};
	// Messung gestartet; Anzeigefelder und Buttonbeschriftungen anpassen,
	// Zeitzaehlung beginnen, bei Mehrfachbelichtung Timer starten, bei
	// Einfachbelichtung Steering fuer Nachregelung starten

	// Klier Protokollbuch  gendert 01.03.2003
	if ( MessageBox(sth_tp_SaveAtProtocol, sth_tp_MsgBoxProtocol, MBASK) == IDYES)
	{
		SetProtocolTopographyOn();
		SetProtocolTopographyImpMax( m_lnkTopography->GetStrngPeakIntensity() );
		int nOldTopographyMotor= m_lnkTopography->GetMotor();
		m_lnkTopography->SetMotor( Collimator );

		SetProtocolTopographyKKruemmung( m_lnkTopography->GetAngle() );
		m_lnkTopography->SetMotor( nOldTopographyMotor );
		SetProtocolTopographyHWB( m_lnkTopography->GetStrngHwb() );
		SetProtocolTopographyEXPT( m_lnkTopography->GetMeasurementTime() );
		SetProtocolTopographyINCDF( m_lnkTopography->GetControlStep() );
		SetProtocolTopographyNBEXPOS( m_lnkTopography->GetNumberCycle() );
		if ( m_lnkTopography->IsMultipleShot() )
		{
			SetProtocolTopographyRSA( m_lnkTopography->GetStartAngle() );
			SetProtocolTopographyABE( m_lnkTopography->GetAngleBetweenShots() );
		}
		else
		{
			SetProtocolTopographyCRT( m_lnkTopography->GetMaxTime() );
			SetProtocolTopographySTE();
			SetProtocolTopographyCONLIMIT();
			if ( m_lnkTopography->IsSmallAngleSide() )
				SetProtocolTopographyCONPOINT( -(m_lnkTopography->GetWorkPoint()) );
			else
				SetProtocolTopographyCONPOINT( m_lnkTopography->GetWorkPoint() );
		}
		ViewOnProtocolTopographyDlg();
	}
	else
		SetProtocolTopographyOff();

	m_lnkTopography->SetExceptionOccured( FALSE );
	m_lnkTopography->SetCurrentTime( 0 );
	GetTimeString(buf, m_lnkTopography->GetCurTime());
	CtrlSetText( id_RemainderTime, buf);

	// Millisekunden seit Systemstart
	m_lnkTopography->SetStartTime( GetTickCount() );
	m_lnkTopography->SetCurrentTime( GetTickCount() );
	m_lnkTopography->SetTimeRunning( TRUE );
	CtrlSetText( id_Text, "");

	// bei Mehrfachbelichtung
	if ( m_lnkTopography->IsMultipleShot() )
	{
		// Startposition wurde anfahren
		m_lnkTopography->SetRestShots( m_lnkTopography->GetNumberCycle() );
		m_lnkTimerMultiExpose->StartTimer();
		CtrlSetText( cm_SwitchControl, sth_tp_StopMultiExposure);
	}
	else
	{
		m_lnkTopography->SetAngleWidth( m_lnkTopography->GetControlStep() ); // Motorschrittweite setzen
		// Millisekunden seit Systemstart
		m_lnkTopography->SetStartTime( GetTickCount() );
		m_lnkTopography->SetCurrentTime( GetTickCount() );
		sprintf(buf, m_lnkTopography->GetDigitFormat( eDF ), m_lnkTopography->GetControlRange() ); // Parameter fr Nachregelung laden
		// ausfuehren des Kommandos ControlFlank
		if ( m_lnkTopography->IsSmallAngleSide() )
			m_lnkTopography->DoStartMeasure(ControlFlank, SmallSide, 0, buf, GetHandle() );
		else
			m_lnkTopography->DoStartMeasure(ControlFlank, LargeSide, 0, buf, GetHandle() );
		CtrlSetText( cm_SwitchControl, sth_tp_StopSingleExposure);
	};
	m_lnkTopography->SetControlActive( TRUE );
	CtrlSetEnabled(cm_GotoWorkPoint, FALSE);
};
//-----------------------------------------------------------------------------
// Behandlung der Nachricht cm_Initialize

void TTopographyExecDlg::Dlg_OnInitialize( void )
{
	char buf[2*MaxString];

	// Detector mit Zeitintervall zum Zaehlen, maximaler Impulsrate und Messfehlerwert (0.02) initialisieren
	m_lnkTopography->DoSetDetectorParams( m_lnkTopography->GetMaxTime(), m_lnkTopography->GetMaxCounts(), FALSE );
	// Eintragen von Werten im Fenster 'Topographie'
	CtrlSetFloat( id_ExposureTime, m_lnkTopography->GetMaxTime(), 1, _DECIMAL);

	m_lnkTopography->SetRestShots( m_lnkTopography->GetNumberCycle() );
	CtrlSetLong( id_RemainderCyclus, m_lnkTopography->GetRestShots());

	m_lnkTopography->SetAngleWidth( m_lnkTopography->GetControlStep() );
	// Warnung, falls Motor oder Counter vor Aufruf von 'Topographie Durchfhrung' nicht gesetzt sind
	if ( !m_lnkTopography->IsSetupOk() )
	{
		char *stSettingsIncomplete1= LoadString(sth_tp_SettingsIncomplete1);
		strcpy(buf, stSettingsIncomplete1 );
		_FREELIST(stSettingsIncomplete1);

		char *stSettingsIncomplete2= LoadString(sth_tp_SettingsIncomplete2);
		strcat(buf, stSettingsIncomplete2 );
		_FREELIST(stSettingsIncomplete2);

		char *stSettingsIncomplete3= LoadString(sth_tp_SettingsIncomplete3);
		strcat(buf, stSettingsIncomplete3 );
		_FREELIST(stSettingsIncomplete3);

		CtrlSetText(id_Text, (LPSTR)buf);
	};
};
//-----------------------------------------------------------------------------
// Behandlung der Nachricht cm_GotoWorkPoint

void TTopographyExecDlg::Dlg_OnGotoWorkPoint( void )
{
	char buf[2*MaxString];

	if ( (m_lnkTopography->IsStartPointOk()) && (m_lnkTopography->IsMultipleShot()) )
	{
		// prevents the multiple relative movement
		MessageBox(sth_tp_StartpointReached, sth_tp_MsgBoxInfo, MBINFO);
		return;
	};
	CtrlSetEnabled(cm_SwitchControl, FALSE); // 'Startposition anfahren' und
	CtrlSetEnabled(cm_GotoWorkPoint, FALSE); // 'Regelung starten' bzw 'Topographie starten' deaktivieren
	CtrlSetText( id_Text, sth_tp_GoToStartpoint);

	m_lnkTopography->SetAngleWidth( m_lnkTopography->GetMoveStep() ); // Motorschrittweite setzen
	// Anfahren des um dStartAngle vom aktuellen Stand entfernten Punktes
	if ( m_lnkTopography->IsMultipleShot() )
	{
		sprintf(buf, m_lnkTopography->GetDigitFormat( eDF ), m_lnkTopography->GetStartAngle());
		m_lnkTopography->DoStartMeasure(MoveToPoint, Relative, 0, buf, GetHandle());
	}
	else
		m_lnkTopography->DoStartMeasure(m_lnkTopography->GetMWorkPoint(), GetHandle()); // Anfahren
};
//-----------------------------------------------------------------------------
// Behandlung der Nachricht cm_SetupPosition

void TTopographyExecDlg::Dlg_OnSetupPosition( void )
{
	char buf[2*MaxString];

	CtrlSetEnabled(cm_SwitchControl, TRUE); // 'Startposition anfahren' und
	CtrlSetEnabled(cm_GotoWorkPoint, TRUE); // 'Regelung starten' bzw 'Topographie starten' aktivieren

	// Ausgabe Hinweistext
	char *stSettingsComplete1= LoadString(sth_tp_SettingsComplete1);
	strcpy(buf, stSettingsComplete1 );
	_FREELIST(stSettingsComplete1);

	char *stSettingsComplete2= LoadString(sth_tp_SettingsComplete2);
	strcat(buf, stSettingsComplete2 );
	_FREELIST(stSettingsComplete2);

	char *stSettingsComplete3= LoadString(sth_tp_SettingsComplete3);
	strcat(buf, stSettingsComplete3 );
	_FREELIST(stSettingsComplete3);

	CtrlSetText( id_Text, (LPSTR)buf);

	m_lnkTopography->SetStartPointOk( TRUE );
	if ( m_lnkTopography->IsMultipleShot() )
	{
		// 19.11.02 Steering.dStartPoint wird auf die aktuelle Motorposition plus Verschiebungsweite gesetzt
		// Warum??? dStartPoint wird im Topographie Zusammenhang eigentlich nicht genutzt
		m_lnkTopography->SetStrngStartPoint( m_lnkTopography->GetAngle() + m_lnkTopography->GetStartAngle());
		Dlg_OnParamSet();
	};
};
//-----------------------------------------------------------------------------
// Behandlung der Nachricht cm_SteeringReady

void TTopographyExecDlg::Dlg_OnSteeringReady( void )
{
	// falls Steering Aktion bezgl. eines Umsetzungsschritts bei der Mehrfachbelichtung
	// --> Timer fuer die naechste Runde neu initialisieren
	if ( (m_lnkTopography->IsControlActive()) && (m_lnkTopography->IsMultipleShot()) )
	{
		m_lnkTopography->SetStartTime( GetTickCount() );
		m_lnkTopography->SetCurrentTime( GetTickCount() );
		m_lnkTopography->SetTimeRunning( TRUE );
		m_lnkTimerMultiExpose->StartTimer();
		return;
	};
	// Steering-Aktion hat sich auf Arbeitspunkt bzw. Startposition anfahren bezogen
	PostMessage(GetHandle(), WM_COMMAND, cm_SetupPosition, 0);
};
//-----------------------------------------------------------------------------
// Behandlung der Nachricht cm_CounterSet

void TTopographyExecDlg::Dlg_OnCounterSet( void )
{
	if (m_lnkTopography->IsMultipleShot())
		return;

	m_lnkTopography->DoSaveMonitorSignal();
	m_lnkTopography->GetDetector()->UpdateViews(TRUE); // Anzeige des Zaehlerfensters aktualisieren
	m_lnkTopography->DetectorRequest();
	// Anzeige der Regel-Gren
	Dlg_OnParamSet();
	PostMessage(GetHandle(), WM_COMMAND, cm_InquireExeption, 0 );
};
//-----------------------------------------------------------------------------
// Neue Eingabe im Feld Messzeit

void TTopographyExecDlg::Dlg_OnExposureTime( void )
{
	BOOL bValid;
	if ( !m_lnkTopography->SetMaxTime( CtrlGetFloat(id_ExposureTime, 1, _DECIMAL, bValid)) )
	{
		MessageBox( sth_tp_ValueMaxTime, sth_tp_MsgBoxError, MBSTOP );
		return;
	}
	m_lnkTopography->DoSetDetectorParams( m_lnkTopography->GetMaxTime(), m_lnkTopography->GetMaxCounts(), FALSE);
	CtrlSetFloat( id_ExposureTime, m_lnkTopography->GetMaxTime(), 1, _DECIMAL); // Ausgabe der neuen Detektormesszeit
};
//-----------------------------------------------------------------------------
// Behandlung bei Nachricht cm_TopographyParam

void TTopographyExecDlg::Dlg_OnTopographyParam( void )
{
	TModalDlg *dlg;

	// 'Topographie Einstellungen' Dialog
	dlg= new TTopographyAdjustDlg(TRUE);
	if ( dlg ) dlg->ExecuteDialog( GetHandle() );
	_FREEOBJ(dlg);

	m_lnkTopography->DoSetDetectorParams( m_lnkTopography->GetMaxTime(), m_lnkTopography->GetMaxCounts(), TRUE);
	m_lnkTopography->SetAngleWidth(m_lnkTopography->GetControlStep()); // neue Motorschrittweite setzen
	CtrlSetFloat( id_ExposureTime, m_lnkTopography->GetMaxTime(), 1, _DECIMAL); // Ausgabe der neuen Detektormesszeit
};
//-----------------------------------------------------------------------------
// Behandlung bei Nachricht cm_ParamSet

void TTopographyExecDlg::Dlg_OnParamSet( void )
{
	char buf[MaxString];

	CtrlSetLong( id_RemainderCyclus, m_lnkTopography->GetRestShots());
	if ( !m_lnkTopography->IsControlActive() )
		return;

	// abgelaufende Zeit berechnen
	if ( m_lnkTopography->IsTimeRunning() )
	{
		// klier Protokollbuch
		if ( (m_lnkTopography->GetCurTime() == m_lnkTopography->GetStartTime()) && (!m_lnkTopography->IsMultipleShot() && IsProtocolTopographyOn()) )
			SetProtocolTopographyImpsvor( m_lnkTopography->GetStrngIntensity() );
		m_lnkTopography->SetCurrentTime( GetTickCount() );
		m_lnkTopography->SetCurrentTime( m_lnkTopography->GetCurTime() - m_lnkTopography->GetStartTime() );
	};
	CtrlSetDouble( id_Angle, m_lnkTopography->GetAngle() - m_lnkTopography->GetStrngStartPoint(), m_lnkTopography->GetMotorDigits(eDF), _DECIMAL); // Drift anzeigen
	// Ausgabe der zustzlichen Mezeit
	GetTimeString(buf, m_lnkTopography->GetCurTime());
	if ( m_lnkTopography->HasAdditionalTime() )
		CtrlSetText( id_RemainderTime, buf);
	else
		CtrlSetText( id_MeasurementTime, buf);
	return;
};
//-----------------------------------------------------------------------------
// Behandlung bei Nachricht cm_InquireException

void TTopographyExecDlg::Dlg_OnInquireException( void )
{
	if ( m_lnkTopography->IsExeptionOccured() )
		return;

	// Intensitaet unter Minimum
	if ( m_lnkTopography->GetStrngIntensity() < (0.01 * m_lnkTopography->GetStrngStartIntensity()) )
	{
		m_lnkTopography->SetExceptionOccured( TRUE );
		m_lnkTimerMeasuring->StartTimer(); // Beep Timer
		MessageBox( sth_tp_ExceptionOnIntensity, sth_tp_MsgBoxInfo, MBINFO );
		m_lnkTimerMeasuring->StopTimer();
	};
	if ( m_lnkTopography->IsMultipleShot() )
		return;

	// Abweichung vom Arbeitspunkt ist zu gross
	if ( m_lnkTopography->GetMaxAngleEscape() < fabs(m_lnkTopography->GetStrngStartPoint() - m_lnkTopography->GetStrngDistance()) )
	{
		m_lnkTopography->SetExceptionOccured( TRUE );
		MessageBox( sth_tp_ExceptionOnWorkPoint, sth_tp_MsgBoxInfo, MBINFO );
		m_lnkTimerMeasuring->StopTimer();
	};
	// Messzeit ist abgelaufen
	if ( (m_lnkTopography->GetCurTime() / 1000) > (DWORD) m_lnkTopography->GetMeasurementTime() )
	{
		m_lnkTopography->SetExceptionOccured( TRUE );
		m_lnkTopography->SetTimeFinish( TRUE );
		m_lnkTopography->SetAdditionalTime( TRUE );
		m_lnkTopography->SetStartTime( GetTickCount() );

		m_lnkTimerMeasuring->StartTimer(); // Beep Timer
		MessageBox( sth_tp_ExceptionOnTime,  sth_tp_MsgBoxInfo, MBINFO );
		m_lnkTimerMeasuring->StopTimer();
		PostMessage(GetHandle(), WM_COMMAND, cm_SwitchControl, 0 );
	};
};
//-----------------------------------------------------------------------------
// Behandlung verschiedener Nachrichten

void TTopographyExecDlg::Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	static HWND hDlgItem= NULL;

	switch ( id )
	{
		// Aenderung des Messzustandes (Messung laeuft <-> keine Messung)
		case cm_SwitchControl:
			Dlg_OnSwitchControl();
			break;

		// Initalisierung
		case cm_Initialize:
			Dlg_OnInitialize();
			break;
		
		// Button "Startposition einstellen" wurde ausgewaehlt
		case cm_GotoWorkPoint:
			Dlg_OnGotoWorkPoint();
			break;
		
		// Arbeitspunkt wurde eingestellt
		case cm_SetupPosition:
			Dlg_OnSetupPosition();
			break;
		
		// Steering ist mit seiner Aktion fertig
		case cm_SteeringReady:
			Dlg_OnSteeringReady();
			break;
		
		// es liegen neue Zaehlerwerte vor
		case cm_CounterSet:
			Dlg_OnCounterSet();
			break;
		
		// Nutzer hat Eingabefeld Detektor-Messzeit angewaehlt
		case id_ExposureTime:
			hDlgItem= Ctrl( id_ExposureTime);
			break;
		
		// Nutzer hat Return gedrueckt
		case IDOK:
			// falls das Return sich nicht auf ein Dialogfeld bezieht, Standardbehandlung durch TModalDlg
			if ( !hDlgItem )
			{
				TModalDlg::Dlg_OnCommand(hwnd, id, hwndCtl, codeNotify);
				break;
			};
			// Der Nutzer hat mit Return eine neue Mezeit eingegeben
			if ( hDlgItem == Ctrl( id_ExposureTime) )
				Dlg_OnExposureTime();
			hDlgItem= NULL;
			CtrlSetFocus(cm_SwitchControl);
			break;
		
		// Button "Einstellungen" wurde betaetigt
		case cm_TopographyParam:
			Dlg_OnTopographyParam();
			break;
		
		// Anzeigen im Topographie Ausfuehren Fenster aktualisieren
		case cm_ParamSet:
			Dlg_OnParamSet();
			break;
		
		// es ist moeglicherweise ein Abbruchfall eingetreten
		case cm_InquireExeption:
			Dlg_OnInquireException();
			break;
		
		case IDCANCEL:
			if ( IsProtocolTopographyOn() )
				SetProtocolTopographyOff(); //! neu: klier Protokollbuch
			// Kullmann+Reinecker: Hier muss kein break hin, weiter bei TModalDlg::Dlg_OnCommand!

		default:
			TModalDlg::Dlg_OnCommand(hwnd, id, hwndCtl, codeNotify);
	};
};
//-----------------------------------------------------------------------------
// Beenden des 'Topographie Durchfhrung' Fenster

void TTopographyExecDlg::LeaveDialog( void )
{
	m_lnkTimerMeasuring->StopTimer(); // Beep Timer
	m_lnkTimerMultiExpose->StopTimer(); // Beep Timer
	m_lnkTopography->DoLoadMotorSettings(); // Settings des aktuellen Motors zurckschreiben
	m_lnkTopography->DoResetDetectorParams(); // Detektor zuruecksetzen
	m_lnkTopography= 0; // Funktionalitt freigeben, ABER diese existiert noch
};

//#############################################################################
// TTopographyAdjustDlg
//#############################################################################

//-----------------------------------------------------------------------------
// Konstruktor
// aCtrlStatus gibt an, ob das Fenster von 'Topographie Durchfhrung' aus gerufen wurde

TTopographyAdjustDlg::TTopographyAdjustDlg(BOOL aCtrlStatus) : TModalDlg( "TOPOGRAPHYADJUSTDLG", GetMainInstance() )
{
	m_lnkTopography= TTopography::GetInstance(); //POLLING, d.h. KEINE Referenz auf uns bergeben
	m_bCtrlStatus= aCtrlStatus;
	// falls Detektor noch nicht gesetzt, auf aktuellen Detektor setzen
	m_lnkTopography->DoInitDetectors();
	// Motor (wenn mglich) auf Omega, sonst auf aktuellen Motor setzen
	// aktuelle Settings im Motor speichern, um diese beim Verlassen wiederherzustellen
	// FALSE -> Omega ist nicht verfgbar
	if ( !m_lnkTopography->DoInitMotor() )
		MessageBox( sth_tp_NoOmega, sth_tp_MsgBoxError, MBSTOP );
};
// ----------------------------------------------------------------------------
// Initalisierung

BOOL TTopographyAdjustDlg::Dlg_OnInit ( HWND, HWND, LPARAM )
{
	if ( m_bCtrlStatus )
	{
		// Wird das Einstellungsfenster von dem Topographie Fenster aus aufgerufen,
		// so ist nur fuer einige Werte die Eingabe moeglich. Alle anderen Eingabefelder
		// werden hier deaktiviert und ausgegraut.
		CtrlSetEnabled(id_ChooseMotor, FALSE);
		// 02.08.2002 Toter Code : EnableWindow(Ctrl(cm_RotateMotor),FALSE);
		CtrlSetEnabled(id_ChooseDetector, FALSE);
		CtrlSetEnabled(id_WorkPoint, FALSE);
		CtrlSetEnabled(id_Shots, FALSE);
		CtrlSetEnabled(id_MaxAngleEscape, FALSE);
		CtrlSetEnabled(id_StepWidth, FALSE);
		CtrlSetEnabled(id_From, FALSE);
		CtrlSetEnabled(id_ControlRange, FALSE);
		CtrlSetEnabled(id_MoveStep, FALSE);
		CtrlSetEnabled(id_MultipleShot, FALSE);
		CtrlSetChecked(id_MultipleShot, m_lnkTopography->IsMultipleShot());
		// 19.11.02 : EnableWindow(Ctrl(id_ControlStep),FALSE);
	}
	else
	{
		CtrlSetChecked(id_MultipleShot, m_lnkTopography->IsMultipleShot());
		CtrlSetEnabled(id_StepWidth, m_lnkTopography->IsMultipleShot());
		CtrlSetEnabled(id_From, m_lnkTopography->IsMultipleShot());
		CtrlSetEnabled(id_Shots, m_lnkTopography->IsMultipleShot());
	};
	// Motoren- /Detektorenauswahlliste aufbauen
	TDetectorManager::DetectorManager().FillDetectorComboBox( Ctrl(id_ChooseDetector) );
	for (int i= 0; i < m_lnkTopography->GetMotorCount(); i++)
		CtrlAddString( id_ChooseMotor, m_lnkTopography->GetMotorName(i) );
	Dlg_OnParamSet();
	return TRUE;
};
// ----------------------------------------------------------------------------
// Behandlung der Nachricht cm_ParamSet

void TTopographyAdjustDlg::Dlg_OnParamSet()
{
	CtrlSetSelString(id_ChooseMotor, m_lnkTopography->GetMotorName(m_lnkTopography->GetActMotor()) );
	CtrlSetSelString(id_ChooseDetector, m_lnkTopography->GetDetector()->GetCharacteristic() );

	// Steuerelement: 'Arbeitspunkt' -> 'Lage'
	float fValue= m_lnkTopography->GetWorkPoint() * 100;
	if ( m_lnkTopography->IsSmallAngleSide() )
		fValue *= -1.0;
	CtrlSetFloat( id_WorkPoint, fValue, 1, _DECIMAL);

	// Steuerelement: 'Arbeitspunkt' -> 'Anfahren mit'
	CtrlSetFloat( id_MoveStep, m_lnkTopography->GetMoveStep(), m_lnkTopography->GetMotorDigits(eSF), _DECIMAL);
	CtrlSetText( id_MoveStepUnit, m_lnkTopography->GetMotorUnit() ); // die Einheit des aktuellen Antriebs

	// Steuerelement: 'Actuator' -> 'Beschrnkung auf'
	CtrlSetFloat( id_MaxAngleEscape, m_lnkTopography->GetMaxAngleEscape(), 1, _DECIMAL);
	CtrlSetText( id_Unit, m_lnkTopography->GetMotorUnit()); // die Einheit des aktuellen Antriebs

	// Steuerelement: 'Belichtungsregelung' -> 'Schritt'
	CtrlSetFloat( id_ControlStep, m_lnkTopography->GetControlStep(), m_lnkTopography->GetMotorDigits(eSF), _DECIMAL);
	CtrlSetText( id_ControlStepUnit, m_lnkTopography->GetMotorUnit() ); // die Einheit des aktuellen Antriebs

	// Steuerelement: 'Belichtungsregelung' -> 'Bereich'
	CtrlSetFloat( id_ControlRange, m_lnkTopography->GetControlRange() * 100, 2, _DECIMAL);

	// Steuerelement: 'Detektor-Parameter' -> 'Zeit'
	CtrlSetFloat( id_MaxTime, m_lnkTopography->GetMaxTime(), 1, _DECIMAL);

	// Steuerelement: 'Mehrfachbelichtung'
	CtrlSetChecked( id_MultipleShot, m_lnkTopography->IsMultipleShot() );

	// Steuerelement: 'Mehrfach-Belichtung' -> 'Schrittweite'
	CtrlSetFloat( id_StepWidth, m_lnkTopography->GetAngleBetweenShots(), m_lnkTopography->GetMotorDigits(eSF), _DECIMAL);

	// Steuerelement: 'Mehrfach-Belichtung' -> 'Startwert'
	CtrlSetDouble( id_From, m_lnkTopography->GetStartAngle(), m_lnkTopography->GetMotorDigits(eDF), _DECIMAL);
	CtrlSetText( id_FromUnit, m_lnkTopography->GetMotorUnit() ); // die Einheit des aktuellen Antriebs

	// Steuerelement: 'Mehrfach-Belichtung' -> 'Anzahl'
	CtrlSetLong( id_Shots, m_lnkTopography->GetNumberCycle());

	// Steuerelement: 'Belichtungszeit'
	CtrlSetLong( id_MeasurementTime, (LONG)(m_lnkTopography->GetMeasurementTime() / 60) );

	// Steuerelement: 'Detektor-Parameter' -> 'Counts'
	CtrlSetLong( id_MaxCounts, m_lnkTopography->GetMaxCounts());

	//CtrlSetFocus( IDCANCEL ); 11.12.2002 Kullmann. Reinecker funktioniert nicht richtig
	// das Ctrl hat zwar den Focus, dieser ist aber nicht sichtbar
};
// ----------------------------------------------------------------------------
// Behandlung der Nachricht id_ChooseMotor

void TTopographyAdjustDlg::Dlg_OnChooseMotor( UINT aCodeNotify )
{
	if ( aCodeNotify != CBN_SELCHANGE )
		return; // nur Auswahl eines Eintrags interessiert uns

	BOOL bValid= FALSE;
	int idx= CtrlGetSel( id_ChooseMotor );
	if ( (m_lnkTopography->GetMotor() != idx) && (idx != CB_ERR) )
	{ // wirklich ein neuer Motor gewhlt?
		m_lnkTopography->SetMotor( idx ); // initialisiert den ausgewaehlten Motor
		// Aktualisierung der Antriebsliste und aller antriebsspezifischen Einheiten und Nachkommastellen
		CtrlSetSelString(id_ChooseMotor, m_lnkTopography->GetMotorName(m_lnkTopography->GetActMotor()) );

		CtrlSetFloat( id_MoveStep, CtrlGetFloat(id_MoveStep, m_lnkTopography->GetMotorDigits(eSF), _DECIMAL, bValid), m_lnkTopography->GetMotorDigits(eSF), _DECIMAL);
		CtrlSetText( id_MoveStepUnit, m_lnkTopography->GetMotorUnit() ); // die Einheit des aktuellen Antriebs

		CtrlSetText( id_Unit, m_lnkTopography->GetMotorUnit()); // die Einheit des aktuellen Antriebs

		CtrlSetFloat( id_ControlStep, CtrlGetFloat(id_ControlStep, m_lnkTopography->GetMotorDigits(eSF), _DECIMAL, bValid), m_lnkTopography->GetMotorDigits(eSF), _DECIMAL);
		CtrlSetText( id_ControlStepUnit, m_lnkTopography->GetMotorUnit() ); // die Einheit des aktuellen Antriebs

		CtrlSetFloat( id_StepWidth, CtrlGetFloat(id_StepWidth, m_lnkTopography->GetMotorDigits(eSF), _DECIMAL, bValid), m_lnkTopography->GetMotorDigits(eSF), _DECIMAL);
		CtrlSetText( id_StepWidthUnit, m_lnkTopography->GetMotorUnit() ); // die Einheit des aktuellen Antriebs

		CtrlSetFloat( id_From, CtrlGetFloat(id_From, m_lnkTopography->GetMotorDigits(eDF), _DECIMAL, bValid), m_lnkTopography->GetMotorDigits(eDF), _DECIMAL);
		CtrlSetText( id_FromUnit, m_lnkTopography->GetMotorUnit() ); // die Einheit des aktuellen Antriebs
	};
};
// ----------------------------------------------------------------------------
// Behandlung der Nachricht id_ChooseDetector

void TTopographyAdjustDlg::Dlg_OnChooseDetector( UINT aCodeNotify )
{
	if ( aCodeNotify != CBN_SELCHANGE )
		return; // nur Auswahl eines Eintrags interessiert uns

	int idx= CtrlGetSel( id_ChooseDetector);
	if ( (m_lnkTopography->GetDetector() != TDetectorManager::DetectorManager().GetDetector(idx)) && (idx != CB_ERR) )
	{
		// setzt den ausgewaehlten Detektor als aktuellen Detektor ein, aktualisiert die Anzeige
		m_lnkTopography->SetDetector( TDetectorManager::DetectorManager().GetDetector(idx) );
		// Aktualisierung der Detektorliste
		CtrlSetSelString(id_ChooseDetector, m_lnkTopography->GetDetector()->GetCharacteristic());
	};
};
// ----------------------------------------------------------------------------
// Behandlung der Nachricht von id_MultipleShot

void TTopographyAdjustDlg::Dlg_OnMultipleShot()
{
	// Bei Aktivierung der Checkbox MultipleShot
	// Eingabefelder im Bereich Mehrfachbelichtung werden aktiviert
	// Arbeitspunkt, Lage und Belichtungsregelung Bereich deaktiviert
	// Bei Deaktivierung entsprechend umgekehrte Aktivierung/Deaktivierung der Eingabefelder
	m_lnkTopography->SetMultipleShot( CtrlIsChecked( id_MultipleShot ) );
	CtrlSetEnabled(id_Shots, m_lnkTopography->IsMultipleShot());
	CtrlSetEnabled(id_From, m_lnkTopography->IsMultipleShot());
	CtrlSetEnabled(id_StepWidth, m_lnkTopography->IsMultipleShot());
	CtrlSetEnabled(id_WorkPoint, !m_lnkTopography->IsMultipleShot());
	CtrlSetEnabled(id_ControlRange, !m_lnkTopography->IsMultipleShot());
};
// ----------------------------------------------------------------------------
// Bearbeitung der Nachrichten an das 'Topographie Einstellen' Fenster

void TTopographyAdjustDlg::Dlg_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch ( id )
	{
		// aktualisieren der Anzeige/Eingabefelder
		case cm_ParamSet:
			Dlg_OnParamSet();
			break;

		// Motor Auswahlbox wurde angewaehlt
		case id_ChooseMotor:
			Dlg_OnChooseMotor( codeNotify );
			break;
		
		// Detektor Auswahlbox wurde angewaehlt
		case id_ChooseDetector:
			Dlg_OnChooseDetector( codeNotify );
			break;

		// 02.08.02 Toter Code : war Schnellwahl-Knopf fr Motorauswahl
		/*
		case cm_RotateMotor:
			  mlSetAxis(nMotor+1);
			  nMotor= mlGetAxis();
			  FORWARD_WM_COMMAND(hwnd,cm_ParamSet,0,0,SendMessage);
			  break;
			*/

		// Aktivieren bzw. Deaktivieren der Mehrfachbelichtungs-Checkbox
		case id_MultipleShot:
			Dlg_OnMultipleShot();
			break;
		
		default:
			TModalDlg::Dlg_OnCommand(hwnd, id, hwndCtl, codeNotify);
	};
};
// ----------------------------------------------------------------------------
// Testet alle Eingabefelder, ob die Werte im Wertebereich liegen und uebernimmt sie in TTopographie

BOOL TTopographyAdjustDlg::CanClose(void)
{
	BOOL bValid= TRUE;
	BOOL bInputOk= TRUE;

	m_lnkTopography->SetMultipleShot( CtrlIsChecked( id_MultipleShot ) ); // Feststellen ob Mehrfachbelichtung ausgewaehlt

	// Steuerelement: 'Belichtungsregelung' -> 'Bereich'
	if ( !m_lnkTopography->SetControlRange( CtrlGetFloat(id_ControlRange, 2, _DECIMAL, bValid)) )
	{
		MessageBox( sth_tp_ValueExposureRange, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	// Steuerelement: 'Belichtungsregelung' -> 'Schritt'
	if ( !m_lnkTopography->SetControlStep( CtrlGetFloat(id_ControlStep, m_lnkTopography->GetMotorDigits(eSF), _DECIMAL, bValid)) )
	{
		MessageBox( sth_tp_ValueExposureStep, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	// Steuerelement: 'Belichtungszeit'
	if ( !m_lnkTopography->SetMeasurementTime( (LONG)(CtrlGetLong(id_MeasurementTime, bValid) * 60) ))
	{
		MessageBox( sth_tp_ValueExposureTime, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	// Steuerelement: 'Mehrfach-Belichtung' -> 'Anzahl'
	if ( !m_lnkTopography->SetNumberCycle( (WORD)CtrlGetLong(id_Shots, bValid)) )
	{
		MessageBox( sth_tp_ValueCycles, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	// Steuerelement: 'Mehrfach-Belichtung' -> 'Schrittweite'
	if ( !m_lnkTopography->SetAngleBetweenShots( CtrlGetFloat(id_StepWidth, m_lnkTopography->GetMotorDigits(eSF), _DECIMAL, bValid)) )
	{
		MessageBox( sth_tp_ValueMultiStep, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	// Steuerelement: 'Mehrfach-Belichtung' -> 'Startwert'
	if ( !m_lnkTopography->SetStartAngle( CtrlGetFloat(id_From, m_lnkTopography->GetMotorDigits(eDF), _DECIMAL, bValid)) )
	{
		MessageBox( sth_tp_ValueMultiStart, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	// Steuerelement: 'Arbeitspunkt' -> 'Anfahren mit'
	if ( !m_lnkTopography->SetMoveStep( CtrlGetFloat(id_MoveStep, m_lnkTopography->GetMotorDigits(eSF), _DECIMAL, bValid)) )
	{
		MessageBox( sth_tp_ValueWorkpointStep, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	// Steuerelement: 'Arbeitspunkt' -> 'Lage'
	if ( !m_lnkTopography->SetWorkPoint( CtrlGetFloat(id_WorkPoint, 1, _DECIMAL, bValid)) )
	{
		MessageBox( sth_tp_ValueWorkpointPos, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	m_lnkTopography->SetSmallAngleSide( CtrlGetFloat(id_WorkPoint, 1, _DECIMAL, bValid) < 0.0 );

	// Steuerelement: 'Actuator' -> 'Beschrnkung auf'
	if ( !m_lnkTopography->SetMaxAngleEscape( CtrlGetFloat(id_MaxAngleEscape, 1, _DECIMAL, bValid)) )
	{
		MessageBox( sth_tp_ValueMaxAngle, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	// Steuerelement: 'Detektor-Parameter' -> 'Zeit'
	if ( !m_lnkTopography->SetMaxTime( CtrlGetFloat(id_MaxTime, 1, _DECIMAL, bValid)) )
	{
		MessageBox( sth_tp_ValueMaxTime, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	// Steuerelement: 'Detektor-Parameter' -> 'Counts'
	if ( !m_lnkTopography->SetMaxCounts( CtrlGetLong(id_MaxCounts, bValid)) )
	{
		MessageBox( sth_tp_ValueCounts, sth_tp_MsgBoxError, MBSTOP );
		bInputOk= FALSE;
	};
	return ( bValid && bInputOk );
};
// ----------------------------------------------------------------------------
// Aufraeumen beim Schliessen des Einstellungen Topographie Fenster

void TTopographyAdjustDlg::LeaveDialog(void)
{
	m_lnkTopography->DoLoadMotorSettings(); // Settings des aktuellen Motors zurckschreiben
	m_lnkTopography= 0;
};
// ----------------------------------------------------------------------------
// __LastLine__
