//#############################################################################
//                                                                           //
// U_FILES.CPP                                                               //
// Subsystem: Utilities - Dateizugriff                                       //
// Deklaration: U_FILES.H                                                    //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand:   03.08.2002                                                       //
//                                                                           //
//#############################################################################

#include <stdlib.h> // fr: min(), max()

#include "utils\u_files.h" // SCHNITTSTELLE fr diese Datei
#include "utils\u_values.h" // IsDouble(), Double2String()

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//#############################################################################
// globale Methoden (Lesen/ Schreiben von String-, Double- und Long-Werten aus/ in ini-Dateien)
//#############################################################################

void _CURVECLASS WINAPI IniReadString ( char *aReturn,
		LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		LPCSTR aDef, const UINT aLength )
{
	GetPrivateProfileString( aSection, aKey, aDef,
							 aReturn, aLength, aFile );
}
//-----------------------------------------------------------------------------

BOOL _CURVECLASS WINAPI IniWriteString ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		LPCSTR aString )
{
	return WritePrivateProfileString( aSection, aKey, aString, aFile );
}
//*****************************************************************************

BOOL _CURVECLASS WINAPI IniReadBool ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const BOOL aDef )
{
	return IniReadLong( aFile, aSection, aKey,
						0, (LONG)aDef, 1 );
}
//-----------------------------------------------------------------------------

BOOL _CURVECLASS WINAPI IniWriteBool ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const BOOL aBool )
{
	return IniWriteLong( aFile, aSection, aKey,
						 (LONG)aBool );
}
//*****************************************************************************

LONG _CURVECLASS WINAPI IniReadLong ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const LONG aDef )
{ //WIE IniReadDouble, nur ohne Nachkommastellen
	return (LONG)IniReadDouble( aFile, aSection, aKey,
								(LONG)aDef, 0);
}
//-----------------------------------------------------------------------------

LONG _CURVECLASS WINAPI IniReadLong ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const LONG aMin, const LONG aDef, const LONG aMax )
{ //WIEVOR, jedoch mit Intervallgrenzen
	LONG result= IniReadLong( aFile, aSection, aKey,
							   aDef );
	//gelesenen Wert ins Intervall [<aMin>..<aMax>] einpassen
	return max( aMin, min( aMax, result ) );
}
//-----------------------------------------------------------------------------

BOOL _CURVECLASS WINAPI IniWriteLong( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const LONG aLong )
{
	return IniWriteDouble( aFile, aSection, aKey,
						   (LONG)aLong, 0 );
}
//*****************************************************************************

float _CURVECLASS WINAPI IniReadFloat ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const float aDef, const UINT aDigits )
{
	return (float)IniReadDouble( aFile, aSection, aKey,
						      Float2Double(aDef, aDigits), aDigits );
}
//-----------------------------------------------------------------------------

float _CURVECLASS WINAPI IniReadFloat ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const float aMin, const float aDef, const float aMax, const UINT aDigits )
{ //WIEVOR, jedoch mit Intervallgrenzen
	float result= IniReadFloat( aFile, aSection, aKey,
								 aDef, aDigits );
	//gelesenen Wert ins Intervall [<aMin>..<aMax>] einpassen
	return max( aMin, min( aMax, result ) );
}
//-----------------------------------------------------------------------------

BOOL _CURVECLASS WINAPI IniWriteFloat ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const float aFloat, const UINT aDigits )
{
	return IniWriteDouble( aFile, aSection, aKey,
						   Float2Double(aFloat, aDigits), aDigits );
}

//*****************************************************************************

double _CURVECLASS WINAPI IniReadDouble ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const double aDef, const UINT aDigits )
{
	//String aus ini-Datei lesen
	char value[51]; //max. 50 Zeichen + Nullterminierung
	IniReadString( value, aFile, aSection, aKey, "", 50 );
	if ( !IsDouble( value ) )
		return aDef; //Eintrag ist nicht vorhanden oder kein Double

	return String2Double( value, aDigits, FALSE, '.' );
}
//-----------------------------------------------------------------------------

double _CURVECLASS WINAPI IniReadDouble ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const double aMin, const double aDef, const double aMax, const UINT aDigits )
{ //WIEVOR, jedoch mit Intervallgrenzen
	double result= IniReadDouble( aFile, aSection, aKey,
								   aDef, aDigits );
	//gelesenen Wert ins Intervall [<aMin>..<aMax>] einpassen
	return max( aMin, min( aMax, result ) );
}
//-----------------------------------------------------------------------------

BOOL _CURVECLASS WINAPI IniWriteDouble ( LPCSTR aFile, LPCSTR aSection, LPCSTR aKey,
		const double aDouble, const UINT aDigits )
{
	char value[51]; //max. 50 Zeichen + Nullterminierung
	Double2String( value, aDouble, aDigits, FALSE, '.' ); //Double nach String konvertieren
	return IniWriteString( aFile, aSection, aKey, value ); //String in ini-Datei speichern
}

//#############################################################################
// TTxtRead
//#############################################################################

TTxtRead::TTxtRead ( const char *aFilename ) : m_File(0), m_Line(0)
{
	m_File= fopen( aFilename, "r" );
	m_Exists= (m_File != 0);
	m_Eof= ( (m_File == 0) || (feof(m_File)) ); //nicht vorhanden oder leer

	//Anzahl der Zeilen ermitteln und zurck zum Dateianfang
	m_Lines= 0;
	if ( (!m_Exists) || (m_Eof) )
		return;

	do
	{
		char c= fgetc( m_File );
		if ( (feof(m_File)) || (c == '\n') || (c == '\r') )
			m_Lines++; //eof || new line || carrige return
	} while ( !feof( m_File) );
	Restart();
}
//-----------------------------------------------------------------------------

TTxtRead::~TTxtRead ()
{
	_FREELIST(m_Line);
	if ( m_File )
	{
		fclose( m_File );
		m_File= 0;
	}
}
//-----------------------------------------------------------------------------

void TTxtRead::Restart() {
	_FREELIST(m_Line);
	if ( m_File ) fseek(m_File, 0, SEEK_SET);
}
//-----------------------------------------------------------------------------

BOOL TTxtRead::NextLine (string &result)
{
	result= "";
	_FREELIST(m_Line);
	if ( m_Eof )
		return FALSE;

	//Zeile einlesen
	BOOL eol= FALSE;
	do
	{
		char c= fgetc(m_File);
		if ( !feof(m_File) )
		{
			if ( (c != '\n') && (c != '\r') )
			{
				string temp= "1"; // genau ein Zeichen !!!
				temp[0]= c;
				result += temp; // eigentlich reicht result+= c;
			}
			else
				eol= TRUE; //new line || carrige return
		}
		else
			m_Eof= TRUE;
	} while ( (!m_Eof) && (!eol) );
	clearerr(m_File);
	return TRUE;
}
//-----------------------------------------------------------------------------

char *TTxtRead::NextLine ()
{
	string temp;
	if ( !NextLine(temp) )
		return 0; // Fehler oder Dateiende

	// in char* konvertieren, temp wird freigegeben
	m_Line= new char[ temp.length() + 1 ]; // + Nullterminierung

	string::size_type i;

	for ( i= 0; i < temp.length(); i++)
		m_Line[i]= temp[i];
	m_Line[i]= '\0'; // Nullterminierung nicht vergessen
	return m_Line;
}
//-----------------------------------------------------------------------------

BOOL TTxtRead::IsEOF ()
{
	return m_Eof;
}
//-----------------------------------------------------------------------------

BOOL TTxtRead::Exists ()
{
	return m_Exists;
}
//-----------------------------------------------------------------------------

UINT TTxtRead::GetLines ()
{
	return m_Lines;
}

//#############################################################################
// LASTLINE
//#############################################################################
