//#############################################################################
//                                                                           //
// U_LISTS.CPP                                                               //
// Subsystem: Utilities - verkettete Listen                                  //
// Deklaration: U_LISTS.H                                                    //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand:   02.04.2004                                                       //
//                                                                           //
//#############################################################################

#include "utils\u_lists.h" // SCHNITTSTELLE fr diese Datei

//#############################################################################
// TLnkContainer
//#############################################################################

TLnkContainer::TLnkContainer(void *const aData, TLnkList &aList) : m_Data(aData), m_List(aList), m_Prev(0), m_Next(0) {
}
//-----------------------------------------------------------------------------

bool TLnkContainer::IsElementOf(const TLnkList *const aList) const {
	return ( &m_List==aList );
}

//#############################################################################
// TLnkList
//#############################################################################

TLnkList::TLnkList(const bool aFailIfNull, const bool aFailIfExists, const bool aAutoDelete, const bool aCheckOnDel) : m_FailIfNull(aFailIfNull), m_FailIfExists(aFailIfExists), m_AutoDelete(aAutoDelete), m_CheckOnDel(aCheckOnDel), m_First(0), m_Last(0) {
}
//-----------------------------------------------------------------------------

TLnkList::~TLnkList() {
	Clear();
}
//-----------------------------------------------------------------------------

TLnkContainer *TLnkList::Find(void *aData, TLnkContainer *aStart) {
	TLnkContainer *result= 0;
	if ( !aStart ) result= m_First; // am Anfang anfangen
	else if ( !aStart->IsElementOf(this) ) return aStart->m_List.Find(aData, aStart); // nicht Element dieser Liste
	else result= aStart; // ab angegebenem Container

	while ( result ) {
		if ( result->m_Data==aData ) return result; // gefunden
		else result= result->m_Next;
	}
	return 0; // nicht gefunden
}
//-----------------------------------------------------------------------------

void *TLnkList::GetAt(TLnkContainer &aContainer) const {
	if ( !aContainer.IsElementOf(this) ) return aContainer.m_List.GetAt(aContainer);
	return aContainer.m_Data; // ggf. 0
}
//-----------------------------------------------------------------------------

bool TLnkList::Insert(void *const aData) {
	if ( m_FailIfNull && !aData ) return false; // Null-Elemente nicht aufnehmen
	if ( m_FailIfExists && Find(aData) ) return false; // ist schon in der Liste

	TLnkContainer *toAdd= new TLnkContainer(aData, *this);
	// mit Folgeelement verlinken
	if ( m_First ) m_First->m_Prev= toAdd;
	toAdd->m_Next= m_First; // ggf. 0
	m_First= toAdd;
	if ( !m_Last ) m_Last= toAdd; // die Liste war leer
	return true;
}
//-----------------------------------------------------------------------------

bool TLnkList::Append(void *const aData) {
	if ( m_FailIfNull && !aData ) return false; // Null-Elemente nicht aufnehmen
	if ( m_FailIfExists && Find(aData) ) return false; // ist schon in der Liste

	TLnkContainer *toAdd= new TLnkContainer(aData, *this);
	// mit Vorgnger verlinken
	if ( m_Last ) m_Last->m_Next= toAdd;
	toAdd->m_Prev= m_Last; // ggf. 0
	m_Last= toAdd;
	if ( !m_First ) m_First= toAdd; // die Liste war leer
	return true;
}
//-----------------------------------------------------------------------------

bool TLnkList::InsBefore(TLnkContainer *aContainer, void *const aData) {
	if ( !aContainer ) return Insert(aData); // am Anfang einfgen
	if ( !aContainer->IsElementOf(this) ) return aContainer->m_List.InsBefore(aContainer, aData); // nicht Element dieser Liste

	if ( m_FailIfNull && !aData ) return false; // Null-Elemente nicht aufnehmen
	if ( m_FailIfExists && Find(aData) ) return false; // ist schon in der Liste

	TLnkContainer *prev= aContainer->m_Prev; // ggf. 0
	TLnkContainer *toAdd= new TLnkContainer(aData, *this);
	// mit Vorgnger verlinken
	if ( prev ) prev->m_Next= toAdd; 
	else m_First= toAdd; // kein Vorgnger: wir sind die ersten
	toAdd->m_Prev= prev; // ggf. 0
	// mit Folgeelement verlinken
	toAdd->m_Next= aContainer;
	aContainer->m_Prev= toAdd; 
	return true;
}
//-----------------------------------------------------------------------------

bool TLnkList::AddAfter(TLnkContainer *aContainer, void *const aData) {
	if ( !aContainer ) return Append(aData); // am Ende einfgen
	if ( !aContainer->IsElementOf(this) ) return aContainer->m_List.AddAfter(aContainer, aData); // nicht Element dieser Liste
	
	if ( m_FailIfNull && !aData ) return false; // Null-Elemente nicht aufnehmen
	if ( m_FailIfExists && Find(aData) ) return false; // ist schon in der Liste

	TLnkContainer *next= aContainer->m_Next; // ggf. 0
	TLnkContainer *toAdd= new TLnkContainer(aData, *this);
	// mit Vorgnger verlinken
	aContainer->m_Next= toAdd;
	toAdd->m_Prev= aContainer;
	// mit Folgeelement verlinken
	if ( next ) next->m_Prev= toAdd;
	else m_Last= toAdd; // kein Folgeelement: wir sind die letzten
	toAdd->m_Next= next; // ggf. 0
	return true;
}
//-----------------------------------------------------------------------------

void *TLnkList::Remove(TLnkContainer &aContainer) {
	if ( !aContainer.IsElementOf(this) ) return aContainer.m_List.Remove(aContainer); // nicht Element dieser Liste

	TLnkContainer *p= aContainer.m_Prev; // ggf. 0
	TLnkContainer *n= aContainer.m_Next; // ggf. 0
	// Verlinkung entfernen
	if ( p ) p->m_Next= n;
	else m_First= n; // kein Vorgnger: wir waren die ersten; ggf. 0
	if ( n ) n->m_Prev= p;
	else m_Last= p; // kein Nachfolger: wie waren die letzten; ggf. 0

	// Container freigeben
	TLnkContainer *toDel= &aContainer;
	void *result= toDel->m_Data;
	_FREEOBJ(toDel);

	// Element nicht automatisch freigeben
	if ( m_AutoDelete && result ) { // Element automatisch freigeben
		if ( !m_CheckOnDel || m_FailIfExists || !Find(result) ) { // <result> ist einzigartig in der Liste
			FreeData(result); // einzigartiges Element freigeben
			result= 0;
		}
	}	
	return result;
}
//-----------------------------------------------------------------------------

void TLnkList::Clear() {
	if ( m_First ) CutList(*m_First); // alle Elemente entfernen
}
//-----------------------------------------------------------------------------

void TLnkList::CutList(TLnkContainer &aContainer) {
	if ( !aContainer.IsElementOf(this) ) { // nicht Element dieser Liste
		aContainer.m_List.CutList(aContainer); 
		return;
	}

	TLnkContainer *act= &aContainer;
	// Vorgnger ist neues Listenende
	TLnkContainer *p= act->m_Prev; // ggf. 0
	if ( p ) p->m_Next= 0;
	else m_First= 0; // kein Vorgnger: wir waren die ersten
	m_Last= p; // ggf. 0

	// dieses und Folgeelemente entfernen
	while ( act ) {
		TLnkContainer *toDel= act;
		act= act->m_Next; // weitersetzen, vor _FREEOBJ

		// Container freigeben
		void *result= toDel->m_Data;
		_FREEOBJ(toDel);

		if ( m_AutoDelete && result ) { // Element automatisch freigeben
			if ( m_CheckOnDel && !m_FailIfExists ) { // alle weiteren Vorkommen, die weiter hinten folgen, entfernen
				// alle nachfolgenden Container entfernen, die auch <result> beinhalten
				TLnkContainer *next= 0;
				if ( act ) next= Find(result, act); // nur suchen, wenn Folgeelement existiert
				while ( next ) {
					TLnkContainer *temp= next;
					next= Find(result, next->m_Next); // erst weitersuchen, dann freigeben

					// Verlinkung aktualisierien, <m_First> und <m_Last> wurden oben schon gesetzt - drfen nicht mehr gesetzt werden
					TLnkContainer *p= temp->m_Prev;
					TLnkContainer *n= temp->m_Next;				
					if ( p ) p->m_Next= n; 
					if ( n ) n->m_Prev= p; 

					// Container freigeben
					_FREEOBJ(temp);
				}
			}
			// wenn <result> noch in der Liste ist, die am Ende von CutList brig bleibt: <result> NICHT freigeben		
			if ( !m_CheckOnDel || m_FailIfExists || !Find(result) ) { // <result> ist einzigartig in der Liste
				FreeData(result); // einzigartiges Element freigeben
				result= 0;
			}
		}
	}
}
//-----------------------------------------------------------------------------

void TLnkList::FreeData(void *aData) const {
	_FREEOBJ(aData);
}

//##############################################################################
// TIndList
//##############################################################################

TIndList::TIndList(const bool aFailIfNull, const bool aFailIfExists, const bool aAutoDelete, const bool aCheckOnDel, const TIndex &aTemp) : TLnkList(aFailIfNull, aFailIfExists, aAutoDelete, aCheckOnDel) {
	m_Count= 0;
	DoInvalidAccelerator();
}
//-----------------------------------------------------------------------------

void TIndList::DoInvalidAccelerator() {
	m_AccelIndex= 0;
	m_AccelContainer= 0;
}
//-----------------------------------------------------------------------------

TLnkContainer *TIndList::FindForward(TIndex aIndex, const TIndex aActIdx, TLnkContainer *aActContainer) const {
	TLnkContainer *result= aActContainer;
	while ( (aIndex>aActIdx) && (result) ) {
		result= result->m_Next;
		aIndex--;
	}
	return result;
}
//-----------------------------------------------------------------------------

TLnkContainer *TIndList::FindBackward(TIndex aIndex, const TIndex aActIdx, TLnkContainer *aActContainer) const {
	TLnkContainer *result= aActContainer;
	while ( (aIndex<aActIdx) && (result) ) {
		result= result->m_Prev;
		aIndex++;
	}
	return result;
}
//-----------------------------------------------------------------------------

TLnkContainer *TIndList::FindIdx(TIndex aIndex, bool aAdoptAccelerator) const {
	if ( GetCount()==0 ) { // Liste ist leer
		TraceErr("INDEX!!! Die Liste ist leer!\nDie angeforderte Operation ist nicht durchfhrbar.");
		return 0;
	}
	if ( aIndex > GetCount()-1 ) { // Index ist zu gro
		TraceErr("INDEX!!! Der angegebene Index ueberschreitet das Listenende!\nDas letzte Element wird fuer die angeforderte Operation verwendet.");
		aIndex= GetCount()-1;
	}

	TLnkContainer *result= 0;
	if ( m_AccelContainer && aIndex<m_AccelIndex && m_AccelIndex-aIndex<aIndex-0 ) // angeforderter Index liegt vor dem zuletzt benutzten Container und dieser ist besser als am Anfang anzufangen
		result= FindBackward(aIndex, m_AccelIndex, m_AccelContainer);
	else if ( m_AccelContainer && m_AccelIndex<=aIndex && aIndex-m_AccelIndex<GetCount()-aIndex ) // angeforderter Index liegt nach dem zuletzt benutzten Container und dieser ist besser als am Ende anzufangen
		result= FindForward(aIndex, m_AccelIndex, m_AccelContainer);
	else if ( aIndex<GetCount()/2 || GetCount()==0 ) // am Anfang anfangen und vorwrts suchen
		result= FindForward(aIndex, 0, m_First);
	else // bei Ende anfangen und rckwrts suchen
		result= FindBackward(aIndex, GetCount()-1, m_Last);

	if ( aAdoptAccelerator && result ) {
		static_cast<TIndex>(m_AccelIndex)= aIndex;
		const_cast<TLnkContainer*>(m_AccelContainer)= result;
	}
	return result;
}
//-----------------------------------------------------------------------------

void *TIndList::GetAt(TIndex aIndex, bool &aValid) const {
	TLnkContainer *temp= FindIdx(aIndex);
	aValid= (temp!=0);
	if ( temp ) return TLnkList::GetAt(*temp);
	else return 0; // leere oder fehlerhafte Liste
}
//-----------------------------------------------------------------------------

void *TIndList::GetAt(TIndex aIndex) const {
	bool bValid= false;
	return GetAt(aIndex, bValid);
}
//-----------------------------------------------------------------------------

void *TIndList::operator[] (TIndex aIndex) const {
	return GetAt(aIndex); 
}
//-----------------------------------------------------------------------------

bool TIndList::Insert(void *const aData) {
	bool result= TLnkList::Insert(aData);
	if ( result ) {
		DoInvalidAccelerator();
		m_Count++;
	}
	return result;
}
//-----------------------------------------------------------------------------

bool TIndList::Append(void *const aData) {
	bool result= TLnkList::Append(aData);
	if ( result ) m_Count++;
	return result;
}
//-----------------------------------------------------------------------------

bool TIndList::InsBefore(TIndex aIndex, void *const aData) {
	bool result= false;
	if ( GetCount()==0 ) { // Liste ist leer
		TraceErr("INDEX!!! Die Liste ist leer!\nDas Element wird am Listenende eingefuegt.");
		result= TLnkList::Append(aData);
	} else {
		TLnkContainer *temp= FindIdx(aIndex, false);
		result= ( temp && TLnkList::InsBefore(temp, aData) );
		if ( result && aIndex<=m_AccelIndex ) DoInvalidAccelerator();
	}
	if ( result ) m_Count++;
	return result; 
}
//-----------------------------------------------------------------------------

bool TIndList::AddAfter(TIndex aIndex, void *const aData) {
	bool result= false;
	if ( GetCount()==0 ) { // Liste ist leer
		TraceErr("INDEX!!! Die Liste ist leer!\nDas Element wird am Listenende eingefuegt.");
		result= TLnkList::Append(aData);
	} else {
		TLnkContainer *temp= FindIdx(aIndex);
		result= ( temp && TLnkList::AddAfter(temp, aData) );
		if ( result && aIndex<m_AccelIndex ) DoInvalidAccelerator();
	}
	if ( result ) m_Count++;
	return result;
}
//-----------------------------------------------------------------------------

void *TIndList::Remove(TIndex aIndex) {
	TLnkContainer *temp= FindIdx(aIndex, false);
	if ( temp ) {
		if ( aIndex<=m_AccelIndex ) DoInvalidAccelerator();
		void *result= TLnkList::Remove(*temp); // entfernt <*temp> in jedem Fall; Rckgabewert aber ggf. 0
		m_Count--;
		return result;
	} else return 0; // leere oder fehlerhafte Liste
}
//-----------------------------------------------------------------------------

void *TIndList::RemoveData(void *aData) {
	TLnkContainer *temp= m_First;
	TIndex idx= 0;
	while ( temp ) {
		if ( temp->m_Data==aData ) return Remove(idx); // gefunden
		temp= temp->m_Next;
		idx++;
	}
	return 0; // leere Liste oder <aData> nicht gefunden
}
//-----------------------------------------------------------------------------

void TIndList::Clear() {
	DoInvalidAccelerator();
	TLnkList::Clear();
	m_Count= 0;
}
//-----------------------------------------------------------------------------

void TIndList::CutList(TIndex aCount) {
	if ( GetCount()==0 ) {
		TraceErr("INDEX!!! Die Liste ist leer!\nDie angeforderte Operation ist nicht durchfhrbar."); // Liste ist leer
		return;
	} else if ( aCount > GetCount()-1 ) {
		TraceErr("INDEX!!! Der angegebene Index ueberschreitet das Listenende!\nDie angeforderte Operation ist nicht durchfhrbar."); // Index ist zu gro
		return;
	}

	TLnkContainer *temp= FindIdx(aCount, false);
	if ( temp ) {
		TLnkList::CutList(*temp); // entfernt <temp> und Folgeelemente
		m_Count= aCount;
		if ( aCount<=m_AccelIndex ) DoInvalidAccelerator();
	}
}
//-----------------------------------------------------------------------------

TIndex TIndList::GetCount() const {
	return m_Count;
}

//##############################################################################
// TIdxContainer
//##############################################################################

TIdxContainer::TIdxContainer(const TIndex &aSize) : m_Data(0), m_Size(0), m_Prev(0), m_Next(0) {
	static_cast<TIndex>(m_Size)= max(1, aSize); // mind. ein Element sollte enthalten sein
	m_Data= new void*[m_Size];
	for (TIndex idx= 0; idx<GetSize(); idx++) m_Data[idx]= 0;
}
//-----------------------------------------------------------------------------

TIdxContainer::~TIdxContainer() {
	_FREELIST(m_Data);
}
//-----------------------------------------------------------------------------

TIndex TIdxContainer::GetSize() const {
	return m_Size;
}
//-----------------------------------------------------------------------------

void *TIdxContainer::ShiftRight(TIndex aStartIdx) {
	if ( aStartIdx>=GetSize() ) {
		TraceErr("INDEX!!! Der Index ist zu gro!\nDie gesamte Liste wird nach hinten geshiftet."); // Index zu gro
		aStartIdx= 0; // vom Anfang der Liste
	}	
	void *result= m_Data[ GetSize()-1 ];
	for (TIndex idx= GetSize()-1; idx>aStartIdx; idx--) m_Data[idx]= m_Data[idx-1];
	m_Data[aStartIdx]= 0; // nun leer
	return result;
}
//-----------------------------------------------------------------------------

void *TIdxContainer::ShiftLeft(TIndex aStartIdx) {
	if ( aStartIdx>=GetSize() ) {
		TraceErr("INDEX!!! Der Index ist zu gro!\nDie gesamte Liste wird nach vorn geshiftet."); // Index zu gro
		aStartIdx= GetSize()-1; // bis zum Ende der Liste
	}
	void *result= m_Data[ aStartIdx ];
	for (TIndex idx= aStartIdx+1; idx<GetSize()-1; idx++) m_Data[idx-1]= m_Data[idx];
	m_Data[GetSize()-1]= 0; // nun leer
	return result;
}

//##############################################################################
// TIdxList
//##############################################################################

TIdxList::TIdxList(const bool aFailIfNull, const bool aFailIfExists, const bool aAutoDelete, const bool aCheckOnDel, const TIndex &aSize) : m_FailIfNull(aFailIfNull), m_FailIfExists(aFailIfExists), m_AutoDelete(aAutoDelete), m_CheckOnDel(aCheckOnDel), m_Size(aSize), m_First(0), m_Last(0), m_Count(0), m_Used(0) {
}
//-----------------------------------------------------------------------------

TIdxList::~TIdxList() {
	Clear();
}
//-----------------------------------------------------------------------------

TIdxContainer *TIdxList::FindIdx(TIndex &aIndex) const {
	if ( GetCount()>0 && aIndex>GetCount()-1 ) {
		TraceErr("INDEX!!! Der Index ist zu gro!\nDas letzte Element wird verwendet."); // Index zu gro
		aIndex= GetCount()-1;
	} else if ( !m_First ) TraceErr("INDEX!!! Die Liste ist leer!\nDie angeforderte Operation ist nicht durchfhrbar."); // Liste ist leer

	TIdxContainer *result= 0;
	if ( aIndex<GetCount()/2 || GetCount()==0 ) { // am Anfang anfangen und vorwrts suchen
		result= m_First;
		while ( result && aIndex>=result->GetSize() ) {
			aIndex-= result->GetSize();
			result= result->m_Next;
		}
		return result;
	} else { // bei Ende anfangen und rckwrts suchen
		result= m_Last;
		TIndex index= GetCount();
		while ( result ) {
			if ( !result->m_Next ) index-= m_Used; // Container ist u.U. noch nicht voll
			else index-= result->GetSize();
			if ( index<=aIndex ) { // gefunden
				aIndex-= index;
				return result;
			}
			result= result->m_Prev;
		}
		// nicht gefunden
		aIndex= 0;
		return 0;
	}
}
//-----------------------------------------------------------------------------

TIdxContainer *TIdxList::Find(void *aData, TIndex &aIndex) const {
	aIndex= 0;
	TIdxContainer *result= m_First;
	while ( result ) {
		TIndex last= result->GetSize();
		if ( !result->m_Next ) last= m_Used; // das letzte Element mit einbeziehen
		for (TIndex idx= 0; idx<last; idx++) {
			if ( result->m_Data[idx]==aData ) return result; // gefunden
			aIndex++;
		}
		result= result->m_Next;
	}
	aIndex= 0;
	return 0;
}
//-----------------------------------------------------------------------------

void *TIdxList::GetAt(TIndex aIndex, bool &aValid) const {
	TIdxContainer *temp= FindIdx(aIndex); // sucht den Container und verndert <aIndex>
	aValid= (temp!=0);
	if ( temp ) return temp->m_Data[aIndex];
	else return 0; // leere Liste
}
//-----------------------------------------------------------------------------

void *TIdxList::GetAt(TIndex aIndex) const {
	bool bValid= false;
	return GetAt(aIndex, bValid);
}
//-----------------------------------------------------------------------------

void *TIdxList::operator[] (TIndex aIndex) const {
	return GetAt(aIndex);
}
//-----------------------------------------------------------------------------

bool TIdxList::Insert(void *const aData) {
	if ( !m_First ) return Append(aData); // Liste ist leer: Vor den berprfungen Append aufrufen - fhrt auch diese berprfungen aus
	else return InsBefore(0, aData); // gibt bei <m_First>==0 eine Fehlermeldung aus
}
//-----------------------------------------------------------------------------

bool TIdxList::Append(void *const aData) {
	TIndex i;
	if ( m_FailIfNull && !aData ) return false; // Null-Elemente nicht aufnehmen
	if ( m_FailIfExists && Find(aData, i) ) return false; // ist schon in der Liste

	if ( !m_Last ) { // Liste ist leer
		TIdxContainer *toAdd= m_Last= new TIdxContainer(m_Size);
		m_Used= 1;
		toAdd->m_Data[m_Used-1]= aData;
		// mit Vorgnger verlinken
		m_First= m_Last= toAdd;
		m_Count= 1;
		return true;
	}

	if ( m_Used>=m_Last->GetSize() ) { // dieser (letzte) Container ist voll: einen neuen anhngen
		TIdxContainer *toAdd= new TIdxContainer(m_Size);
		m_Used= 1;
		toAdd->m_Data[m_Used-1]= aData;
		// mit Vorgnger verlinken
		m_Last->m_Next= toAdd;
		toAdd->m_Prev= m_Last;
		m_Last= toAdd;
	} else { // ansonsten ist im letzten Container noch genug Platz
		m_Used++; 
		m_Last->m_Data[m_Used-1]= aData;
	}
	m_Count++;
	return true;
}
//-----------------------------------------------------------------------------

bool TIdxList::InsBefore(TIndex aIndex, void *const aData) {
	if ( !m_First ) { // Liste ist leer
		TraceErr("INDEX!!! Die Liste ist leer!\nDas Element wird am Listenende eingefuegt.");
		return Append(aData);
	}
	TIndex i;
	if ( m_FailIfNull && !aData ) return false; // Null-Elemente nicht aufnehmen
	if ( m_FailIfExists && Find(aData, i) ) return false; // ist schon in der Liste

	void *shifted= aData; // als Element, das beim shiften frei wird
	TIdxContainer *act= FindIdx(aIndex); // sucht den Container und verndert <aIndex>
	while ( act ) {
		void *temp= act->ShiftRight(aIndex);
		act->m_Data[aIndex]= shifted;
		shifted= temp;
		aIndex= 0; // nun sofort nur noch am Anfang einfgen
		if ( !act->m_Next ) { // wir haben den letzten Container erreicht
			if ( m_Used>=act->GetSize() ) { // dieser (letzte) Container ist voll: einen neuen anhngen
				TIdxContainer *toAdd= new TIdxContainer(m_Size);
				m_Used= 1;
				toAdd->m_Data[m_Used-1]= shifted;
				// mit Vorgnger verlinken
				m_Last->m_Next= toAdd;
				toAdd->m_Prev= m_Last;
				m_Last= toAdd;
			} else m_Used++; // ansonsten ist im letzten Container noch genug Platz
			break; // fertig					
		} else act= act->m_Next; // im nchsten Container weitermachen
	}
	m_Count++;
	return true;
}
//-----------------------------------------------------------------------------

bool TIdxList::AddAfter(TIndex aIndex, void *const aData) {
	if ( m_First && aIndex>=GetCount()-1 ) return Append(aData); // das einzige, was InsBefore nicht kann; bei <m_First>==0 auch InsBefore aufrufen, wegen der Fehlermeldung
	else return InsBefore(aIndex+1, aData); // einfach VOR dem nachfolgenden Element einfgen
}
//-----------------------------------------------------------------------------

void *TIdxList::Remove(TIndex aIndex) {
	void *result= 0; // falls Liste leer
	TIdxContainer *act= FindIdx(aIndex); // sucht den Container und verndert <aIndex>
	if ( act ) {
		// alle nachfolgenden Elemente eins nach vorn kopieren
		result= act->ShiftLeft(aIndex);
		while ( act->m_Next ) {
			void *shifted= act->m_Next->ShiftLeft(0);
			act->m_Data[ act->GetSize()-1 ]= shifted;
		}
		m_Used--;
		if ( m_Used==0 ) { // letzter Container ist nun leer: diesen entfernen und m_Used auf Gre des nun letzten Containers (if one)
			act= m_Last->m_Prev; // ggf. 0
			_FREEOBJ(m_Last); // Container freigeben			
			if ( act ) {
				m_Last= act; // neues letztes Element
				m_Last->m_Next= 0; // ohne Nachfolger
				m_Used= m_Last->GetSize();
			} else m_First= m_Last= 0; // Liste ist nun leer
		}
		m_Count--;
		
		if ( m_AutoDelete && result ) { // Element automatisch freigeben
			TIndex temp;
			if ( !m_CheckOnDel || m_FailIfExists || !Find(result, temp) ) { // <result> ist einzigartig in der Liste
				FreeData(result);
				result= 0;
			}
		}
	}
	return result;
}
//-----------------------------------------------------------------------------

void TIdxList::CutList(const TIndex &aCount) {
	if ( GetCount()==0 ) {
		TraceErr("INDEX!!! Die Liste ist leer!\nDie angeforderte Operation ist nicht durchfhrbar."); // Liste ist leer
		return;
	} else if ( aCount>GetCount()-1 ) {
		TraceErr("INDEX!!! Der angegebene Index ueberschreitet das Listenende!\nDie angeforderte Operation ist nicht durchfhrbar."); // Index ist zu gro
		return;
	}

	// dieses und alle nachfolgenden Elemente entfernen
	TIndex aIndex= aCount;
	TIdxContainer *act= FindIdx(aIndex); // sucht den Container und verndert <aIndex>
	if ( !act ) return;

	// Liste vor dem Element abschneiden
	TIdxContainer *next= act->m_Next;
	TIndex used= m_Used;
	if ( aIndex==0 ) { // <act> wird komplett geleert; d.h. sein Vorgnger wird neuer, letzter Container
		m_Last= act->m_Prev; // ggf. 0
		if ( !m_Last ) { // Liste ist nun leer
			m_First= 0; 
			m_Used= 0;
		} else m_Used= m_Last->GetSize(); // den neue letzten Container komplett benutzen (der ist voll)
	} else {
		m_Last= act;
		m_Used= aIndex;
	}	
	if ( m_Last ) m_Last->m_Next= 0;
	m_Count= aCount;
	// Elemente freigeben
	while ( act ) {
		TIndex last= act->GetSize();
		if ( !next ) last= used; // das letzte Element mit einbeziehen
		for (TIndex idx= aIndex; idx<last; idx++) {
			void *result= act->m_Data[idx];
			act->m_Data[idx]= 0;

			if ( m_AutoDelete && result ) { // Element automatisch freigeben
				if ( m_CheckOnDel && !m_FailIfExists ) { // alle weiteren Vorkommen, die weiter hinten folgen, entfernen					
					// bis zum Ende des aktuellen Containers
					for (TIndex idx2= idx+1; idx2<last; idx2++) if ( act->m_Data[idx2]==result ) act->m_Data[idx2]= 0;
					TIdxContainer *act2= next;
					// alle weiteren Container
					while ( act2 ) {
						TIndex last2= act2->GetSize();
						if ( !act2->m_Next ) last2= used; // das letzte Element mit einbeziehen
						for (idx2= 0; idx2<last2; idx2++) if ( act2->m_Data[idx2]==result ) act2->m_Data[idx2]= 0;
						act2= act2->m_Next;
					}
				}
				TIndex temp;
				if ( !m_CheckOnDel || m_FailIfExists || !Find(result, temp) ) { // <result> ist einzigartig in der Liste
					FreeData(result);
					result= 0;
				}
			}
		}
		if ( aIndex==0 ) _FREEOBJ(act); // Container komplett entfernen
		aIndex= 0; // nchsten Container komplett
		act= next; // bei dem ber FindIdx gefundenen Container ist m_Next==0, um das neue Listenende zu kennzeichnen, deshalb <next> verwenden
		if ( act ) next= act->m_Next;
		else next= 0;
	}
}
//-----------------------------------------------------------------------------

void TIdxList::Clear() {
	if ( GetCount() ) CutList(0);
	m_Count= 0;
}
//-----------------------------------------------------------------------------

TIndex TIdxList::GetCount() const {
	return m_Count;
}
//-----------------------------------------------------------------------------

void TIdxList::FreeData(void *aData) const { // berschreiben, wenn in der Liste nur Objekte eines bestimmten Typs enthalten sind (dann <aData> auf diesen Typ casten und freigeben)
	_FREEOBJ(aData);
}

//##############################################################################
// TIntList
//##############################################################################

TIntList::TIntList(const TIndex &aCount, const int &aDefaultValue, const TIndex &aSize) : TBaseList(true, false, true, false, aSize), m_DefaultValue(aDefaultValue) {
	SetCount(aCount);
}
//-----------------------------------------------------------------------------

TIntList::TIntList(const TIntList &aList) : TBaseList(aList.m_FailIfNull, aList.m_FailIfExists, aList.m_AutoDelete, aList.m_CheckOnDel, aList.m_Size), m_DefaultValue(aList.m_DefaultValue) {
	if ( &aList==this ) return; // Selbstzuweisung

	Clear();
	bool bValid= false;
	for (TIndex i=0; i<aList.GetCount(); i++) Append( aList.GetAt(i, bValid) );
}
//-----------------------------------------------------------------------------

TIntList &TIntList::operator= (const TIntList &aList) { 
	if ( &aList==this ) return *this; // Selbstzuweisung

	m_DefaultValue= aList.m_DefaultValue;
	Clear();
	bool bValid= false;
	for (TIndex i=0; i<aList.GetCount(); i++) Append( aList.GetAt(i, bValid) );
	return *this;
}
//-----------------------------------------------------------------------------

int TIntList::GetAt(const TIndex &aIndex, bool &aValid) const {
	int *temp= (int*)TBaseList::GetAt(aIndex, aValid);
	if ( aValid && temp ) return *temp;
	else return m_DefaultValue;
}
//-----------------------------------------------------------------------------

int TIntList::GetAt(const TIndex &aIndex) const {
	bool bValid= false;
	return GetAt(aIndex, bValid);
}
//-----------------------------------------------------------------------------

int &TIntList::operator[] (const TIndex &aIndex) {
	bool bValid= false;
	int *temp= (int*)TBaseList::GetAt(aIndex, bValid);
	if ( !bValid || !temp ) { // <m_DefaultValue> kopieren, sonst kann <m_DefaultValue> beim Schreibzugriff gendert werden
		temp= new int;
		*temp= m_DefaultValue;
	}
	return *temp;
}
//-----------------------------------------------------------------------------

bool TIntList::SetAt(const TIndex &aIndex, const int &aValue) {
	if ( GetCount()>0 && aIndex<GetCount() ) {
		int *temp= (int*)TBaseList::GetAt(aIndex);
		if ( temp ) {
			*temp= aValue;
			return true;
		}
	}
	if ( GetCount()==0 ) TraceErr("INDEX!!! Die Liste ist leer!\nDie angeforderte Operation ist nicht durchfhrbar."); // Liste ist leer
	else if ( aIndex > GetCount()-1 ) TraceErr("INDEX!!! Der angegebene Index ueberschreitet das Listenende!\nDie angeforderte Operation ist nicht durchfhrbar."); // Index ist zu gro
	return false;
}
//-----------------------------------------------------------------------------

bool TIntList::Insert(const int &aValue) {
	int *temp= new int;
	*temp= aValue;
	bool result= TBaseList::Insert(temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

bool TIntList::Append(const int &aValue) {
	int *temp= new int;
	*temp= aValue;
	bool result= TBaseList::Append(temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

bool TIntList::InsBefore(const TIndex &aIndex, const int &aValue) {
	int *temp= new int;
	*temp= aValue;
	bool result= TBaseList::InsBefore(aIndex, temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

bool TIntList::AddAfter(const TIndex &aIndex, const int &aValue) {
	int *temp= new int;
	*temp= aValue;
	bool result= TBaseList::AddAfter(aIndex, temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

void TIntList::SetCount(const TIndex &aCount, const int &aDefaultValue) {
	// Liste verkrzen
	if ( GetCount()>aCount ) TBaseList::CutList(aCount); // CutList verringert GetCount() 
	// Liste verlngern		
	else while ( GetCount()<aCount && Append(aDefaultValue) ) /* Append erhht GetCount() */;
}
//-----------------------------------------------------------------------------

void TIntList::SetCount(const TIndex &aCount) {
	SetCount(aCount, m_DefaultValue);
}

//##############################################################################
// TFloatList
//##############################################################################

TFloatList::TFloatList(const TIndex &aCount, const float &aDefaultValue, const TIndex &aSize) : TBaseList(true, false, true, false, aSize), m_DefaultValue(aDefaultValue) {
	SetCount(aCount);
}
//-----------------------------------------------------------------------------

TFloatList::TFloatList(const TFloatList &aList) : TBaseList(aList.m_FailIfNull, aList.m_FailIfExists, aList.m_AutoDelete, aList.m_CheckOnDel, aList.m_Size), m_DefaultValue(aList.m_DefaultValue) {
	if ( &aList==this ) return; // Selbstzuweisung

	Clear();
	bool bValid= false;
	for (TIndex i=0; i<aList.GetCount(); i++) Append( aList.GetAt(i, bValid) );
}
//-----------------------------------------------------------------------------

TFloatList &TFloatList::operator= (const TFloatList &aList) { 
	if ( &aList==this ) return *this; // Selbstzuweisung

	m_DefaultValue= aList.m_DefaultValue;
	Clear();
	bool bValid= false;
	for (TIndex i=0; i<aList.GetCount(); i++) Append( aList.GetAt(i, bValid) );
	return *this;
}

//-----------------------------------------------------------------------------

float TFloatList::GetAt(const TIndex &aIndex, bool &aValid) const {
	float *temp= (float*)TBaseList::GetAt(aIndex, aValid);
	if ( aValid && temp ) return *temp;
	else return m_DefaultValue;
}
//-----------------------------------------------------------------------------

float TFloatList::GetAt(const TIndex &aIndex) const {
	bool bValid= false;
	return GetAt(aIndex, bValid);
}
//-----------------------------------------------------------------------------

float &TFloatList::operator[] (const TIndex &aIndex) {
	bool bValid= false;
	float *temp= (float*)TBaseList::GetAt(aIndex, bValid);
	if ( !bValid || !temp ) { 
		temp= new float;
		*temp= m_DefaultValue;
	}
	return *temp;
}
//-----------------------------------------------------------------------------

bool TFloatList::SetAt(const TIndex &aIndex, const float &aValue) {
	if ( GetCount()>0 && aIndex<GetCount() ) {
		float *temp= (float*)TBaseList::GetAt(aIndex);
		if ( temp ) {
			*temp= aValue;
			return true;
		}
	}
	if ( GetCount()==0 ) TraceErr("INDEX!!! Die Liste ist leer!\nDie angeforderte Operation ist nicht durchfhrbar."); // Liste ist leer
	else if ( aIndex > GetCount()-1 ) TraceErr("INDEX!!! Der angegebene Index ueberschreitet das Listenende!\nDie angeforderte Operation ist nicht durchfhrbar."); // Index ist zu gro
	return false;
}
//-----------------------------------------------------------------------------

bool TFloatList::Insert(const float &aValue) {
	float *temp= new float;
	*temp= aValue;
	bool result= TBaseList::Insert(temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

bool TFloatList::Append(const float &aValue) {
	float *temp= new float;
	*temp= aValue;
	bool result= TBaseList::Append(temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

bool TFloatList::InsBefore(const TIndex &aIndex, const float &aValue) {
	float *temp= new float;
	*temp= aValue;
	bool result= TBaseList::InsBefore(aIndex, temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

bool TFloatList::AddAfter(const TIndex &aIndex, const float &aValue) {
	float *temp= new float;
	*temp= aValue;
	bool result= TBaseList::AddAfter(aIndex, temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

void TFloatList::SetCount(const TIndex &aCount, const float &aDefaultValue) {
	// Liste verkrzen
	if ( GetCount()>aCount ) TBaseList::CutList(aCount); // CutList verringert GetCount() 
	// Liste verlngern		
	else while ( GetCount()<aCount && Append(aDefaultValue) ) /* Append erhht GetCount() */;
}
//-----------------------------------------------------------------------------

void TFloatList::SetCount(const TIndex &aCount) {
	SetCount(aCount, m_DefaultValue);
}

//##############################################################################
// TPoint
//##############################################################################

TPoint::TPoint() : m_Pt(0) {
	m_Pt= new float[s_Size];
	for (TIndex i= 0; i<s_Size; i++) m_Pt[i]= 0.0;
	Valid= FALSE;
}
//-----------------------------------------------------------------------------

TPoint::~TPoint() {
	_FREELIST(m_Pt);
}
//-----------------------------------------------------------------------------

TPoint::TPoint(const TPoint &aPoint) : m_Pt(0) {
	if ( &aPoint==this ) return; // Selbstzuweisung

	m_Pt= new float[s_Size];
	for (TIndex i= 0; i<s_Size; i++) m_Pt[i]= aPoint.m_Pt[i];
	Valid= aPoint.Valid;
}
//-----------------------------------------------------------------------------

TPoint &TPoint::operator= (const TPoint &aPoint) {
	if ( &aPoint==this ) return *this; // Selbstzuweisung

	for (TIndex i= 0; i<s_Size; i++) m_Pt[i]= aPoint.m_Pt[i];
	Valid= aPoint.Valid;
	return *this;
}
//-----------------------------------------------------------------------------

float TPoint::GetAt(const TIndex &aIndex, bool &aValid) const {
	aValid= false;
	if ( aIndex>=s_Size ) { // Index ist zu gro
		TraceErr("INDEX!!! Der angegebene Index ueberschreitet das Listenende!\nDas letzte Element wird fuer die angeforderte Operation verwendet.");
		aValid= false;
	} else aValid= true;
	return m_Pt[ min(aIndex, s_Size-1) ];
}
//-----------------------------------------------------------------------------

float TPoint::GetAt(const TIndex &aIndex) const {
	bool temp;
	return GetAt(aIndex, temp);
}
//-----------------------------------------------------------------------------

float &TPoint::operator[] (const TIndex &aIndex) {
	if ( aIndex>=s_Size ) TraceErr("INDEX!!! Der angegebene Index ueberschreitet das Listenende!\nDas letzte Element wird fuer die angeforderte Operation verwendet."); // Index ist zu gro
	return m_Pt[ min(aIndex, s_Size-1) ];	
}
//-----------------------------------------------------------------------------

bool TPoint::SetAt(const TIndex &aIndex, const float &aValue) {
	if ( aIndex<s_Size ) {
		m_Pt[ aIndex ]= aValue;
		return true;
	} else {
		TraceErr("INDEX!!! Der angegebene Index ueberschreitet das Listenende!\nDie angeforderte Operation ist nicht durchfhrbar."); // Index ist zu gro
		return false;
	}
}

//##############################################################################
// TPointList
//##############################################################################

TPointList::TPointList(const TIndex &aCount, const TIndex &aSize) : TBaseList(true, false, true, false, aSize) {	
	SetCount(aCount);
}
//-----------------------------------------------------------------------------

TPointList::~TPointList() {
	Clear(); // sonst wird nicht TPointList::FreeData verwendet
}
//-----------------------------------------------------------------------------

TPointList::TPointList(const TPointList &aList) : TBaseList(aList.m_FailIfNull, aList.m_FailIfExists, aList.m_AutoDelete, aList.m_CheckOnDel, aList.m_Size), m_DefaultValue(aList.m_DefaultValue) {
	if ( &aList==this ) return; // Selbstzuweisung

	Clear();
	bool bValid= false;
	for (TIndex i=0; i<aList.GetCount(); i++) Append( aList.GetAt(i, bValid) );
}
//-----------------------------------------------------------------------------

TPointList &TPointList::operator= (const TPointList &aList) { 
	if ( &aList==this ) return *this; // Selbstzuweisung

	m_DefaultValue= aList.m_DefaultValue;
	Clear();
	bool bValid= false;
	for (TIndex i=0; i<aList.GetCount(); i++) Append( aList.GetAt(i, bValid) );
	return *this;
}

//-----------------------------------------------------------------------------

TPoint &TPointList::GetAt(const TIndex &aIndex, bool &aValid) const {
	TPoint *temp= (TPoint*)TBaseList::GetAt(aIndex, aValid);
	if ( !aValid && temp ) {
		FreeData(temp);
		temp= 0;
	}
	if ( !aValid || !temp ) temp= new TPoint(m_DefaultValue);
	return *temp;
}
//-----------------------------------------------------------------------------

TPoint &TPointList::GetAt(const TIndex &aIndex) const {
	bool bValid= false;
	return GetAt(aIndex, bValid);
}
//-----------------------------------------------------------------------------

TPoint &TPointList::operator[] (const TIndex &aIndex) const {
	return GetAt(aIndex); 
}
//-----------------------------------------------------------------------------

bool TPointList::Insert(const TPoint &aValue) {
	TPoint *temp= new TPoint(aValue);
	bool result= TBaseList::Insert(temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

bool TPointList::Append(const TPoint &aValue) {
	TPoint *temp= new TPoint(aValue);
	bool result= TBaseList::Append(temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

bool TPointList::InsBefore(const TIndex &aIndex, const TPoint &aValue) {
	TPoint *temp= new TPoint(aValue);
	bool result= TBaseList::InsBefore(aIndex, temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

bool TPointList::AddAfter(const TIndex &aIndex, const TPoint &aValue) {
	TPoint *temp= new TPoint(aValue);
	bool result= TBaseList::AddAfter(aIndex, temp);
	if ( !result ) FreeData(temp);
	return result;
}
//-----------------------------------------------------------------------------

void TPointList::SetCount(const TIndex &aCount, const TPoint &aDefaultValue) {
	// Liste verkrzen
	if ( GetCount()>aCount ) TBaseList::CutList(aCount); // CutList verringert GetCount() 
	// Liste verlngern		
	else while ( GetCount()<aCount && Append(aDefaultValue) ) /* Append erhht GetCount() */;
}
//-----------------------------------------------------------------------------

void TPointList::SetCount(const TIndex &aCount) {
	SetCount(aCount, m_DefaultValue);
}
//-----------------------------------------------------------------------------

void TPointList::FreeData(void *aData) const {
	TPoint *cast= (TPoint*)aData;
	_FREEOBJ(cast); // sonst wird TPoint::~TPoint() nicht gerufen
}