// (C) 1994 by Heiko Damerow MPG AG "Roentgenbeugung"
//#############################################################################
//                                                                           //
// M_STEERG.CPP                                                              //
//                                                                           //
// Subsystem : Ablaufsteuerung - Makroverarbeitung, Kommandos                //
// Deklaration: WORKFLOW.H, M_STEERG.H                                       //
//---------------------------------------------------------------------------//
// letzte Bearbeitung: Thomas Kullmann, Gnther Reinecker                    //
// Stand:   27.07.2002                                                       //
//                                                                           //
//#############################################################################
//## Diese Kommentare (mit zwei Doppelkreuzen) sind von Marlies Gollnick ~:)

#include "internls\evrythng.h" // GermanVersion
#include "winresrc\rc_def.h"   // Ressourcen-IDs
#pragma hdrstop

//#include "datavisa\datavisa.h"
#include "motrstrg\motrstrg.h"
//#include "difrkmty\difrkmty.h"

#include "workflow\workflow.h"
#include "workflow\m_steerg.h"

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//#############################################################################
// global Data
//#############################################################################

extern TMain Main; // in MAIN.CPP deklariertes Hauptprogramm

// Modul-Instance
TSteering Steering; //Datum: 28.07.2002 = TSteering()

// Weiterfhrung der Bewegung nach ca. 6 Sekunden; 30 * 200= 60.000
const UINT TSteering::nAskTicks= 30; // Pausierung der Kommandoverarbeitung [in Millisek.]
//static  const int     nMaxAskNumber= 200;

//bestimmt wie viele Elemente maximal in dDistance und fIntensity enthalten sein sollen
const UINT TGotoIntensityCmd::nLifo= 3;
const UINT TGotoPeakCmd::nLifo= 3;
const UINT TControlFlankCmd::nLifo= 4;

//Datum: 28.07.2002  Groteil nun static in TSteering; Einiges entfernt
/*
 
TDetector  *Monitor= 0;
 
BOOL  bMonitorUsed= FALSE;
 
HGDIOBJ  testobc;
 
DWORD  dwStartTimeTicks;
 
// for interrupt and restart any macro
// in TChooseDetector und TSteering
TExposureSettings  exposure;
 
static  const int    nAskTicks     = 30;
static  const int     nMaxAskNumber= 200;
static  int          nAskIdx;
 
UINT  nEvent;
 
HWND  hHostWindow;
 
extern "C" {
TIMECALLBACK RecallSteering;
}*/

//#############################################################################
// globale Methode aus L_LAYER.CPP
//#############################################################################

// Prototypen
ExScanType ParsingXScanType ( LPSTR xst )
{
	if ( !strcmp( xst, "Standard" ) )
		return stStandardScan;
	if ( !strcmp( xst, "Omega2Theta" ) )
		return stOmega2ThetaScan;
	if ( !strcmp( xst, "Continuous" ) )
		return stContinuousScan;
	if ( !strcmp( xst, "PsdEnergyScan" ) )
		return stPsdEnergyScan;
	if ( !strcmp( xst, "PsdPositionScan" ) )
		return stPsdPositionScan;
	if ( !strcmp( xst, "Omega2ThetaList" ) )
		return stOmega2ThetaScanList;
	if ( !strcmp( xst, "ScanList" ) )
		return stScanList;
	return stUnknown;
};

//#############################################################################
// TCmd
//#############################################################################
//##allgemeine Kommandoklasse

//*****************************************************************************
// Konstruktor-Destruktor
//*****************************************************************************

TCmd::TCmd( TCmdTag ct ) : bMoveOk(0)
{
	bMoveOk= new BOOL [mlGetAxisNumber()];
	memset(bMoveOk, FALSE, mlGetAxisNumber()*sizeof(BOOL));
	eCmdId= ct.Id;
	bPositionValid= TRUE;
	bNoMeasure= FALSE;
	nFailureId= 0;
	eStep= CReady;
};
//-----------------------------------------------------------------------------

TCmd::~TCmd()
{
	Steering.StopTimer();
	_FREELIST(bMoveOk);
};

//*****************************************************************************
// Schritte der Kommandoverarbeitung
//*****************************************************************************

//##Auswahl der naechsten Aktion anhand des Zustandsmerkers eStep
ECmdCode TCmd::DoAction ( void)
{
	switch (eStep)
	{
		case CFirstStep :
			return FirstStep();

		case CControlStep :
			return ControlStep();
		
		case CReadyStep :
			return ReadyStep();
		
		case CReady :
			return Ready();
	}
	MessageBox(GetFocus(), "Unbekannter Schritt", "Meldung", MBINFO);
	return CStop;
}

//*****************************************************************************
// angegebenen Antrieb bewegen
//*****************************************************************************

//##Bewegt angegebenen Motor zur Position dDistance
BOOL TCmd::StartMove (const int mid, double dDistance )
{
	//*** does no change the actual motor
	//##Ausfuehren der Motorbewegung und Ueberpruefung des Rueckgabewertes,
	//##falls erfolgreich, setze Zustandsvariablen entsprechend
	if (mlMoveToDistance(mid, dDistance) )
	{
		bMoveOk[mid]= FALSE;
		bPositionValid= FALSE;
		//Datum: 28.07.2002 nAskIdx= nMaxAskNumber;
		//##warte 20 Millisekunden -> warum wird dies hier benoetigt???
		//##muss nach Motorbewegungen gewartet werden, dann gehoert dies
		//##aber in die Motorfunktionen direkt und nicht hierher
		DelayTime(20);
		return TRUE;
	}
	return FALSE;
};

//*****************************************************************************
// Accessor-Methoden
//*****************************************************************************

ECmdCode TCmd::WakeUp ( void )
{
	//*** does no change the actual motor
	int mid;
	double dd;
	BOOL bAllReady= TRUE;
	for (mid= 0; mid < mlGetAxisNumber(); mid++)
		if ( !bMoveOk[mid] )
		{
			if ( mlIsMoveFinish(mid) )
			{
				DelayTime(5);
				if (mlGetDistance(mid, dd))
					bMoveOk[mid]= TRUE;
				else
					bAllReady= FALSE;
			}
			else
				bAllReady= FALSE;
		}

	if ( bAllReady )
	{
		bPositionValid= TRUE;
		if ( bNoMeasure )
		{
			bNoMeasure= FALSE;
			return DoAction();
		}
		else
			return CMeasure;
	}
	else
		return CRecall;
	/*
	if(nAskIdx > 0)
	return CRecall;

	else
	return DoAction();
	*/
};

//Datum: 26.07.2002
/*
//-----------------------------------------------------------------------------
BOOL  TCmd::DetectorRequest ( void ) {
  return TRUE;
};
//-----------------------------------------------------------------------------
 
LPSTR  TCmd::GetFailure ( void ) {
  return (LPSTR)"Fehler";
};*/

//#############################################################################
// TChooseAxisCmd
//#############################################################################

TChooseAxisCmd::TChooseAxisCmd ( TCmdTag ct ) : TCmd(ct)
{
	bNoMeasure= TRUE;
	if (!mlSetAxis(mlGetIdByName((EAxisType)ct.P1)))
		nFailureId= 1;
	eStep= CReady;
};
//-----------------------------------------------------------------------------

BOOL TChooseAxisCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
		if (nFailureId)
		{
#ifdef GermanVersion
			sprintf(buffer, "\"ChooseAxis\"-Kommando fehlgeschlagen.");
#else

			sprintf(buffer, "\"ChooseAxis\"-Command failed.");
#endif

		}
		else
		{
#ifdef GermanVersion
			sprintf(buffer, "\"ChooseAxis\"-Kommando erfolgreich.");
#else

			sprintf(buffer, "\"ChooseAxis\"-Command ready.");
#endif

		}
	}
	return TRUE;
};

//#############################################################################
// TSetWidthCmd
//#############################################################################

TSetWidthCmd::TSetWidthCmd ( TCmdTag ct ) : TCmd(ct)
{
	bNoMeasure= TRUE;
	mSetValue(Width, atof(ct.P3));
	eStep= CReady;
};
//-----------------------------------------------------------------------------

BOOL TSetWidthCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
#ifdef GermanVersion
		sprintf(buffer, "\"SetWidth\"-Kommando erfolgreich.");
#else

		sprintf(buffer, "\"SetWidth\"-Command ready.");
#endif

	}
	return TRUE;
};

//#############################################################################
// TLoadPointCmd
//#############################################################################

TLoadPointCmd::TLoadPointCmd ( TCmdTag ct ) : TCmd(ct)
{
	bNoMeasure= TRUE;
	switch ((ECmdParam)ct.P1)
	{
		case Argument:
			Steering.dCalcArg[ct.P2]= Steering.dGoalDistance;
			break;

		case Start:   // start
			Steering.dStartPoint= Steering.dGoalDistance;
			break;

		case Peak:   // peak
			Steering.dPeakPoint= Steering.dGoalDistance;
			break;
	}
	eStep= CReady;
};
//-----------------------------------------------------------------------------

BOOL TLoadPointCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
#ifdef GermanVersion
		sprintf(buffer, "\"LoadPoint\"-Kommando erfolgreich.");
#else

		sprintf(buffer, "\"LoadPoint\"-Command ready.");
#endif

	}
	return TRUE;
};

//#############################################################################
// TMoveToPointCmd
//#############################################################################
//##Punkt anfahren verschiedene Moeglichkeiten der Wertangabe werden
//##fuer StartMove "uebersetzt"

TMoveToPointCmd::TMoveToPointCmd ( TCmdTag ct ) : TCmd(ct)
{
	double distance;

	bNoMeasure= TRUE;
	switch (ct.P1)
	{
		case Result:
			StartMove(mlGetAxis(), Steering.dCalcResult);
			break;

		case Peak:
			StartMove(mlGetAxis(), Steering.dPeakPoint);
			break;

		case Start:
			StartMove(mlGetAxis(), Steering.dStartPoint);
			break;

		//##bewegen zur unter dGoalDistance angegebenen Position
		case LastGoal:
			StartMove(mlGetAxis(), Steering.dGoalDistance);
			break;

		//##angegebene Wert ist relativer Wert zum aktuellen Punkt
		case Relative:
			mlGetDistance(mlGetAxis(), distance);
			StartMove(mlGetAxis(), distance + atof(ct.P3));
			break;

		default:
			StartMove(mlGetAxis(), atof(ct.P3));
	}
	eStep= CReadyStep;
};
//-----------------------------------------------------------------------------

BOOL TMoveToPointCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReadyStep )
	{
#ifdef GermanVersion
		sprintf(buffer, "\"MoveToPoint\"-Kommando erfolgreich.");
#else
		sprintf(buffer, "\"MoveToPoint\"-Command ready.");
#endif
	}
	return TRUE;
};

//Datum: 28.07.2002
/*//-----------------------------------------------------------------------------
ECmdCode  TMoveToPoint::ReadyStep ( void ) {
  eStep= CReady;
  return CReady;
};*/

//#############################################################################
// TChooseDetectorCmd
//#############################################################################

TChooseDetectorCmd::TChooseDetectorCmd ( TCmdTag ct ) : TCmd(ct)
{
	float time, fl;
	DWORD counts;

	bNoMeasure= TRUE;
	if ( TDetectorManager::DetectorManager().SetDetector( ct.P1 ) )
	{
		TExposureSettings exposure= TDetectorManager::DetectorManager().GetDetector()->GetExposureSettings();
		sscanf(ct.P3, "%f %ld %f", &time, &counts, &fl);
		//JP new algorithm
		if ( time != 0 )
			exposure.SetExposureTime(time);
		if ( counts != 0 )
			exposure.SetExposureCounts(counts);
		TDetectorManager::DetectorManager().GetDetector()->SetExposureSettings(exposure);
	}
	else
		nFailureId= 1;

	eStep= CReady;
};
//-----------------------------------------------------------------------------

BOOL TChooseDetectorCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
		if (nFailureId)
		{
#ifdef GermanVersion
			sprintf(buffer, "\"ChooseDetector\"-Kommando fehlgeschlagen.");
#else

			sprintf(buffer, "\"ChooseDetector\"-Command failed.");
#endif

		}
		else
		{
#ifdef GermanVersion
			sprintf(buffer, "\"ChooseDetector\"-Kommando erfolgreich.");
#else

			sprintf(buffer, "\"ChooseDetector\"-Command ready.");
#endif

		}
	}
	return TRUE;
};

//#############################################################################
// TGotoIntensityCmd
//#############################################################################

//*****************************************************************************
// Konstruktor-Destruktor
//*****************************************************************************

//berechnet Zielintensitaet und bewegt Motor um zwei Schritte nach links bzw. rechts
TGotoIntensityCmd::TGotoIntensityCmd ( TCmdTag ct ) : TCmd(ct), dDistance(0), fIntensity(0)
{
	nMotor= mlGetAxis();
	bSmallAngleSide= (ct.P1 == SmallSide);
	nReadyAction= (ECmdParam)ct.P2;
	// Bezug auf Vorgeschichte
	//##berechnet anzusteuernde Intensitaet
	fSearchIntensity= Steering.fPeakIntensity * atof(ct.P3);
	//##falls diese Null ist, liegt ein Fehler vor
	if (!fSearchIntensity)
	{
		nFailureId= 11;
		eStep= CReady;
		return;
	}
	dDistance= new double [nLifo];
	fIntensity= new float [nLifo];
	//##bewege Motor um zweifache Schrittweite nach links
	if (bSmallAngleSide)
		StartMove(nMotor, mGetValue(Distance) - 2.0 * mGetValue(Width));
	//##bewege Motor um zweifache Schrittweite nach rechts
	else
		StartMove(nMotor, mGetValue(Distance) + 2.0 * mGetValue(Width));
	eStep= CFirstStep;
};
//-----------------------------------------------------------------------------

TGotoIntensityCmd::~TGotoIntensityCmd()
{
	_FREELIST(dDistance);
	_FREELIST(fIntensity);
};

//*****************************************************************************
// Fortschritt- Kommandoinformtionen
//*****************************************************************************

BOOL TGotoIntensityCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
#ifdef GermanVersion
		sprintf( buffer, "\"GotoIntensity\"-Kommando erfolgreich." );
#else

		sprintf( buffer, "\"GotoIntensity\"-Command ready." );
#endif

	}
	else if ( eStep == CFirstStep )
	{
#ifdef GermanVersion
		sprintf(buffer, "Ziel-Intensitt %.2f", fSearchIntensity);
#else

		sprintf(buffer, "Serachintensitt %.2f", fSearchIntensity);
#endif

	}
	else if ( eStep == CControlStep )
	{
		sprintf(buffer, "D:%.3lf  I:%.2f", dDistance[0], fIntensity[0]);
	}
	return TRUE;
};

//*****************************************************************************
// Schritte der Kommandoverarbeitung
//*****************************************************************************

ECmdCode TGotoIntensityCmd::FirstStep ( void )
{
	eStep= CControlStep;
	//##Intensitaet und Position erfassen
	fIntensity[0]= Steering.GetIntensity();
	dDistance[0]= Steering.GetDistance();
	//##um einen Schritt bewegen
	if (bSmallAngleSide)
		StartMove(nMotor, dDistance[0] - mGetValue(Width));
	else
		StartMove(nMotor, dDistance[0] + mGetValue(Width));
	return CRecall;
};
//-----------------------------------------------------------------------------

ECmdCode TGotoIntensityCmd::ControlStep ( void )
{
	double dDist;
	float idif0, idif1;

	//##alle Intensitaets und Positionswerte um eins verschieben
	memmove((LPSTR)(fIntensity + 1), (LPSTR)fIntensity, (nLifo - 1) * sizeof(float));
	memmove((LPSTR)(dDistance + 1), (LPSTR)dDistance, (nLifo - 1) * sizeof(double));
	//##aktuellen Intensitaets- und Positionswerte bestimmen
	fIntensity[0]= Steering.GetIntensity();
	dDistance[0]= Steering.GetDistance();
	//##Abstand der aktuellen Intensitaet zur Zielintensitaet
	idif0= fSearchIntensity - fIntensity[0];
	//##Intensitaetsabstand eine Runde vorher
	idif1= fSearchIntensity - fIntensity[1];
	//*** Feststellen, ob Level durchschritten wurde
	if ((idif0 * idif1) < 0.0)
	{
		eStep= CReadyStep;
		//##berechnen der endgueltigen Position und anfahren
		switch (nReadyAction)
		{
			case Interpolation:
				dDist= (fSearchIntensity - fIntensity[1]) * dDistance[0];
				dDist += (fIntensity[0] - fSearchIntensity) * dDistance[1];
				dDist /= (fIntensity[0] - fIntensity[1]);
				StartMove(nMotor, dDist);
				break;

			case BackMove:
				StartMove(nMotor, dDistance[1]);
				break;
		}
		return CRecall;
	}

	//##!! Hier wird nach links/rechts von der Flanke unterschieden und einmal
	//##der aktuelle Intensitaetsabstand (idif0) betrachtet (links von der Flanke)
	//##und das andere Mal der Intensitaetsabstand eine Runde zuvor (idif1) (rechts
	//##von der Flanke)-> Hier liegt ein Fehler vor!
	if (bSmallAngleSide)
	{
		if (idif0 > 0.0)
			StartMove(nMotor, dDistance[0] + mGetValue(Width));
		else
			StartMove(nMotor, dDistance[0] - mGetValue(Width));
	}
	else
	{
		if (idif1 > 0)
			StartMove(nMotor, dDistance[0] - mGetValue(Width));
		else
			StartMove(nMotor, dDistance[0] + mGetValue(Width));
	}
	return CRecall;
};
//-----------------------------------------------------------------------------

ECmdCode TGotoIntensityCmd::ReadyStep ( void )
{
	double mm;
	switch (nReadyAction)
	{
		case Interpolation:
			mm= (fIntensity[1] - fIntensity[0]);
			if (!mm)
			{
				Steering.dGoalDistance= dDistance[0];
				Steering.fGoalIntensity= fIntensity[0];
				eStep= CReady;
				return CReady;
			}
			//##Was wird hier berechnet?
			mm= (dDistance[1] - dDistance[0]) * (fSearchIntensity - fIntensity[0]) / mm;
			Steering.dGoalDistance= dDistance[0] + mm;
			Steering.fGoalIntensity= fSearchIntensity;
			eStep= CReady;
			return CReady;

		default:
			Steering.dGoalDistance= dDistance[0];
			Steering.fGoalIntensity= fIntensity[0];
			eStep= CReady;
			return CReady;
	}
};

//#############################################################################
// TGotoPeakCmd
//#############################################################################
//##Flanke suchen

//*****************************************************************************
// Konstruktor-Destruktor
//*****************************************************************************

TGotoPeakCmd::TGotoPeakCmd ( TCmdTag ct ) : TCmd(ct), dDistance(0), fIntensity(0)
{
	bFlankDetected= FALSE;
	bSmallAngleSide= (ToSmallerAngle == ct.P1);
	nReadyAction= (ECmdParam)ct.P2;
	nMotor= mlGetAxis();
	mlGetDistance(nMotor, Steering.dStartPoint);
	dDistance= new double [nLifo];
	fIntensity= new float [nLifo];
	//Datum: 28.07.2002 bDirectionChanged= FALSE;
	//Datum: 28.07.2002 bDynamicStepWidth= FALSE;
	eStep= CFirstStep;
};
//-----------------------------------------------------------------------------

TGotoPeakCmd::~TGotoPeakCmd ()
{
	//Datum: 28.07.2002 bMonitorUsed= FALSE;
	Steering.Monitor= 0;

	_FREELIST(dDistance);
	_FREELIST(fIntensity);
};

//*****************************************************************************
// Fortschritt- Kommandoinformtionen
//*****************************************************************************

BOOL TGotoPeakCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
#ifdef GermanVersion
		sprintf( buffer, "\"GotoPeak\"-Kommando erfolgreich." );
#else

		sprintf( buffer, "\"GotoPeak\"-Command ready." );
#endif

	}
	else if ( eStep == CControlStep )
	{
		sprintf(buffer, "D:%.3lf  I:%.2f", dDistance[0], fIntensity[0]);
	}
	return TRUE;
};

//*****************************************************************************
// Schritte der Kommandoverarbeitung
//*****************************************************************************

//##ermittelt die aktuelle Position und Intensitaet, bewegt Motor um Width
//##nach rechts (bzw. links) wenn wir links (bzw. rechts) vom Peak sind
ECmdCode TGotoPeakCmd::FirstStep ( void )
{
	eStep= CControlStep;
	//##ermittelt die aktuelle Intensitaet
	fIntensity[0]= Steering.GetIntensity();
	//##ermitteln der aktuellen Motorposition (Distance)
	//##Steering.GetDistance() ruft auch nur mGetValue(Distance)
	dDistance[0]= Steering.GetDistance();
	Steering.dStartPoint= dDistance[0];
	//##wenn wir uns links vom Peak befinden, nach rechts bewegen,
	//##sonst nach links
	if (bSmallAngleSide)
		StartMove(nMotor, mGetValue(Distance) + mGetValue(Width));
	else
		StartMove(nMotor, mGetValue(Distance) - mGetValue(Width));
	return CRecall;
};
//-----------------------------------------------------------------------------

ECmdCode TGotoPeakCmd::ControlStep ( void )
{
	float idif0, idif1;
	//##verschiebt alle Werte im Intensitaets- und Distanzfeld um eine Position
	memmove((LPSTR)(fIntensity + 1), (LPSTR)fIntensity, (nLifo - 1) * sizeof(float));
	memmove((LPSTR)(dDistance + 1), (LPSTR)dDistance, (nLifo - 1) * sizeof(double));
	//##ermittelt den aktuellen Intensitaets- und Distanzwert
	fIntensity[0]= Steering.GetIntensity();
	dDistance[0]= Steering.GetDistance();
	//*** Keine Reaktion unterhalb des Rausch-Level
	if (fIntensity[0] < Steering.GetNoiseLevel())
	{
		//##Motor ausserhalb des erlaubten Bereichs
		if (mIsRangeHit())
		{
			//##wechselt die Seite vom Peak, Bewegung zurueck zum Ausgangspunkt
			bSmallAngleSide= !bSmallAngleSide;
			StartMove(nMotor, Steering.dStartPoint);
			return CRecall;
		}
		goto Exit;
	}
	// Variablen belegen
	//##berechnet den Intensitaetsunterschied
	idif0= fIntensity[0] - fIntensity[1];
	//*** Abbruch-Bedingung
	//##falls die Flanke entdeckt wurde, und der letzte Intensitaetsunterschied
	//##negativ (aktuelle Intensitaet ist kleiner als die zu letzt gemessene)
	if (bFlankDetected && (idif0 < 0.0))
	{
		eStep= CReadyStep;
		return CRecall;
	}
	//##hinreichend grosse Intensitaetsaenderung
	if (fabs(idif0) > (3.0 * Steering.GetSigma() * fIntensity[1]))
	{
		bFlankDetected= TRUE;
		//##ist die aktuelle Intensitaet kleiner als die zuletzt gemessene tausche
		//##Seite bzgl. Peak
		if (idif0 < 0.0)
			bSmallAngleSide= !bSmallAngleSide;
		goto Exit;
	}
	//##Intensitaetsunterschied der Messung zuvor
	//##!!!Beim zweiten Messchritt (erste Aufruf des ControlStep)
	//##ist fIntensity[2] unbestimmt, da erst [0] und [1] definierte
	//##Werte erhalten haben
	idif1= fIntensity[1] - fIntensity[2];
	//##hat die Intensitaet bei den beiden letzten Messungen abgenommen,
	//##dann, Flanke entdeckt, Wechsel der Seite bzgl. Flanke
	if ((idif0 < 0.0) && (idif1 < 0.0))
	{
		bFlankDetected= TRUE;
		bSmallAngleSide= !bSmallAngleSide;
		goto Exit;
	}

Exit:
	//##Bewegung auf den Peak zu
	if ( bSmallAngleSide )
		StartMove(nMotor, dDistance[0] + mGetValue(Width));
	else
		StartMove(nMotor, dDistance[0] - mGetValue(Width));
	return CRecall;
};
//-----------------------------------------------------------------------------

ECmdCode TGotoPeakCmd::ReadyStep ( void )
{
	if (bBackMoveActive)
	{
		Steering.dPeakPoint= Steering.dGoalDistance= dDistance[0];
		Steering.fPeakIntensity= Steering.fGoalIntensity= fIntensity[0];
		eStep= CReady;
		return CReady;
	}
	switch ( nReadyAction )
	{
		//##Was ist die Grundlage dieser Berechnung ?
		case Interpolation:
			Steering.dGoalDistance= (fSearchIntensity - fIntensity[1]) * dDistance[0];
			Steering.dGoalDistance += (fIntensity[0] - fSearchIntensity) * dDistance[1];
			Steering.dGoalDistance /= (fIntensity[0] - fIntensity[1]);
			Steering.dPeakPoint= Steering.dGoalDistance;
			Steering.fPeakIntensity= Steering.fGoalIntensity= fIntensity[1];
			eStep= CReady;
			return CReady;

		//##geht zurueck zur letzten Position
		default:   // auch BackMove
			StartMove(nMotor, dDistance[1]);
			bBackMoveActive= TRUE;
			return CRecall;
	}
};

//#############################################################################
// TShowValueCmd
//#############################################################################

TShowValueCmd::TShowValueCmd ( TCmdTag ct ) : TCmd(ct)
{
	char buf[MaxString];

	eStep= CReady;
	bNoMeasure= TRUE;
	switch (ct.P1)
	{
		case Hwb:
			sprintf(buf, "Halbwertsbreite : %.2f arcsec", Steering.GetHwb());
			MessageBox(GetFocus(), buf, "Information", MBINFO);
			break;

		case Peak:
			sprintf(buf, "Peak (D,I) : (%.2lf,%.2f)", Steering.dPeakPoint, Steering.fPeakIntensity);
			MessageBox(GetFocus(), buf, "Information", MBINFO);
			break;

		case Start:
			sprintf(buf, "Startposition : %.2lf", Steering.dStartPoint);
			MessageBox(GetFocus(), buf, "Information", MBINFO);
			break;
	}
};
//-----------------------------------------------------------------------------

BOOL TShowValueCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	/* ! neu Kullmann+Reinecker sprachspezifische Statusinformationen if ( eStep==CReady ) {
	  #ifdef GermanVersion
	    sprintf( buffer, "\"ShowValue\"-Kommando erfolgreich." );
	  #else
	    sprintf( buffer, "\"ShowValue\"-Command ready." );
	  #endif
	}*/ 
	return TRUE;
};

//#############################################################################
// TCalculateCmd
//#############################################################################

TCalculateCmd::TCalculateCmd ( TCmdTag ct ) : TCmd(ct)
{
	bNoMeasure= TRUE;
	float hwb;
	switch ((ECmdParam)ct.P1)
	{
		case Difference:
			Steering.dCalcResult= Steering.dCalcArg[2] - Steering.dCalcArg[1];
			break;

		case Opposite:
			Steering.dCalcResult= 2.0 * Steering.dPeakPoint - Steering.dCalcArg[1];
			break;

		case Hwb:
			hwb= Steering.dCalcArg[2] - Steering.dCalcArg[1];
			if ( (mGetUnit() == utGrad) || (mGetUnit() == utMinuten) )
				hwb *= 60.0; // FEHLER? muss bei Grad nicht mit 3600 multipliziert werden
			Steering.SetHwb( hwb );
			break;

		case Middle:
			Steering.dCalcResult= (Steering.dCalcArg[1] + Steering.dCalcArg[2]) / 2.0;
			break;
	}
	eStep= CReady;
};
//-----------------------------------------------------------------------------

BOOL TCalculateCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
#ifdef GermanVersion
		sprintf(buffer, "\"Calculate\"-Kommando erfolgreich.");
#else

		sprintf(buffer, "\"Calculate\"-Command ready.");
#endif

	}
	return TRUE;
};

//Datum: 28.07.2002
/*//#############################################################################
// TInquireCmd
//#############################################################################
TInquireCmd::TInquireCmd(TCmdTag ct) : TCmd(ct)
{
  eStep= CReady;
  bNoMeasure= TRUE;
};
 
BOOL TInquireCmd::GetShowData(LPSTR buffer) {  
  buffer[0]= 0;
  /* ! neu Kullmann+Reinecker sprachspezifische Statusinformationen if ( eStep==CReady ) {
    #ifdef GermanVersion
      sprintf( buffer, "\"Inquire\"-Kommando erfolgreich." );
    #else
      sprintf( buffer, "\"Inquire\"-Command ready." );
    #endif
  }* /
  return TRUE;
};*/

//#############################################################################
// TControlFlankCmd
//#############################################################################
//##Topo Nachregeln Kommando

//*****************************************************************************
// Konstruktor-Destruktor
//*****************************************************************************

TControlFlankCmd::TControlFlankCmd ( TCmdTag ct ) : TCmd(ct), dDistance(0), fIntensity(0)
{
	nMotor= mlGetAxis();
	bSmallAngleSide= (ct.P1 == SmallSide);
	fControlRange= atof(ct.P3);
	//##neben dem aktuellen Werten sollen die letzten drei Werte gespeichert
	//##werden -> es wird aber nur der aktuelle und der zuletzt gemessene
	//##Werte betrachtet -> nLifo= 2 wuerde ausreichen
	dDistance= new double [nLifo];
	fIntensity= new float [nLifo];
	//## Setzt Steuercode (Zustandsmerker) auf CFirstStep
	eStep= CFirstStep;
};
//-----------------------------------------------------------------------------

TControlFlankCmd::~TControlFlankCmd ()
{
	_FREELIST(dDistance);
	_FREELIST(fIntensity);
};

//*****************************************************************************
// Fortschritt- Kommandoinformtionen
//*****************************************************************************

//##Ausgabe des aktuellen Zustands in der Kontrollzeile anhand des
//##aktuellen Steuer-Codes fuer Commandos (eStep)
BOOL TControlFlankCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
#ifdef GermanVersion
		sprintf( buffer, "\"ControlFlank\"-Kommando erfolgreich." );
#else

		sprintf( buffer, "\"ControlFlank\"-Command ready." );
#endif

	}
	else if ( eStep == CControlStep )
	{
		sprintf(buffer, "D:%.3lf  I:%.2f", dDistance[0], fIntensity[0]);
	}
	return TRUE;
};

//*****************************************************************************
// Schritte der Kommandoverarbeitung
//*****************************************************************************

//##erste Aktion
ECmdCode TControlFlankCmd::FirstStep ( void )
{
	// Feststellen, ob die gewhlten Parameter konsistent sind

	//##setzt Steuercode auf ControlStep
	eStep= CControlStep;
	//##aktuellen Intensitaetswert aus fIncomingIntensity auslesen
	//##fIncomingIntensity wird mit jedem TSteering::DetectorRequest
	//##aktualisiert
	fIntensity[0]= Steering.GetIntensity();
	//##aktuelle Motorposition ruft mGetValue(Distance)
	dDistance[0]= Steering.GetDistance();
	//##Ausgangswerte merken
	fSearchIntensity= fIntensity[0];
	Steering.dStartPoint= dDistance[0];

	// Mess-Zeit, Start-Intensitaet und Regel-Bereich
	//##? Hier steht nichts von Messzeit!

	//##Berechnen der Regelgrenzen
	fMinIntensity= fSearchIntensity * (1.0 - fControlRange / 2.0);
	fMaxIntensity= fSearchIntensity * (1.0 + fControlRange / 2.0);
	fIntensityRange= (fMaxIntensity - fMinIntensity) / 2.0;
	//##Rueckgabe des Commando_Steuercodes CMeasure
	return CMeasure;
};
//-----------------------------------------------------------------------------

//##Pruefen ob Intensitaet im vorgegebenen Bereich und gegebenenfalls
//##nachregeln
ECmdCode TControlFlankCmd::ControlStep ( void )
{
	float steprel, idif0, idif1;
	//##Verschiebt alle gemerkten Intensitaetswerte und Distanzwerte um eins
	memmove((LPSTR)(fIntensity + 1), (LPSTR)fIntensity, (nLifo - 1) * sizeof(float));
	memmove((LPSTR)(dDistance + 1), (LPSTR)dDistance, (nLifo - 1) * sizeof(double));
	//##aktuelle Werte
	fIntensity[0]= Steering.GetIntensity();
	dDistance[0]= Steering.GetDistance();
	//##Berechnet vom aktuellen Wert und vom zuletzt gemessenen Wert die Abweichung
	//##vom Startwert (positive Wert -> Intensitaet ist kleiner, negative Wert ->
	//##Intensitaet ist groesser)
	idif0= fSearchIntensity - fIntensity[0];
	idif1= fSearchIntensity - fIntensity[1];

	//ANFANG: Kullmann+Reinecker (22.04.2003) while(1)-Schleifen durch if-then-else ersetzt

	// Fall 1: Intensitaet liegt im Bereich
	if (fabs(idif0) < fIntensityRange)
	{
		//##ueber Arbeitspunkt bewegt
		if (((idif1*idif0) < 0) && (fabs(idif1) > fIntensityRange))
			mSetValue(Width, 0.75 * mGetValue(Width)); // Schrittweite vermindern
		return CRecall;

		// Fall 2: Nur mit der letzten Messung wurde der Bereich verlassen
	}
	else if (fabs(idif1) < fIntensityRange)
	{
		//##d.h. aktueller Wert liegt ausserhalb des Bereichs
		// nichts zu tun

		// Fall 3: Weiter in falsche Richtung bewegt
	}
	else if (((idif1 * idif0) > 0) && (fabs(idif0) > fabs(idif1)))
	{
		//##aktueller Wert und Wert davor liegen ausserhalb des Bereichs
		//##aktuelle Abweichung ist groesser, als Abweichung im Schritt zuvor

		//##Hier wird zwar etwas berechnet, die Aktion selbst ist aber auskommentiert
		//##-> Was war hier eigentlich geplant?

		// Ist die Flanke bestimmbar ?

		//##Hier wird die Bewegungsrichtung bestimmt, ist steprel negativ,
		//##so wurde nach links bewegt, positiv nach rechts und bei null lag
		//##keine Bewegung vor. Es gibt keine Aussage darueber, ob wir uns
		//##links oder rechts von der Flanke befinden.
		steprel= (dDistance[0] - dDistance[1]);
		steprel= fabs(steprel / mGetValue(Width));
		if (steprel > 0.7)
		{
			// FlankenFlag ndern
			// bSmallAngleSide= !bSmallAngleSide;
		}

		// Fall 4: Bewegung durch Arbeitspunkt und zu weit
	}
	else if ((idif1 * idif0) < 0)
	{
		mSetValue(Width, 0.75 * mGetValue(Width)); // Schrittweite vermindern

	} // Kullmann+Reinecker: Gibt es hier keinen else-Fall mehr? Die Mglichkeiten fr iDif0 und iDif1 sind nicht ausgeschpft!!!
	//ENDE: Kullmann+Reinecker (22.04.2003)

	//! float dummy= mGetValue(Width);
	//##eigentliches Nachregeln
	//##rechts vom Pik -> Intensitaet faellt bei Bewegung nach rechts ab
	if (!bSmallAngleSide)
	{
		//##Intensitaet ist kleiner -> rechts vom Arbeitspunkt
		if (idif1 > 0.0)
			StartMove(nMotor, dDistance[0] - mGetValue(Width)); //bewegt Probe nach links
		//##Intensitaet ist groesser -> links vom Arbeitspunkt
		else
			StartMove(nMotor, dDistance[0] + mGetValue(Width));

		//##links vom Pik -> Intensitaet faellt bei Bewegung nach links ab
	}
	else
	{
		//##Intensitaet ist groesser -> rechts vom Arbeitspunkt
		if (idif1 < 0.0)
			StartMove(nMotor, dDistance[0] - mGetValue(Width)); //bewegt Probe nach links
		//##Intensitaet ist kleiner -> links vom Arbeitspunkt
		else
			StartMove(nMotor, dDistance[0] + mGetValue(Width));
	}
	return CRecall;
};
//-----------------------------------------------------------------------------

//##Abschlusschritt
ECmdCode TControlFlankCmd::ReadyStep ( void )
{
	eStep= CReady;
	return CReady;
};

//#############################################################################
// TSetupScanCmd
//#############################################################################

TSetupScanCmd::TSetupScanCmd ( TCmdTag ct ) : TCmd(ct)
{
	ScanW= (TScanWindow*)Main.ScanWindow;
	if (!ScanW)
	{
		MessageBox(GetFocus(), "Kein Scan-Fenster offen", "Fehler", MBSTOP);
		return;
	}
	ScanW->nMotor= mlGetAxis();
	ScanW->Detector= TDetectorManager::DetectorManager().GetDetector();
	ScanW->eScanType= (ExScanType)ct.P1;
	// Startup Steerng
	Steering.StartUp(ScanW->GetHandle(), mlGetAxis(), ScanW->Detector);
	switch (ScanW->eScanType)
	{
		case stOmega2ThetaScan:
			sscanf(ct.P3, "%lf %lf %lf", &ScanW->dArgumentMin, &ScanW->dArgumentWidth, &ScanW->dArgumentMax);
			break;

		case stStandardScan:
			sscanf(ct.P3, "%lf %lf %lf", &ScanW->dArgumentMin, &ScanW->dArgumentWidth, &ScanW->dArgumentMax);
			break;
	}
	eStep= CReady;
	bNoMeasure= TRUE;
};
//-----------------------------------------------------------------------------

BOOL TSetupScanCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
		if (nFailureId == 1)
		{
#ifdef GermanVersion
			sprintf( buffer, "\"SetupScan\"-Kommando fehlgeschlagen." );
#else

			sprintf( buffer, "\"SetupScan\"-Command failed." );
#endif

		}
		else
		{
#ifdef GermanVersion
			sprintf( buffer, "\"SetupScan\"-Kommando erfolgreich." );
#else

			sprintf( buffer, "\"SetupScan\"-Command ready." );
#endif

		}
	}
	return TRUE;
};

//#############################################################################
// TSetFileNameCmd
//#############################################################################

TSetFileNameCmd::TSetFileNameCmd ( TCmdTag ct ) : TCmd(ct)
{
	eStep= CReady;
	switch (ct.P1)
	{
		case ForScan:
			ScanW= (TScanWindow*)Main.ScanWindow;
			if (ScanW)
				ScanW->SetFileName(ct.P3);
			break;

		case ForAreaScan:
			AreaScanW= (TAreaScanWindow*)Main.ScanWindow;
			if (AreaScanW)
				AreaScanW->SetFileName(ct.P3);
			break;
	}
	bNoMeasure= TRUE;
};
//-----------------------------------------------------------------------------

BOOL TSetFileNameCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
		if (nFailureId == 1)
		{
#ifdef GermanVersion
			sprintf(buffer, "\"SetFileName\"-Kommando fehlgeschlagen.");
#else

			sprintf(buffer, "\"SetFileName\"-Command failed.");
#endif

		}
		else
		{
#ifdef GermanVersion
			sprintf(buffer, "\"SetFileName\"-Kommando erfolgreich.");
#else

			sprintf(buffer, "\"SetFileName\"-Command ready.");
#endif

		}
	}
	return TRUE;
};

//#############################################################################
// TSaveDataCmd
//#############################################################################

TSaveDataCmd::TSaveDataCmd ( TCmdTag ct ) : TCmd(ct)
{
	ScanW= (TScanWindow*)Main.ScanWindow;
	AreaScanW= (TAreaScanWindow*)Main.ScanWindow;
	eStep= CReady;
	switch (ct.P1)
	{
		case ForScan:
			if ( (ScanW) && (!ScanW->SaveFile(atNoAsk)) )
				nFailureId= 1;
			break;

		case ForAreaScan:
			if ( (AreaScanW) && (!AreaScanW->SaveFile(atNoAsk)) )
				nFailureId= 1;
			break;
	}
	bNoMeasure= TRUE;
};
//-----------------------------------------------------------------------------

BOOL TSaveDataCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
		if (nFailureId != 1)
		{
#ifdef GermanVersion
			if (ScanW)
				sprintf(buffer, "Speichere Daten in %s.", ScanW->FileName);
			if (AreaScanW)
				sprintf(buffer, "Speichere Daten in %s.", AreaScanW->FileName);
#else

			if (ScanW)
				sprintf(buffer, "Save Data on %s.", ScanW->FileName);
			if (AreaScanW)
				sprintf(buffer, "Save Data on %s.", AreaScanW->FileName);
#endif

		}
		else
		{
#ifdef GermanVersion
			sprintf( buffer, "\"SaveData\"-Kommando fehlgeschlagen." );
#else

			sprintf( buffer, "\"SaveData\"-Command failed." );
#endif

		}
	}
	return TRUE;
};

//#############################################################################
// TScanCmd
//#############################################################################
//! kommentiert  von Diffr.-gruppe

//*****************************************************************************
// Konstruktor-Destruktor
//*****************************************************************************

TScanCmd::TScanCmd ( TCmdTag ct ) : TCmd( ct ), Scaler(0)
{
	//! gibt fuer areascan im standardscan an ob psd-offset verwendet wird
	bChannelOffset= FALSE;

	//! FIX Fehler 72
	bRestart= FALSE;
	//! Schrittweitensteuerung deaktivieren + Art des Scans aus Kommando holen
	bDynamicStepWidth= FALSE;
	nStepScaleLevels= 0;
	Scaler= NULL;
	eCaller= ct.P1;
	//! abhaengig von  der Art des Scans
	switch ( ct.P1 )
	{
		//! wenn AreaScan-kommando
		case ForAreaScan:
			//! zum Kommando gehoeriges AreaScanFenster ermitteln + wenn keins
			//! existiert dann Fehlermeldung + abbruch
			AreaScanW= (TAreaScanWindow*) Main.AreaScanWindow;
			if ( !AreaScanW )
			{
#ifdef GermanVersion
				MessageBox( GetFocus(), "Kein AreaScan-Fenster offen", "Fehler", MBSTOP );
#else
				MessageBox( GetFocus(), "There is no AreaScan-Window opened", "Failure", MBSTOP );
#endif
				return;
			}
			//! uebernehmen von Werten aus AreaScan + loeschen Datenbasis +
			//! neuzeichnen Fenster
			bAbsorberUsed= AreaScanW->ReportUse.bAbsorber;
			eScanType= AreaScanW->eScanType;
			AreaScanW->New();
			AreaScanW->bMeasurementActive= TRUE;
			AreaScanW->UpdateWnd();
			//Datum: 28.07.2002 bMonitorUsed=  AreaScanW->ReportUse.bMonitor;
			//Datum: 28.07.2002 if ( bMonitorUsed ) Monitor= AreaScanW->Monitor;
			if ( AreaScanW->Monitor != 0 )
				Steering.Monitor= AreaScanW->Monitor;

			switch ( eScanType )
			{
				//! bewegen omega + theta
				case stOmega2ThetaScan:
					//! uebernehmen von Werten aus AreaScan
					bOmega2Theta= TRUE;
					//! berechnen der Min- und Maxwerte incl. offset
					dArgumentMin= AreaScanW->dOmegaMin - AreaScanW->dOffsetOmega;
					dArgumentWidth= AreaScanW->dOmegaWidth;
					dArgumentMax= AreaScanW->dOmegaMax - AreaScanW->dOffsetOmega;
					dMoveRelation= AreaScanW->dMoveRelation;
					dThetaMin= AreaScanW->d2ThetaStart - AreaScanW->dOffsetTheta + AreaScanW->dPsdOffset;
					break;

				//! nur omega bewegen
				case stStandardScan:
					//! uebernehmen von Werten aus AreaScan
					//! uebernehmen der 2Thetaposition ,die waehrend des scans
					//! beibehalten werden soll
					dThetaMin= AreaScanW->d2ThetaStart;
					bChannelOffset= mlIsAxisValid(Theta);
					if ( AreaScanW->nMeasurementChannel != 0 )
						dThetaMin += AreaScanW->dPsdOffset;
					bOmega2Theta= FALSE;
					//! berechnen der Min- und Maxwerte incl. offset
					dArgumentMin= AreaScanW->dOmegaMin - AreaScanW->dOffsetOmega;
					dArgumentWidth= AreaScanW->dOmegaWidth;
					dArgumentMax= AreaScanW->dOmegaMax - AreaScanW->dOffsetOmega;
					break;

				default:
#ifdef GermanVersion
					MessageBox( GetFocus(), "Scan-Start fehlgeschlagen.", "Steuerung", MBSTOP );
#else
					MessageBox( GetFocus(), "Start Scan failed.", "Steering", MBSTOP );
#endif
			} // switch (eScanType)
			break; //ForAreaScan

		//! wenn Stepscan
		case ForScan:
			//! zum Kommando gehriges StepScanFenster ermitteln + wenn keins
			//! existiert, dann Fehlermeldung + abbruch
			ScanW= (TScanWindow*) Main.ScanWindow;
			if ( !ScanW )
			{
#ifdef GermanVersion
				MessageBox( GetFocus(), "Kein Scan-Fenster offen", "Fehler", MBSTOP );
#else
				MessageBox( GetFocus(), "There is no Scan-Window opened", "Failure", MBSTOP );
#endif
				return;
			}
			//! FIX Fehler 15
			//Datum: 28.07.2002 bMonitorUsed= ScanW->bMonitorUsed;
			//Datum: 28.07.2002 if ( bMonitorUsed ) Monitor= TDList::GetDetectorListPtr()->GetDetectorPtr(ScanW->Monitor->GetId());
			if ( ScanW->Monitor != 0 )
				Steering.Monitor= ScanW->Monitor;

			//! Absorber deaktivieren (nur bei Areascan)
			bAbsorberUsed= FALSE;
			//! Werte fr dynamische Schrittweitensteuerung aus Scanfenster holen
			bDynamicStepWidth= ScanW->bDynamicStepWidth;
			nStepScaleLevels= ScanW->nStepScaleLevels;
			Scaler= &ScanW->Scaler;
			//! eingestellten Scantyp ermitteln (Standard oder Omega2Theta)
			eScanType= ScanW->eScanType;
			//! Maincurve lschen, Fenster neuzeichnen ...
			ScanW->New();
			//! Messung als nicht unterbrochen und aktiv kennzeichnen
			ScanW->bInterrupted= FALSE;
			ScanW->bMeasurementActive= TRUE;
			//! neu zeichnen
			ScanW->UpdateWnd();
			//! abhaengig von eingest. Scantyp
			switch ( eScanType )
			{
				//! wenn Omega2ThetaScan
				case stOmega2ThetaScan:
					//! Omega2ThetaScan als aktiv kennzeichnen + Parameter wie
					//! Startwinkel,Endwinkel und normale Schrittweite des Omegamotors
					//! fuer den Scan aus Scanfenster holen + schrittverhaeltnis von Om.
					//! zu Theta auf 2 setzen
					bOmega2Theta= TRUE;
					dArgumentMin= ScanW->dArgumentMin;
					dArgumentWidth= ScanW->dArgumentWidth;
					dArgumentMax= ScanW->dArgumentMax;
					dMoveRelation= 2.0;
					//! 2Thetastartpos. aus Scanfenster
					dThetaMin= ScanW->d2ThetaStart;
					break;

				//! wenn Standard-scan
				case stStandardScan:
					//! Omega2ThetaScan als inaktiv kennzeichnen + Parameter wie
					//! Startwinkel, Endwinkel und normale Schrittweite des eingest.
					//! motors fuer den Scan aus Scanfenster holen
					bOmega2Theta= FALSE;
					dArgumentMin= ScanW->dArgumentMin;
					dArgumentWidth= ScanW->dArgumentWidth;
					dArgumentMax= ScanW->dArgumentMax;
					break;

				//! unbekannter Scantyp(weder Standard- noch Omega2ThetaScan)
				default:
#ifdef GermanVersion
					MessageBox( GetFocus(), "Scan-Start fehlgeschlagen.", "Steuerung", MBSTOP );
#else
					MessageBox( GetFocus(), "Start Scan failed.", "Steering", MBSTOP );
#endif
			} // switch ( eScanType )
			break; //ForScan

		//! unbekannte Scanart(weder Step- noch AreaScan)
		default:
#ifdef GermanVersion
			MessageBox( GetFocus(), "Scan-Start", "Fehler", MBSTOP );
#else
			MessageBox( GetFocus(), "Start Scan", "Steering", MBSTOP );
#endif
			eStep= CReady;
			bNoMeasure= TRUE;
			return;
	} // switch ( ct.P1 )

	//! aktiven motor + aktive Pos. des Motors bestimmen
	nMotor= mlGetAxis();  // Startup
	mlGetDistance( nMotor, Steering.dStartPoint );
	//! motor zum Startwinkel des Scans bewegen + dieser ist erster Messpunkt
	StartMove( nMotor, dArgumentMin );
	dArgument= dArgumentMin;
	//! bei Om.2Th. akt. Winkel des Thetamotors bestimmen + auf Startpos. bewegen
	//! bei psd-offset im areascan(standardmodus) nur auf startposition fahren
	if ( bOmega2Theta || bChannelOffset )
	{
		nTheta= mlGetIdByName( Theta );
		mlGetDistance( nTheta, dStartPointTheta );
		//! Thetaschrittweite berechnen
		dThetaWidth= dArgumentWidth * dMoveRelation;
		StartMove( nTheta, dThetaMin );
		dArgument2= dThetaMin;
	}
	//! ?
	//! wenn absorber benutzt wird (nur AreaScan) dann aktuelle Pos. des
	//! Absorbermotors ermitteln und ihn auf Pos. 0.0 bewegen
	if ( bAbsorberUsed )
	{
		nAbsorber= mlGetIdByName( Absorber );
		//21.04.2004 bAbsorberMovedIn= FALSE;
		//21.04.2004 bAbsorberActivated= FALSE;
		mlGetDistance( nAbsorber, dStartPointAbsorber );
		StartMove( nAbsorber, 0.0 );
	}
	//! Ausgabe in Statuszeile + akt. Schritt des Scans auf ersten Schritt
	//! setzen + Anzahl der Voraussichtlich (ohne dyn. Schrittweite) zu
	//! erwartende Messpunkte
#ifdef GermanVersion
	SetInfo( "Anfahren der Startposition ..." );
#else

	SetInfo( "Move to startposition ..." );
#endif
	eStep= CFirstStep;
	nPointNumber= ( dArgumentMax - dArgumentMin ) / dArgumentWidth;
	bNoMeasure= TRUE;
	dwStartTimeTicks= 0;
};

//*****************************************************************************
// Fortschritt- Kommandoinformtionen
//*****************************************************************************

//! realisiert die Ausgaben in die Statusleiste whrend der Scan laeuft
//! (sollte eigentlich nur den Ausg.string liefern die ausgabe sollte dann mit
//! SendReport() getaetigt werden)
//! 1. Param. Buffer enhlt nach aufruf den ausgegeben String der Statusleiste
//! Rueckkehrcode immer true
BOOL TScanCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	float fTimeRemaining;
	char fmt[ MaxString ];
	DWORD dwElapsedTicks;

	//! wenn Scan fertig
	if ( eStep == CReady )
	{
#ifdef GermanVersion
		sprintf( buffer, "Scan abgeschlossen" );
#else

		sprintf( buffer, "Scan Ready" );
#endif
		//Datum: 24.07.2002  SetInfo( buffer );  //!? scheinbar ueberfluessig (siehe Fkt.beschreibung)

		//! wenn Scan laeuft
	}
	else if ( eStep == CControlStep )
	{
		// calculate the remaining time
		//! ermittelt zeit seit scanstart
		dwElapsedTicks= GetTickCount() - dwStartTimeTicks;
		//! bei ersten schritt verbleibende zeit =0.0 danach jeweils berechnen
		//! wieviel zeit bisher pro Messpunkt gebraucht wurde und umrechnen ,
		//! wieviel dann fr die folgenden Mepunkte noch gebraucht wird
		//! (funktioniert nicht bei dynam. Schrittweitensteuerung
		if (nPointIdx)
		{
			fTimeRemaining= ( dwElapsedTicks / 60000.0 ) / nPointIdx;
			fTimeRemaining *= ( nPointNumber - nPointIdx + 1 );
		}
		else
			fTimeRemaining= 0.0;

#ifdef GermanVersion
		sprintf( fmt, "Scan ...  Winkel= " );
		strcat( fmt, mGetDF() );
		strcat( fmt, " Intensitt= %.3f" );
		//! Aenderung Dynamische Schrittweite
		//! Ausgabe der verbleibenden Zeit nur, wenn keine dyn. Schrittweite
		if (!bDynamicStepWidth)
			strcat( fmt, " verbl. Zeit= %.1f Min." );
#else
		sprintf( fmt, "Scan ...  Angle= " );
		strcat( fmt, mGetDF() );
		strcat( fmt, " Intensity= %.3f" );
		//! Aenderung Dynamische Schrittweite
		//! Ausgabe der verbleibenden Zeit nur, wenn keine dyn. Schrittweite
		if (!bDynamicStepWidth)
			strcat( fmt, " Time rem.= %.1f min." );
#endif
		//! string mit Param. fllen
		sprintf( buffer, fmt, mGetValue( Distance ), Steering.GetIntensity(), fTimeRemaining );
		//Datum: 24.07.2002  SetInfo( buffer );  //!? scheinbar ueberfluessig (siehe Fkt.beschreibung)
		//Datum: 24.07.2002  buffer[ 0 ]= 0;    //!? scheinbar ueberfluessig (siehe Fkt.beschreibung)
	}
	return TRUE;
};

//*****************************************************************************
// Schritte der Kommandoverarbeitung
//*****************************************************************************

//! aufruf aus DoAction() bei erstem Schritt des Scans
//! Rueckkehrcode immer CMeasure
ECmdCode TScanCmd::FirstStep ( void )
{
	//! akt. Schritt= Controlschritt + anzahl bereits gemessener Punkte auf 0 +
	//! akt. Zeitwert  sichern
	eStep= CControlStep;
	nPointIdx= 0;
	dwStartTimeTicks= GetTickCount();
	return CMeasure;
};
//-----------------------------------------------------------------------------

//! aufruf aus DoAction() waehrend Scan
//! Rueckkehrcode immer CRecall
ECmdCode TScanCmd::ControlStep ( void )
{
	float fStepScaler= 1.0, fIntensity, fNorm;
	int nScaleLevel= 0;

	//! wenn dyn. Schrittweite eingestellt dann vergleichen der gemessenen
	//! Intensitaet mit den Intensitaetswerten aus Scaler bei erstem
	//! Scaler-int.-wert, der groesser ist, wird die zu diesem scaler-int.-wert
	//! gehoerig Schrittweite als Schrittweitenfaktor festgelegt ansonsten
	//! bleibt dieser Faktor auf 1.0
	//! FIX Fehler 72 Diff./Refl.
	if (!bRestart)
	{
		if ( bDynamicStepWidth )
		{
			fIntensity= Steering.GetIntensity();
			//! auch die Scalerwerte muessen durch Monit.int. normiert werden
			fNorm= ( Steering.GetNorm() != 0.0 ) ? Steering.GetNorm() : 1.0;
			while ( nScaleLevel < nStepScaleLevels )
			{
				if ( fIntensity > ( Scaler->GetAt(nScaleLevel) / fNorm ) )
				{
					nScaleLevel++;
					continue;
				}
				fStepScaler= Scaler->GetAt(nScaleLevel + nMaxScaleIdx);
				break;
			}
		}

		//! wenn aktuelle Pos. des motors noch kleiner als endwinkel des Scans dann
		//! anzahl gemessener Punkte erhoehen + naechsten Mepunkt des Motors
		//! berechnen (dabei schrittweite beachten) + anfahren dieses Punktes
		//! bei Omega2Theta den Thetamotor auf neue pos. bewegen
		//! FIX Fehler 75
		if ( Steering.GetDistance() < dArgumentMax )
		{
			nPointIdx++;
			dArgument += dArgumentWidth * fStepScaler;
			StartMove( nMotor, dArgument );
			if ( bOmega2Theta )
			{
				dArgument2 += dThetaWidth * fStepScaler;
				StartMove( nTheta, dArgument2 );
			}
			return CRecall;
		}
	}
	else
	{ //! FIX Fehler 72 Diff./Refl.
		bRestart= FALSE;
		StartMove( nMotor, dArgument );
		if ( bOmega2Theta)
			StartMove( nTheta, dArgument2 );
		return CRecall;
	}

	//! wenn endwinkel erreicht (bzw. darueber hinaus) dann naechsten Schritt als
	//! Endschritt kennzeichnen + aktiven Motor (und ev. Theta)auf Position vor
	//! start des Scan zurueckbewegen
	eStep= CReadyStep;
	bNoMeasure= TRUE;
	StartMove( nMotor, Steering.dStartPoint );
	if ( bOmega2Theta || bChannelOffset)
		StartMove( nTheta, dStartPointTheta );
	return CRecall;
};
//-----------------------------------------------------------------------------

//! aufruf aus DoAction() wenn scan fertig ist
//! Rueckkehrcode immer CRecall
ECmdCode TScanCmd::ReadyStep ( void )
{
	//! naechster schritt soll scan beenden (wird ReadyReaction aufrufen)
	eStep= CReady;
	switch ( eCaller )
	{
		//! bei Stepscan die messung als nicht aktiv kennzeichnen
		case ForScan:
			ScanW->bMeasurementActive= FALSE;
			break;
	}
	return CRecall;
};

//#############################################################################
// TAreaScanCmd
//#############################################################################
//! kommentiert von Diffr.gruppe

//*****************************************************************************
// Konstruktor-Destruktor
//*****************************************************************************

TAreaScanCmd::TAreaScanCmd ( TCmdTag ct ) : TCmd( ct )
{
	bRestart= FALSE;
	//! wenn kein Areascanfenster geoeffnet dann Fehlerausgabe + abbruch
	AreaScanW= (TAreaScanWindow*) Main.AreaScanWindow;
	if ( !AreaScanW )
	{
#ifdef GermanVersion
		MessageBox( GetFocus(), "Kein AreaScan-Fenster offen", "Fehler", MBSTOP );
#else
		MessageBox( GetFocus(), "There is no AreaScan-Window opened", "Failure", MBSTOP );
#endif
		return;
	}
	//! uebernehmen von Werten aus AreaScan
	nOmega= AreaScanW->nOmega;
	nTheta= AreaScanW->nTheta;
	AreaScanW->bMeasurementActive= TRUE;
	//! min- und maxwerte fuer Omega und Theta berechnen (incl. offset)
	dOmegaMin= AreaScanW->dOmegaMin - AreaScanW->dOffsetOmega;
	dOmegaMax= AreaScanW->dOmegaMax - AreaScanW->dOffsetOmega;
	dOmegaWidth= AreaScanW->dOmegaWidth;
	dThetaWindow= AreaScanW->dThetaWindow;
	dThetaWidth= AreaScanW->dThetaWidth;
	dMoveRelation= AreaScanW->dMoveRelation;
	//! 2thetastartposition aus Scanfenster
	d2ThetaStart= AreaScanW->d2ThetaStart - AreaScanW->dOffsetTheta;

	//! Anzahl der Omega- und Thetaschritte
	nMaxOmegaPoint= ( dOmegaMax - dOmegaMin ) / dOmegaWidth;
	nMaxThetaPoint= dThetaWindow / dThetaWidth;
	//! akt. Omega- und Thetaschritt auf 0 setzen
	nOmegaPoint= nThetaPoint= 0;
	//! akt. Omegapos. sichern + zum omegastartpunkt fahren
	mlGetDistance( nOmega, Steering.dStartPoint );
	dOmega= dOmegaMin;
	StartMove( nOmega, dOmegaMin );
	//! akt. Thetapos. sichern + zum Thetastartpunkt fahren
	mlGetDistance( nTheta, dStartPointTheta );
	dTheta= d2ThetaStart - dThetaWindow / 2.0;
	StartMove( nTheta, dTheta );
	//! loeschen der Datenbasis + Fenster neuzeichnen + Info in Statuszeile
	AreaScanW->New();
	AreaScanW->UpdateWnd();
#ifdef GermanVersion
	SetInfo( "Anfahren der Startposition ..." );
#else
	SetInfo( "Move to startposition ..." );
#endif
	//! akt. Schritt setzen
	eStep= CFirstStep;
	bNoMeasure= TRUE;
};

//*****************************************************************************
// Fortschritt- Kommandoinformtionen
//*****************************************************************************

//! realisiert die Ausgaben in die Statusleiste waehrend der Scan laeuft
//! (sollte eigentlich nur den Ausg.string liefern die ausgabe sollte dann mit
//! SendReport() getaetigt werden)
//! 1. Param. Buffer enhaelt nach aufruf den ausgegeben String der Statusleiste
//! Rueckkehrcode immer true
BOOL TAreaScanCmd::GetShowData ( LPSTR buffer )
{
	buffer[0]= 0;
	if ( eStep == CReady )
	{
#ifdef GermanVersion
		strcpy( buffer, "Scan abgeschlossen" );
#else

		strcpy( buffer, "Scan Ready" );
#endif

	}
	else if ( eStep == CControlStep )
	{
		char fmt[ MaxString ];

#ifdef GermanVersion

		sprintf( fmt, "Scan luft ...  Winkel= " );
		strcat( fmt, mGetDF() );
		strcat( fmt, " Intensitt= %.3f" );
#else

		sprintf( fmt, "Measurement ...  Angle= " );
		strcat( fmt, mGetDF() );
		strcat( fmt, " Intensity= %.3f" );
#endif

		sprintf( buffer, fmt, mGetValue( Distance ), Steering.GetIntensity() );
		//Datum: 24.07.2002  SetInfo( buffer );
	}
	return TRUE;
};

//*****************************************************************************
// Schritte der Kommandoverarbeitung
//*****************************************************************************

//! aufruf aus DoAction() bei erstem Schritt des Scans
//! Rueckkehrcode immer CMeasure
ECmdCode TAreaScanCmd::FirstStep ( void )
{
	//! akt. Schritt= Controlschritt + anzahl bereits gemessener Punkte auf 0
	eStep= CControlStep;
	bNoMeasure= FALSE;
	return CMeasure;
};
//-----------------------------------------------------------------------------

//! aufruf aus DoAction() waehrend Scan
//! Rueckkehrcode immer CRecall
ECmdCode TAreaScanCmd::ControlStep ( void )
{
	//! FIX Fehler 72 Diff/Refl
	if (bRestart)
	{
		bRestart= FALSE;
		StartMove( nOmega, dOmega );
		StartMove( nTheta, dTheta );
		return CRecall;
	}
	//! wenn letzer Omegaschritt erreicht dann akt. Schritt auf fertig setzen und
	//! Omega- und Theta auf ursprungsposition zurueckfahren
	//! FIX Fehler 31,54
	if ( nThetaPoint >= nMaxThetaPoint )
		if ( nOmegaPoint >= nMaxOmegaPoint )
		{
			eStep= CReadyStep;
			bNoMeasure= TRUE;
			StartMove( nOmega, Steering.dStartPoint );
			StartMove( nTheta, dStartPointTheta );

			//! wenn letzter Thetaschritt erreicht dann dieses ans AreaScanfenster
			//! weitergeben, naechste Omegapos. anfahren + naechste Thetapos. berechnen
			//! und anfahren
		}
		else
		{
			AreaScanW->bThetaScanCompleted= TRUE;
			dOmega += dOmegaWidth;
			nOmegaPoint++;
			StartMove( nOmega, dOmega );
			nThetaPoint= 0;
			d2ThetaStart += dOmegaWidth * dMoveRelation;
			dTheta= d2ThetaStart - dThetaWindow / 2.0;

			StartMove( nTheta, dTheta );

			// Sind die bei Borland bld? Das ist doch ein ELSE-Zweig zu viel! Wann soll der denn eintreten?
		}
	else
	{
		//! naechste Thetapos. anfahren
		nThetaPoint++;
		dTheta += dThetaWidth;
		StartMove( nTheta, dTheta );
	}
	return CRecall;
};
//-----------------------------------------------------------------------------

//! aufruf aus DoAction() wenn scan fertig ist
//! Rueckkehrcode immer CRecall
ECmdCode TAreaScanCmd::ReadyStep ( void )
{
	eStep= CReady;
	return CRecall;
};

//#############################################################################
// TSteering
//#############################################################################

//*****************************************************************************
// KONSTRUKTOR-DESTRUKTOR
//*****************************************************************************

TSteering::TSteering ( void )
{
	// Fuer Liste der aufrufenden Macros
	TheMacro= 0;
	TheCmd= 0;
	nCmdIdx= -1;
	nMacroNumber= 0;
	bSendReport= FALSE;
	hReportList= NULL;
	fNoiseLevel= 300;
	bSteeringActive= FALSE;
	//! neu Kullmann+Reinecker
	bReset= FALSE;
	bInterrupted= FALSE;
	dPeakPoint= 0;
	dStartPoint= 0;
	for (int i= 0; i < nMaxArg; i++)
		dCalcArg[i]= 0;
	fHWB= 0;
	dCalcResult= 0;
	dGoalDistance= 0;
	fIncomingIntensity= 0;
	fPeakIntensity= 0;
	fGoalIntensity= 0;
	nEvent= 0; //Timer ist inaktiv
	hHostWindow= 0;
	Monitor= 0;
	//Datum: 28.07.2002 bTimerActive    = FALSE;
};

// Testgruppe 21.10.2002
TSteering::~TSteering( void )
{
	Reset();
	_FREEOBJ(TheCmd);
	_FREEOBJ(TheMacro);
}

//*****************************************************************************
// Makros aus Dateien laden
//*****************************************************************************

//##laedt Standardmakros ("STANDARD.MAK")
BOOL TSteering::Initialize ( const HWND hwnd, const int mid, TDetector *sensor )
{
	Visualising( FALSE, FALSE );
	StartUp(hwnd, mid, sensor);
	// ! neu Kullmann+Reinecker: LoadMacro ermittelt jetzt selbststaendig den vollstndigen Path
	if ( !LoadMacro("InquireHwb", "standard.mak") )
		return FALSE;
	if ( !LoadMacro("SearchReflection", "standard.mak") )
		return FALSE;
	if ( !LoadMacro("SetupTopography", "standard.mak") )
		return FALSE;
	if ( !LoadMacro("Test", "standard.mak") )
		return FALSE;
	if ( !LoadMacro("AzimutalJustify", "standard.mak") )
		return FALSE;
	return TRUE;
};
//-----------------------------------------------------------------------------

// ldt Makros aus "SCAN.MAK"
BOOL TSteering::LoadMacroByUser ( void )
{
	// ! neu Kullmann+Reinecker: LoadMacro ermittelt jetzt selbststaendig den Path
	if ( !LoadMacro( "ScanJob", "scan.mak" ) )
		return FALSE;
	if ( !LoadMacro( "AreaScanJob", "scan.mak" ) )
		return FALSE;
	return TRUE;
};

//*****************************************************************************
// Kommando-/ Makroverarbeitung vorbereiten
//*****************************************************************************

//##Setzt das Bezugsfenster, aktiviert Motorachse mid, setzt Zaehler auf
//##sensid
BOOL TSteering::StartUp ( const HWND hwnd, const int mid, TDetector *sensor )
{
	hControlWnd= hwnd;
	mlSetAxis(mid);
	Detector= sensor;
	return TRUE;
};
//-----------------------------------------------------------------------------

//## setzt einige Werte, die sich auf spaeter Reportausgaben beziehen
void TSteering::Visualising ( BOOL a, BOOL f )
{
	bSendReport= a;

	//##die naechsten vier Variablen werden hier zwar mit Werten belegt, aber
	//##im weiteren Programmverlauf nicht verwendet
	//Datum: 28.07.2002 b_id_Angle         = b;
	//Datum: 28.07.2002 b_id_PeakIntensity = c;
	//Datum: 28.07.2002 b_id_Intensity     = d;
	//Datum: 28.07.2002 b_id_HalfWidth     = e;
	//##b_id_Report wird nur fuer nachfolgende Zuweisung von hReportList
	//##verwendet
	//Datum: 28.07.2002 b_id_Report        = f;
	hReportList= (f) ? GetDlgItem(hControlWnd, id_Report) : NULL;
};

//*****************************************************************************
// Kommando-/ Makroverarbeitung starten
//*****************************************************************************

BOOL TSteering::StartMacroExecution ( TMacroTag *Macro, HWND host )
{
	// Function tested; Last changes 950814
	if ( !Macro )
		return FALSE; //! Absturzursache beseitigt

	if ( !Macro->bIsReady && !bInterrupted )
	{
		MessageBox(GetFocus(), "Das laufende Macro erst unterbrechen !", "Meldung", MBINFO);
		return FALSE;
	}

	Detector->MeasureStop();
	Detector->SetControlWnd(hControlWnd);
	hHostWindow= host;
	TheMacro= Macro;
	if ( (TheMacro) && (TheMacro->Id == InquireHwb) )
		fHWB= 0; // !neu Kullmann+Reinecker, Halbwertsbreite in diesem Makro NOCH nicht berechnet
	nCmdIdx= 0;
	TheCmdList= Macro->CmdList;
	StartCmdExecution(TheCmdList[nCmdIdx]);
	return TRUE;
};
//-----------------------------------------------------------------------------

//##setzt die uebergebenen Parameter in die Kommandostruktur ein und
//##ruft TSteering::StartCmdExecution mit dieser Kommandostruktur auf
int TSteering::StartCmdExecution ( ECmdId cmdid, int p1, int p2, LPSTR p3, HWND host )
{
	TCmdTag ct;

	hHostWindow= host;
	ct.Id= cmdid;
	ct.P1= (ECmdParam)p1;
	ct.P2= (ECmdParam)p2;
	strcpy(ct.P3, p3);
	return StartCmdExecution(ct);
}

//*****************************************************************************
// Kommando-/ Makroverarbeitung pausieren/ fortsetzen, stoppen
//*****************************************************************************

// Makroverarbeitung abbrechen, kann nicht mehr fortgesetzt werden
BOOL TSteering::Reset ( BOOL aDoNotify )
{
	if (Detector)
		Detector->MeasureStop(); //! neu Kullmann+Reinecker: Absturzursache beseitigt
	StopTimer();
	if ( hHostWindow && aDoNotify )
		SendMessage(hHostWindow, WM_COMMAND, cm_SteeringReset, 0l);
	_FREEOBJ(TheCmd);
	TheMacro= 0; // delete von TheMacro fhrt zur Schutzverletzung
	nCmdIdx= -1;
	bSteeringActive= FALSE;
	bInterrupted= FALSE;
	bReset= TRUE;
	return TRUE;
};
//-----------------------------------------------------------------------------

BOOL TSteering::ToggleInterrupt ( void )
{
	static int nOldMotor; //! FIX Fehler 79
	// Function tested; Last changes 950814
	if (bInterrupted)
	{ // restart the action
		//! FIX Fehler 72 Diff/Refl
		if ( TheCmd ) TheCmd->bRestart= TRUE;
		bInterrupted= FALSE;
		Detector->MeasureStop();
		Detector->SetControlWnd( hControlWnd );
		// Meeinstellungen erneut laden
		Detector->SetExposureSettings( exposure );
		//! FIX Fehler 72 Diff/Refl
		if ( TheCmd ) switch (TheCmd->GetId())
		{
			case Scan:
			case AreaScan:
				mlSetAxis(nMotor= nOldMotor); //! FIX Fehler 79
				DetectorRequest();
				break;

			default:
				Detector->MeasureStart();
		};
		bSteeringActive= TRUE;

		// interrupt the action
	}
	else
	{ // bInterrupted==FALSE
		nOldMotor= mlGetAxis(); //! FIX Fehler 79
		Detector->MeasureStop();
		StopTimer();
		Detector->SetControlWnd(NULL);
		bSteeringActive= FALSE;
		exposure= Detector->GetExposureSettings();
		Detector->PopSettings();
		Detector->MeasureStart();
		bInterrupted= TRUE;
	}
	bReset= FALSE;
	return bInterrupted;
};

//*****************************************************************************
// Schnittstelle zu anderen Subsystemen
//*****************************************************************************

//##fragt den aktuellen Intensitaetswert ab; je nach Kommandozustand
//##unterschiedliche Akionen
BOOL TSteering::DetectorRequest ( void )
{
	// Function tested; Last changes 950814
	//##aktuellen Intensitaetswert abfragen
	Detector->GetIntegral(fIncomingIntensity);
	if (fIntensityNorm != 0.0)
		fIncomingIntensity /= fIntensityNorm;

	//##naechsten Schritt des aktuellen Kommandos auswaehlen
	switch (TheCmd->DoAction())
	{
		case CMeasure:
			//if (bSendReport) SetStatus("Measure");
			Detector->MeasureStart();
			if ( Steering.Monitor )
				Steering.Monitor->MeasureStart();
			break;

		case CRecall:
			// if (bSendReport) SetStatus("Recall");
			StartTimer(1);
			break;

		case CReady:
			ReadyReaction();
			break;
	}
	return TRUE;
};
//-----------------------------------------------------------------------------

void TSteering::WakeUp ( void )
{
	// Function tested; Last changes 950814
	if ( TheCmd->IsReady() )
	{
		ReadyReaction();
		return;
	}
	switch ( TheCmd->WakeUp() )
	{
		case CMeasure:
			if (TheCmd->GetShowData(MessageBuffer))
				SendReport();
			// if ( bSendReport ) SetStatus("Measure");
			Detector->MeasureStart();
			if ( Steering.Monitor )
				Steering.Monitor->MeasureStart();
			break;

		case CRecall:
			//if ( bSendReport ) SetStatus("Recall");
			StartTimer(1);
			break;

		case CReady:
			ReadyReaction();
			break;
	}
	return;
};

//*****************************************************************************
// Makroliste bearbeiten: Makros auslesen, lschen und Makroanzahl ermitteln
//*****************************************************************************

TMacroTag *TSteering::GetMacroById ( EMacroId MacId )
{
	// Function tested; Last changes 950811
	for (int idx= 0; idx < nMacroNumber; idx++)
		if ( MacId == aMacroList[idx].Id )
		{
			if ( aMacroList[idx].bIsReady )
				return &aMacroList[idx];
			else
				return NULL;
		}
	return NULL;
};
//-----------------------------------------------------------------------------

TMacroTag *TSteering::GetMacroByNumber ( int num )
{
	// Function tested; Last changes 950813
	if (num < 0)
		return NULL;
	if (num >= nMacroNumber)
		return NULL; //Datum: 28.07.2002 neu Kullmann+Reinecker
	if ( !aMacroList[num].bIsReady )
		return NULL;
	return &aMacroList[num];
}
//-----------------------------------------------------------------------------

TMacroTag *TSteering::GetMacroByName ( LPSTR name )
{
	// Function tested;  Last changes 950811
	for (int idx= 0; idx < nMacroNumber; idx++)
		if ( !strcmp(name, aMacroList[idx].Name) )
		{
			if ( aMacroList[idx].bIsReady )
				return &aMacroList[idx];
			else
				return NULL;
		}
	return NULL;
};
//-----------------------------------------------------------------------------

BOOL TSteering::DeleteMacro ( TMacroTag *macro )
{
	if ( !macro )
		return FALSE;

	//24.07.2002
	// ! Kullmann+Reinecker: (idx<nMacroNumber) sonst lesen wir ber das Listenende hinweg, wenn das Makro nicht vorhanden ist
	int idx= 0;
	while ( (idx < nMacroNumber) && (strcmp(aMacroList[idx].Name, macro->Name)) )
		idx++;
	if ( idx >= nMacroNumber )
		return FALSE; // das gesuchte Makro ist nicht vorhanden

	// zu gesuchtes Makro lschen
	_FREEOBJ(aMacroList[idx].CmdList);

	// Rest jeweils um eine Position nach vorn kopierem
	idx++;
	while (idx < nMacroNumber)
	{
		aMacroList[idx - 1]= aMacroList[idx];
		idx++;
	}

	// Anzahl der Makros runterzhlen und Speicher freigeben
	nMacroNumber--;
	memset(&(aMacroList[nMacroNumber]), 0, sizeof(TMacroTag));
	return TRUE;
};
//-----------------------------------------------------------------------------

int TSteering::GetMacroCount ( void )
{
	return nMacroNumber;
}

//*****************************************************************************
// POLLING zum Fortschritt der Makroverarbeitung ( ! neu Kullmann+Reinecker )
//*****************************************************************************

BOOL TSteering::IsActive ( void )
{
	return bSteeringActive;
};
//------------------------------------------------------------------------------

BOOL TSteering::IsReset ( void )
{
	return bReset;
}
//------------------------------------------------------------------------------

BOOL TSteering::GetProgress ( int &aCmdIdx, LPSTR aProgress, int &aCmdCount )
{
	aCmdIdx= 0;
	aProgress[0]= 0;
	aCmdCount= 0;
	if ( !IsActive() )
		return FALSE; // derzeit wird nichts verarbeitet, d.h. Verarbeitung bereits beendet ODER es wurde noch kein Kommando verarbeitet

	aCmdIdx= nCmdIdx;
	if ( !TheMacro )
		aCmdCount= 1; //Einzelkommandoverarbeitung
	else
		aCmdCount= TheMacro->Length; //Makroverarbeitung
	if ( !TheCmd )
		return TRUE; // Statusinformationen sind nicht verfgbar

	TheCmd->GetShowData( aProgress ); //Statusinformationen ermitteln
	if ( strlen(aProgress) == 0 )
	{
		TheCmd->GetName( aProgress );
		if ( strlen(aProgress) > 0 )
		{
			char *buf= new char [ strlen(aProgress) + 1 ];
			strcpy( buf, aProgress );
#ifdef GermanVersion

			sprintf( aProgress, "\"%s\"-Kommando...", buf );
#else

			sprintf( aProgress, "\"%s\"-Command...", buf );
#endif

			_FREELIST(buf);
		}
	}
	return TRUE;
}

//*****************************************************************************
// Accessor-/ Mutator-Methoden (einiges neu oder vormals als inline deklariert)
//*****************************************************************************

BOOL TSteering::IsPositionValid ( void )
{
	if ( TheCmd )
		return TheCmd->IsPositionValid();
	else
		return FALSE;
};
//-----------------------------------------------------------------------------

double TSteering::GetDistance ( void )
{
	return mGetValue(Distance);
};
//-----------------------------------------------------------------------------

double TSteering::GetStartPoint ( void )
{
	return dStartPoint;
};
//-----------------------------------------------------------------------------
void TSteering::SetStartPoint ( double aValue)
{
	dStartPoint= aValue;
};
//-----------------------------------------------------------------------------

float TSteering::GetIntensity ( void )
{
	return fIncomingIntensity;
};
//-----------------------------------------------------------------------------

float TSteering::GetSigma ( void )
{
	return Detector->GetSigma();
};
//-----------------------------------------------------------------------------

float TSteering::GetNoiseLevel ( void )
{
	return fNoiseLevel;
};
//-----------------------------------------------------------------------------

void TSteering::SetNoiseLevel ( float nl )
{
	fNoiseLevel= nl;
};
//-----------------------------------------------------------------------------

float TSteering::GetNorm ( void )
{
	return fIntensityNorm;
};
//-----------------------------------------------------------------------------

void TSteering::SetNorm ( float norm )
{
	fIntensityNorm= norm;
};
//-----------------------------------------------------------------------------

BOOL TSteering::IsResponding ( void )
{
	return bSendReport;
};
//-----------------------------------------------------------------------------

void TSteering::SetResponse ( BOOL sr )
{
	bSendReport= sr;
}
//------------------------------------------------------------------------------

double TSteering::GetHwb ( void )
{
	return fHWB;
}
//------------------------------------------------------------------------------

void TSteering::SetHwb ( double aHwb )
{
	fHWB= aHwb;
}

//*****************************************************************************
// PRIVATE METHODS
//*****************************************************************************

BOOL TSteering::LoadMacro (LPSTR makname, LPSTR fn )
{
	TTxtRead file( GetMacroFile(fn) ); // ! neu Kullmann+Reinecker
	char *line;
	while ( (line= file.NextLine()) != 0 )
	{ // suche den Anfang des nchsten Makros
		if ( !strstr(line, "[Common]") )
			continue; // bis zum nchsten "Common", berlesen wir alle Zeilen

		//*** [Common]
		while ( (line= file.NextLine()) != 0 )
		{ // "[Common] ber- und Makro einlesen
			if ( (strlen(line) == 0) || (line[0] == '#') )
				continue; // Leer- und Kommentarzeilen berlesen

			char *token= strtok( line, " =\r\n\t" );
			//*** Name=<Makroname>
			if ( !strcmp(token, "Name") )
			{
				token= strtok(NULL, " =\r\n\t");
				if ( strcmp(makname, token) )
					break; // es ist nicht das gesuchte Makro, also zurck zu "bis das nchste Makro beginnt, berlesen wir alle Zeilen"

				ParsingMacroId(aMacroList[nMacroNumber], token);
				strcpy(aMacroList[nMacroNumber].FileName, fn);
				continue;
			}
			//*** Length=<Anzahl der Kommandos>
			if ( !strcmp(token, "Length") )
			{
				token= strtok(NULL, " =\r\n\t");
				aMacroList[nMacroNumber].Length= atoi(token);
				continue;
			}
			//*** [Commands]
			if ( !strcmp(token, "[Commands]") )
			{ //in den nchsten Zeilen folgen bis zu <Anzahl der Kommandos> Kommandos
				aMacroList[nMacroNumber].CmdList= new TCmdTag[aMacroList[nMacroNumber].Length];
				int cnt= 0;
				while ( (line= file.NextLine()) != 0 )
				{ // [Commandos] ber- und Kommandos (samt -parametern) lesen
					if ( (strlen(line) == 0) || (line[0] == '#') )
						continue; // Leer- und Kommentarzeilen berlesen

					if ( (strstr(line, "[End]")) || (cnt > aMacroList[nMacroNumber].Length) )
						break; // Ende der Kommandoliste ODER Maximalanzahl erreicht: Rest ignorieren

					// das nchste Makro beginnt, ohne das dieses Makro beendet wird (d.h. [End]): Fehlerstabilisierung
					if ( strstr(line, "[Common]") )
					{
						MessageBox( GetFocus(), "Kommandoliste unvollstaendig!", makname, MBSTOP );
						return FALSE;
					}
					// Kommandoparameter
					LPSTR cmd= strtok(line, " \r\n\t");
					LPSTR p1= strtok(NULL, " \r\n\t");
					LPSTR p2= strtok(NULL, " \r\n\t");
					LPSTR p3= strtok(NULL, "\r\n\t");
					if ( ParsingCmd(aMacroList[nMacroNumber].CmdList[cnt], cmd, p1, p2, p3) )
						cnt++; // FALSE: dieses Kommando mit dem nchsten Kommando (ggf. Stop) berschrieben
					if (cnt == aMacroList[nMacroNumber].Length)
						break; // maximale Kommandoanzahl erreicht: nicht weiterlesen
				} // Kommandos (samt -parametern) einlesen

				// wenn kein fehlerfreies Kommando gefunden wurde
				if ( cnt == 0 )
				{
					MessageBox( GetFocus(), "Makro ohne Kommandos!", makname, MBSTOP );
					return FALSE;
				}
				aMacroList[nMacroNumber].Length= cnt; //so viele fehlerfreie Kommandos haben wirklich gelesem
				aMacroList[nMacroNumber].CmdList[cnt - 1].Id= Stop; // das letzte Kommando muss "Stop" sein: Ist dies nicht der Fall wird das letzte Kommando berschrieben
				aMacroList[nMacroNumber].bIsReady= TRUE;
				nMacroNumber++;

				// durchsuche die Liste ob das Makro schon existiert; ist der der Fall wird das alte gelscht
				int idx= nMacroNumber - 1;
				while ( idx > 0 )
				{
					idx--; // wir berspringen das soebene eingelesene (letzte) Makro, sonst ist die Dopplung vorprogrammiert
					if ( !strcmp(aMacroList[nMacroNumber - 1].Name, aMacroList[idx].Name) )
					{ // das soeben gelesene Makro existsiert schon: wir lschen das alte Makro
						DeleteMacro( &aMacroList[idx] ); // ! neu Kullmann+Reiecker: wir verwenden lieber DeleteMacro()
						return TRUE; // das gesuchte Makro wurde erfolgreich einlesen; Dopplung zuvor entfernt
					}
				}
				return TRUE; // das gesuchte Makro wurde erfolgreich einlesen
			} // if "[Commands]"
		} // while "Makro einlesen"
	} // while "suche den Anfang des nchsten Makros"

	// Makrodatei oder gesuchtes Makro nicht vorhanden
	if ( !file.Exists() )
		MessageBox( GetFocus(), "Makrodatei fehlt!", fn, MBSTOP ); //! Makrodatei ist nicht in dem Verzeichnis aus dem die EXE gestartet wurde
	else
		MessageBox( GetFocus(), "Makro fehlt!", makname, MBSTOP );
	return FALSE;
};
//-----------------------------------------------------------------------------

//##ein Kommando ausfuehren: legt Objekt an, das diesem Kommando entspricht
int TSteering::StartCmdExecution ( TCmdTag Cmd )
{
	// Function tested; Last changes 950814
	//##das evtl. gerade ausgefuehrte Kommando stoppen, Werte auf Initial-
	//##zustand zuruecksetzen, neues aktives Kommando erzeugen
	_FREEOBJ(TheCmd);
	Detector->MeasureStop();
	Detector->SetControlWnd(hControlWnd);
	fIntensityNorm= 0.0;
	//Datum: 28.07.2002 nFailureId     = 0;
	bInterrupted= FALSE;
	bReset= FALSE;
	//##dem Kommando entsprechendes Objekt anlegen
	switch (Cmd.Id)
	{
		case GotoPeak:
			TheCmd= (TGotoPeakCmd*)new TGotoPeakCmd(Cmd);
			break;

		case GotoIntensity:
			TheCmd= (TGotoIntensityCmd*)new TGotoIntensityCmd(Cmd);
			break;

		case ChooseAxis:
			TheCmd= (TChooseAxisCmd*)new TChooseAxisCmd(Cmd);
			break;

		case ChooseDetector:
			TheCmd= (TChooseDetectorCmd*)new TChooseDetectorCmd(Cmd);
			break;

		case SetupScan:
			TheCmd= (TSetupScanCmd*)new TSetupScanCmd(Cmd);
			break;

		case SetWidth:
			TheCmd= (TSetWidthCmd*)new TSetWidthCmd(Cmd);
			break;

		case Scan:
			TheCmd= (TScanCmd*)new TScanCmd(Cmd);
			break;
		
		case ControlFlank: //## Nachregeln der Topographie
			TheCmd= (TControlFlankCmd*)new TControlFlankCmd(Cmd);
			break;

		case ShowValue:
			TheCmd= (TShowValueCmd*)new TShowValueCmd(Cmd);
			break;

		case SetFileName:
			TheCmd= (TSetFileNameCmd*)new TSetFileNameCmd(Cmd);
			break;

		case SaveData:
			TheCmd= (TSaveDataCmd*)new TSaveDataCmd(Cmd);
			break;

		case Calculate:
			TheCmd= (TCalculateCmd*)new TCalculateCmd(Cmd);
			break;

		case LoadPoint:
			TheCmd= (TLoadPointCmd*)new TLoadPointCmd(Cmd);
			break;
		
		case MoveToPoint:
			TheCmd= (TMoveToPointCmd*)new TMoveToPointCmd(Cmd);
			break;

		case AreaScan:
			TheCmd= (TAreaScanCmd*)new TAreaScanCmd(Cmd);
			break;

		default:
			return FALSE;
	}

	if ( !TheCmd->nFailureId )
	{ // das gestartete Kommando hat einen Fehlercode zurckgegeben
		bSteeringActive= TRUE;
		StartTimer(0);
		return TRUE;
	}
	return FALSE;
};
//-----------------------------------------------------------------------------

BOOL TSteering::ReadyReaction ( void )
{
	// Function tested; Last changes 950814
	if (TheCmd->GetShowData(MessageBuffer))
		SendReport();
	if (TheMacro)
		ExecuteNextCmd();
	else
		NotifyCmdReady();
	return TRUE;
};
//-----------------------------------------------------------------------------

BOOL TSteering::ExecuteNextCmd ( void )
{
	if ( TheCmd->nFailureId )
	{
		MessageBox(GetFocus(), "Makroausfhrung ist fehlgeschlagen !", "Steuerung", MBSTOP);
		_FREEOBJ(TheCmd);
		return FALSE;
	}
	_FREEOBJ(TheCmd);
	// Function tested; Last changes 950814
	if ( TheCmdList[nCmdIdx].Id == Inquire )
		nCmdIdx += (bInquireResult) ? 2 : 1;
	else
		nCmdIdx++;
	switch (TheCmdList[nCmdIdx].Id)
	{
		case Stop:
			// Kontrolle an Steuerfenster bergeben
			NotifyMacroReady();
			return FALSE;

		case GotoLine:
			nCmdIdx= TheCmdList[nCmdIdx].P1;
			if ( (TheCmdList[nCmdIdx].Id == Stop) || (nCmdIdx > TheMacro->Length - 1) )
			{
				NotifyMacroReady();
				return FALSE;
			}
			return TRUE;

		default:
			StartCmdExecution(TheCmdList[nCmdIdx]);
			return TRUE;
	}
};
//-----------------------------------------------------------------------------

void TSteering::NotifyCmdReady ( void )
{
	// Laeuft ein Macro, dann naechsten Befehl laden
	bSteeringActive= FALSE;
	Detector->MeasureStop();
	Detector->SetControlWnd(NULL);
	Detector->PopSettings();
	Detector->MeasureStart();
	if ( hHostWindow )
		PostMessage(hHostWindow, WM_COMMAND, cm_SteeringReady, 0l);
	hHostWindow= 0;
};
//-----------------------------------------------------------------------------

void TSteering::NotifyMacroReady ( void )
{
	bSteeringActive= FALSE;
	Detector->MeasureStop();
	Detector->SetControlWnd(NULL);
	Detector->PopSettings();
	Detector->MeasureStart();
	if ( hHostWindow )
		PostMessage(hHostWindow, WM_COMMAND, cm_SteeringReady, 0l);
	hHostWindow= 0;
	TheMacro= NULL; // ! neu Kullmann+Reinecker: Bei der nchsten Kommandoverarbeitung wissen wir, ob es ein Makro ( !TheMacro ) verarbeitet wird ODER nur ein Einzelkommendo ( SONST ).
};
//-----------------------------------------------------------------------------

// Callback Timer starten

BOOL TSteering::StartTimer ( BOOL restart )
{
	if ( Steering.nEvent != 0 )
		return FALSE; // bereits aktiv

	//Datum: 28.07.2002 if (restart) nAskIdx--;
	//Datum: 28.07.2002 else nAskIdx= nMaxAskNumber;
	Steering.nEvent= timeSetEvent( nAskTicks, 3, (LPTIMECALLBACK) & TSteering::RecallSteering, 0, TIME_ONESHOT );
	return ( Steering.nEvent != 0 );
};
//-----------------------------------------------------------------------------

//##falls Timer Callback aktiv, diesen stoppen
BOOL TSteering::StopTimer ( void )
{
	// Function tested; Last changes 960322
	if ( Steering.nEvent == 0 )
		return TRUE; //bereits gestoppt

	timeKillEvent( Steering.nEvent );
	Steering.nEvent= 0;
	return TRUE;
};
//-----------------------------------------------------------------------------

void CALLBACK TSteering::RecallSteering ( UINT, UINT, DWORD, DWORD, DWORD )
{
	// Function tested;  Last changes 950814
	//Datum: 28.07.2002 Steering.bTimerActive= FALSE;
	Steering.nEvent= 0;
	PostMessage( GetFrameHandle(), GetWakeUpMsg(), 0, 0l);
	// Steering.WakeUp();
};
//-----------------------------------------------------------------------------

void TSteering::SendReport ( void )
{
	if ( !bSendReport )
		return;
	if ( !strlen(MessageBuffer) )
		return;
	if ( hReportList )
		ListBox_AddString( hReportList, MessageBuffer );
	else
		SetInfo( MessageBuffer );
};

//-----------------------------------------------------------------------------
BOOL TSteering::ParsingMacroId (TMacroTag& macro, LPSTR name )
{
	// Makro: SEARCHREFLECTION
	if ( !strcmp(name, "SearchReflection") )
	{
		macro.Id= SearchReflection;
		strcpy(macro.Name, name);

		// Makro: INQUIREHWB
	}
	else if ( !strcmp(name, "InquireHwb") )
	{
		macro.Id= InquireHwb;
		strcpy(macro.Name, name);

		//Datum: 28.07.2002
		/*// Makro: BATCHMACRO
		} else if ( !strcmp(name,"BatchMacro") ) {
		  macro.Id= BatchMacro;
		  strcpy(macro.Name, name);*/

		// Makro: SCANJOB
	}
	else if ( !strcmp(name, "ScanJob") )
	{
		macro.Id= ScanJob;
		strcpy(macro.Name, name);

		// Makro: AREASSCANJOB
	}
	else if ( !strcmp(name, "AreaScanJob") )
	{
		macro.Id= AreaScanJob;
		strcpy(macro.Name, name);

		//Datum:  28.07.2002
		/*// Makro: MIDDLEOFVALLEY
		} else if ( !strcmp(name,"MiddleOfValley") ) {
		macro.Id= MiddleOfValley;
		strcpy(macro.Name, name);*/

		// Makro:AZIMUTALJUSTIFY
	}
	else if ( !strcmp(name, "AzimutalJustify") )
	{
		macro.Id= AzimutalJustify;
		strcpy(macro.Name, name);

		// Makro: SETUPTOPOGRAPHY
	}
	else if ( !strcmp(name, "SetupTopography") )
	{
		macro.Id= SetupTopography;
		strcpy(macro.Name, name);

		// Makro: TEST
	}
	else if ( !strcmp(name, "Test") )
	{
		macro.Id= Test;
		strcpy(macro.Name, name);

		// UNBEKANNTES MAKRO
	}
	else
	{
		macro.Id= (EMacroId)nMacroNumber;
		sprintf(macro.Name, "Macro%d", nMacroNumber);
		SetInfo("No Match for this Macro-Name.");
		return FALSE;
	}

	// ParsingMacroId darf nur fr noch nicht vorhandene Makros benutzt werden, sonst wird eine Fehlermeldung ausgegeben
	for (int idx= 0;idx < nMacroNumber; idx++)
		if (macro.Id == aMacroList[idx].Id)
		{
			SetInfo("Macro is already loaded.");
			return FALSE;
		}
	return TRUE;
};
//-----------------------------------------------------------------------------

BOOL TSteering::ParsingCmd ( TCmdTag& cmd, LPSTR pcmd, LPSTR p1, LPSTR p2, LPSTR p3 )
{
	int nFailure= 0; //bislang kein Fehler
	char buf[MaxString];

	if ( !strcmp(pcmd, "GotoPeak") )// Kommando: GOTOPEAK
	{
		cmd.Id= GotoPeak;
		cmd.P1= ParsingCmdParam(p1);
	}
	else if ( !strcmp(pcmd, "SaveData") )// Kommando: SAVEDATA
	{
		cmd.Id= SaveData;
		cmd.P1= ParsingCmdParam(p1);
		switch (cmd.P1)
		{
			case 0:
				nFailure= 5;
				break;

			case ForScan:
			case ForAreaScan:
				break;

			default:
				nFailure= 2;
		}
	}
	else if ( !strcmp(pcmd, "SetFileName") )// Kommando: SETFILENAME
	{
		cmd.Id= SetFileName;
		cmd.P1= ParsingCmdParam(p1);
		switch (cmd.P1)
		{
			case 0:
				nFailure= 5;
				break;

			case ForScan:
			case ForAreaScan:
				strcpy(cmd.P3, strtok(p2, " "));
				break;
			
			default:
				nFailure= 2;
		}

		//Datum: 28.07.2002
		/*} else if ( !strcmp(pcmd,"SetupAreaScan") ) {// Kommando: SETUPAREASCAN
		  cmd.Id= SetupAreaScan;
		  cmd.P1= (ECmdParam)ParsingXScanType(p1);
		  switch (cmd.P1) {
				case 0:                                   
					nFailure= 5; 
					break;

				case StandardScan: 
				case Omega2ThetaScan:  
					sprintf(cmd.P3,"%s ",p2); 
					strcat(cmd.P3,p3); 
					break;

				default:                                  
					nFailure= 2; 
		  }*/
	}
	else if ( !strcmp(pcmd, "SetupScan") )// Kommando: SETUPSCAN
	{
		cmd.Id= SetupScan;
		cmd.P1= (ECmdParam)ParsingXScanType(p1);
		switch (cmd.P1)
		{
			case stUnknown:
				nFailure= 5;
				break;

			case stStandardScan:
			case stOmega2ThetaScan:
				if ( !p2 && !p3 )
				{
					nFailure= 5;
					break;
				}
				sprintf(cmd.P3, "%s ", p2);
				strcat(cmd.P3, p3);
				break;

			default:
				nFailure= 2;
		}
	}
	else if ( !strcmp(pcmd, "Scan") ) // Kommando: SCAN
	{
		cmd.Id= Scan;
		cmd.P1= ParsingCmdParam(p1);
		switch (cmd.P1)
		{
			case 0:
				nFailure= 5;
				break;

			case ForScan:
			case ForAreaScan:
				cmd.P2= (ECmdParam)ParsingXScanType(p2);
				switch (cmd.P2)
				{
					case stUnknown:
						nFailure= 5;
						break;
					
					case stOmega2ThetaScan:
					case stStandardScan:
						break;
					
					default:
						nFailure= 2;
				}
				break;

			default:
				nFailure= 2;
		}		
	}
	else if ( !strcmp(pcmd, "AreaScan") ) // Kommando: AREASCAN
	{
		cmd.Id= AreaScan;
		cmd.P1= ParsingCmdParam(p1);
		switch (cmd.P1)
		{
			case 0:
				nFailure= 5;
				break;
			
			case ForAreaScan:
				cmd.P2= (ECmdParam)ParsingXScanType(p2);
				switch (cmd.P2)
				{
					case stUnknown:
						nFailure= 5;
						break;

					case stOmega2ThetaScan:
					case stStandardScan:
						break;

					default:
						nFailure= 2;
				}
				break;

			default:
				nFailure= 2;
		}
	}
	else if ( !strcmp(pcmd, "ShowValue") ) // Kommando: SHOWVALUE
	{ 
		cmd.Id= ShowValue;
		cmd.P1= ParsingCmdParam(p1);
		switch (cmd.P1)
		{
			case 0:
				nFailure= 5;
				break;
	
			case Reflection:
			case Hwb:
				break;
			
			default:
				nFailure= 2;
		}
	}
	else if ( !strcmp(pcmd, "LoadPoint") ) // Kommando: LOADPOINT
	{
		cmd.Id= LoadPoint;
		cmd.P1= ParsingCmdParam(p1);
		switch (cmd.P1)
		{
			case 0:
				nFailure= 5;
				break;
	
			case Argument:
				cmd.P2= (ECmdParam)atoi(p2);
				if ( (cmd.P2 > nMaxArg) || (cmd.P2 < 0) )
					nFailure= 3;
				break;
			
			case Start:
			case Peak:
				break;
	
			default:
				nFailure= 2;
		}		
	}
	else if ( !strcmp(pcmd, "SetWidth") ) // Kommando: SETWIDTH
	{
		cmd.Id= SetWidth;
		if ( atof(p1) != 0.0 )
			strcpy(cmd.P3, p1);
		else
			nFailure= 2;		
	}
	else if ( !strcmp(pcmd, "MoveToPoint") ) // Kommando: MOVETOPOINT
	{ 
		cmd.Id= MoveToPoint;
		cmd.P1= ParsingCmdParam(p1);
		switch (cmd.P1)
		{
			case Start:
			case Peak:
			case Result:
			case LastGoal:
				break;

			case Relative:
				if (atof(p2) == 0.0)
					nFailure= 5;
				else
					strcpy(cmd.P3, p2);
				break;

			case 0:
				if (atof(p1) == 0.0)
					nFailure= 5;
				else
					strcpy(cmd.P3, p1);
				break;

			default:
				nFailure= 2;
		}		
	}
	else if ( !strcmp(pcmd, "Calculate") ) // Kommando: CALCULATE
	{
		cmd.Id= Calculate;
		cmd.P1= ParsingCmdParam(p1);
		switch (cmd.P1)
		{
			case Middle:
			case Hwb:
			case Difference:
			case Opposite:
				break;

			default:
				nFailure= 2;
		}
	}
	else if ( !strcmp(pcmd, "Stop") ) // Kommando: STOP
	{
		cmd.Id= Stop;		
	}
	else if ( !strcmp(pcmd, "GotoLine") ) // Kommando: GOTOLINE
	{
		cmd.Id= GotoLine;
		cmd.P1= (ECmdParam)atoi(p1);
		if ((cmd.P1 <= 0) || (cmd.P1 > aMacroList[nMacroNumber].Length))
			nFailure= 2;
	}
	else if ( !strcmp(pcmd, "GotoIntensity") ) // Kommando: GOTOINTENSITY
	{
		cmd.Id= GotoIntensity;
		cmd.P1= ParsingCmdParam(p1);
		switch (cmd.P1)
		{
			case SmallSide:
			case LargeSide:
				if ( atof(p2) != 0.0 )
				{
					strcpy(cmd.P3, p2);
					break;
				}
				cmd.P2= ParsingCmdParam(p2);
				switch (cmd.P2)
				{
					case Interpolation:
					case BackMove:
						if (atof(p3) == 0.0 )
						{
							nFailure= 5;
							break;
						}
						else
							strcpy(cmd.P3, strtok(p3, " "));
						break;

					case 0:
						nFailure= 2;
						break;
					
					default:
						nFailure= 4;
				} // switch cmd.P2
				break;

			case 0:
				nFailure= 2;
				break;

			default:
				nFailure= 5;
		}		
	}
	else if ( !strcmp(pcmd, "ControlFlank") ) // Kommando: CONTROLFLANK
	{
		cmd.Id= ControlFlank;
	}
	else if ( !strcmp(pcmd, "ChooseAxis") ) // Kommando: CHOOSEAXIS
	{
		cmd.Id= ChooseAxis;
		cmd.P1= (ECmdParam)mlParsingAxis(p1);
		if (!cmd.P1)
			nFailure= 2;		
	}
	else if ( (!strcmp(pcmd, "ChooseDetector")) || (!strcmp(pcmd, "ChooseDevice")) ) // Kommando: CHOOSEDETECTOR
	{
		cmd.Id= ChooseDetector;
		cmd.P1= (ECmdParam)TDetectorManager::DetectorManager().GetIdByDescription(p1);
		if (cmd.P1 != -1)
		{
			if (p2)
			{
				sprintf(cmd.P3, "%s ", p2);
				if (p3)
					strcat(cmd.P3, p3);
			}
		}
		else
			nFailure= 2;		
	}
	else // UNBEKANNTES KOMMANDO
	{
		cmd.Id= Stop;
		nFailure= 1;
	}

	if ( nFailure != 0 )
	{ //Fehler-Id in Klartext umsetzen und Meldungsfenster ausgeben
		switch (nFailure)
		{
			case 1:
				sprintf(buf, "Makrobefehl >%s< wird nicht untersttzt !", pcmd);
				break;

			case 2:
				sprintf(buf, "Paramter %s fr >%s< wird nicht untersttzt !", p1, pcmd);
				break;

			case 3:
				sprintf(buf, "Paramter %s fr >%s< wird nicht untersttzt !", p2, pcmd);
				break;
			
			case 4:
				sprintf(buf, "Paramter %s fr >%s< wird nicht untersttzt !", p3, pcmd);
				break;
			
			case 5:
				sprintf(buf, "Paramter fr >%s< sind unvollstndig !", pcmd);
				break;
		}
		MessageBox(GetFocus(), buf, "Macro einlesen", MBINFO);
		return FALSE;
	}
	else
		return TRUE; //keine Probleme :-)
};
//-----------------------------------------------------------------------------

ECmdParam TSteering::ParsingCmdParam ( LPSTR param )
{
	if ( param == NULL )
		return (ECmdParam)0;

	if ( !strcmp(param, "AbsorberUsed") )
		return AbsorberUsed;
	if ( !strcmp(param, "AreaScanResult") )
		return AreaScanResult;
	if ( !strcmp(param, "Argument") )
		return Argument;
	if ( !strcmp(param, "Array") )
		return Array;
	if ( !strcmp(param, "BackMove") )
		return BackMove;
	if ( !strcmp(param, "DecreaseWidth") )
		return DecreaseWidth;
	if ( !strcmp(param, "DynamicWidth") )
		return DynamicWidth;
	if ( !strcmp(param, "Difference") )
		return Difference;
	if ( !strcmp(param, "Equidistant") )
		return Equidistant;

	if ( !strcmp(param, "ForAreaScan") )
		return ForAreaScan;
	if ( !strcmp(param, "ForScan") )
		return ForScan;
	if ( !strcmp(param, "Hwb") )
		return Hwb;
	if ( !strcmp(param, "IncreasePeak") )
		return IncreasePeak;
	if ( !strcmp(param, "Interpolation") )
		return Interpolation;
	if ( !strcmp(param, "LargeSide") )
		return LargeSide;
	if ( !strcmp(param, "LastGoal") )
		return LastGoal;
	if ( !strcmp(param, "List") )
		return List;
	if ( !strcmp(param, "Max") )
		return Max;
	if ( !strcmp(param, "MaximizeCollimator") )
		return MaximizeCollimator;
	if ( !strcmp(param, "MaximizeGradient") )
		return MaximizeGradient;
	if ( !strcmp(param, "MaximizeTilt") )
		return MaximizeTilt;
	if ( !strcmp(param, "Middle") )
		return Middle;
	if ( !strcmp(param, "Min") )
		return Min;

	if ( !strcmp(param, "Opposite") )
		return Opposite;
	if ( !strcmp(param, "Peak") )
		return Peak;
	if ( !strcmp(param, "Reflection") )
		return Reflection;
	if ( !strcmp(param, "Relative") )
		return Relative;
	if ( !strcmp(param, "Result") )
		return Result;
	if ( !strcmp(param, "ScanResult") )
		return ScanResult;
	if ( !strcmp(param, "SmallSide") )
		return SmallSide;
	if ( !strcmp(param, "Standard") )
		return Standard;
	if ( !strcmp(param, "Start") )
		return Start;
	if ( !strcmp(param, "ToSmallerAngle") )
		return ToSmallerAngle;
	if ( !strcmp(param, "ToSmallerAngle") )
		return ToSmallerAngle;

	return (ECmdParam)0;
}

//*****************************************************************************
// TOTER CODE
//*****************************************************************************

//Datum: 28.07.2002
/*int  TSteering::CallMacro ( TMacroTag* )
{
  /*
  if(pMacro == NewMacroPtr)
	 {
	 // Macro sollte sich nicht selbst aufrufen
//    nFailureId= 12;
	 CmdExecuted();
	 return FALSE;
	 }
  // Sichern von Einstellungen, die Befehlsbergreifend sind
  PushParam();
  return StartMacroExecution(NewMacroPtr);
  * /
  return TRUE;
};
BOOL  TSteering::IsMacroCalled( void ) {
  if ( nCmdIdx < 0 ) return FALSE;
  return TRUE;
};
int  TSteering::ReturnMacroCall( void ) {
  /*
  PopParam();
  nMacroStep++;
  StartCmdExecution(pMacro[nMacroStep]);
  * /
  return TRUE;
};*/

//#############################################################################
// TMacroExecuteDlg
//#############################################################################

TMacroExecuteDlg::TMacroExecuteDlg(TModelessDlg **aRef): TModelessDlg("EXECUTEMACRO", aRef, GetMainInstance() )
{
	bInterrupted= FALSE;
	bActionCaused= FALSE;
};

BOOL TMacroExecuteDlg::Dlg_OnInit(HWND hwnd, HWND hwndCtl, LPARAM)
{
	int idx;

	// Initialisieren der Ablaufsteuerung
	CheckDlgButton(hwnd, id_DetailedReport, Steering.IsResponding());
	CheckDlgButton(hwnd, id_SaveReport, TRUE);
	EnableWindow(GetDlgItem(hwnd, cm_Interrupt), bActionCaused);
	hMacroList= GetDlgItem(hwnd, id_ChooseMacro);
	idx= 0;
	do
	{
		TheMacro= Steering.GetMacroByNumber(idx++);
		if (TheMacro)
			ComboBox_AddString(hMacroList, TheMacro->Name);
	} while (TheMacro); // 22.04.2003 Kullmann+Reinecker: anstatt while (1) { ... if (!TheMacro) break;... }
	TheMacro= Steering.GetMacroByNumber(0);
	if (TheMacro)
		ComboBox_SelectString(hMacroList, 0, TheMacro->Name);
	else
		ComboBox_SelectString(hMacroList, 0, "");
	FORWARD_WM_COMMAND(hwnd, cm_ParamSet, hwndCtl, 0, SendMessage);
	return TRUE;
};

void TMacroExecuteDlg::Dlg_OnCommand(HWND hwnd, int cmd, HWND hwndCtl, UINT codeNotify)
{
	//!  static HWND   hDlgItem;
	int id;
	EMacroId OldId;

	switch (cmd)
	{
		case cm_CounterSet:   //***********************************
			//** Ausgabe der aktuellen Position
			TDetectorManager::DetectorManager().GetDetector()->UpdateViews(TRUE);
			Steering.DetectorRequest();
			break;

		case cm_SteeringReady:
			EnableWindow(GetDlgItem(GetHandle(), cm_Start), TRUE);
			EnableWindow(GetDlgItem(GetHandle(), cm_Interrupt), FALSE);
			if ( TDetectorManager::DetectorManager().GetDetector()->GetCounterWnd() != 0)
				TDetectorManager::DetectorManager().GetDetector()->MeasureStart();
			break;

		case cm_Start:   //****************************************
			if (bInterrupted)
			{
				bInterrupted= Steering.ToggleInterrupt();
				PostMessage(GetHandle(), WM_COMMAND, (WPARAM)cm_ParamSet, 0);
				PostMessage(GetHandle(), WM_COMMAND, (WPARAM)cm_ParamSet, 0);
				break;
			}
			Steering.StartUp(GetHandle(), mlGetAxis(), TDetectorManager::DetectorManager().GetDetector());
			// Testgruppe: Fehler behoben 09.08.2002
			// Steering.Visualising( TRUE, TRUE ); ignorierte Zustand von bSendReport
			Steering.Visualising( Steering.IsResponding(), TRUE );
			if (Steering.StartMacroExecution(TheMacro, GetHandle()))
			{
				bActionCaused= TRUE;
				ListBox_ResetContent(GetDlgItem(GetHandle(), id_Report));
			}
			else
				MessageBox("Fehler beim Starten", "Meldung", MBINFO);
			PostMessage(GetHandle(), WM_COMMAND, (WPARAM)cm_ParamSet, 0);
			break;

		case cm_Interrupt:
			if (!bActionCaused)
				break;
			if (bInterrupted)
			{
				// Macro beenden
				Steering.Reset();
				bActionCaused= FALSE;
				bInterrupted= FALSE;
			}
			else
				bInterrupted= Steering.ToggleInterrupt();
			PostMessage(GetHandle(), WM_COMMAND, (WPARAM)cm_ParamSet, 0);
			break;

		case cm_ParamSet:
#ifdef GermanVersion
			if (bActionCaused && bInterrupted)
			{
				SetWindowText(GetDlgItem(GetHandle(), cm_Start), "Weiter");
				SetWindowText(GetDlgItem(GetHandle(), cm_Interrupt), "Beenden");
			}
			else
			{
				SetWindowText(GetDlgItem(GetHandle(), cm_Start), "Start");
				SetWindowText(GetDlgItem(GetHandle(), cm_Interrupt), "Anhalten");
			}
#else
			if (bActionCaused && bInterrupted)
			{
				SetWindowText(GetDlgItem(GetHandle(), cm_Start), "Continue");
				SetWindowText(GetDlgItem(GetHandle(), cm_Interrupt), "Stop");
			}
			else
			{
				SetWindowText(GetDlgItem(GetHandle(), cm_Start), "Start");
				SetWindowText(GetDlgItem(GetHandle(), cm_Interrupt), "Interrupt");
			}
#endif
			EnableWindow(GetDlgItem(GetHandle(), cm_Start), bInterrupted == bActionCaused);
			EnableWindow(GetDlgItem(GetHandle(), cm_Interrupt), !(!bInterrupted && !bActionCaused));
			EnableWindow(GetDlgItem(GetHandle(), cm_DeleteMacro), !bActionCaused);
			EnableWindow(GetDlgItem(GetHandle(), cm_LoadMacro), !bActionCaused);
			break;

		case id_DetailedReport:
			Steering.SetResponse(!Steering.IsResponding());
			CheckDlgButton(GetHandle(), id_DetailedReport, Steering.IsResponding());
			break;

		case cm_LoadMacro:
			id= ComboBox_GetCount(hMacroList);
			while (id--)
				ComboBox_DeleteString(hMacroList, id);
			Steering.LoadMacroByUser();
			//! Diese (bisherige) Version fuegte am Ende der startbaren Makros eine
			//! Leerzeile ein, die bei Auswahl und anschl. Start zum Absturz fuehrte
			/*
			id= 0;
			for(;;)
			{
				TheMacro= Steering.GetMacroByNumber(id++);
				if(!TheMacro) break;
				ComboBox_AddString(hMacroList,TheMacro->Name);
			}
			*/ 
			//! Workaround nutzt tatsaechliche Makro-Anzahl
			for (id= 0; id < Steering.GetMacroCount(); id++)
			{
				TheMacro= Steering.GetMacroByNumber(id);
				ComboBox_AddString(hMacroList, TheMacro->Name);
			}
			TheMacro= Steering.GetMacroByNumber(0);
			ComboBox_SelectString(hMacroList, 0, TheMacro->Name);
			break;

		case cm_DeleteMacro:
			id= ComboBox_GetCount(hMacroList);
			while (id--)
				ComboBox_DeleteString(hMacroList, id);
			Steering.DeleteMacro(TheMacro);
			id= 0;
			do
			{
				TheMacro= Steering.GetMacroByNumber(id++);
				if (TheMacro)
					ComboBox_AddString(hMacroList, TheMacro->Name);
			} while (TheMacro); // 22.04.2003 Kullmann+Reinecker: anstatt while (1) { ... if (!TheMacro) break;... }
			TheMacro= Steering.GetMacroByNumber(0);
			if (TheMacro)
				ComboBox_SelectString(hMacroList, 0, TheMacro->Name);
			else
				ComboBox_SelectString(hMacroList, 0, "");
			break;

		case id_ChooseMacro:
			if (codeNotify==CBN_SELCHANGE) 
			{
				id= ComboBox_GetCurSel(hMacroList);
				if (CB_ERR == id)
					break;
				OldId= TheMacro->Id;
				TheMacro= Steering.GetMacroByNumber(id);
				if (!TheMacro)
					TheMacro= Steering.GetMacroByNumber(OldId);
			}
			break;

		default:
			TModelessDlg::Dlg_OnCommand(hwnd, cmd, hwndCtl, codeNotify);
	}
};

void TMacroExecuteDlg::LeaveDialog(void)
{
	int count;
	int hFile;
	HWND hwndCtl;
	char lpszBuffer[MaxString];
	OFSTRUCT of;

	if ( IsDlgButtonChecked(GetHandle(), id_SaveReport) )
	{
		hwndCtl= GetDlgItem(GetHandle(), id_Report);
		hFile= OpenFile("macro.log", &of, OF_CREATE);
		MessageLine= ListBox_GetCount(hwndCtl);
		for (count= 0; count < MessageLine; count++)
		{
			ListBox_GetText(hwndCtl, count, lpszBuffer);
			_lwrite(hFile, (LPSTR)lpszBuffer, strlen(lpszBuffer));
			_lwrite(hFile, (LPSTR)"\r\n", strlen("\r\n"));
		}
		_lclose(hFile);
	}
	if (bActionCaused)
	{
		if (!bInterrupted)
			Steering.ToggleInterrupt();
		Steering.Reset();
	}
};

BOOL TMacroExecuteDlg::CanClose(void)
{
	return TRUE;
};

/*27.04.2004
//#############################################################################
// TEditWindow
//#############################################################################

TEditWindow::TEditWindow(LPSTR aFileName) : TMDIWindow()
{
	bIsNewFile= TRUE;
	bFileChanged= FALSE;
	if (aFileName)
		lstrcpy((LPSTR)FileName, aFileName);
	WindowType= wtEdit;

	HMENU TheMenu= CreatePopupMenu();
	AppendMenu(TheMenu, MF_STRING, -1, "Item");
};

TEditWindow::~TEditWindow()
{};

void TEditWindow::Create(HWND)
{
	if (strcmp(FileName, ""))
		ReadFile();
	SetTitle();
};

void TEditWindow::Size(HWND hWnd, WPARAM, LPARAM)
{
	GetWindowRect(hWnd, &WndRect);
};

BOOL TEditWindow::SetTitle()
{
	strcpy(m_Title, "Text:");
	if ( !strcmp(FileName, "") )
	{
		strcat(m_Title, "<untitled.txt>");
		bFileChanged= TRUE;
	}
	else
		strcat(m_Title, FileName);
	return TMDIWindow::SetTitle();
};

void TEditWindow::Paint(HDC hdc, PAINTSTRUCT *ps)
{
	char buf[MaxString];
	HFONT hFont;

	SetMapMode(hdc, MM_TEXT);
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfHeight= 12;
	lf.lfWidth= 4;
	lf.lfWeight= 600;
	lf.lfUnderline= FALSE;
	lstrcpy(lf.lfFaceName, "Courier New");
	hFont= CreateFontIndirect(&lf);
	//Datum: 15.08.2002 das zurckgegebene Objekt wird nur geschrieben, am Ende des Zeichnungsvorgangs aber nicht wiederhergestellt (damit Warning) HGDIOBJ testobc=
	SelectObject(hdc, hFont); // das zurckgegeben Objekt sollte am Ende des Zeichnungsvorgangs wieder ausgewhlt werden!!!
	// Zeichnen der Daten
	sprintf(buf, "%s", "Text");
	DrawText(hdc, (LPSTR)buf, strlen(buf) - 1, &(ps->rcPaint), DT_RIGHT | DT_SINGLELINE);
	DeleteObject(hFont);
};

BOOL TEditWindow::CanClear()
{
	char buf[MaxString];
	int rslt;

	if (IsModified())
	{
		if (!FileName)
			strcpy(buf, "Untitled file has changed. Save?");
		else
			sprintf(buf, "File \"%s\" has changed .  Save?", FileName);
		rslt= MessageBox(GetHandle(), buf, "File Changed", MBASK);
		return (rslt == IDYES ? SaveFile(def) : rslt != IDNO);
	}
	else
		return TRUE;
};

BOOL TEditWindow::ReadFile()
{
	BOOL ret= TRUE;
	return ret;
};

BOOL TEditWindow::NewFile()
{
	if (CanClear())
	{
		ClearWindow();
		ClearModify();
		bIsNewFile= TRUE;
		strcpy(FileName, "");
		SetTitle();
	}
	return TRUE;
};

BOOL TEditWindow::SaveFile(EAskType)
{
	if (IsModified())
		if (FileName) {
			// Inhalt des Fensters speichern
		} else return SaveFileAs();		
	return TRUE;
};*/

//#############################################################################
// LASTLINE
//#############################################################################

